MODULE arrayutil

implicit none

   interface add_points
       module procedure add_point, add_pointv
   end interface

CONTAINS

   integer function clocate(xv,x)
!
!  Cerca la locazione del punto del vettore xv piu' vicino a x
!
   real, dimension(:), intent(in) :: xv
   real              , intent(in) :: x
   integer                        :: n
   integer                        :: jl,ju,jm
!
   n = size(xv)
   if (x <= xv(1)) then
       clocate=1
   else if (x >= xv(n)) then
       clocate=n
   else
       jl = 1  !!!0
       ju = n  !!!!n + 1
       do
         if (ju-jl <= 1) exit
         jm = (ju+jl)/2
         if (x >= xv(jm)) then
             jl = jm
         else
             ju = jm
         endif
       enddo
       if (x-xv(jl) < xv(ju)-x) then
           clocate=jl
       else
           clocate=jl+1
       endif
   endif
!
   end function clocate

! -------------------------------------------------------------------------

   subroutine resize_points(points,minp,maxp)
!
!  Ridimensiona un vettore di tipo point alla dimensione tra tmin e tmax
!
   USE progtype
   USE realloc
   type(point_type), allocatable, dimension(:), intent(inout) :: points
   real, intent(in), optional                                 :: minp,maxp
   integer                                                    :: ipmin,ipmax
   integer                                                    :: newdim,olddim
!
   olddim = size(points)
   if (present(minp)) then
       ipmin = clocate(points%x,minp)                  ! locazione del punto + vicino
       if (points(ipmin)%x < minp) ipmin = ipmin + 1   ! ma non deve essere < di minp
   else
       ipmin = 1
   endif
   if (present(maxp)) then
       ipmax = clocate(points%x,maxp)                ! locazione del punto + vicino
       if (points(ipmax)%x > maxp) ipmax = ipmax - 1 ! ma non deve essere > di maxp
   else
       ipmax = olddim
   endif
   newdim = ipmax - ipmin + 1
   if (newdim > 0 .and. newdim /= olddim) then
       points(1:newdim) = points(ipmin:ipmax)
       call reallocate(points,newdim)
   endif
!
   end subroutine resize_points

! -------------------------------------------------------------------------

   subroutine add_point(points,dati,iposi)
!
!  Aggiunge punto alla fine o in posizione ipos
!
   USE progtype
   USE realloc
   type(point_type), allocatable, dimension(:), intent(inout) :: points
   type(point_type), allocatable, dimension(:) :: pointsc
   type(point_type), intent(in)                               :: dati
   integer, intent(in), optional                              :: iposi
   integer                                                    :: ipos
   integer                                                    :: np
   integer                                                    :: iperr
!
   np = size(points)
!
!  Definisci la posizione in cui inserire il punto
   iperr = 0
   if (present(iposi)) then
       if (iposi > 0 .and. iposi <= np) then
           ipos = iposi
           call reallocate(points,np+1)
           call reallocate(pointsc,np-ipos+1)
           pointsc = points(ipos:np)
           !points(ipos+1:np+1) = points(ipos:np)  ! problema con ifort
           points(ipos+1:) = pointsc(:)
       else
           iperr = 1   ! iposi non ha un valore ragionevole
       endif
   else
       ipos = np+1
       call reallocate(points,np+1)
   endif
!
   if (iperr == 0) then
       points(ipos) = dati
   endif
!
   end subroutine add_point

! -------------------------------------------------------------------------

   subroutine add_pointv(points,dati,iposi)
!
!  Aggiunge punto alla fine o in posizione ipos
!
   USE progtype
   USE realloc
   type(point_type), allocatable, dimension(:), intent(inout) :: points
   type(point_type), allocatable, dimension(:)                :: pointsc
   type(point_type), dimension(:), intent(in)                 :: dati
   integer, intent(in), optional                              :: iposi
   integer                                                    :: ipos
   integer                                                    :: np,nd
   integer                                                    :: iperr
   integer                                                    :: i
!
   np = size(points)
   nd = size(dati)
!
!  Definisci la posizione in cui inserire il punto
   iperr = 0
   if (present(iposi)) then
       if (iposi > 0 .and. iposi <= np) then
           ipos = iposi
           call reallocate(points,np+nd)
           call reallocate(pointsc,np-ipos+1)
           pointsc = points(ipos:np)
           !points(ipos+1:np+1) = points(ipos:np)  ! problema con ifort
           points(ipos+nd:) = pointsc(:)
       else
           iperr = 1   ! iposi non ha un valore ragionevole
       endif
   else
       ipos = np+1
       call reallocate(points,np+nd)
   endif
!
   if (iperr == 0) then
       do i=1,nd
          points(ipos+i-1) = dati(i)
       enddo
   endif
!
   end subroutine add_pointv

! -------------------------------------------------------------------------

   subroutine del_point(points,ipos)
   USE progtype
   USE realloc
   type(point_type), allocatable, dimension(:), intent(inout) :: points
   integer, intent(in)                                        :: ipos
   integer                                                    :: np
!
   np = size(points)
   points(ipos:np-1) = points(ipos+1:np)
   call reallocate(points,np-1)
!
   end subroutine del_point

! -------------------------------------------------------------------------

   logical function check_container(vet1,vet2)  result(okcont)
!
!  okcont=.true. se tutto vet2 è contenuto in vet1
!
   integer, dimension(:), intent(in) :: vet1,vet2
   integer                           :: i
   integer                           :: nv1,nv2
!
   nv1 = size(vet1)
   nv2 = size(vet2)
   if (nv2 > nv1) then
       okcont = .false.
   else
       okcont = .true.
       do i=1,size(vet2)
          if (.not.any(vet1 == vet2(i))) then
              okcont = .false.
              exit
          endif
       enddo
   endif
!
   end function check_container


END MODULE arrayutil
