module atom_basic

  implicit none

  integer, parameter :: SIZELAB = 16
  type atom_type
     real, dimension(3)     :: xc         ! x,y,z
     real                   :: biso       ! fattore termico
     real, dimension(6)     :: bij = 0.0  ! fattori termici anisotropi: b11,b22,b33,b12,b13,b23
     real                   :: och = 1.0  ! occupanza chimica
     real                   :: ocry = 1.0 ! occupanza cristall.
     real                   :: inte       ! intensita di picco
     integer                :: nz = 0     ! nz
     character(len=SIZELAB) :: lab  = ' ' ! label
     integer, dimension(5)  :: rcod = 0   ! posizione nella matrice jac. del parametro se affinato; 0 se non affinato
     integer                :: ptab = 0   ! pointer to table of elements
     real, dimension(3)     :: xsd = 0.0  ! standard deviation on x,y,z
     real                   :: bsd = 0.0  ! standard deviation on biso
     real                   :: osd = 0.0  ! standard deviation on och
  end type atom_type

!F slabnum(slab,num)                        Concatena label con numero d'ordine dell'atomo
!S slabvet(vatm,llab,str,ier)               Genera nella stringa str una lista di atomi separati da virgola
!S get_from_label(label,specl,numb,ierror)  Estrai dalla label la stringa e la sequenza numerica finale


CONTAINS 

   function slabnum(slab,num)
!
!  Concatena label con numero d'ordine dell'atomo
!
   character(len=*), intent(in)     :: slab
   integer, intent(in)              :: num
   character(len=len_trim(slab)+12) :: slabnum
!
   slabnum = ' '
   write(slabnum,'(a,i0,a)')trim(slab)//'(',num,')'
!
   end function slabnum

 !----------------------------------------------------------------------------------

   function slabvet(vatm,llab,showord,maxlen)  result(str)
!
!  Genera nella stringa str una lista di atomi separati da virgola
!
   integer, dimension(:), intent(in)          :: vatm    ! puntatori alle labels
   character(len=*), dimension(:), intent(in) :: llab    ! labels
   logical, intent(in), optional              :: showord ! se vero mostra il numero d'ordine
   integer, intent(in), optional              :: maxlen  ! maximum length of string
   logical                                    :: showord1
   character(len=:), allocatable              :: str    
   integer                                    :: i
   integer                                    :: ipos
   integer                                    :: natlm
!
   if (present(showord)) then
       showord1 = showord
   else
       showord1 = .false. 
   endif
!
   str = ' '
   natlm = size(vatm) 
   if (natlm == 0) return
   ipos = vatm(1)
   if (showord1) then
       str = trim(slabnum(llab(ipos),ipos))
       do i=2,natlm
          ipos = vatm(i)
          if (present(maxlen)) then
              if (len(str) + len_trim(slabnum(llab(ipos),ipos)) + 1 > maxlen) then
                  str = str//' ...'
                  exit
              endif
          endif
          str = str//','//trim(slabnum(llab(ipos),ipos))
       enddo
   else
       str = trim(llab(ipos))
       do i=2,natlm
          ipos = vatm(i)
          if (present(maxlen)) then
              if (len(str) + len_trim(llab(ipos)) + 1 > maxlen) then
                  str = str//' ...'
                  exit
              endif
          endif
          str = str//','//trim(llab(ipos))
       enddo
   endif
!
   end function slabvet

 !----------------------------------------------------------------------------------

   subroutine get_from_label(label,specl,numb,ierror,letter)
!
!  Estrai dalla label la stringa e il seriale
!
   USE strutil
   character(len=*), intent(in)            :: label   !label atomo
   character(len=*), intent(out), optional :: specl   !porzione carattere
   integer, intent(out)                    :: numb    !seriale
   character(len=1), intent(out), optional :: letter  !lettera dopo il seriale
   integer, intent(out)                    :: ierror
   integer                                 :: kpos1,kpos2
   integer                                 :: lenl
   integer                                 :: lenn
   integer                                 :: i
!
   numb = 0
   lenl = len_trim(label)
!
!  Cerca una sequenza di numeri a partire dalla fine
   kpos1 = 0   ! posizione iniziale del numero
   kpos2 = 0   ! posizione finale del numero
   do i=lenl,1,-1
      if (kpos2 == 0 .and. ch_is_digit(label(i:i))) kpos2 = i
      if (kpos2 > 0 .and. .not.ch_is_digit(label(i:i))) then
          kpos1 = i+1
          exit
      endif
   enddo
!
!  Solo un carattere finale e' consentito (es. C23A ma non C23pippo)
   if (kpos1 > 1 .and. kpos2 >= lenl - 1) then
       call s_to_i(label(kpos1:kpos2),numb,ierror,lenn)
   else
       ierror = 1  ! sequenza di soli numeri o sole lettere
   endif
!
   if (present(specl)) then
       if (ierror == 0) then
           specl = label(1:kpos1-1)
       else
           specl = trim(label)
       endif
   endif
!
   if (present(letter)) then
       letter = ' '
       if (ierror == 0) then
           if (kpos2 == lenl-1 .and. ch_is_alpha(label(lenl:lenl))) letter = label(lenl:lenl)
       endif
   endif
   !write(0,*)'label=',trim(label),' numb=',numb,' lab=',trim(specl),' ier=',ierror
!
   end subroutine get_from_label

end module atom_basic
