#include <iostream>
static int gpuhard = 0, cuda = 0;
#include <utils.h>
#include <fftw3.h>

#ifdef HAVE_CUDA
#include <cuda.h>
#include <cuda_runtime.h>
#include <cufft.h>
bool CudaDetect()
{
    int device, deviceCount, gpuDeviceCount=0;
    struct cudaDeviceProp properties;
    cudaError_t cudaResultCode = cudaGetDeviceCount(&deviceCount);
    if(cudaResultCode != cudaSuccess)
       deviceCount = 0;
    for (device=0; device<deviceCount; ++device)
    {
        cudaGetDeviceProperties(&properties, device);
        if(properties.major != 9999)
           ++gpuDeviceCount;
    }
    return (gpuDeviceCount > 0);
}
#endif

extern "C" 
{
   void setcuda_(int *n, int *h)
   {
       cuda = *n;
       gpuhard = *h;
//       g_print("cuda %d gpuhard %d\n", cuda, gpuhard);
   }

#ifdef HAVE_CUDA
   void cudadetectc_(int *n)
   {
       *n = (CudaDetect())?1:0;
   }
  
   cufftHandle sfftw_plan_dft_r2c_1d_cu(int n1)
   {
       cufftHandle plan;
       if(cufftPlan1d(&plan, n1, CUFFT_R2C, 1) != CUFFT_SUCCESS)
       {
          std::cerr << "CUFFT error: Plan creation failed\n";
          exit (1);
       }
       return plan;
   }

   cufftHandle sfftw_plan_dft_c2r_3d_cu(int n1, int n2, int n3)
   {
       cufftHandle plan;
       if(cufftPlan3d(&plan, n1, n2, n3, CUFFT_C2R) != CUFFT_SUCCESS)
       {
          std::cerr << "CUFFT error: Plan creation failed\n";
          exit (1);
       }
       return plan;
   }

   cufftHandle sfftw_plan_dft_r2c_3d_cu(int n1, int n2, int n3)
   {
       cufftHandle plan;
       if(cufftPlan3d(&plan, n1, n2, n3, CUFFT_R2C) != CUFFT_SUCCESS)
       {
          std::cerr << "CUFFT error: Plan creation failed\n";
          exit (1);
       }
       return plan;
   }

   void sfftw_executer2ccu_1d(int n1, cufftHandle *p, cufftReal *xmap,
        cufftComplex *xmapC)
   {
       cufftReal *cu_xmap;
       cufftComplex *cu_xmapC;
       cufftResult outresult;
       cudaError_t out;

       if(cudaMalloc((void **)&cu_xmap, sizeof(cufftReal)*n1) != cudaSuccess)
       {
          std::cerr << "CUDA error: cudaMalloc for cu_xmap failed\n";
          exit (1);
       }
       if(cudaMalloc((void **)&cu_xmapC, sizeof(cufftComplex)*(n1/2+1)) != cudaSuccess)
       {
          std::cerr << "CUDA error: cudaMalloc for cu_xmapC failed\n";
          exit (1);
       }
       if(cudaMemcpy(cu_xmap, xmap, sizeof(cufftReal)*n1,
            cudaMemcpyHostToDevice) != cudaSuccess)
       {
          std::cerr << "CUDA error: cudaMemcpy from xmap to cu_xmap failed\n";
          exit (1);
       }
       outresult = cufftExecR2C(*p, cu_xmap, cu_xmapC);
       if ( outresult == CUFFT_INVALID_PLAN ) 
            std::cerr << "invalid plan\n";
       if ( outresult == CUFFT_INVALID_VALUE ) 
            std::cerr << "invalid value\n";
       if ( outresult == CUFFT_INTERNAL_ERROR ) 
            std::cerr << "internal error\n";
       if ( outresult == CUFFT_EXEC_FAILED ) 
            std::cerr << "exec failed\n";
       if ( outresult == CUFFT_SETUP_FAILED ) 
            std::cerr << "setup failed\n";
       if ( outresult != CUFFT_SUCCESS )
       {
          std::cerr << "CUFFT error: ExecR2C failed\n";
          exit (1);
       }
       out = cudaMemcpy(xmapC, cu_xmapC, sizeof(cufftComplex)*(n1/2+1),
            cudaMemcpyDeviceToHost);
       if ( out == cudaErrorInvalidValue )
            std::cerr << "invalid value\n";
       if ( out == cudaErrorInvalidDevicePointer ) 
            std::cerr << "invalid device pointer\n";
       if ( out == cudaErrorInvalidMemcpyDirection ) 
            std::cerr << "invalid memcpy direction\n";
       if ( out != cudaSuccess )
       {
          std::cerr << "CUDA error: cudaMemcpy from cu_xmapC to xmapC failed\n";
          exit (1);
       }
       cudaFree(cu_xmap);
       cudaFree(cu_xmapC);
   }

   void sfftw_executer2ccu(int n1, int n2, int n3, cufftHandle *p,
         cufftReal *xmap, cufftComplex *xmapC)
   {
       cufftReal *cu_xmap;
       cufftComplex *cu_xmapC;
       cufftResult outresult;
       cudaError_t out;

       if(cudaMalloc((void **)&cu_xmap, sizeof(cufftReal)*n1*n2*n3) != cudaSuccess)
       {
          std::cerr << "CUDA error: cudaMalloc for cu_xmap failed\n";
          exit (1);
       }
       if(cudaMalloc((void **)&cu_xmapC, sizeof(cufftComplex)*n1*n2*(n3/2+1)) != cudaSuccess)
       {
          std::cerr << "CUDA error: cudaMalloc for cu_xmapC failed\n";
          exit (1);
       }
       if(cudaMemcpy(cu_xmap, xmap, sizeof(cufftReal)*n1*n2*n3,
            cudaMemcpyHostToDevice) != cudaSuccess)
       {
          std::cerr << "CUDA error: cudaMemcpy from xmap to cu_xmap failed\n";
          exit (1);
       }
       outresult = cufftExecR2C(*p, cu_xmap, cu_xmapC);
       if ( outresult == CUFFT_INVALID_PLAN ) 
            std::cerr << "invalid plan\n";
       if ( outresult == CUFFT_INVALID_VALUE ) 
            std::cerr << "invalid value\n";
       if ( outresult == CUFFT_INTERNAL_ERROR ) 
            std::cerr << "internal error\n";
       if ( outresult == CUFFT_EXEC_FAILED ) 
            std::cerr << "exec failed\n";
       if ( outresult == CUFFT_SETUP_FAILED ) 
            std::cerr << "setup failed\n";
       if ( outresult != CUFFT_SUCCESS )
       {
          std::cerr << "CUFFT error: ExecR2C failed\n";
          exit (1);
       }
       out = cudaMemcpy(xmapC, cu_xmapC, sizeof(cufftComplex)*n1*n2*(n3/2+1),
            cudaMemcpyDeviceToHost);
       if ( out == cudaErrorInvalidValue )
            std::cerr << "invalid value\n";
       if ( out == cudaErrorInvalidDevicePointer ) 
            std::cerr << "invalid device pointer\n";
       if ( out == cudaErrorInvalidMemcpyDirection ) 
            std::cerr << "invalid memcpy direction\n";
       if ( out != cudaSuccess )
       {
          std::cerr << "CUDA error: cudaMemcpy from cu_xmapC to xmapC failed\n";
          exit (1);
       }
       cudaFree(cu_xmap);
       cudaFree(cu_xmapC);
   }

   void sfftw_executec2rcu(int n1, int n2, int n3, cufftHandle *p,
           cufftComplex *xmapC, cufftReal *xmap)
   {
       cufftReal *cu_xmap;
       cufftComplex *cu_xmapC;
       cufftResult outresult;
       if(cudaMalloc((void**)&cu_xmap, sizeof(cufftReal)*n1*n2*n3)!=cudaSuccess)
       {
          std::cerr << "CUDA error: cudaMalloc for cu_xmap failed \n";
          exit(1);
       }
       if(cudaMalloc((void**)&cu_xmapC, sizeof(cufftComplex)*n1*n2*(n3/2+1))!=cudaSuccess)
       {
          std::cerr << "CUDA error: cudaMalloc for cu_xmapC failed \n";
          exit(1);
       }
       if(cudaMemcpy(cu_xmapC, xmapC, sizeof(cufftComplex)*n1*n2*(n3/2+1),cudaMemcpyHostToDevice)!=cudaSuccess)
       {
          std::cerr << "CUDA error: cudaMemcpy from xmapC to cu_xmapC failed\n";
          exit(1);
       }
       outresult=cufftExecC2R(*p,cu_xmapC,cu_xmap);
       if( outresult == CUFFT_INVALID_PLAN ) std::cerr << "invalid plan\n";
       if( outresult == CUFFT_INVALID_VALUE ) std::cerr << "invalid value\n";
       if( outresult == CUFFT_INTERNAL_ERROR ) std::cerr << "internal error\n";
       if( outresult == CUFFT_EXEC_FAILED ) std::cerr << "exec failed\n";
       if( outresult == CUFFT_SETUP_FAILED ) std::cerr << "setup failed\n";
       if( outresult != CUFFT_SUCCESS )
       {
          std::cerr << "CUFFT error: ExecC2R failed \n";
          exit(0);
       }
       if (cudaMemcpy(xmap, cu_xmap, n1*n2*n3*sizeof(cufftReal),cudaMemcpyDeviceToHost)!=cudaSuccess)
       {
           std::cerr << "CUDA error: cudaMemcpy from cu_xmap to xmap failed\n";
           exit(1);
       }
      cudaFree(cu_xmap);
      cudaFree(cu_xmapC);
   }

   void sfftw_destroy_plancu_(cufftHandle *p)
   {
       cufftDestroy(*p);
   }

#else

   void cudadetectc_(int *n)
   {
       *n = 0;
   }
  
#endif

   void sfftw_executec_(fftwf_plan *p)
   {
        fftwf_execute(*p);
   }

   void sfftw_execute_r2c_1d_(int* n1, void *p, float *vett,
          fftwf_complex *vettC)
   {
#ifdef HAVE_CUDA
     if(cuda && gpuhard)
     {
        cufftHandle *f = (cufftHandle *) p;
        sfftw_executer2ccu_1d(*n1, f, vett, (cufftComplex *) vettC);
        return;
     }
#endif
     fftwf_plan *f = (fftwf_plan *)p;
     fftwf_execute(*f);
   }

   void sfftw_execute_r2c_(int* n1, int* n2, int* n3, void *p,
           float *xmap, fftwf_complex *xmapC)
   {
#ifdef HAVE_CUDA
     if(cuda && gpuhard)
     {
        cufftHandle *f = (cufftHandle *) p;
        sfftw_executer2ccu(*n1, *n2, *n3, f, xmap, (cufftComplex *) xmapC);
        return;
     }
#endif
     fftwf_plan *f = (fftwf_plan *)p;
     fftwf_execute(*f);
   }

   void sfftw_execute_c2r_(int *n1, int *n2, int *n3, void *p,
           fftwf_complex *xmapC, float *xmap)
   {
#ifdef HAVE_CUDA
     if(cuda && gpuhard)
     {
        cufftHandle *f = (cufftHandle *) p;
        sfftw_executec2rcu(*n1, *n2, *n3, f, (cufftComplex *)xmapC, xmap);
        return;
     }
#endif
     fftwf_plan *f = (fftwf_plan *)p;
     fftwf_execute(*f);
   }

long int sfftw_plan_dft_r2c_1dc_(int *n1, float *vInp, fftwf_complex *vOut, int *f)
{
     long int plan;
#ifdef HAVE_CUDA
     if(cuda && gpuhard)
        plan = (long int) sfftw_plan_dft_r2c_1d_cu(*n1);
     else
#endif
        plan = (long int) fftwf_plan_dft_r2c_1d(*n1, vInp, vOut, *f);
     if(!plan)
     {
        std::cerr << "Null plan\n";
        exit(1);
     }
     return plan;
}

long int sfftw_plan_dft_c2r_3dc_(int *n1, int *n2, int *n3, fftwf_complex *xmapC, float *xmap, int *f)
{
     long int plan;
#ifdef HAVE_CUDA
     if(cuda && gpuhard)
        plan = (long int) sfftw_plan_dft_c2r_3d_cu(*n3, *n2, *n1);
     else
#endif
        plan = (long int) fftwf_plan_dft_c2r_3d(*n3, *n2, *n1, xmapC,xmap, *f);
     if(!plan)
     {
        std::cerr << "Null plan\n";
        exit(1);
     }
     return plan;
}

long int sfftw_plan_dft_r2c_3dc_(int *n1, int *n2, int *n3, float *xmap, fftwf_complex *xmapC, int *f)
{
     long int plan;
#ifdef HAVE_CUDA
     if(cuda && gpuhard)
        plan = (long int) sfftw_plan_dft_r2c_3d_cu(*n3, *n2, *n1);
     else
#endif
        plan = (long int) fftwf_plan_dft_r2c_3d(*n3, *n2, *n1, xmap,xmapC, *f);
     if(!plan)
     {
        std::cerr << "Null plan\n";
        exit(1);
     }
     return plan;
}

void sfftw_destroy_planc_(void *p)
{
#ifdef HAVE_CUDA
     if(cuda && gpuhard)
     {
        cufftHandle *f = (cufftHandle *) p;
        sfftw_destroy_plancu_(f);
        return;
     }
#endif
     fftwf_plan *f = (fftwf_plan *)p;
     fftwf_destroy_plan(*f);
}

} // end extern "C"
