MODULE chem_file_fmt

implicit none

CONTAINS

   subroutine read_molfile(j_in,cell,atom,legm,error)
!
!  Legge da MDL (Molecular Design Limited) MolFile specie,coord,connettivita'
!   
   USE realloc
   USE unit_cell
   USE atom_type_util
   USE strutil
   USE errormod
!corr   USE progtype
   USE cgeom
   USE connect_mod
   USE elements
!
   integer, intent(in)                                       :: j_in  ! unita' di lettura del file
   real, dimension(6), intent(in)                            :: cell  ! cella elementare   
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom  ! atomi in uscita
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   type(error_type), intent(out)                             :: error ! errore in lettura
   integer                                                   :: nat
   integer                                                   :: i
   character(len=500)                                        :: line,line2
   integer                                                   :: pos
   integer                                                   :: nconn
   real, dimension(50)                                       :: vet
   integer, dimension(50)                                    :: ivet
!corr   integer                                                   :: iv
   integer                                                   :: icc
   integer                                                   :: nline
   integer                                                   :: ier
   character(len=*), parameter                               :: stralf = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
   integer                                                   :: nlong1,nlong2
   character(len=6)                                          :: version
   integer                                                   :: ierror
   integer                                                   :: iordb
!
!corr   call reset_error(error)   
!
!  Don't read header block
   nline = 1
   read(j_in,*,err=10,end=10)
   nline = nline + 1
   read(j_in,*,err=10,end=10)
   nline = nline + 1
   read(j_in,*,err=10,end=10)
!
!  Read versione, number of atoms e numero di legami  
   nline = nline + 1
   read(j_in,'(i3,i3,27x,a6)',err=10,end=10)nat,nconn,version
   if(index(version,'V3000') /= 0) then 
!     versione 3000 non e' supportata           
!corr      call set_error(error,'File is not MDL Molfile') 
      call error%set('File is not MDL Molfile') 
      go to 20
   endif
!
   call reallocate(atom,nat,.false.) ! allocate on number of atoms
!
!  Read coordinates and labels   
!  xxxxx.xxxxyyyyy.yyyyzzzzz.zzzz aaaddcccssshhhbbbvvvHHHrrriiimmmnnneee
!  0...30   x y z = atom coordinates
!  31..33   aaa = atom symbol
!  34..35   dd = mass difference: -3, -2, -1, 0, 1, 2, 3, 4 ('M  ISO' lines take precedence)
!  36..38   ccc = charge  ('M  CHG' and 'M  RAD' lines take precedence)
!  39..41   sss = atom stereo parity (ignored)
!          ... = query/reaction related
!  48..50   vvv = valence (ignored unless 15, which means 0)
   do i=1,nat
      nline = nline + 1
      read(j_in,'(a)',iostat=ier)line
!
!     read coordinates
      if (len_trim(line) < 30) go to 10
      call s_to_r(line(1:10), vet(1), ier)
      call s_to_r(line(11:20), vet(2), ier)
      call s_to_r(line(21:30), vet(3), ier)
      if (ier > 0) go to 10
      atom(i)%xc = vet(1:3)
!
!     read atom symbol
      pos = scan(line,stralf) ! trova il primo carattere della linea
      line2 = line(pos:)
      call cutst(line2,nlong1,line,nlong2)
      if (nlong2 == 0) go to 10
      atom(i)%lab = line(:nlong2)
   enddo 
!
!  Leggi la connettivita'
   icc = 0
   call reallocate_legm(legm,nconn,savevet=.false.)
   do i=1,nconn
      nline = nline + 1
      read(j_in,'(i3,i3,i3)',err=10,iostat=ierror)ivet(1:3)
      if (ierror < 0) exit
      if (ivet(1) > nat) then
          write(0,'(a,i0,a)')'WARNING:Atom with serial #',ivet(1),' was not found'
          cycle
      endif    
      if (ivet(2) > nat) then
          write(0,'(a,i0,a)')'WARNING:Atom with serial #',ivet(2),' was not found'
          cycle
      endif    
      select case (ivet(3))
         case (1,5,6); iordb = 1
         case (2,4,7); iordb = 2
         case (3);     iordb = 3
         case default; iordb = 1
      end select
      icc = icc + 1
      legm(icc) = bond_type(ivet(1),ivet(2),distanzaC(atom(ivet(1))%xc,atom(ivet(2))%xc),0.03,iordb)
   enddo   
   if (icc /= nconn) call reallocate_legm(legm,icc)
!
!  Jump other rows up to end-row
   do 
      nline = nline + 1
      read(j_in,'(a)',err=10,iostat=ierror)line
      if (ierror < 0) exit
      if (index(line,'END') > 0) exit
   enddo    
!
!  Converti in coordinate cristallografiche
   call cart_to_frac(atom,orthomatrixi_std(cell))
!
!  Metti in nz il puntatore al file xen
   do i=1,nat
      atom(i)%nz = pxen_from_specie(atom(i)%lab)
   enddo
!
   return      
!      
10 error%signal = .true.
   write(error%message,'(a,i0,a)')'Error on reading line ',nline,' of MDL MolFile'
!
20 continue
!
   end subroutine read_molfile  

!-------------------------------------------------------------------------------------------------

   subroutine read_mol2file(j_in,atom,legm,spg,cellmol2,error)
!
!  Legge da MDL (Molecular Design Limited) MolFile specie,coord,connettivita'
!   
   USE realloc
   !USE CCRYST
   USE atom_type_util
   USE strutil
   USE cgeom
   USE errormod
   USE connect_mod
   USE spginfom
   USE elements
   USE fileutil
!
   integer, intent(in)                                       :: j_in  ! unita' di lettura del file
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom  ! atomi in uscita
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm  ! connettivita'
   type(spaceg_type), intent(out)                            :: spg   ! space group
   real, dimension(6), intent(out)                           :: cellmol2
   type(error_type), intent(out)                             :: error ! errore in lettura
   integer                                                   :: i,j
   character(len=500)                                        :: line,line2
   integer                                                   :: pos
   integer                                                   :: natom
   integer                                                   :: nconn
   real, dimension(50)                                       :: vet
   integer, dimension(50)                                    :: ivet
   integer                                                   :: iv
   integer                                                   :: icc
   integer                                                   :: nline
   integer                                                   :: ier
   character(len=10)                                         :: strn
   integer                                                   :: nlong1,nlong2
!
!  Leggi blocco @<TRIPOS>MOLECULE
   nline = 0
   do
      nline = nline + 1
      read(j_in,'(a)',iostat=ier)line
      if (ier < 0) then
          call error%set('It does not seem mol2 file')
          go to 20
      endif    
      if(index(line,'@<TRIPOS>MOLECULE') > 0 ) exit
   enddo
   nline = nline + 1
   read(j_in,'(a)',err=10)line
   nline = nline + 1
   read(j_in,'(a)',err=10)line
   call Getnum(line,vet,ivet,iv)
   if (iv < 2) go to 10
   natom = ivet(1)
   if (natom == 0) then
       go to 10
   else           
       allocate(atom(natom))
   endif
   nconn = ivet(2)
   spg = init_spaceg_type()
   cellmol2(:) = (/10.0,10.0,10.0,90.,90.,90./) 
!   
!  Leggi blocco @<TRIPOS>ATOM
   !do 
   !   nline = nline + 1
   !   read(j_in,'(a)',iostat=ier)line
   !   if (ier < 0) then
   !       call set_error(error,'It does not seem mol2 file')
   !       go to 20
   !   endif    
   !   if(index(line,'@<TRIPOS>ATOM') > 0 ) exit
   !enddo
   call find_key_file(j_in,'@<TRIPOS>ATOM',nline,line,ier)
   if (ier < 0) then
!corr       call set_error(error,'It does not seem mol2 file')
       call error%set('It does not seem mol2 file')
       go to 20
   endif    
   do i=1,natom
      nline = nline + 1
      read(j_in,'(a)',err=10)line2
      call cutst(line2,nlong1,line,nlong2)
      call cutst(line2,nlong1,line,nlong2)
!      
!     Leggi le coordinate
      do j=1,3
         call cutst(line2,nlong1,line,nlong2)      
         if (nlong2 == 0) go to 10
         call s_to_r(line,atom(i)%xc(j),ier,nlong1)
         if (ier > 0) go to 10
      enddo
!      
!     Leggi atom type
      call cutst(line2,nlong1,line,nlong2)
      if (nlong2 == 0) go to 10
      pos = index(line,'.') - 1
      if (pos <= 0) then
          atom(i)%lab = line(:nlong2)
      else              
          atom(i)%lab = line(:pos)
      endif
   enddo
!
!  Leggi blocco @<TRIPOS>BOND
   !nline = nline + 1
   !read(j_in,'(a)',err=10)line
   !if(index(line,'@<TRIPOS>BOND') == 0 .and. nconn > 0) then
   !   call set_error(error,'Cannot find <TRIPOS>BOND section')
   !   go to 20
   !endif   
   call find_key_file(j_in,'@<TRIPOS>BOND',nline,line,ier)
   if (ier < 0) then
!corr       call set_error(error,'Cannot find <TRIPOS>BOND section')
       call error%set('Cannot find <TRIPOS>BOND section')
       go to 20
   endif    
   call reallocate_legm(legm,nconn,savevet=.false.)
   icc = 1
   do i=1,nconn
      nline = nline + 1
      read(j_in,'(a)',err=10)line
      call Getnum(line,vet,ivet,iv)
      if (iv < 3) go to 10
      if (ivet(2) > natom) then
          write(0,30)nline,ivet(2)
          cycle
      endif    
      if (ivet(3) > natom) then
          write(0,30)nline,ivet(3)
          cycle
      endif    
      legm(i)%n1 = ivet(2)
      legm(i)%n2 = ivet(3)
      legm(i)%dist = distanzaC(atom(ivet(2))%xc,atom(ivet(3))%xc)
      legm(i)%sigma = 0.03
   enddo   
!
!  Read crystallografic information
   call find_key_file(j_in,'@<TRIPOS>CRYSIN',nline,line,ier)
   if (ier == 0) then
       read(j_in,'(a)',iostat=ier)line
       if (ier == 0) then
           call Getnum(line,vet,ivet,iv)
           if (iv == 8) then
               cellmol2(:6) = vet(:6)
               spg = init_spaceg_type(numb = ivet(7),code=ivet(8))
           endif
       endif
       !!!!write(0,*)'LINE=',vet(:8)
   endif
   !do 
   !   nline = nline + 1
   !   read(j_in,'(a)',iostat=ier)line
   !   if (ier < 0) exit
   !   if (index(line,'@<TRIPOS>CRYSIN') > 0) then
   !       read(j_in,'(a)',iostat=ier)line
   !       if (ier == 0) then
   !           call Getnum(line,vet,ivet,iv)
   !           if (iv == 8) then
   !               cellmol2(:6) = vet(:6)
   !               spg = init_spaceg_type(numb = ivet(7),code=ivet(8))
   !           endif
   !           !!!!write(0,*)'LINE=',vet(:8)
   !           exit
   !       endif
   !   endif
   !enddo
!
!  Converti in coordinate cristallografiche
!!!!!corr   call cart_to_frac(atom,orthomatrixi(cell))
!
!  Metti in nz il puntatore al file xen
   do i=1,natom
      atom(i)%nz = pxen_from_specie(atom(i)%lab)
   enddo
!
   return      
!      
10 write(strn,'(i0)')nline
!corr   call set_error(error,'Error on reading line '//trim(strn)//' of mol2 file')   
   call error%set('Error on reading line '//trim(strn)//' of mol2 file')   
30 format('WARNING (line ',i0,'): Atom with serial #',i0,' was not found')
20 continue
!
   end subroutine read_mol2file  

!-------------------------------------------------------------------------------------------------

   subroutine read_xyzfile(j_in,cellc,atom,error)
   USE strutil
   USE atom_type_util
   USE errormod
   USE unit_cell
   USE realloc
   USE elements
   integer, intent(in)                                       :: j_in    ! unita' di lettura del file
   real, dimension(6), intent(in)                            :: cellc   ! cella corrente
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom    ! atomi in uscita
   type(error_type), intent(out)                             :: error   ! errore in lettura
   integer                                                   :: nline
   character(len=500)                                        :: line,line2
   integer                                                   :: nlong1,nlong2
   real, dimension(50)                                       :: vet
   integer, dimension(50)                                    :: ivet
   integer                                                   :: iv
   integer                                                   :: natom
   integer                                                   :: i
   character(len=10)                                         :: strn
   integer                                                   :: ier
   integer                                                   :: n,nread
!
!corr   call reset_error(error)   
!
!  Leggi il numero di atomi dalla prima riga
   nline = 1
   read(j_in,*,err=10)natom
   if (natom == 0) then
!corr       call set_error(error,'Cannot read this file')
       call error%set('Cannot read this file')
       go to 20
   else
       allocate(atom(natom))
   endif
!
!  Salta riga con commento  
   nline = nline + 1
   read(j_in,*,err=10)
!
!  Leggi natom atomi
   nread = 0   ! conta il numero di atomi letti
   do n=1,natom
      nline = nline + 1
      read(j_in,'(a)',iostat=ier)line
      if (ier < 0) exit
!
!     Leggi simbolo chimico 
      call cutst(line,nlong1,line2,nlong2)
      if (nlong2 > 2 .or. .not.s_is_alpha(line2(:nlong2))) go to 10   
      nread = nread + 1
      atom(nread)%lab = line2(1:2)
!
!     Leggi coordinate frazionarie
      call Getnum(line,vet,ivet,iv)
      if (iv /= 3) go to 10
      atom(nread)%xc = vet(1:3)
   enddo
!  considera che per errore
!  nread potrebbe essere inferiore ad natom
   call reallocate_model(atom,nread)
!
!  Non modificare la cella corrente 
   call cart_to_frac(atom,orthomatrixi_std(cellc))
!
!  Metti in nz il puntatore al file xen
   do i=1,natom
      atom(i)%nz = pxen_from_specie(atom(i)%lab)
   enddo
!
   return
10 write(strn,'(i0)')nline
!corr   call set_error(error,'Error on reading line '//trim(strn))   
   call error%set('Error on reading line '//trim(strn))   
!20 call print_error_message(error)
20 continue
!
   end subroutine read_xyzfile

!-------------------------------------------------------------------------------------------------

   subroutine create_molfile(filename,jfile,cella,atom,legm,progname,error)
!
!  Crea un  MDL (Molecular Design Limited) MolFile 
!   
   USE errormod
   USE atom_type_util
   USE unit_cell   
   USE fileutil
   USE connect_mod
   USE elements, only: NLEN_LAB
   character(len=*), intent(in), optional     :: filename  ! se presente apre il file
   integer, intent(in), optional              :: jfile     ! unita' di lettura del file
   real, dimension(6), intent(in)             :: cella     ! cella elementare   
   type(atom_type), dimension(:), allocatable, intent(in)  :: atom  
   type(atom_type), dimension(size(atom))     :: atomc 
   type(bond_type), dimension(:), allocatable, intent(in)  :: legm      ! connettivita'        
   character(len=*), intent(in)                            :: progname
   type(error_type), intent(out)              :: error     ! errore in scrittura
   integer                                    :: num_atoms,num_bonds
   integer                                    :: chiral_flag
   integer                                    :: nprop_lines
   integer, parameter                         :: izero = 0
   character(len=5), parameter                :: version = 'V2000'
   character(len=NLEN_LAB)                    :: labat
   integer                                    :: i,j
   character(len=10)                          :: date,time
   integer                                    :: j_in
   integer                                    :: ier
   type(bond_type), dimension(:), allocatable  :: legmol 
!
!  Apertura del file
   if (present(filename)) then
       call get_unit(j_in)
       open(unit=j_in,file=filename,iostat=ier,status='unknown')
   else
       j_in = jfile   ! in questo caso il file e' stato gia' aperto
   endif
!
!corr   call reset_error(error)      
!   
!  Header block (nome del programma, data, ora)
   call date_and_time(date,time)
   write(j_in,'(/2x,a/)')trim(progname)//date(5:8)//date(3:4)//time(1:4)
!
!  Counts line (number of atoms,number of bonds,...)
   num_atoms = size(atom)
   num_bonds = numbonds(legm)
   chiral_flag = 0    ! non gestito al momento
   nprop_lines = 1
   if (num_atoms > 999) then
!corr       call set_error(error,'Number of atoms is too high')
       call error%set('Number of atoms is too high')
       go to 10    
   endif
   if (num_bonds > 999) then
!corr       call set_error(error,'Number of bonds is too high')
       call error%set('Number of bonds is too high')
       go to 10    
   endif   
   write(j_in,'(i3,i3,i3,i3,i3,15x,i3,1x,a)')num_atoms,num_bonds,izero,izero,chiral_flag,nprop_lines,version
!
!  Atom block
   atomc = atom
   call frac_to_cart(atomc,orthomatrix_std(cella))                !converti in coord. cartesiasane
   do i=1,num_atoms
      labat = atomic_specie(atomc(i))
      write(j_in,'(3f10.4,1x,a2,1x,6i3)')atomc(i)%xc,labat,(izero,j=1,6)
   enddo          
!
!  Bond block
   call copia_legm(legmol,legm)
   call bond_type_perception(atom,legmol)
   do i=1,numbonds(legmol)
      write(j_in,'(4i3)')legmol(i)%n1,legmol(i)%n2,legmol(i)%ord,izero
   enddo
!
!  Properties block
   write(j_in,'(a)')'M  END'
!
10 if (present(filename)) close(j_in)
!
   end subroutine create_molfile

!-------------------------------------------------------------------------------------------------

   subroutine read_zmtfile(j_in,cell,atom,error)
!   
!  Legge Fenske-Hall zmatrix
!  Generica riga i: 
!    label(i) Conn(1,i) I_coor(1,i) Conn(2,i) I_coor(2,i) Conn(3,i) I_coor(3,i)
!
   USE unit_cell
   USE atom_type_util
   USE strutil
   USE errormod
   !USE ptable
   USE elements
   integer, intent(in)                                       :: j_in
   real, dimension(:), intent(in)                            :: cell
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom    ! atomi in uscita
   real, dimension(:,:), allocatable                         :: I_coor  ! Internal coordinates (d,ang,tors)
   integer, dimension(:,:), allocatable                      :: Conn    ! interi in colonna 2,4,6
   integer                                                   :: natom   ! numero di righe = num. di atomi
   logical                                                   :: kpr = .false.
   integer                                                   :: nline
   type(error_type), intent(out)                             :: error 
   character(len=500)                                        :: line1,line2
   integer                                                   :: nlong1,nlong2
   integer                                                   :: i
   real, dimension(50)                                       :: vet
   integer, dimension(50)                                    :: ivet
   integer                                                   :: iv
   character(len=10)                                         :: strn
!
!corr   call reset_error(error)   
!   
!  La prima riga e' vuota
   nline = 1
   read(j_in,*,err=10)
!   
!  La seconda riga contiene il numero di righe da leggere
   nline = nline + 1
   read(j_in,*,err=10)natom
   if (natom <= 0) then
!corr       call set_error(error,'Error in zmt file: not atoms are defined')
       call error%set('Error in zmt file: not atoms are defined')
       go to 20
   endif        
   allocate(atom(natom),Conn(3,natom),I_coor(3,natom))
   Conn(:,:) = 0
   I_coor(:,:) = 0.0
!   
   do i=1,natom
      nline = nline + 1
      read(j_in,'(a)',err=10)line1
!      
!     Leggi la specie sulla prima colonna
      call cutst(line1,nlong1,line2,nlong2)
      if (nlong2 > 2) go to 10
      atom(i)%lab = line2(:nlong2)
!      
!     Leggi il resto della riga
      call Getnum(line1,vet,ivet,iv)
      select case (i) 
         case (1)       ! prima riga
           if(iv < 1) go to 10        
           Conn(1,i) = ivet(1)
           if (kpr) write(0,'(a,1x,i4)')atom(i)%lab,Conn(1,i)

         case (2)       ! seconda riga
           if(iv < 2) go to 10        
           Conn(1,i) = ivet(1)
           I_coor(1,i) = vet(2)
           if (kpr) write(0,'(a,1x,i4,1x,f10.4)')atom(i)%lab,Conn(1,i),I_coor(1,i)

         case (3)       ! terza riga      
           if(iv < 4) go to 10        
           Conn(1,i) = ivet(1)
           I_coor(1,i) = vet(2)
           Conn(2,i) = ivet(3)
           I_coor(2,i) = vet(4)
           if (kpr) write(0,'(a,2(1x,i4,1x,f10.4))')atom(i)%lab,Conn(1,i),I_coor(1,i), &
                    Conn(2,i),I_coor(2,i)

         case (4:)      ! le altre righe  
           if(iv < 6) go to 10        
           Conn(1,i) = ivet(1)
           I_coor(1,i) = vet(2)
           Conn(2,i) = ivet(3)
           I_coor(2,i) = vet(4)
           Conn(3,i) = ivet(5)
!corr           I_coor(3,i) = dangle_norm(vet(6))
   ! cosi si corregge errore segnalato da Akihiro
           I_coor(3,i) = -dangle_norm(vet(6))
           if (kpr) write(0,'(a,3(1x,i4,1x,f10.4))')atom(i)%lab,Conn(1,i),I_coor(1,i),  &
                    Conn(2,i),I_coor(2,i),Conn(3,i),I_coor(3,i)

      end select
   enddo
!
!  Converti coordinate interne in coordinate cartesiane
   call Zmatrix_to_Cartesian(natom,I_coor,conn)
   if (kpr) then
       do i=1,natom
          write(0,'(i4,3f10.3,3i6)')i,I_coor(:3,i),conn(:3,i)
       enddo
   endif        
!   
!  Trasferisci in atom le coordinate cartesiane
   do i=1,natom
      atom(i)%xc = I_coor(:3,i)
   enddo
!
!  Converti in coordinate cristallografiche
   call cart_to_frac(atom,orthomatrixi_std(cell))
!
!  Metti in nz il puntatore al file xen
   do i=1,natom
      atom(i)%nz = pxen_from_specie(atom(i)%lab)
   enddo
! 
   return
!   
10 write(strn,'(i0)')nline
!corr   call set_error(error,'Error on reading line '//trim(strn)//' of zmt file')   
   call error%set('Error on reading line '//trim(strn)//' of zmt file')   
!
20 continue
!   
   end subroutine read_zmtfile


!-------------------------------------------------------------------------------------------------

   function dangle_norm(angle) result(dang)
!   
!  Normalizza un angolo in gradi tra -180 e 180
!
   real, intent(in) :: angle
   real             :: dang
!   
   dang = mod(angle+1080,360.0)   ! 1080= 180*6
   if (dang > 180) dang = dang - 360
!
   end function dangle_norm

!-------------------------------------------------------------------------------------------------

   Subroutine Zmatrix_to_Cartesian(na,I_coor,conn)
   USE cgeom
   USE trig_constants
   integer, intent(in)                    :: na
   real, dimension(:,:), intent(inout)    :: I_coor
   integer, dimension(:,:), intent(inout) :: conn
   integer                                :: i,j,k,n
   real                                   :: dist, ang
   real, dimension(3)                     :: ci,ri,rj,rk,rn
!
!  First atom is always at origin (Z-matrix)
   I_coor(:,1) = 0.0
   conn(:,1) = 0
!   
!  Second atom is always along "x"
   I_coor(2:3,2) = 0.0
   conn(2:3,2) = 0
   conn(1,2)   = 1
!
!  Third atom is always in the "xy" plane       !A(i) d_ij  ang_ijk   dang_ijkl  j k l
   if (conn(1,3) == 1) then
       conn(2,3) = 2
       conn(3,3) = 0
       dist= I_coor(1,3)
       ang = I_coor(2,3)*dtor
       I_coor(1,3) = dist * cos(ang)
       I_coor(2,3) = dist * sin(ang)
       I_coor(3,3) = 0.0
    else
       conn(1,3) = 2
       conn(2,3) = 1
       conn(3,3) = 0
       dist= I_coor(1,3)
       ang = I_coor(2,3)*dtor
       I_coor(1,3) = dist * cos(pi-ang) +  I_coor(1,2)
       I_coor(2,3) = dist * sin(pi-ang)
       I_coor(3,3) = 0.0
    end if

    do i=4,na
       ci(:) = I_coor(:,i)
       j     = conn(1,i)         !The connectivity is needed for the Z-matrix description
       k     = conn(2,i)         !If the connectivity is given it is possible to transform to
       n     = conn(3,i)         !Z-matrix if cartesian/spherical coordinates are given.
       if (j == 0 .or. k == 0 .or. n == 0) cycle
       rj(:) = I_coor(:,j)
       rk(:) = I_coor(:,k)
       rn(:) = I_coor(:,n)
       call get_cartesian_from_Z(ci,ri,rj,rk,rn)
       I_coor(:,i) = ri
    end do

    end subroutine Zmatrix_to_Cartesian
   
!------------------------------------------------------------------------------------------
   subroutine create_zmtfile(filename,jfile,atom,legm,cella)
!
!  Crea un file z-matric secondo lo standard stan
!
   USE atom_type_util
   USE connect_mod
   USE fileutil
   USE cgeom
   USE unit_cell
   character(len=*), intent(in), optional                  :: filename  ! se presente apre il file
   integer, intent(in), optional                           :: jfile     ! unita' di lettura del file
   type(atom_type), dimension(:), intent(in)               :: atom      ! atomi in uscita
   type(bond_type), dimension(:), allocatable, intent(in)  :: legm      ! atomi in uscita
   real, dimension(6), intent(in)                          :: cella     ! cella elementare   
   type(atom_type), dimension(size(atom))                  :: atomc     ! atomi in uscita   
   type(bond_type), dimension(:), allocatable              :: legmc 
   integer                                                 :: j_in
   integer                                                 :: ier
   real, dimension(3,size(atom))                           :: I_coor 
   integer, dimension(3,size(atom))                        :: conn 
   integer                                                 :: nat
           integer :: i
!
!  Apertura del file
   if (present(filename)) then
       call get_unit(j_in)
       open(unit=j_in,file=filename,iostat=ier,status='unknown')
   else
       j_in = jfile   ! in questo caso il file e' stato gia' aperto
   endif
!
   call copia_legm(legmc,legm)
!
   nat = size(atom)
!
!  Connetti forzatamente atomi isolati
   do i=1,nat
      if (number_of_bonds(legmc,i) == 0) then
          call force_connectivity(atom,legmc,i)
      endif
   enddo
!
!  Converti in coord. cartesiane
   atomc(:) = atom
   call frac_to_cart(atomc,orthomatrix_std(cella))
!
   call cartesian_to_zmatrix(atomc,legmc,I_coor,conn)

   write(j_in,'(/i0)')nat
   write(j_in,'(a,2x,i6)')atomic_specie(atomc(1)),conn(1,1)
   if (nat > 1) then
       write(j_in,'(a,2x,i6,f8.3)')atomic_specie(atomc(2)),conn(1,2),I_coor(1,2)
       if (nat > 2) write(j_in,'(a,2x,i6,f8.3,i6,f10.3)')atomic_specie(atomc(3)),conn(1,3),I_coor(1,3),conn(2,3),I_coor(2,3)
   endif
   do i=4,nat
      if (I_coor(3,i) < 0) I_coor(3,i) = 360 + I_coor(3,i) ! normalizzazione tra 0-360
      write(j_in,'(a,2x,i6,f8.3,i6,f10.3,i6,f10.1)')atomic_specie(atomc(i)),conn(1,i),   &
      I_coor(1,i),conn(2,i),I_coor(2,i),conn(3,i),I_coor(3,i)
   enddo
!
   if (present(filename)) close(j_in)
!
   end subroutine create_zmtfile

!------------------------------------------------------------------------------------------

   subroutine cartesian_to_zmatrix(atom,legm,I_coor,conn)
   USE connect_mod
   USE cgeom
   USE trig_constants
!corr   USE General, only:rtod
   USE atom_type_util
   type(atom_type), intent(inout), dimension(:) :: atom
   type(bond_type), intent(in), dimension(:) :: legm
   real, dimension(:,:), intent(out)         :: I_coor
   integer, dimension(:,:), intent(out)      :: conn
   integer, dimension(size(atom),size(atom)) :: T_Conn
   real, dimension(size(atom),size(atom))    :: T_Dist
   integer                                   :: i,j,k,l,m,n
   integer                                   :: nc1,nc2,nc3
   integer                                   :: na
   real, dimension(3)                        :: ri,rj,rk,rn,ci
   real                                      :: dist,ang
!
!  Genera da legm tabella delle distanze   
   T_Conn(:,:) = 0
   T_Dist(:,:) = 0.0
   do i=1,size(legm)
      nc1 = legm(i)%n1
      nc2 = legm(i)%n2
      if (nc1 < nc2) then
          T_Conn(nc1,nc2) = nc1
          T_Conn(nc2,nc1) = nc1
      else
          T_Conn(nc1,nc2) = nc2
          T_Conn(nc2,nc1) = nc2
      endif
      T_Dist(nc1,nc2) = legm(i)%dist
      T_Dist(nc2,nc1) = legm(i)%dist
        write(6,*)i,'leg=',nc1,nc2
   enddo
!
   na = size(atom)
   do i=1,na
!      
!     Distances: Fill N1
      j=minloc(T_Dist(i,1:i-1),dim=1,mask=(T_Dist(i,1:i-1) > 0.0))
      conn(1,i)=j
!
!     Angles: Fill N2
      if (i > 2) then
         nc1=count((T_Conn(j,1:i-1) > 0 .and. T_Conn(j,1:i-1) /=j),dim=1)
         nc2=count((T_Conn(i,1:i-1) > 0 .and. T_Conn(i,1:i-1) /=j),dim=1)
         k=0
         if (nc1 > 0) then
            do
               k=minloc(T_Dist(j,1:i-1),dim=1, mask=(T_Dist(j,1:i-1) > 0.0))
               if (k == j) then
                  T_Dist(j,k)=-T_Dist(j,k)
                  cycle
               else
                  exit
               end if
            end do
         elseif (nc2 > 0) then
            do
               k=minloc(T_Dist(i,1:i-1),dim=1, mask=(T_Dist(i,1:i-1) > 0.0))
               if (k == j) then
                  T_Dist(i,k)=-T_Dist(i,k)
                  cycle
               else
                  exit
               end if
            end do
         end if
         if (k == 0) then
            !scegli uno qualsiasi
            do l=1,i-1
               if (l == j) cycle
               k=l
               exit
            end do
         end if
         Conn(2,i)=k
      end if
      T_Dist=abs(T_Dist)
!
!     Torsion 
      if (i > 3) then
         nc1=count((T_Conn(k,1:i-1) > 0 .and. T_Conn(k,1:i-1) /=j .and. T_Conn(k,1:i-1) /=k),dim=1)
         nc2=count((T_Conn(j,1:i-1) > 0 .and. T_Conn(j,1:i-1) /=j .and. T_Conn(j,1:i-1) /=k),dim=1)
         nc3=count((T_Conn(i,1:i-1) > 0 .and. T_Conn(i,1:i-1) /=j .and. T_Conn(i,1:i-1) /=k),dim=1)

         l=0
         if (nc1 > 0) then
            do
               l=minloc(T_Dist(k,1:i-1),dim=1, mask=(T_Dist(k,1:i-1) > 0.0))
               if (l == j .or. l == k) then
                  T_Dist(k,l)=-T_Dist(k,l)
                  cycle
               else
                  exit
               end if
            end do
         elseif (nc2 > 0) then
            do
               l=minloc(T_Dist(j,1:i-1),dim=1, mask=(T_Dist(j,1:i-1) > 0.0))
               if (l == j .or. l == k) then
                  T_Dist(j,l)=-T_Dist(j,l)
                  cycle
               else
                  exit
               end if
            end do
         elseif (nc3 > 0) then
            do
               l=minloc(T_Dist(i,1:i-1),dim=1, mask=(T_Dist(i,1:i-1) > 0.0))
               if (l == j .or. l == k) then
                  T_Dist(i,l)=-T_Dist(i,l)
                  cycle
               else
                  exit
               end if
            end do
         end if
         if (l==0) then
            !scegli uno qualsiasi
            do m=1,i-1
               if (m == j .or. m == k) cycle
               l=m
               exit
            end do
         end if
         conn(3,i)=l
      end if
      T_Dist=abs(T_Dist)
   enddo
!
!  First atom is always at origin (Z-matrix)
   I_Coor(:,1) = 0.0
   conn(1,1) = 1
   conn(2:3,1) = 0
!
!  Second atom is always along "x"
   if (na > 1) then
       ri= atom(2)%xc - atom(1)%xc
       dist=sqrt(dot_product(ri,ri))
       I_Coor(1,2)   = dist
       I_Coor(2:3,2) = 0.0
       conn(2:3,2)   = 0
       conn(1,2)     = 1
!
!      Third atom is always in the "xy" plane ----!
!      A(i) d_ij  ang_ijk   dang_ijkl  j k l
       if (na > 2) then
           if (conn(1,3) == 1) then
              conn(2,3) = 2
              conn(3,3) = 0
              ri = atom(3)%xc - atom(1)%xc
              rj = atom(2)%xc - atom(1)%xc
              dist= sqrt(dot_product(ri,ri))
              !ang = acosd(dot_product(ri,rj)/dist/sqrt(dot_product(rj,rj)))
              ang = rtod*acos(dot_product(ri,rj)/dist/sqrt(dot_product(rj,rj)))
              I_coor(1,3) = dist
              I_coor(2,3) = ang
              I_coor(3,3) = 0.0
           else
              conn(1,3) = 2
              conn(2,3) = 1
              conn(3,3) = 0
              ri = atom(3)%xc - atom(2)%xc
              rj = atom(1)%xc - atom(2)%xc
              dist= sqrt(dot_product(ri,ri))
              !ang = acosd(dot_product(ri,rj)/dist/sqrt(dot_product(rj,rj)))
              ang = rtod*acos(dot_product(ri,rj)/dist/sqrt(dot_product(rj,rj)))
              I_coor(1,3) = dist
              I_coor(2,3) = ang
              I_coor(3,3) = 0.0
           end if
       endif
   endif
!
   do i=4,na       
      ri = atom(i)%xc
      j  = conn(1,i)  
      k  = conn(2,i) 
      n  = conn(3,i)
!
      rj = atom(j)%xc 
      rk = atom(k)%xc
      rn = atom(n)%xc
      call get_Z_from_cartesian(ci,ri,rj,rk,rn)
      I_coor(:,i) = ci
   end do
! 
   end subroutine cartesian_to_zmatrix

!------------------------------------------------------------------------------------------

   subroutine read_mopcartfile(j_in,cell,atom,error)
!
!  Read MOPAC Cartesian format
!   
!corr   USE progtype
   USE unit_cell
   USE atom_type_util
   USE strutil
   USE errormod
   !USE ptable
   USE elements
   integer, intent(in)                                       :: j_in
   real, dimension(:), intent(in)                            :: cell
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom  ! atomi in uscita
   character(len=10)                                         :: strn
   integer                                                   :: i
   type(error_type), intent(out)                             :: error 
   character(len=500)                                        :: line1,line2
   integer                                                   :: nlong1,nlong2
   integer                                                   :: ier
   integer                                                   :: nline
   real, dimension(50)                                       :: vet
   integer, dimension(50)                                    :: ivet
   integer                                                   :: iv
   integer                                                   :: natom
   integer                                                   :: ncomm
!   
!corr   call reset_error(error)   
!
!  Controlla se le 3 righe di commento esistono
   ncomm = 0
   do i=1,3
      read(j_in,'(a)',iostat=ier)line1
      if (ier < 0) exit
!      
!     controlla se esiste una stringa di 1-2 caratteri (la specie chimica)
      call cutst(line1,nlong1,line2,nlong2)
      if (nlong2 == 1 .or. nlong2 == 2) then
!
!         controlla il numero di campi numerici
          call Getnum(line1,vet,ivet,iv)
          if (iv == 6) exit
      endif
      ncomm = ncomm + 1
   enddo
!
   if (ncomm /= 3) backspace (j_in)  ! rileggi l'ultima riga
!
!  Conta le righe nel file per allocare 
   nline = 0
   do 
      read(j_in,'(a)',iostat=ier)line1
      if (ier < 0) exit
      if (len_trim(line1) == 0) exit ! righe vuote a fine file
      nline = nline + 1
   enddo
   natom = nline !!!!!- ncomm
   allocate(atom(natom))
   rewind (j_in)
!   
!  Non legge le prime 3 righe se ci sono
   nline = 0
   do i=1,ncomm
      nline = nline + 1
      read(j_in,*,err=10)
   enddo
!   
   do i=1,natom
      nline = nline + 1
      read(j_in,'(a)',err=10)line1
!      
!     Leggi la specie sulla prima colonna
      call cutst(line1,nlong1,line2,nlong2)
      if (nlong2 > 2) go to 10
      atom(i)%lab = line2(:nlong2)
!      
!     Leggi il resto della riga
      call Getnum(line1,vet,ivet,iv)
      if (iv < 6) go to 10 
      atom(i)%xc(1) = vet(1)
      atom(i)%xc(2) = vet(3)
      atom(i)%xc(3) = vet(5)
   enddo
!
!  Converti in coordinate cristallografiche
   call cart_to_frac(atom,orthomatrixi_std(cell))
!
!  Metti in nz il puntatore al file xen
   do i=1,natom
      atom(i)%nz = pxen_from_specie(atom(i)%lab)
   enddo
!
   return
!
10 write(strn,'(i0)')nline
!corr   call set_error(error,'Error on reading line '//trim(strn)//' of MOPAC file')   
   call error%set('Error on reading line '//trim(strn)//' of MOPAC file')   
!   call print_error_message(error)
!   
   end subroutine read_mopcartfile

!------------------------------------------------------------------------------------------
   
   subroutine create_mopcrtfile(filename,jfile,cella,atom,title)
!
!  Create MOP file in cartesian format
!   
   USE atom_type_util
!corr   USE variables, only:ortom
   USE unit_cell
!corr   USE progtype
   USE fileutil
   USE elements, only: NLEN_LAB
!
   character(len=*), intent(in), optional     :: filename  ! se presente apre il file
   integer, intent(in), optional              :: jfile     ! unita' di lettura del file
   real, dimension(6), intent(in)             :: cella     ! cella elementare   
   character(len=*), intent(in)               :: title
   type(atom_type), dimension(:), intent(in)  :: atom  
   type(atom_type), dimension(size(atom))     :: atomc 
   integer                                    :: j_in
   integer                                    :: ier
   character(len=NLEN_LAB)                    :: labat
   integer                                    :: ocode
   integer                                    :: i,j
!
!  Apertura del file
   if (present(filename)) then
       call get_unit(j_in)
       open(unit=j_in,file=filename,iostat=ier,status='unknown')
   else
       j_in = jfile   ! in questo caso il file e' stato gia' aperto
   endif
!
!  Header block
   write(j_in,'(a)')'PM6'                        ! tipo di ottimizzazione
   write(j_in,'(a)')trim(title)                  ! titolo
   write(j_in,'(a)')'All coordinates are cartesian'
!
!  Atom block
!corr   ortom = orthomatrix(cella) 
   atomc = atom
   ocode = 1       ! codice di ottimizzazione
!corr   call frac_to_cart(atomc,ortom)                !converti in coord. cartesiasane
   call frac_to_cart(atomc,orthomatrix_std(cella))                !converti in coord. cartesiasane
   do i=1,size(atomc)
      labat = atomic_specie(atomc(i))
      write(j_in,'(a,3(f10.4,i5))')labat,(atomc(i)%xc(j),ocode,j=1,3)
   enddo          
!
   end subroutine create_mopcrtfile

!------------------------------------------------------------------------------------------

   subroutine read_mopintfile(j_in,cell,atom,error)
!
!  Read MOPAC Internal format
!   
!corr   USE progtype
   USE unit_cell
   USE atom_type_util
   USE strutil
   USE errormod
   USE elements
   integer, intent(in)                                       :: j_in
   real, dimension(:), intent(in)                            :: cell
   type(atom_type), allocatable, dimension(:), intent(inout) :: atom    ! atomi in uscita
   real, dimension(:,:), allocatable                         :: I_coor  ! Internal coordinates (d,ang,tors)
   integer, dimension(:,:), allocatable                      :: Conn    ! connettivita in zmt
   character(len=10)                                         :: strn
   integer                                                   :: i
   type(error_type), intent(out)                             :: error 
   character(len=500)                                        :: line1,line2
   integer                                                   :: nlong1,nlong2
   integer                                                   :: ier
   integer                                                   :: nline
   real, dimension(50)                                       :: vet
   integer, dimension(50)                                    :: ivet
   integer                                                   :: iv
   integer                                                   :: natom
   logical                                                   :: kpr = .false.
   integer                                                   :: ncomm
!   
!corr   call reset_error(error)   
!
!  Controlla se le 3 righe di commento esistono
   ncomm = 0
   do i=1,3
      read(j_in,'(a)',iostat=ier)line1
      if (ier < 0) exit
!      
!     controlla se esiste una stringa di 1-2 caratteri (la specie chimica)
      call cutst(line1,nlong1,line2,nlong2)
      if (nlong2 == 1 .or. nlong2 == 2) then
!
!         controlla il numero di campi numerici
          call Getnum(line1,vet,ivet,iv)
          if (iv == 9) exit
      endif
      ncomm = ncomm + 1
   enddo
!
   if (ncomm /= 3) backspace (j_in)  ! rileggi l'ultima riga
!
!  Conta le righe nel file per allocare 
   nline = 0
   do 
      read(j_in,'(a)',iostat=ier)line1
      if (ier < 0) exit
      if (len_trim(line1) == 0) exit ! righe vuote a fine file
      nline = nline + 1
   enddo
   natom = nline !!!!!- ncomm
   allocate(atom(natom),Conn(3,natom),I_coor(3,natom))
   rewind (j_in)
!   
!  Non legge le prime 3 righe se ci sono
   nline = 0
   do i=1,ncomm
      nline = nline + 1
      read(j_in,*,err=10)
   enddo
!   
   do i=1,natom
      nline = nline + 1
      read(j_in,'(a)',err=10)line1
!      
!     Leggi la specie sulla prima colonna
      call cutst(line1,nlong1,line2,nlong2)
      if (nlong2 > 2) go to 10
      atom(i)%lab = line2(:nlong2)
!      
!     Leggi il resto della riga
      call Getnum(line1,vet,ivet,iv)
      if (iv < 9) go to 10 
      I_coor(1:3,i) = vet(1:6:2)
   ! cosi si corregge errore segnalato da Akihiro
      !if(i>3)I_coor(3,i) = dangle_norm(I_coor(3,i))
      if(i>3)I_coor(3,i) = -dangle_norm(I_coor(3,i))
      conn(1:3,i) = ivet(7:9)
   enddo
!
!  Converti coordinate interne in coordinate cartesiane
   call Zmatrix_to_Cartesian(natom,I_coor,conn)
   if (kpr) then
       do i=1,natom
          write(0,'(i4,3f10.3,3i6)')i,I_coor(:3,i),conn(:3,i)
       enddo
   endif        
!   
!  Trasferisci in atom le coordinate cartesiane
   do i=1,natom
      atom(i)%xc = I_coor(:3,i)
   enddo
!
!  Converti in coordinate cristallografiche
   call cart_to_frac(atom,orthomatrixi_std(cell))
!
!  Metti in nz il puntatore al file xen
   do i=1,natom
      atom(i)%nz = pxen_from_specie(atom(i)%lab)
   enddo
!
!  Converti conn in icon per usare la connettivit� del file zmt
!!!!!   icon(:) = 0
!
   return
!
10 write(strn,'(i0)')nline
!corr   call set_error(error,'Error on reading line '//trim(strn)//' of zmt file')   
   call error%set('Error on reading line '//trim(strn)//' of zmt file')   
!   call print_error_message(error)
!   
   end subroutine read_mopintfile

!------------------------------------------------------------------------------------------

   subroutine create_xyzfile(filename,jfile,atom,cella,comment)
!
!  Crea un XYZ chemical file format
!   
   USE elements, only:NLEN_LAB
   USE atom_basic, only: atom_type
   USE unit_cell
   USE atom_type_util
   USE fileutil
   character(len=*), intent(in), optional    :: filename  ! se presente apre il file
   integer, intent(in), optional             :: jfile     ! unita' di lettura del file
   type(atom_type), dimension(:), intent(in) :: atom
   real, dimension(6), intent(in)            :: cella
   character(len=*), intent(in)              :: comment
   type(atom_type), dimension(size(atom))    :: atomc
   integer                                   :: num_atoms
   character(len=NLEN_LAB)                   :: labat
   integer                                   :: i
   integer                                   :: ier
   integer                                   :: j_in
!
!  Apertura del file
   if (present(filename)) then
       call get_unit(j_in)
       open(unit=j_in,file=filename,iostat=ier,status='unknown')
   else
       j_in = jfile   ! in questo caso il file e' stato gia' aperto
   endif
!
!  Scrvi il numero di atomi sulla prima riga
   num_atoms = size(atom)
   write(j_in,'(i0)')num_atoms
!
!  Scrivi un commento sulla seconda riga
   write(j_in,'(a)')trim(comment)
!
!  Scrivi simbolo chimico e coordinate cartesiane sulle altre righe
!corr   ortom = orthomatrix(cella) 
   atomc = atom
!corr   call frac_to_cart(atomc,ortom)             !converti in coord. cartesiasane
   call frac_to_cart(atomc,orthomatrix_std(cella))             !converti in coord. cartesiasane
   do i=1,num_atoms
      labat = atomic_specie(atomc(i))
      write(j_in,'(a2,3(1x,f10.5))')labat,atomc(i)%xc
   enddo          
!
   if (present(filename)) close(j_in)
!
   end subroutine create_xyzfile

!------------------------------------------------------------------------------------------

   subroutine create_povrayfile(filename,jfile,atom,legm,elem,cols,colc,iscell,rmat)
   USE atom_type_util
   USE connect_mod
   USE fileutil
   USE elements
   USE strutil
   USE general, only:cell
   USE Atomi, only:nzConst
   USE unit_cell
   USE cgeom
   USE realloc
   character(len=*), intent(in), optional                 :: filename  ! se presente apre il file
   integer, intent(in), optional                          :: jfile     ! unita' di lettura del file
   type(atom_type), dimension(:), intent(in)              :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legm
   type(element_type), dimension(:), intent(in)           :: elem
   real, dimension(:,:), intent(in)                       :: cols      ! col(i,j) = colore della specie i
   real, dimension(3), intent(in)                         :: colc      ! colore della cella
   logical, intent(in)                                    :: iscell
   real, dimension(3,3)                                   :: rmat
   integer                                                :: ier
   integer                                                :: j_in
   integer                                                :: i    !!!!!, j
   integer                                                :: nspec
   type(atom_type), dimension(:), allocatable             :: atomc
   integer                                                :: iz
   real, dimension(3)                                     :: bar
   real                                                   :: fact
   real                                                   :: wire = 0.1
   integer                                                :: n1,n2
   real                                                   :: rad
   real, dimension(3)                                     :: cpos
   real, dimension(3)                                     :: xm
   character(len=200)                                     :: texture
   real, dimension(3,3)                                   :: orto
   integer                                                :: nat
!!mr
   real :: xmin, xmax, ymin, ymax, scalapov, dx, dy
!!end mr
!
!  Apertura del file
   if (present(filename)) then
       call get_unit(j_in)
       open(unit=j_in,file=filename,iostat=ier,status='unknown')
   else
       j_in = jfile   ! in questo caso il file e' stato gia' aperto
   endif
!
   nspec = size(elem)
   nat = size(atom)
!
!  Colori delle specie
   write(j_in,'(a)')'#include "colors.inc"'
   do i=1,nspec
      write(j_in,'(a,2(f0.2,","),f0.2,a)')'#declare color_'//trim(elem(i)%lab)//' = '//'rgb < ',cols(i,:),' >;'
   enddo
!
!  Colore della cella
   if (iscell) then
      write(j_in,'(a,2(f0.2,","),f0.2,a)')'#declare color_cell = rgb < ',colc(:),' >;'
   endif
!
!  Costanti numeriche
   write(j_in,'("#declare Wire =",f0.3,"; // Wire thickness")')wire
!
!  Se presente la cella aggiungi 8 atomi in più in corrispondenza dei vertici   
   if(iscell) then
      call reallocate(atomc,nat+8)
      atomc(:nat) = atom(:)
      do i=nat+1,nat+8
         atomc(i) = atomc(1)
      enddo
      atomc(nat+1)%xc = (/0,0,0/)
      atomc(nat+2)%xc = (/1,0,0/)
      atomc(nat+3)%xc = (/0,1,0/)
      atomc(nat+4)%xc = (/0,0,1/)
      atomc(nat+5)%xc = (/1,1,1/)
      atomc(nat+6)%xc = (/0,1,1/)
      atomc(nat+7)%xc = (/1,0,1/)
      atomc(nat+8)%xc = (/1,1,0/)
   else
      call reallocate(atomc,nat)
      atomc(:) = atom
   endif
!
!  Converti in cartesiane
   orto = orthomatrix(cell)
   call frac_to_cart(atomc,orto)
   do i=1,nat
      atomc(i)%xc = matmul(rmat,atomc(i)%xc)
   enddo
   write(j_in,'(a)')'#declare Surface = finish { ambient < 0.5 0.5 0.5 1 1 > phong 1 phong_size 10 }'
   write(j_in,'(a)')'background { color White }'
!
!  Locate the camera. La camera guarda il baricentro della molecola
!!!!!  Posiziona la camera ad una distanza pari a 1.8*rad dal bar. lungo la retta che unisce il bar. all'origine (rimosso)
   call get_radius_molecule(atomc,rad,bar)
!!mr
   call translate_atoms(atomc,-bar)
   xmin = minval(atomc(:)%xc(1))
   xMax = maxval(atomc(:)%xc(1))
   ymin = minval(atomc(:)%xc(2))
   ymax = maxval(atomc(:)%xc(2))
   !xmin = 9.e33
   !xMax = -9.e33
   !ymin = 9.e33
   !yMax = -9.e33
   !do i=1,nat
   !   do j=1,3
   !      atomc(i)%xc(j) = atomc(i)%xc(j) - bar(j)
   !   enddo
   !   if (xmin > atomc(i)%xc(1)) xmin = atomc(i)%xc(1)
   !   if (xMax < atomc(i)%xc(1)) xMax = atomc(i)%xc(1)
   !   if (ymin > atomc(i)%xc(2)) ymin = atomc(i)%xc(2)
   !   if (yMax < atomc(i)%xc(2)) yMax = atomc(i)%xc(2)
   !enddo
   dx = xMax - xmin
   dy = yMax - ymin
   if (dx.gt.dy) then
       if (dx > 0.001) then
           scalapov = 0.5 / dx
       else
           scalapov = 0.5
       endif
   else
       if (dy > 0.001) then
           scalapov = 0.5 / dy
       else
           scalapov = 0.5
       endif
   endif
!!end mr

   !!!!!!cpos =  bar(:) + bar(:) * 1.8 * rad/distanzac(bar,(/0.0,0.0,0.0/))
!!!FIXME - improve scale
   cpos = (/0.0,0.0,1.8*rad/)
   write(j_in,'(a)')'camera { orthographic'
   write(j_in,'(a,2(f0.2,","),f0.2,a)')'         location < ',cpos,' >'
   !write(j_in,'(a,2(f0.2,","),f0.2,a)')'         look_at  < ',bar(:),' >'
   !write(j_in,'(a,f0.2,a)')'         location < 0 0 ',cpos(3),' >'
   !write(j_in,'(a,f0.2,a)')'         location < 0 ',cpos(3),' 0 >'
   write(j_in,'(a)')'         look_at  < 0 0 0 >'
   write(j_in,'(a)')'         up y'
   write(j_in,'(a)')'         right -4/3*x'
   write(j_in,'(a)')'       }'
!
!!!!!!FIXME - improve light position
   write(j_in,'(a,2(f0.2,","),f0.2,a)')'light_source { <',cpos,'> color White }'
!
!  Disegna atomi
   fact = 0.5
   write(j_in,'(a)')'#declare molecule = union {'
   do i=1,nat
      iz = iabs(atomc(i)%nz/nzConst)
      if (iz == 0) cycle   ! non disegno i ghost
      write(j_in,'(a)')'// Atom '//trim(atomc(i)%lab)
      write(j_in,'(a,2(f0.5,","),f0.5,a,f0.3)')'sphere{<',atomc(i)%xc,'>,',fact*elem(iz)%c_radius
      write(j_in,'(a)')'       texture { pigment {'//' color color_'//trim(elem(iz)%lab)//' } finish { Surface } } }'
   enddo
!
!  Disegna i legami
   do i=1,numbonds(legm)
      n1 = legm(i)%n1
      n2 = legm(i)%n2
      write(j_in,'(a)')'// Bond '//trim(atomc(i)%lab)//'-'//trim(atomc(i)%lab)
      xm(:) = (atomc(n1)%xc + atomc(n2)%xc) / 2
      iz = iabs(atomc(n1)%nz/nzConst)
      texture = '       texture { pigment { color color_'//trim(elem(iz)%lab)//' } finish { Surface } }'
      call draw_cylinder_pov(j_in,atomc(n1)%xc,xm,texture)
      iz = iabs(atomc(n2)%nz/nzConst)
      texture = '       texture { pigment { color color_'//trim(elem(iz)%lab)//' } finish { Surface } }'
      call draw_cylinder_pov(j_in,atomc(n2)%xc,xm,texture)
   enddo
!
!  Disegna la cella ovvero disegna 12 linee
   if (iscell) then
       texture = '       texture { pigment { color color_cell } finish { Surface } }'
       write(j_in,'(a)')'// Cell '
       call draw_cylinder_pov(j_in,atomc(nat+1)%xc,atomc(nat+2)%xc,texture)
       call draw_cylinder_pov(j_in,atomc(nat+1)%xc,atomc(nat+3)%xc,texture)
       call draw_cylinder_pov(j_in,atomc(nat+1)%xc,atomc(nat+4)%xc,texture)
       call draw_cylinder_pov(j_in,atomc(nat+5)%xc,atomc(nat+6)%xc,texture)
       call draw_cylinder_pov(j_in,atomc(nat+5)%xc,atomc(nat+7)%xc,texture)
       call draw_cylinder_pov(j_in,atomc(nat+5)%xc,atomc(nat+8)%xc,texture)
       call draw_cylinder_pov(j_in,atomc(nat+7)%xc,atomc(nat+4)%xc,texture)
       call draw_cylinder_pov(j_in,atomc(nat+7)%xc,atomc(nat+2)%xc,texture)
       call draw_cylinder_pov(j_in,atomc(nat+4)%xc,atomc(nat+6)%xc,texture)
       call draw_cylinder_pov(j_in,atomc(nat+3)%xc,atomc(nat+6)%xc,texture)
       call draw_cylinder_pov(j_in,atomc(nat+3)%xc,atomc(nat+8)%xc,texture)
       call draw_cylinder_pov(j_in,atomc(nat+8)%xc,atomc(nat+2)%xc,texture)
   endif
!
   write(j_in,'(a)')'                          }'
!
   write(j_in,'(a)')'object { molecule'
   write(j_in,'(a)')'         translate <0.0, 0.0, 0.0>'
   write(j_in,'(a)')'         rotate    <0.0, 0.0, 0.0>'
!!!   write(j_in,'(a)')'         scale     <1.0, 1.0, 1.0>'
   write(j_in,'(a,2(f0.5,","),f0.5,a)')'         scale     < ',scalapov, scalapov, scalapov,' >'
   write(j_in,'(a)')'       }'
!
   if (present(filename)) close(j_in)
!
   end subroutine create_povrayfile

!------------------------------------------------------------------------------------------

   subroutine draw_cylinder_pov(junit,x1,x2,text)
   integer, intent(in)            :: junit
   real, dimension(3), intent(in) :: x1,x2
   character(len=*), intent(in)   :: text
!
   write(junit,'("cylinder{ <",2(f0.5,","),f0.5,">,")')x1
   write(junit,'("          <",2(f0.5,","),f0.5,">, Wire")')x2
   write(junit,'(a)')trim(text)//' }'
!
   end subroutine draw_cylinder_pov

!------------------------------------------------------------------------------------------

   subroutine create_frafile(filename,jfile,atom,cella,comment)
!
!  Crea un Free Form Fractional file 
!   
   !USE General, only:pi
   USE trig_constants, only:pi
   USE fileutil
   USE atom_type_util
   USE elements, only:NLEN_LAB
   character(len=*), intent(in), optional    :: filename  ! se presente apre il file
   integer, intent(in), optional             :: jfile      ! unita' di lettura del file
   type(atom_type), dimension(:), intent(in) :: atom
   real, dimension(6), intent(in)            :: cella
   character(len=*), intent(in)              :: comment
   character(len=NLEN_LAB)                   :: labat
   integer                                   :: i
   integer                                   :: j_in
   integer                                   :: ier
!
!  Apertura del file
   if (present(filename)) then
       call get_unit(j_in)
       open(unit=j_in,file=filename,iostat=ier,status='unknown')
   else
       j_in = jfile   ! in questo caso il file e' stato gia' aperto
   endif
!
!  Scrivi un commento sulla prima riga
   write(j_in,'(a)')trim(comment)
!
!  Scrivi la cella sulla seconda riga
   write(j_in,'(6(1x,f10.5))')cella(1:6)
!
!  Scrivi simbolo chimico e coordinate frazionarie sulle altre righe
   do i=1,size(atom)
      labat = atomic_specie(atom(i))
      write(j_in,'(a2,3(1x,f10.5))')labat,atom(i)%xc
   enddo          
!
   if (present(filename)) close(j_in)
!
   end subroutine create_frafile

!------------------------------------------------------------------------------------------

   function orthomatrixi_pdb_babel(cell)  result(om)
   USE trig_constants
!corr   USE CCRYST
!
!  Genera matrice di conversione  co. cartesiane --> co. cristallografiche
! 
   real, dimension(6), intent(in) :: cell
   real                           :: sinAlpha,sinBeta,sinGamma
   real                           :: cosAlpha,cosBeta,cosGamma
   real                           :: v
   real, dimension(3,3)           :: om
!
   sinAlpha = sin(cell(4)*dtor)
   sinBeta = sin(cell(5)*dtor)
   sinGamma = sin(cell(6)*dtor)
   cosAlpha = cos(cell(4)*dtor)
   cosBeta = cos(cell(5)*dtor)
   cosGamma = cos(cell(6)*dtor)
!
   v = sqrt(1 - cosAlpha**2 - cosBeta**2 - cosGamma**2 + 2*cosAlpha*cosBeta*cosGamma)
!
   om(1,1)=1/cell(1)
   om(2,1)=-cosGamma / (cell(1)*sinGamma)
   om(3,1)=(cosGamma * cosAlpha - cosBeta) / (cell(1) * v *sinGamma)
   om(1,2)=0.0
   om(2,2)=1.0 / (cell(2)*sinGamma)
   om(3,2)=(cosGamma * cosBeta - cosAlpha) / (cell(2) * v * sinGamma)
   om(1,3)=0.0
   om(2,3)=0.0
   om(3,3)=sinGamma / (cell(3) * v)
!
   end function orthomatrixi_pdb_babel

END MODULE chem_file_fmt

!------------------------------------------------------------------------------------------
!------------------------------------------------------------------------------------------

MODULE compare_struct

USE atom_basic, only: atom_type
USE progtype, only: reflection_type

implicit none

type compare_info_type
   integer                                    :: nfound     ! numero di trovati nel modello 2
   real                                       :: distmin    ! distanza minima
   real                                       :: distm      ! distanza media
   real                                       :: rmsd       ! RMSD
   integer, dimension(:), allocatable         :: nv1        ! atomo 1 dell'associazione
   integer, dimension(:), allocatable         :: nv2        ! aromo 2 dell'associazione
   real, dimension(:), allocatable            :: vdist      ! distanza dell'associazione
   real                                       :: errph      ! errore di fase
   integer                                    :: nref       ! numero di riflessi per l'errore di fase
   real, dimension(4)                         :: shift      ! shift applicato
   type(atom_type), dimension(:), allocatable :: at1        ! modello 1
   type(atom_type), dimension(:), allocatable :: at2        ! modello 2
   integer                                    :: nass       ! numero di atomi associati nel modello 1
end type compare_info_type

CONTAINS
   
   subroutine to_coord_ref(code)   
   USE variables, only:ref
   USE riflessi, only:iphaz,wt, iwork1
   USE General, only:knw, QualePgm, EXPOPGM, nrefl
   USE Dialogo, only:ModelToCompare
   integer, intent(in)        :: code
   integer                    :: nrrf
   integer                    :: knwsav
   integer                    :: i, j
!
!  Passa le fasi in iphaz per consentire l'uso della to_coord
   if ((knw == 1 .or. ModelToCompare /= 0) .and. allocated(ref)) then
       if (QualePgm == EXPOPGM) then
           nrrf = size(ref)
           iphaz(:nrrf) = ref%ph
           wt(:nrrf) = 1.0
       else
           nrrf = nrefl
           wt(:) = 0.0
           do i=1, size(ref)
              j = iwork1(i)
              wt(j) = 1.0
              iphaz(j) = ref(i)%ph
           enddo
       endif
       knwsav = knw
       knw = 1
       call to_coord_sir(code, nrrf)
       knw = knwsav
   endif
!   
   end subroutine to_coord_ref   
   
!-------------------------------------------------------------------------------------------------------  

   subroutine drawup_models(atom1,legm1,atom2,legm2,shifte,kfix,kenant,kinorg)
!
!  Avvicina il modello atom2 al modello atom1. atom1 viene mantenuto fisso se kfix e' true
!
   USE connect_mod
   USE fragmentmod
   USE General, only:cell,StructureName
   USE errormod
   USE chem_file_fmt
   USE atom_type_util
   USE Symmetry, only:icent
   USE molpnew
   USE Molcom, only:PkgName
   USE unit_cell
   type(atom_type), dimension(:), intent(inout)           :: atom1    ! modello fisso
   type(bond_type), dimension(:), allocatable, intent(in) :: legm1    ! legami del modello fisso
   type(atom_type), dimension(:), intent(inout)           :: atom2    ! modello da avvicinare
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm2    ! legami modello da avvicinare
   real, dimension(:), intent(in), optional               :: shifte   ! shift (usa examin)
   logical, intent(in), optional                          :: kfix     ! se true atom1 viene fissato
   logical, intent(in), optional                          :: kenant   ! se true puoi eseguire l'inversione
   logical, intent(in), optional                          :: kinorg   ! se true inorganic fragment are disconnected
   logical                                                :: kfix1,kenant1,kinorg1
   type(atom_type), dimension(:), allocatable             :: atomu
   type(bond_type), dimension(:), allocatable             :: legm
   integer                                                :: nat1,nat2
   type(fragment_type), dimension(:), allocatable         :: frag1,frag2   !!!!!,frag3
   integer                                                :: nfrag1,nfrag2
   integer                                                :: i,j
   logical                                                :: kexport = .false.
   type(error_type)                                       :: error
   integer                                                :: ninorg
!
   nat1 = size(atom1)
   nat2 = size(atom2)
!
   if (present(kenant)) then
       kenant1 = kenant
   else
       kenant1 = .true.
   endif
!
   if (present(kinorg)) then
       kinorg1 = kinorg
   else
       kinorg1 = .true.
   endif
!
!  Applica shift ad atom2
   if (present(shifte)) then
       if (shifte(4) > 0) then
           call translate_atoms(atom2,-shifte(1:3))
       else
           if (icent == 1 .or. kenant1) then   ! puoi passare all'enantiomorfo
               do i=1,nat2
                  atom2(i)%xc = -(atom2(i)%xc + shifte(1:3))
               enddo
           else   
               call translate_atoms(atom2,-shifte(5:7))
           endif
       endif
   endif
!
!  Il modello potrebbe andare fuori cella!
   call translate_in_cell(atom2)
!
   if (present(kfix)) then
       kfix1 = kfix
   else
       kfix1 = .false.
   endif
!
!  Unisci i 2 modelli
   allocate(atomu(nat1+nat2))
   atomu(:nat1) = atom1(:)
   atomu(nat1+1:) = atom2(:)
!
!  estrai frammenti da atom1
   call get_fragments(atom1,legm1,nfrag1,frag1)
   if (kfix1) then
       do i=1,nfrag1
          frag1(i)%rcod(:3) = 0  ! blocca i frammenti di atom1
       enddo
   endif
!
!  build fragments from atom2
   call get_fragments(atom2,legm2,nfrag2,frag2)
!
   ninorg = 0
   if (kinorg1) then
!
!      Delete connectivity for inorganic fragment
       do i=1,nfrag2
          if (frag2(i)%nat > 1 .and. is_organic(atom2(frag2(i)%pos)) == 0) then
              ninorg = ninorg + 1
              call remove_bond_from_atom(legm2,frag2(i)%pos)
              do j=1,frag2(i)%nat
                 call add_fragment(frag2,atom2,[frag2(i)%pos(j)])
              enddo
              frag2(i)%nat = 0  ! frag2(i) will be removed
          endif
       enddo
!   
!      pack the array to exclude fragments with nat == 0     
       if (ninorg > 0) then
           nfrag2 = count(frag2%nat > 0) 
           frag2(:nfrag2) = pack(frag2, mask=frag2%nat > 0)
       endif
   endif
!
   call overlay_fragment(frag1,frag2(:nfrag2),atom1,atom2)
   if (ninorg > 0) call create_connectivity(atom2,legm2,set_cell_type(cell))
!
!  export both models if .mol file
   if (kexport) then
       call copia_legm(legm,legm1)
       call combine_legm(legm,legm2,nat1)
       allocate(atomu(nat1+nat2))
       atomu(:nat1) = atom1(:)
       atomu(nat1+1:) = atom2(:)
       call create_molfile(trim(StructureName)//'_comp.mol',cella=cell,atom=atomu,legm=legm,progname=trim(PkgName),error=error)
   endif
!
   end subroutine drawup_models
   
!---------------------------------------------------------------------      

   subroutine overlay_models(atom1,legm1,atom2,legm2)
!
!  Sovrappone atom2 su atom1
!
   USE connect_mod
   USE PhaseModule, only:shiftexam
   USE General, only:nrefl, cell
   USE factor
   USE reflection_type_util
   USE proginterface
   USE sinterface
   USE Symmetry, only: kmat,tmat,nsym,icent,latt,jsys
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom1
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom2
   type(bond_type), dimension(:), allocatable, intent(in)    :: legm1   !!!!,legm2
   type(bond_type), dimension(:), allocatable, intent(inout)    :: legm2
   type(reflection_type), dimension(:), allocatable          :: ref1
   type(reflection_type), dimension(:), allocatable          :: ref2
   integer                                                   :: i
   integer                                                   :: ier
   real, parameter                                           :: DMAX = 1.7
   real                                                      :: rhomax 
   integer                                                   :: nref
   integer, dimension(3)                                     :: ihmx
!
!  Genera i riflessi se non esistono
   ier = 0
   if (nrefl == 0) then
       rhomax = 1 / (2*DMAX)**2
       call create_reflections(0.0,rhomax,cell(:6),spaceg_string(),kmat,tmat,nsym,jsys,icent,latt,ref1,nref,ihmx)
       if (nref == 0) ier = 1
   else
       call fillref(ref1)  
       nref = nrefl
   endif
!
!  Calcolo dello shift di origine tra i 2 modelli
   if (ier == 0) then
       allocate(ref2(nref))
       do i=1,nref
          ref2(i) = ref1(i)
       enddo
       call fcalcang(ref1,atom1)  ! calcola fasi del modello 1
       call fcalcang(ref2,atom2)  ! calcola fasi del modello 2
       call AllocaTestStuff(1)
       call examing(ref1,ref2)
       call AllocaTestStuff(2)
   else
       shiftexam(1:7) = 0; shiftexam(4) = 1
   endif
!
!  Sovrappono i modelli
   call drawup_models(atom1,legm1,atom2,legm2,shiftexam,kfix=.true.)
!
   end subroutine overlay_models

!---------------------------------------------------------------------      

   subroutine compare_models(atom1,atom2,dist,kpr,infoc)
!
!  Confronta 2 modelli. Nella stampa solo le coordinate di atom1 vengono modificate.
!
   USE atom_type_util
   USE PhaseModule, only:shiftexam,erore
   USE General, only:nrefl, cell
   USE reflection_type_util, only: fillref, fcalcang, create_reflections
   USE cgeom
   USE general, only:xgg
   USE Symmetry
   USE proginterface
   USE sinterface
   type(atom_type), dimension(:), allocatable, intent(in) :: atom1
   type(atom_type), dimension(:), allocatable, intent(in) :: atom2
   real, optional, intent(in)                             :: dist
   integer, intent(in), optional                          :: kpr
   type(compare_info_type), optional, intent(out)         :: infoc
   type(reflection_type), dimension(:), allocatable       :: ref1
   type(reflection_type), dimension(:), allocatable       :: ref2
   integer                                                :: i,j,k
   integer                                                :: nat1,nat2
   integer, dimension(:), allocatable                     :: npunt
   real                                                   :: distm
   integer                                                :: jpunt
   real, dimension(3,(icent+1)*nsym*ncentr)               :: xeq
   real                                                   :: dd
   real, dimension(3)                                     :: ktra,ktramin
   type(compare_info_type)                                :: info
   integer                                                :: kprint
   real, dimension(:), allocatable                        :: distrmsd
   integer                                                :: ier
   real, parameter                                        :: DMAX = 1.7
   real                                                   :: rhomax 
   integer                                                :: nref
   integer, dimension(3)                                  :: ihmx
!
   nat1 = numatomtype(atom1)
   nat2 = numatomtype(atom2)
   if (nat1 == 0 .or. nat2 == 0) return
!
   allocate(npunt(nat2),distrmsd(nat1))
   call init_compare_info_type(info,nat1)
   call copia_atoms(info%at2,atom2)
   call copia_atoms(info%at1,atom1)
!
!  Genera i riflessi se non esistono
   ier = 0
   if (nrefl == 0) then
       rhomax = 1 / (2*DMAX)**2
       call create_reflections(0.0,rhomax,cell(:6),spaceg_string(),kmat,tmat,nsym,jsys,icent,latt,ref1,nref,ihmx)
       if (nref == 0) ier = 1
   else
       call fillref(ref1)
       nref = nrefl
   endif
!
!  Calcola shift di fase
   if (ier == 0) then
       allocate(ref2(nref))
       do i=1,nref
          ref2(i) = ref1(i)
       enddo
       call fcalcang(ref1,atom1)  ! calcola fasi del modello 1
       call fcalcang(ref2,atom2)  ! calcola fasi del modello 2
       call AllocaTestStuff(1)
       call examing(ref1,ref2)
       call AllocaTestStuff(2)
   else
       erore = 0
       shiftexam(1:7) = 0; shiftexam(4) = 1
   endif
!
   info%errph = erore  ! errore di fase
   info%nref = nref
   info%shift(:) = shiftexam(:4)
!
!  Applica lo shift di fase al modello 1
   if (shiftexam(4) > 0) then
       call translate_atoms(info%at1,-shiftexam(1:3))
   else
       do i=1,nat2
          info%at1(i)%xc = -(info%at1(i)%xc + shiftexam(1:3))
       enddo
   endif
!
   if (present(kpr)) then
       kprint = kpr
   else
       kprint = -1
   endif
!
   if (present(dist)) then
       info%distmin = dist
   else
       info%distmin = 0.6
   endif
!
   npunt(:) = 0
   distrmsd(:) = huge(1.0)
   do i=1,nat1
      call get_equivalent1(info%at1(i)%xc,xeq)   !!!!!!,neq)
      jpunt = 0
      distm = huge(1.0)
      do j=1,nat2
!!!!!         if (npunt(j) /= 0) cycle  ! atomi gia' associato
         do k=1,nsymtot
            call xdisteqs(atom2(j)%xc,xeq(:,k),xgg,dd,ktra)
            if (dd < distrmsd(i)) distrmsd(i) = dd
            if (dd < distm .and. dd < info%distmin) then
                distm = dd
                ktramin(:) = ktra(:)
                jpunt = j
            endif
         enddo
      enddo
      if (jpunt > 0) then
          info%nass = info%nass + 1   ! conta le associazioni nel modello 1
          info%nv1(info%nass) = i
          info%nv2(info%nass) = jpunt
          info%vdist(info%nass) = distm
          info%at1(i)%xc = atom2(jpunt)%xc + ktramin
          if (npunt(jpunt) == 0) info%nfound = info%nfound + 1  ! conta i trovati nel modello 2
          npunt(jpunt) = i
      endif
   enddo
   info%rmsd = sqrt(sum(distrmsd(:)**2))/nat1
   if (info%nfound > 0) then
       info%distm = sum(info%vdist(:info%nfound))/info%nfound
   endif
!
   if (kprint > 0) call print_compare_info(info,kprint)
!
   if (present(infoc)) then
       infoc = info
   endif
!
   end subroutine compare_models

!---------------------------------------------------------------------      

   subroutine init_compare_info_type(info,npos)
!corr   USE realloc
   type(compare_info_type), intent(inout) :: info
   integer, intent(in)                    :: npos
!
   info%nfound = 0
   allocate(info%nv1(npos))
   allocate(info%nv2(npos))
   allocate(info%vdist(npos))
   info%distm = 0.0
   info%nass = 0
!
   end subroutine init_compare_info_type

!---------------------------------------------------------------------      

   subroutine print_compare_info(info,kpr)
   type(compare_info_type), intent(in) :: info
   integer, intent(in)                 :: kpr
   integer                             :: i
   integer                             :: n1,n2
!
   write(kpr,'(a,i0)')         'Atoms in model1:      ',size(info%at1)
   write(kpr,'(a,i0)')         'Atoms in model2:      ',size(info%at2)
   write(kpr,'(a,i0)')         'Matches found:        ',info%nfound
   if (info%shift(4) > 0) then
       write(kpr,'(a,3f10.3,a)')     'Shift applied:        ',info%shift(:3),' Sign is +'
   else
       write(kpr,'(a,3f10.3,a)')     'Shift applied:        ',info%shift(:3),' Sign is -'
   endif
   write(kpr,'(a,f0.3,a,i0,a)')'Mean Phase Error:     ',info%errph,' using ',info%nref,' reflections'
   write(kpr,'(a,f0.3)')       'Distance limit:       ',info%distmin
   write(kpr,'(a,f0.3)')       '<Dist>                ',info%distm
   write(kpr,'(14("-")," Model 1 ",15("-"),5x,"Distance",4x,10("-")," Model 2 ",10("-"))')
   do i=1,info%nfound
      n1 = info%nv1(i)
      n2 = info%nv2(i)
      write(kpr,'(a8,3f10.3,f12.5,3f10.3,2x,a8)')info%at1(n1)%lab,info%at1(n1)%xc,info%vdist(i),info%at2(n2)%xc,info%at2(n2)%lab
   enddo
!
   end subroutine print_compare_info

END MODULE compare_struct
