MODULE importexport

USE progtype

implicit none

integer, parameter :: CIF_FILE = 1
integer, parameter :: PDB_FILE = 2
integer, parameter :: SHELX_FILE = 3
integer, parameter :: FRA_FILE = 4
integer, parameter :: XYZ_FILE = 5
integer, parameter :: ZMT_FILE = 6
integer, parameter :: MOL_FILE = 7
integer, parameter :: MOL2_FILE = 8
integer, parameter :: MOP_FILE = 9
integer, parameter :: POV_FILE = 10
integer, parameter :: GAMESS_OUT = 11
integer, parameter :: NWCHEM_OUT = 12

integer, private, dimension(:), allocatable :: vmodel_cif
integer, private                            :: selected_cif

CONTAINS

   subroutine import_structure(filename,ftype,atom,legm,kadd,ier,at_info,nsec,sec_struct)
   USE fileutil
   USE General, only:cell, celem, neutro
   USE errormod
   USE chem_file_fmt
   USE realloc
   USE connect_mod
   USE molpnew
   USE Atomi
   USE atom_type_util
   USE model_util
   USE strutil
   USE elements
   USE Molcom, only:jscreen
   USE sinterface
   USE spginfom
   USE unit_cell
!!!corr   USE Conteggi, only:npunti
!!!corr   USE enable_amb
   USE cif_frm
   USE pdb_frm
   USE fra_frm
   USE shelx_frm
   USE crystal_phase
      USE proginterface
      USE Symmetry, only: nsymtot
      USE wlist
              !USE sannel
   character(len=*), intent(in)                              :: filename
   integer, intent(in)                                       :: ftype ! if 0 the file type is set from filename
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   type(bond_type), dimension(:), allocatable                :: legmc
   type(atom_type), dimension(:), allocatable                :: atomc
   integer                                                   :: j_in
   logical, intent(in)                                       :: kadd
   integer                                                   :: ier    !!!!,ierspace
   type(pdb_atom_info_type), dimension(:), allocatable, optional, intent(out) :: at_info   ! additional info for pdb file
   integer, intent(out), optional :: nsec
   type(secondary_struct_type), dimension(:), allocatable, optional, intent(out) :: sec_struct ! secondary structure from pdb
   logical                                                   :: is_bfac,is_occ
   logical                                                   :: make_conn
   logical                                                   :: trasl
   type(error_type)                                          :: err_read
   type(error_type), dimension(:), allocatable               :: errcif
   type(spaceg_type)                                         :: spgfile
   integer                                                   :: i
   logical                                                   :: make_label
   logical                                                   :: make_occ
   logical                                                   :: is_conect
   logical                                                   :: is_cell
   character(len=16)                                         :: spacepdb
   integer                                                   :: nmodel
   logical, dimension(2)                                     :: optconn
   real, dimension(6)                                        :: cellfile
   integer                                                   :: filetype
   integer, parameter                                        :: NMAXATOM = 500
   integer                                                   :: nblockcif
   type(crystal_phase_type), dimension(:), allocatable       :: phase_file
   type(element_type), dimension(:), allocatable             :: elemf
   integer, parameter                                        :: CODE_CIFLIST = 101
   type(file_handle)                                         :: fileh
!

!!NEW mr
   real, dimension(6)                                        :: celltemp
   logical                                                   :: check_cell
   real                                                      :: a1, a2
!
   celltemp(1:6) = cell(1:6)
   check_cell = .false.
   if (abs(SUM(celltemp)) > 1e-6) then
      check_cell = .true.
   endif
!!End NEW 

   ier = 0
   call fileh%fopen(filename)
!
   if (fileh%good()) then
       j_in = fileh%handle()
       make_conn = .false.     
       is_bfac = .false.
       trasl = .false.
       make_label = .false.
       make_occ = .false.
       optconn = (/.true.,.true./)
       if (ftype == 0) then
           filetype = code_fragfilen(filename)
       else
           filetype = ftype
       endif

       select case (filetype)

         case (FRA_FILE)    ! fra or frac file
           is_cell = is_cell_defined()           
           call read_fra_file(j_in,atomc,is_bfac,is_occ,err_read)
!      
!          se si genera errore leggi il file come standard fractional file
           if (err_read%signal) then
               call reallocate(atomc,0)
               rewind (j_in)
               call read_fractional(j_in,cell(1:6),is_cell,atomc,err_read)
               if (.not.is_cell) call set_cell(cell(1:6))
               is_bfac = .false.
               is_occ = .false.
           else
               if (.not.is_cell) call create_cell()
           endif
           if (.not.err_read%signal) then
               if (.not.is_space_defined()) call set_spaceg_new(init_spaceg_type())   ! force 'P 1'
               make_conn = .true.
               make_occ = .not.is_occ
               make_label = .true.
           endif

         case (XYZ_FILE)    ! cartesian file
           if (.not.is_cell_defined()) call create_cell()
           if (.not.is_space_defined()) call set_spaceg_new(init_spaceg_type())   ! force 'P 1'
           call read_xyzfile(j_in,cell(1:6),atomc,err_read)
           if (.not.err_read%signal) then
               make_conn = .true.
               make_occ = .true.
               trasl = .true.
               make_label = .true.
           endif

         case (SHELX_FILE)    ! ShelX file (.ins,.res)
           is_cell = is_cell_defined_list()           
           call read_shelxfile(j_in,atomc,cell,is_cell,spgfile,err_read)
           if (.not.err_read%signal) then
               if (.not.is_cell) call set_cell(cell(1:6))
               if (.not.is_space_defined_list()) call set_spaceg_new(spgfile)
               make_conn = .true.
               is_bfac = .true.
           endif

         case (PDB_FILE)    ! pdb file
           is_cell = is_cell_defined_list()
           if (present(at_info) .and. present(nsec) .and. present(sec_struct)) then
               call read_PDBfile(j_in,cell,is_cell,atomc,legmc,is_conect,spacepdb,err_read,at_info,nsec=nsec,sec_struct=sec_struct)
           else
               call read_PDBfile(j_in,cell,is_cell,atomc,legmc,is_conect,spacepdb,err_read)
           endif
           if (.not.err_read%signal) then
               if (.not.is_cell) call set_cell(cell(1:6))
               if (is_conect)call copia_legm(legm,legmc)
               if (.not.is_space_defined_list()) then
                   spgfile = init_spaceg_type(spacepdb)
                   call set_spaceg_new(spgfile)
               endif
               make_conn = .not.is_conect
               is_bfac = .true.
               trasl = .true.
               make_occ = .true.
           endif

         case (CIF_FILE)    ! cif file
           if (kadd) then
               is_cell = is_cell_defined()  ! in 'add' check only for cell
           else
               is_cell = is_cell_defined_list() ! in 'import' check for cell in list
           endif
           call read_CIFfile(j_in,phase_file,nblockcif,is_cell,cell,errcif)
           if (any(errcif(:nblockcif)%signal .eqv. .false.)) then
               nmodel = 0
               allocate(vmodel_cif(nblockcif))
               vmodel_cif(:) = 0
               do i=1,nblockcif
                  if (numatomtype(phase_file(i)%at) > 0) then
                      nmodel = nmodel + 1
                      vmodel_cif(nmodel) = i
                  endif
               enddo
               if (nmodel > 0) then
                   selected_cif = vmodel_cif(1)
                   if (nmodel > 1 .and. jscreen > 0) then
                       call wlist_create(CODE_CIFLIST,1,nmodel,['Data block name'],'CIF file',   &
                       trim(i_to_s(nmodel))//' crystal structures. Select one of them.',2,1,cif_func,     &
                       [set_column((/(trim(phase_file(vmodel_cif(i))%phase_name)//' ('//    &
                       trim(phase_file(vmodel_cif(i))%spg%symbol_xhm)//')',i=1,nmodel)/))])
                   endif
               else
                   selected_cif = 1
               endif
               deallocate(vmodel_cif)
               err_read = errcif(selected_cif)
!
!              controlla la cella 
               if (.not.is_cell) then
                   call set_cell(phase_file(selected_cif)%cell%get_par())
               endif
               if (.not.is_space_defined_list()) then
                   if (phase_file(selected_cif)%exist_spg()) then
                       call set_spaceg_new(phase_file(selected_cif)%spg)
                   else
                       call set_spaceg_new(init_spaceg_type())   ! force 'P 1'
                   endif
               endif
               if (numatomtype(phase_file(selected_cif)%at) > 0) then
                   call copia_atoms(atomc,phase_file(selected_cif)%at)
                   make_conn = .true.
                   is_bfac = .true.
                   trasl = .false.
                   make_occ = .false.
               endif
           else
               err_read = errcif(1)
           endif

         case (ZMT_FILE)    ! z-matrix
           if (.not.is_cell_defined()) call create_cell()
           if (.not.is_space_defined()) call set_spaceg_new(init_spaceg_type())   ! force 'P 1'
           call read_zmtfile(j_in,cell,atomc,err_read)
           if (.not.err_read%signal) then
               make_conn = .true.
               make_label = .true.
               make_occ = .true.
           endif

         case (MOP_FILE)    ! mop file
           if (.not.is_cell_defined()) call create_cell()
           if (.not.is_space_defined()) call set_spaceg_new(init_spaceg_type())   ! force 'P 1'
!
!          Prova a leggere il file come mop in formato coordinate interne
           call read_mopintfile(j_in,cell,atomc,err_read)
!      
!          se si genera errore leggi il file come mop a coordinate cartesiane
           if (err_read%signal) then
               call reallocate(atomc,0)
               rewind (j_in)
               call read_mopcartfile(j_in,cell,atomc,err_read)
           endif
           if (.not.err_read%signal) then
               make_conn = .true.
               make_occ = .true.
               make_label = .true.
               trasl = .true.
           endif

         case (MOL_FILE)    ! mol file
           if (.not.is_cell_defined()) call create_cell()
           if (.not.is_space_defined()) call set_spaceg_new(init_spaceg_type())   ! force 'P 1'
           call read_molfile(j_in,cell,atomc,legmc,err_read)
           if (.not.err_read%signal) then
               call copia_legm(legm,legmc)
               trasl = .true.
               make_occ = .true.
               make_label = .true.
           endif

         case (MOL2_FILE)    ! mol2 file
           call read_mol2file(j_in,atomc,legmc,spgfile,cellfile,err_read)
           if (.not.err_read%signal) then
               if (.not.is_cell_defined()) call set_cell(cellfile)
               call cart_to_frac(atomc,orthomatrixi_std(cell))
               if (.not.is_space_defined()) call set_spaceg_new(spgfile)
               call copia_legm(legm,legmc)
               trasl = .true.
               make_occ = .true.
           endif

         case default 
           call err_read%set('Unknown file format. File '//trim(filename))

       end select

       close (j_in)

   else
       !if (.not.file_exist(filename)) then
       !   call err_read%set(PkgName(1:lPkgName)//" can't find file "//trim(filename))
       !else
       !   call err_read%set('Error opening the file '//trim(filename))
       !endif
       call err_read%set('Cannot open: '//trim(filename)//char(10)//' Message: '//trim(fileh%err_msg()))
   endif
!
   if (err_read%signal) then
       call err_read%print()
       ier = 1
   else

!!NEW mr
       if (check_cell) then
           do i=1, 6
              a1 = abs(cell(i)-celltemp(i))
              a2 = a1 / cell(i)
              if (a2 > 0.01) then
                  call err_read%set('Previous and actual cell parameters differ. ' // 'File is abandoned.')
                  call err_read%print()
                  ier = 1
                  cell(1:6) = celltemp(1:6)
                  return
              endif
           enddo
      endif
!!End NEW

       if (numatomtype(atomc) > 0) then  ! cif files can't contain atom coordinates
           call copia_atoms(atom,atomc)
!
!          Trasla il frammento in cella: azione eseguita solo se il file contiene coord. cart.
           if (trasl) call translate_in_cell(atom)
!
!          Update content
           call elem_from_atoms(elemf,atom(:)%nz) 
           if (numelem(elemf) > 0) then
               if(nsymtot > 0) elemf%nw = elemf%nw*nsymtot
               elemf%radtype = neutro
               call ordina_elements(elemf,neutro)
               call add_element(celem,elemf)
               call elem_to_nk(celem)
           endif
           atom(:)%ptab = atom(:)%nz   !!! FIXME- da rimuovere quando ptab sara' defenito per ogni file importato
           atom(:)%nz = nz_from_pxen(atom(:)%nz,celem)
!
!          Creare le label
           if (make_label) call atom_string(atom)  
!
!          Genera connettivita' se non esiste nel file importato
           if (make_conn) then
!              disable because coordinates could be wrong or for too atoms
               !if (err_read%warning .or. size(atom) > NMAXATOM) optconn(:) = .false.   !!FIXME - potresti usare un code error
               if (size(atom) > NMAXATOM) optconn(:) = .false. 
               if (filetype == PDB_FILE) then  ! special connectivity for duplicate residues
                   optconn(:) = .false.
                   call create_connectivity(atom,legm,set_cell_type(cell),check=optconn(1),move=optconn(2), &
                        vet=atom(:)%rcod(1),code=2)
                   atom%rcod(1) = 0
               else
                   call create_connectivity(atom,legm,set_cell_type(cell),check=optconn(1),move=optconn(2))
               endif
           endif
!
!          Genera fattori termici se non esistono nel file
           if(.not.is_bfac)call set_biso(atom)
!
!          Calcola occupanza cristallog. senza modificare le coordinate     
           call calcola_occ(atom,modcoor=.false.)
           if(make_occ) atom%och = 1.0  ! inizializza occ. chimica
!
           call set_intensity(atom)
!
!!!corr           if (npunti > 0) call abilita_tasti('structure',1)
       else
           call reallocate(atom,0)
           call reallocate_legm(legm,0)
       endif
!
       if (err_read%warning) call err_read%print()
!!!!!!!!!!!!!test
            !call make_test(atom,legm)
            !call compute_doc(atom,1.0)
!!!!!!!!!!!!!test
      ! stop
   endif
          !call test_bond_perception(atom,legm); stop
     !call  search_overlapped_atoms_print(atom)
     !call  search_overlapped_atoms(atom,.true.)
     !call stampa_struttura(atom,write_occ=.true.,kpr=0)
!
   end subroutine import_structure

!---------------------------------------------------------------------      

   subroutine cif_func(code)
   integer, intent(in) :: code
   select case (code)
      case (:-1)
        selected_cif = vmodel_cif(-code)
      case (0)  ! open list
      case (1)  ! Ok on list
      case (2)  ! Cancel on list
   end select
   end subroutine cif_func

!---------------------------------------------------------------------      

   subroutine import_from_file(filename,atom,legm,ier)
   USE atom_type_util
!
   character(len=*), intent(in)                              :: filename
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   integer                                                   :: itype
   integer                                                   :: ier
!
   itype = code_fragfilen(filename)  ! codice dall'estensione del file
!
!  importa il nuovo modello e updata il contenuto
   if (itype > 0) then
       call import_structure(filename,itype,atom,legm,.false.,ier)
   else
       ier = 1
   endif
!
   end subroutine import_from_file

!---------------------------------------------------------------------      

   subroutine add_structure(filename,atom,legm,ier)
   USE connect_mod
   USE atom_type_util
   USE general, only: celem
   character(len=*), intent(in)                              :: filename
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   integer, intent(out)                                      :: ier
   type(atom_type), dimension(:), allocatable                :: atom1
   type(bond_type), dimension(:), allocatable                :: legm1
   integer                                                   :: itype
   integer                                                   :: natnew
   integer, dimension(size(atom))                            :: pxen
!
!  Leggi la struttura dal file mettila in atom1/legm1
   itype = code_fragfilen(filename)  ! codice dall'estensione del file
!   
!  salva pxen degli atomi correnti perche' la import_structure modifica il contenuto
   pxen(:) = pxen_from_nz(atom%nz)
!
!  importa il nuovo modello e updata il contenuto
   call import_structure(filename,itype,atom1,legm1,.true.,ier)
!
   if (ier == 0) then
!        
!      ricrea il nuovo nz per atom dal pxen salvato 
       atom(:)%nz = nz_from_pxen(pxen,celem)
!
!      Combina la nuova struttura con quella esistente in atom/legm
       call add_atoms_to_list(atom,atom1,natnew,legm1=legm,legm2=legm1)
   endif
!
   end subroutine add_structure

!---------------------------------------------------------------------      

   subroutine import_fragment(atom,legm,itipo,kpos,kspec1i,kspec2i,kadd,bd)
   USE General, only: cell, celem
   USE atom_type_util
   USE Atomi, only:nzConst
   USE variables, only:ortoi,ortom
   USE unit_cell
   USE connect_mod
   USE realloc
   USE elements
   USE Symmetry, only: nsymtot
   USE proginterface
!
   type(atom_type), dimension(:), allocatable, intent(inout) :: atom   
   type(bond_type), dimension(:), allocatable, intent(inout) :: legm
   integer, intent(in)                                       :: itipo         ! tipo di frammento
   integer, intent(in)                                       :: kpos          ! posizione del frammento
   integer, intent(in)                                       :: kspec1i,kspec2i ! specie nel frammento
   integer                                                   :: kadd          ! 1 se il frammento va aggiunto in atom
   real, intent(in)                                          :: bd            ! distanza 
   type(atom_type), allocatable, dimension(:)                :: atom1
   integer                                                   :: i
   real, dimension(3)                                        :: xpos
   integer                                                   :: kkpos
   logical                                                   :: centr
   integer                                                   :: n1
   integer                                                   :: nat
   integer                                                   :: nadd,nlegadd
   real, dimension(3)                                        :: xrand
   logical                                                   :: addnew1,addnew2
   integer                                                   :: kspec1,kspec2
   integer, dimension(4)                                     :: newsize
!
   nat = numatomtype(atom)
!
!  If kspec is negative add new elements to the content (Z number = -kspec)
   if (kspec1i < 0 .or. kspec2i < 0) then
       if (kpos == 0) then  ! define number of atoms to add
           newsize = [5,7,5,kspec2i]
       else
           newsize = [4,6,4,kspec2i]
       endif
       addnew1 = .false.
       if (kspec1i < 0) then
           call add_element(celem,abs(kspec1i),nw=newsize(itipo)*nsymtot,add=addnew1)
           kspec1 = is_element(celem,abs(kspec1i)) 
       endif
       addnew2 = .false.
       if (kspec2i < 0) then
           call add_element(celem,abs(kspec2i),nw=nsymtot,add=addnew1)
           kspec2 = is_element(celem,abs(kspec2i))  
       endif
       if (addnew1 .or. addnew2) call elem_to_nk(celem)
   endif
   if (kspec1i > 0) kspec1 = kspec1i
   if (kspec2i > 0) kspec2 = kspec2i
!
!  Posizione del frammento
   if (kpos == 0) then
       call random_number(xpos)  ! in posizione random
       kkpos = -1
       centr = .true.
   else
       xpos = atom(kpos)%xc     ! nella posizione dell'atom kpos
       kkpos = kpos - nat      ! serve ad allineare la connettivita
       centr = .false.
   endif
   ortoi = orthomatrixi(cell)
!
!  crea il frammento
   select case(itipo)
       case(1)      ! tetrahedron
         call create_tetrahedron(atom1,bd,ortoi=ortoi,xpos=xpos,kcentr=kkpos,centro=centr)

       case(2)      ! octahedron
         call create_octahedron(atom1,bd,ortoi=ortoi,xpos=xpos,kcentr=kkpos,centro=centr)

       case(3)      ! square plane
         call create_square(atom1,bd,ortoi=ortoi,xpos=xpos,kcentr=kkpos,centro=centr)

       case(4)      ! isolated atoms
         if (kadd == 0) then
             n1 = 1
             nat = kspec2
             call reallocate_legm(legm,0)
         else
             n1 = nat + 1
             nat = kspec2 + nat
         endif
         call reallocate(atom,nat)
         do i=n1,nat
            call random_number(xpos)
            atom(i)%xc = xpos
            atom(i)%nz = kspec1*nzConst
         enddo

   end select
!
!  assegna nz
   if (itipo /= 4) then
       if (kpos == 0) then
           atom1(1)%nz = kspec1*nzConst
           atom1(2:)%nz = kspec2*nzConst
       else
           atom1(1:)%nz = kspec2*nzConst
       endif
!
!      combina i 2 modelli aggiungendo il nuovo frammento in fondo alla lista
       if (kadd == 1) then
           nadd = size(atom1)   ! numero atomi da aggiungere
!
!          Applica una rotazione random intorno al baricentro per kpos/=0 
!          In questo modo frammenti centrati sullo stesso atomo (kpos/=0) non vanno in sovrapposizione
           if (itipo /= 4 .and. kpos /=0) then
               ortom = orthomatrix(cell)
               ortoi = orthomatrixi(cell)
               call random_number(xrand)
               call rand_rotate_atoms(atom1,xrand)
           endif
!
           call add_atoms_to_list(atom,atom1,nat)
           n1 = nat - nadd + 1  ! puntatore al primo atomo aggiunto
           if (kpos == 0) then
               nlegadd = nadd - 1   ! numero legami da aggiungere
               call add_bonds(legm,atom,(/(n1,i=1,nlegadd)/),(/(i+n1,i=1,nlegadd)/))
           else
               nlegadd = nadd       ! numero legami da aggiungere
               call add_bonds(legm,atom,(/(kpos,i=1,nlegadd)/),(/(i+n1-1,i=1,nlegadd)/))
           endif
       else
           call copia_atoms(atom,atom1)
           call reallocate_legm(legm,0)
           nat = size(atom)
           n1 = 1
           call add_bonds(legm,atom,(/(n1,i=1,nat-1)/),(/(i+n1,i=1,nat-1)/))
       endif
   endif
!
!  Calcola occupanza cristallog. senza modificare le coordinate     
   call calcola_occ(atom(n1:nat),modcoor=.false.)
   atom(n1:nat)%och = 1.0  ! inizializza occ. chimica
!
   call set_biso(atom(n1:nat))
!
!  Assegna intensita in base a Z
   call set_intensity(atom(n1:nat))
!          
   call atom_string(atom,vet=(/(i,i=n1,nat)/))
!
   end subroutine import_fragment

!---------------------------------------------------------------------      
   
   subroutine export_structure(filename,atom,legm,ftype,comm,iscell,spg)
   USE General, only:cell,errcell,ifW95,StructureName,nk,celem,alambda=>wavel,package_alt_name,neutro
   USE chem_file_fmt
   USE errormod
   USE connect_mod
   USE proginterface
   USE elements
   USE sinterface
   USE spginfom
   USE cif_frm
   USE pdb_frm
   USE shelx_frm
   character(len=*), intent(in)                                    :: filename
   type(atom_type), dimension(:), intent(in), allocatable          :: atom
   type(bond_type), dimension(:), allocatable, intent(in),optional :: legm
   integer, intent(in)                                             :: ftype
   character(len=*), intent(in), optional                          :: comm
   logical, intent(in), optional                                   :: iscell
   type(spaceg_type), optional                                     :: spg
   type(error_type)                                                :: err_exp
   real, dimension(nk,3)                                           :: cols
   real, dimension(3)                                              :: colcell
   real, dimension(2)                                              :: vfact
   real, dimension(3,3)                                            :: bmat
   type(spaceg_type)                                               :: spaceg
   integer                                                         :: filetype
!corr   real :: alambda = 1.54056 !!! da sistemare
!
   call err_exp%reset
   if (ftype == 0) then
       filetype = code_fragfilen(filename)
   else
       filetype = ftype
   endif
!
   if (filetype == SHELX_FILE .or. filetype == CIF_FILE) then
       if (present(spg)) then
           spaceg = spg
       else
           spaceg = init_spaceg_type(spaceg_string())
       endif
   endif
!
   select case (filetype)
      case (SHELX_FILE)    ! .res file
        call create_shelxfile(filename,atom,legm,cell,errcell,spaceg,alambda,celem,StructureName)

      case (PDB_FILE)     ! pdb file
        call create_pdbfile(filename,atomi=atom,cella=cell,legm=legm,spg=spaceg_string())

      case (FRA_FILE)     ! fra file
        call create_frafile(filename,atom=atom,cella=cell,comment='Structure '//trim(StructureName))

      case (XYZ_FILE)    ! cartesian xyz file
        call create_xyzfile(filename,atom=atom,cella=cell,comment='Structure '//trim(StructureName))

      case (POV_FILE)    ! POV file
        call jav_get_style(cols, colcell, vfact, bmat)
        call create_povrayfile(filename,atom=atom,legm=legm,elem=celem,cols=cols,colc=colcell,iscell=iscell,rmat=bmat)
#if _WIN32
        call Csystem('C:\PROGRA~1\POV-RA~1.5\bin\pvengine.exe '//'+I'//trim(filename)//char(0))
#else
        call Csystem('povray +P +W800 +H600'//' +I'//trim(filename)//' &'//char(0))
#endif

      case (CIF_FILE)    ! CIF file
!
        if (present(comm)) then
            if (present(legm)) then
                call create_ciffile(filename,cella=cell,atom=atom,progname=package_alt_name,    &
                spg=spaceg,elem=celem,commline=comm,legm=legm,wave=alambda,source_type=neutro)
            else
                call create_ciffile(filename,cella=cell,atom=atom,progname=package_alt_name,    &
                spg=spaceg,elem=celem,commline=comm,wave=alambda,source_type=neutro)
            endif
        else
            if (present(legm)) then
                call create_ciffile(filename,cella=cell,atom=atom,progname=package_alt_name,    &
                spg=spaceg,elem=celem,legm=legm,wave=alambda,source_type=neutro)
            else
                call create_ciffile(filename,cella=cell,atom=atom,progname=package_alt_name,    &
                spg=spaceg,elem=celem,wave=alambda,source_type=neutro)
            endif
        endif

      case (ZMT_FILE)    ! zmt file (Fenske-Hall Z-Matrix)
        call create_zmtfile(filename,atom=atom,legm=legm,cella=cell)

      case (MOL_FILE)    ! mol file
        call create_molfile(filename,cella=cell,atom=atom,legm=legm,progname=package_alt_name,error=err_exp)

      case (MOP_FILE)    ! cartesian mop file 
        call create_mopcrtfile(filename,cella=cell,atom=atom,title=StructureName)

      case default 
        call err_exp%set('Unknown file format. File '//trim(filename))
   end select
!
   if (err_exp%signal) then
       call err_exp%print()
   endif
!
   end subroutine export_structure

!---------------------------------------------------------------------      

   subroutine export_info(filename,ftype,vet,atom,legm,legmc,adp_type,infos)
   USE connect_mod
   USE fileutil
   USE atom_type_util
   USE general, only:spacegr
   character(len=*), intent(in)                           :: filename
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legm
   integer, intent(in)                                    :: ftype ! che tipo di informazioni
   integer, dimension(:), intent(in)                      :: vet   ! quali informazioni
   integer, intent(in), optional                          :: adp_type ! adp type: 1 for B, 2 for U
   type(symminfo_type), dimension(:), allocatable, optional :: infos
   integer                                                :: j_in
   integer                                                :: ier
   type(angle_type), dimension(:), allocatable            :: angle
   type(torsion_type), dimension(:), allocatable          :: tors
   type(structure_info_type)                              :: info
   type(bond_type), dimension(:), allocatable, intent(in), optional :: legmc
!
!  Open file
   call get_unit(j_in)
   open(unit=j_in,file=filename,iostat=ier,status='unknown')
!
   if (ier == 0) then
       select case (ftype)
          case (1)    ! general info
            info = structure_info(atom,1)
            call print_structure_info(info,j_in)

          case (2)    ! atomi
            if (present(adp_type)) then
                call stampa_strutturan(atom,kpr=j_in,vet=vet,adp_type=adp_type,infos=infos,spg=spacegr)
            else
                call stampa_strutturan(atom,kpr=j_in,vet=vet,infos=infos,spg=spacegr)
            endif

          case (3)    ! legami
            call print_connect(atom%lab,legm=legm,kpri=j_in,vet=vet)

          case (4)    ! angoli
            call legm_to_ang(legm,atom,angle)
            call print_connect(atom%lab,angle=angle,kpri=j_in,vet=vet)

          case (5)    ! torsioni
            call legm_to_tors(legm,atom,tors)
            call print_connect(atom%lab,tors=tors,kpri=j_in,vet=vet)
        
          case (7)
            call print_connect(atom%lab,legm=legmc,kpri=j_in,vet=vet)

       end select

       close (j_in)
   endif
!
   end subroutine export_info

!---------------------------------------------------------------------      

   integer function code_fragfilen(filedirname)   result(code)
!
!  Utilizza l'estensione del nome del file per individuare il tipo
!
   USE strutil
   USE fileutil
   character(len=*), intent(in)         :: filedirname
   !character(len=len_trim(filedirname)) :: ext
!
   !ext = upper(get_extension(filedirname))
   !select case (ext(:Lung(ext)))
   select case (remc0(upper(get_extension(filedirname))))
     case ('FRA','FRAC')
       code = FRA_FILE

     case ('XYZ')
       code = XYZ_FILE

     case ('RES','INS')
       code = SHELX_FILE

     case ('PDB','ENT')
       code = PDB_FILE

     case ('CIF')
       code = CIF_FILE

     case ('ZMT','FH','FHZ')
       code = ZMT_FILE

     case ('MOP')
       code = MOP_FILE

     case ('MOL')
       code = MOL_FILE

     case ('MOL2','ML2')
       code = MOL2_FILE

     case ('POV')
       code = POV_FILE

     case ('LOG')
       code = GAMESS_OUT

     case ('OUT')
       code = NWCHEM_OUT

     case default 
       code = 0

   end select
!
   end function code_fragfilen

!---------------------------------------------------------------------      

   function structure_info(atom,icalc) result(info)
   USE General, only:nk,nw,no,cell,Species,nrefl,spacegr
   USE unit_cell
   USE Symmetry,only:icent,nsymtot
   USE variables, only:ref
   USE reflection_type_util
   USE atom_type_util
   USE stat
   USE sinterface
   type(structure_info_type)                              :: info
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   integer, intent(in)                                    :: icalc
   integer                                                :: i
!
   info%nspec = nk
   info%cell = cell(1:6)
   do i=1,nk
      info%vspec(i) = nw(i)
      info%zspec(i) = no(i)
   enddo
   info%spaceg = trim(spaceg_string())
   info%icent = icent
!corr   info%nsymmstr = nSymmStrings
   info%nsymmstr = spacegr%nsymop
!
!  Se icalc = 0 non vengono riempite quelle informazioni che richiedono un calcolo
   if (icalc == 1) then
       info%cell_volume = cell_volume(cell(1:6))
       if (numatomtype(atom) > 0) then
           info%volume_per_atom = volume_per_atom(atom, nsymtot, info%cell_volume)
       else
           info%volume_per_atom = volume_per_atom(Species(:nk),nw(:nk),info%cell_volume)
       endif
       if (.not.allocated(ref)) call fillref(ref)
       if (numatomtype(atom) > 0 .and. nrefl > 0) then
           call fcalcang(ref,atom)
           info%r_factor = Rvalue(ref%fo,ref%fc)
       else
           info%r_factor = 0.0
       endif
   endif
!
   end function structure_info

!---------------------------------------------------------------------      

   subroutine print_structure_info(info,kpr)
   USE General, only:StructureName,cell,Species,spacegr
!corr   USE Symmetry
   type(structure_info_type), intent(in) :: info
   integer, intent(in)                   :: kpr
   integer                               :: i
!
   write(kpr,'("Name                : ",a)')trim(StructureName)
   write(kpr,'("Cell Lengths        : ","a ",f0.5," b ",f0.5," c ",f0.5)')cell(1:3)
   write(kpr,'("Cell Angles         : ", "alpha ",f0.5," beta ",f0.5," gamma ",f0.5)')cell(4:6)
   if (info%icent == 1) then
       write(kpr,'("Space Group         : ",a)')trim(info%spaceg)//' (centrosymmetric)'
   else
       write(kpr,'("Space Group         : ",a)')trim(info%spaceg)//' (noncentrosymmetric)'
   endif
   call spacegr%prn_symop(2,kpr)
!corr   write(kpr,'("Symmetry-operations   ")')
!corr   write(kpr,'(4x,i0,a,a)')(i,')',trim(SymmStrings(i)),i=1,nSymmStrings)
!corr   write(kpr,'(4x,i0,a,a)')(i,')',trim(spacegr%symopstr(i)),i=1,spacegr%nsymop)
   write(kpr,'("Cell Volume         : ",f0.3)')info%cell_volume
   write(kpr,'("Cell Content        : ",18(a,1x,i0,1x))')(trim(Species(i)),info%vspec(i),i=1,info%nspec)
   write(kpr,'("Volume per atom     : ",f0.3)')info%volume_per_atom
   write(kpr,'("R-Factor            : ",f0.2)')info%r_factor
!
   end subroutine print_structure_info

!---------------------------------------------------------------------      

   subroutine test_bond_perception(atom,legmv)
   USE fileutil
   USE connect_mod, only: numbonds, copia_legm
   USE atom_type_util
   USE atom_basic
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legmv
   type(bond_type), dimension(:), allocatable             :: legm
   type(file_handle)                                      :: fileh
   integer                                                :: ncorr
   integer                                                :: i
!
   call copia_legm(legm,legmv)
   call bond_type_perception(atom,legm)   ! create new bond order
!
!  Compare legm%ord with legmv%ord e save the result in a external file
   call fileh%fopen('result_bond.txt','w')
   if (fileh%good()) then
       ncorr = 0
       do i=1,numbonds(legmv)
          if (legmv(i)%ord /= legm(i)%ord) then
              ncorr = ncorr + 1
              write(fileh%handle(),'(a,a,i0,a,i0)')'legame sbagliato: ',    &
              trim(slabnum(atom(legm(i)%n1)%lab,legm(i)%n1))//'-'//trim(slabnum(atom(legm(i)%n2)%lab,legm(i)%n2))   &
              //' ord=', legm(i)%ord,' -> ordine corretto=',legmv(i)%ord
              write(0,'(a,a,i0,a,i0)')'legame sbagliato: ',    &
              trim(slabnum(atom(legm(i)%n1)%lab,legm(i)%n1))//'-'//trim(slabnum(atom(legm(i)%n2)%lab,legm(i)%n2))   &
              //' ord=', legm(i)%ord,' -> ordine corretto=',legmv(i)%ord
          endif
       enddo
       if (ncorr == 0) then
           write(fileh%handle(),'("No errors in the bond order!")')
           write(0,'("No errors in the bond order!")')
       endif
       call fileh%fclose()
   endif
   !stop   !uncomment for shell
   end subroutine test_bond_perception


END MODULE importexport

   real function GetBondDistance(ispec1, ispec2)
!
!  Restituisce la distanza di legame date due specie
!      
   USE bondtmod
   USE General
   implicit none
   integer :: ispec1, ispec2
   real    :: bondt                      
   integer :: z1,z2
!
   if (ispec1 < 0) then
       z1 = abs(ispec1)
   else
       z1 = no(ispec1)
   endif
   if (ispec2 < 0) then
       z2 = abs(ispec2)
   else
       z2 = no(ispec2)
   endif
   bondt = bond_table(1,z1,z2)
   if (bondt > 0.000) then
       GetBondDistance = bondt
   else
       GetBondDistance = 1.4
   endif
!
   end function GetBondDistance

MODULE exportfiles

implicit none

CONTAINS
   
   subroutine update_model_and_export(atom,legm,kmodel)
   USE atom_basic
   USE connect_mod
   USE sinterface
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   type(bond_type), dimension(:), allocatable, intent(in) :: legm
   integer, intent(in)                                    :: kmodel
!
   if (kmodel == 1) then
       call update_model(atom,legm)
   endif
!
   end subroutine update_model_and_export

END MODULE exportfiles
