!-----------------------------------------------------------------------
!    MTZ routines
!-----------------------------------------------------------------------
      subroutine ManageMTZ(iAction, celltemp, lix1, lix2, ier)
!
!-- iAction = 1   Open MTZ file, get cell & symmetry
!-- iAction = 2   Initialize reflections reading
!-- iAction = 3   Close MTZ file 
!
      USE General
      USE Symmetry
      USE RemoModule
      USE DataModule
!
      USE spginfom
!
      implicit none
      integer :: iAction, lix1, lix2, ier, kshif(3), ishif
      integer :: i, j, k, k1, k2
      integer :: itype, iform, jop, jopen, ierr, nExitCode, ifail, Lung
      integer :: nPrSym, nSpGroup, Num_Col, nFasi, nFasiS, if_F_F2
      real    :: celltemp(13)
      character (len= 1)  :: lattMTZ
      character (len=10)  :: SpGroupName, PointGroup
!
      if (iAction == 1) then
          itype = 2
          iform = 1
          jop = jopen(kform, MTZfile, itype, iform, ierr)
          if (jop == 0) then
              close(kform)
          else
              ier = -1
              Msg = 'Error opening MTZ file ' // MTZfile
              call MsgWinErr('ERROR', Msg, ERR_WINDOW, nExitCode)
              return
          endif
          MTZprint = 0
          MTZunit  = 1

!
          call mtzini
          call lropen(MTZunit, MTZfile, MTZprint, ifail)
          if (ifail /= 0) then
              ier = -1
              Msg = 'Error managing MTZ file'
              call MsgWinErr('ERROR', Msg, ERR_WINDOW, nExitCode)
              return
          endif
          MTZ_Num_Col = 100
          call lrinfo(MTZunit, MTZSubrVers, MTZ_Num_Col, 
     *                nreflMTZ, MTZranges)
          call lrclab(MTZunit,Lab_Col,tipo_colonne,MTZ_Num_Col)
!
!-- check for R-Free in input
!
          if (QualePGM == SIRPROTPGM) then
              do i=1, MTZ_Num_Col
                 if (tipo_colonne(i) == 'I') then
                     ifRFree = 1
                     exit
                 endif
              enddo
          endif
!
!-- Cell Parameters
!
          if (lix1 == 0) then
              celltemp(:) = 0.0
              call lrcell(MTZunit,celltemp)
              call CreaCellInfo(celltemp)! (1:6))
              cellSav(:) = cell(1:6)
              errcellSav(:) = errcell(:)
              lix1 = 1
          endif
!
!-- Symmetry
!
          if (noSpaceGroup == 0) then
              if (lix2 == 0) then
!
                  OkOrig    = 1
!
                  call lrsymi(MTZunit, nPrSym, lattMTZ, nSpGroup, 
     *                        SpGroupName, PointGroup)
                  if (nSpGroup >= 1 .and. nSpGroup <= 230) then
                      write(cardDiret,'(i5)') nSpGroup
                  else
                      cardDiret = SpGroupName
                  endif
                  call shift(cardDiret,k)
                  read(cardDiret,'(16a1)') nSpaceG
                  call CheckSpaceGroupSymbol(nSpaceG,ier)
                  if (ier.ge.0) then
                      write(SpaceSav,'(16a1)') nSpaceG
                      cardDiret = SpaceSav
                      spacegr = init_spaceg_type(cardDiret) ! set space group in structure spacegr
                      lix2=1
                      jspini = -1
                      ishif = 0
                      kshif(:) = 0
                      call insym(ishif,kshif,ier)
                      nsymTrue = nsym
                      call xspinb
                      call CreaSymmetryInfo(ier)
                      lix2 = 1
                  else
                      ier = -1         ! errore nella direttiva
                      Msg = 'Error processing information in MTZ file'
                      call MsgWinErr('ERROR',Msg, ERR_WINDOW, nExitCode)
                      return
                  endif
              endif
          endif
          Num_Col = 100
          if (ifMTZlab == 0) then
              if (QualePGM == SIRPGM) then
                  knw = 0
                  MTZ_Num_Col = 5
                  Lab_ch(1:MTZ_Num_Col) = (/'H   ', 'K   ', 'L   ',
     *                                      'F   ', 'SIGF'/)
                  do i=1, Num_Col
                     if (Lab_col(i) == 'PUB') then
                         knw = 1
                         MTZ_Num_Col = 6
                         Lab_ch(1:MTZ_Num_Col) = (/
     *                   'H   ', 'K   ', 'L   ','PUB ','F   ', 'SIGF'/)
                         exit
                     endif
                  enddo
              else
                  Lab_ch(1:MTZ_Num_Col) = Lab_col(1:MTZ_Num_Col)
              endif
          endif
          if (QualePGM == SIRPROTPGM) then
              if (ifRFree == 1) then
                 ifRFree = 0
                 do i=1, MTZ_Num_Col
                    if (index(Lab_Ch(i),'FreeR_flag') > 0) then
                        ifRFree = 1
                        exit
                    endif
                 enddo
              endif
          endif
          call Fill_lookup(Num_Col,Lab_Col,Lab_Ch,MTZlookup)
          if (tipo_colonne(MTZlookup(4)) == 'P') then
              knw = 1
          endif
*         if (ifMTZlab == 0) then
*             if (Lab_Col(6)(1:3) == 'PUB'. or. 
*    *            Lab_Col(6)(1:4) == 'PHIC') then
*                 knw = 1
*             endif
*         else
*             if (Lab_Ch(4)(1:3) == 'PUB'. or. 
*    *            Lab_Ch(4)(1:4) == 'PHIC') then
*                 knw = 1
*             endif
*         endif
      elseif (iAction == 2) then
!
          Num_Col = 100
          if (tipo_colonne(1) /= 'H' .or.
     *        tipo_colonne(2) /= 'H' .or.
     *        tipo_colonne(3) /= 'H') then
              ier = -1
              Msg = 'Non standard column types in MTZ file'
              call MsgWinErr('ERROR', Msg, ERR_WINDOW, nExitCode)
              return
          endif

          if (ifMTZlab == 0) then
              if (QualePGM == SIRPGM) then
                  knw = 0
                  MTZ_Num_Col = 5
                  Lab_ch(1:MTZ_Num_Col) = (/'H   ', 'K   ', 'L   ',
     *                                      'F   ', 'SIGF'/)
                  do i=1, Num_Col
                     if (Lab_col(i) == 'PUB') then
                         knw = 1
                         MTZ_Num_Col = 6
                         Lab_ch(MTZ_Num_Col) = 'PUB'
                         exit
                     endif
                  enddo
              else
                  Lab_ch(1:MTZ_Num_Col) = Lab_col(1:MTZ_Num_Col)
              endif
          endif
*         call Fill_lookup(Num_Col,Lab_Col,Lab_Ch,MTZlookup)
!pgMAD131006
          if (Num_Col < 0) then
              if (printMTZerror) then
                  k1 = iabs(Num_Col)
                  k2= Lung(Lab_Ch(k1))
                  Msg = 'Label not found in MTZ file: '
     *               // Lab_Ch(k1)(1:k2)
                  call MsgWinErr('ERROR', Msg, ERR_WINDOW, nExitCode)
              endif
              ier = -1
              return
          endif
!pgMAD131006
          k = 0
          do i=1, Num_Col
             if (MTZlookup(i) == 0) exit
             k = i
          enddo
          MTZ_Num_Col = k
          if (MTZ_Num_Col < (5+knw)) then
              if (printMTZerror) then
                  k1 = MTZ_Num_Col + 1
                  k2= Lung(Lab_Ch(k1))
                  Msg = 'Label not found in MTZ file: '
     *               // Lab_Ch(k1)(1:k2)
                  call MsgWinErr('ERROR', Msg, ERR_WINDOW, nExitCode)
              endif
              ier = -1
              return
          endif
          knw = 0
          nFasi = 0
          do j=1, MTZ_Num_Col
             i = MTZlookup(j)
             if (tipo_colonne(i) == 'P') then
                 nFasi = nFasi + 1
c                exit
              endif
          enddo
          nFasiS = 1
          if (QualePgm == SIRPROTPGM) then
              if (iphase > 0) then
                  nFasiS = 2
              endif
          else
              nFasi = nFasi - ifRecovered
          endif
          if (nFasi >= nFasiS) then
              knw = 1
          endif
          ifo2 = -1
          do j=4, Num_Col
             if (MTZlookup(j) == 0) exit
             i = MTZlookup(j)
             if (tipo_colonne(i) == 'F' .or.
     *           tipo_colonne(i) == 'G') then
                 ifo2 = 1
                 if_F_F2 = 1
                 exit
             elseif (tipo_colonne(i) == 'J' .or.
     *               tipo_colonne(i) == 'K') then
                 ifo2 = 2
                 if_F_F2 = 1
                 exit
              endif
          enddo
          if (ifo2 == -1) then
              ier = -1
              Msg = 'Non standard column types in MTZ file'
              call MsgWinErr('ERROR', Msg, ERR_WINDOW, nExitCode)
              return
          endif
      elseif (iAction == 3) then
          call lrclos(MTZunit)
      endif
!
      return
      end
!----------------------------------------------------------------------
      logical function IsItaNaN(arg)
      character*10 line

      write(line,'(f10.2)') arg
      lun = len_trim(line)

      do i=1, lun
         k = ichar(line(i:i))
         if (line(i:i) /= ' ') then
            exit
         end if
      end do

      line = line(i:lun)
      call lcase(line)
!     if (line(1:3) == 'NaN') then
      if (index(line, 'nan') > 0) then
         IsItaNaN = .true.
      else          
         IsItaNaN = .false.
      end if

      return
      end
!-----------------------------------------------------------------------
      subroutine Fill_lookup(Num_Col,Lab_Col,Lab_Ch,lookup)
      character *(*) Lab_Ch(Num_Col),Lab_Col(Num_Col)
      integer lookup(Num_Col)

      lookup(:) = 0
      do i=1, Num_Col
         LL = Lung(Lab_Ch(i))
         if (LL == 0) then
            exit
         end if
         do j=1, Num_Col
            if (Lab_Col(j) == Lab_Ch(i)) then
               lookup(i) = j
               exit
            end if
         end do
!pgMAD131006
         if (lookup(i) == 0) then
             Num_Col = -i
             return
         endif
!pgMAD131006
      end do
!
      return
      end
!----------------------------------------------------------------------
      subroutine WriteMTZCont()
      USE General
!
      write(lo,90) 
      write(lo,100) MTZfile(1:lung(MTZfile))
      write(lo,110) MTZSubrVers 
      write(lo,120) nreflMTZ
      write(lo,130) MTZ_Num_Col
      write(lo,140)

      if (QualePgm == SIRPROTPGM) then
          write(ktml,'("<b>")')
          write(ktml,100) MTZfile(1:lung(MTZfile))
          write(ktml,'("</b>")')
          write(ktml,110) MTZSubrVers
          write(ktml,120) nreflMTZ
          write(ktml,130) MTZ_Num_Col
          write(ktml,140)
      endif

      do i=1,MTZ_Num_Col
         ii = MTZlookup(i)
         if (ii == 0) cycle
         write(lo,150) ii,tipo_colonne(ii),Lab_Col(ii),MTZranges(1:2,ii)
         if (QualePgm == SIRPROTPGM)
     *   write(ktml,150)i,tipo_colonne(ii),Lab_Col(ii),MTZranges(1:2,ii)
      enddo
      write(lo,90)
      if (QualePgm == SIRPROTPGM)
     *write(ktml,'("<br><br>")')
!
      return
   90 format(' ',78('-'))
  100 format(' MTZ file ',a,/,
     *       ' Summary of content:')
  110 format('   Software version  : ',a)
  120 format('   Reflexions stored : ',i6)
  130 format('   Number of columns : ',i6)
  140 format('   Column# Type   Label',35x,'Min.',11x,'Max.')
  150 format('  ',i5,6x,a1,4x,a30,2f15.3)
      end
!----------------------------------------------------------------------
      subroutine WriteMTZHeader(MTZOUT, filename, ProjectName, 
     *                    CrystalName, DataSetName, MaxLab, MaxSym,
     *                    SPGRNAM, NSPGR, PGNAM, NSYMTZ, RSYM, ier)
      USE General
      USE Symmetry
!
      integer DifferentSettings(23)
      character (len=16) :: SpaceSav
!
      integer MaxSym, MaxLab
      INTEGER i, MTZOUT, NSYMTZ, NSYMP, NSPGR, ISORT(5)
      REAL RSYM(4,4,MaxSym)
      real cell_mtz(6)
      character*(*) filename,  ProjectName, CrystalName, DataSetName
      character  xname(MaxLab)*20, dname(MaxLab)*20
      character  LTYPE*1, SPGRNAM*10, PGNAM*10
      CHARACTER Title*80

!
      nDifferentSettings = 23
      DifferentSettings( 1: 6)  = (/  48,  50,  59,  68,  70,  85 /)
      DifferentSettings( 7:12)  = (/  86,  88, 125, 126, 129, 130 /)
      DifferentSettings(13:18)  = (/ 133, 134, 137, 138, 141, 142 /)
      DifferentSettings(19:23)  = (/ 201, 203, 222, 224, 228      /)
      ISORT  = (/ 1, 2, 3, 0, 0 /)
      cell_mtz(1:6) = cell(1:6)
!
      if (ifMTZopened == 0) then
!-- CCP4 initialisations
!          call ccpfyp
!-- MTZ-specific initialisations
          call mtzini
          ifMTZopened = 1
      endif

      MTZOUT = 1

!-- Open mtz file
      call lwopen(MTZOUT, filename(1:Lung(filename)))

!--Write title to MTZ header
!gl!  write(Title,'(20a4)') itle
      Title = Titolo
      call lwtitl (MTZOUT, Title, 0)

!-- Write cell to MTZ header
      call lwcell(MTZOUT, cell(1:6))

!-- *** For CCP4 5.0 libraries
!-- Store the crystal, project and dataset names plus
!-- associated data in the mtz header:
      call lwidx(MTZOUT, ProjectName, CrystalName, DataSetName,
     +           cell_mtz, wavel)

!-- Store the column labels in the mtz header:
!-- Simpler alternative to LKYOUT/LWASSN
      call lwclab(MTZOUT, Lab_Col, nMTZlab, tipo_colonne, 0)

!-- *** For CCP4 5.0 libraries
!-- Associate columns with crystals and datasets
      do i=1,nMTZlab
        xname(i) = CrystalName
        dname(i) = DataSetName
      enddo
      call lwidasx(MTZOUT, nMTZlab, xname, dname, 0)

!-- Get symmetry info from file SYMOP (syminfo.lib - Env. Var. CLIBD) 
      call SpaceGroupCCP4(NSPGR,SPGRNAM,PGNAM,NSYMP,NSYMTZ,RSYM,LTYPE)
      if (NSYMTZ == 0) then
!
!-- bad space group symbol
!
!-- Close mtz file (print header informations if MTZprint > 0)
          MTZprint = 0
          call lwclos (MTZOUT, MTZprint)
!-- delete mtz file
          itype = 2
          iform = 1
          jop = jopen(kform,filename(1:Lung(filename)),itype,iform,ierr)
          close(kform, status='delete')
          write(SpaceSav,'(16a1)') nSpaceG
          call ucase(SpaceSav(1:1))
          L = Lung(SpaceSav)
          Msg = 'Error writing MTZ file - Bad Space Group Symbol: ' // 
     *          SpaceSav(1:L)
          call MsgWinErr('ERROR', Msg, ERR_WINDOW, nExitCode)
          write(lo,10) filename(1:Lung(filename))
          ier = -1
          return
      endif
      ifAdd = 0
      do i=1, nDifferentSettings
         if (NSPGR == DifferentSettings(i)) then
             ifAdd = 1
             exit
         endif
      enddo
      if (ifAdd == 1) then
          write(SpaceSav,'(16a1)') nSpaceG
          call ucase(SpaceSav(1:1))
          L = Lung(SpaceSav)
          SPGRNAM = ' '
          SPGRNAM(1:L+3) = SpaceSav(1:L) // ' :2'
      endif

!-- Store the symmetry in the mtz header:
      call lwsymm (MTZOUT,NSYMTZ,NSYMP,RSYM,LTYPE,NSPGR,SPGRNAM,PGNAM)

!-- Store program details in history header:
      call lwhstl (MTZOUT, ' ')

!-- Store sort order to be used
      CALL LWSORT(MTZOUT, ISORT)
!
      return
   10 format(/,' MTZ file ',a,' will be deleted !',/)
      end
!----------------------------------------------------------------------
      subroutine WriteMTZFooter(MTZOUT, hkl)
      USE General
!
      real              :: hkl(nMTZlab,nreflMTZ)
!
      do i = 1, nreflMTZ

!-- Initialise MTZdata with Missing Number Flags
         CALL EQUAL_MAGIC(MTZOUT, MTZdata, nMTZlab)

!-- Fill MTZ buffer
         k0 = 3 + knw
         k1 = k0 + 1
         k2 = k1 + 1
         k3 = k2 + 1
         MTZdata(1:k0) = hkl(1:k0,i)
         if (hkl(k1,i) > -998.0) then  ! per non sovrascrivere i valori NaN
             MTZdata(k1:k2) = hkl(k1:k2,i)
         endif
         MTZdata(k3:nMTZlab)=hkl(k3:nMTZlab,i)

!-- Write out reflection
         call lwrefl (MTZOUT, MTZdata)

      enddo

!-- Close mtz file (print header informations if MTZprint > 0)
      MTZprint = 0
      call lwclos (MTZOUT, MTZprint)
!
      return
      end
!-----------------------------------------------------------------------
      subroutine GetHklMTZ(jump, kh, kk, kl, ifEof)
      USE General
!
      logical ifEof, is_a_NaN, IsItaNaN
      integer kh, kk, kl, jj
!
      call lrrefl(MTZunit, resolution, MTZdata, ifEof)
      if (ifEof) then
          return
      endif
      jump = 0
      kh    = MTZdata(MTZlookup(1))
      kk    = MTZdata(MTZlookup(2))
      kl    = MTZdata(MTZlookup(3))
      jj = 4 + knw
      do j=jj,jj+1
         i = MTZlookup(j)
         is_a_NaN = IsItaNaN(MTZdata(i))
         if (is_a_NaN) then
            jump = 1
         end if
      enddo
      if (jump == 0) then
          do i=1, MTZ_Num_Col-3
             jj = MTZlookup(3+i)
             is_a_NaN = IsItaNaN(MTZdata(jj))
             if (is_a_NaN) then
                 valoriMTZ(i) = -999.99
             else
                 valoriMTZ(i) = MTZdata(jj)
             endif

          enddo
      else
          valoriMTZ(1:MTZ_Num_Col-3) = -999.99
      end if
!
      return
      end
!-----------------------------------------------------------------------
      subroutine SpaceGroupCCP4(NSPGR,SPGRNAM,PGNAM,NSYMP,NSYMTZ,RSYM,
     *                          LTYPE)
!      
      USE General
      USE Symmetry
!      
      INTEGER   NSYMP, NSYMTZ, NSPGR, MAXSYM
      parameter (MAXSYM = 192)
      REAL      RSYM(4,4,MAXSYM)
      CHARACTER SPGRNAM*10, SPGRNAM_temp*16, LTYPE*1, PGNAM*10
!
      write(SPGRNAM_temp,'(16a1)') nSpaceG
      call ucase(SPGRNAM_temp)
      SPGRNAM=SPGRNAM_temp(1:10)
      LTYPE=SPGRNAM_temp(1:1)

      NSPGR = nSpaceGroup
      if (LTYPE == 'R') then
          NSPGR = 0
      endif
      if ((jsys == 5 .or. jsys == 7) .and. ifRhombAx == 0) then
          if (LTYPE == 'R') then
              LTYPE = 'H'
              SPGRNAM(1:1) = 'H'
              NSPGR = 0
          endif
      endif
!-- Get symmetry info from file SYMOP (syminfo.lib - Env. Var. CLIBD)
      Buffer1 = 'SYMINFO=' // syminfo
      call SirSetEnv(Buffer1)
      NSYMTZ = 0
      CALL MSYMLB(124,NSPGR,SPGRNAM,PGNAM,NSYMP,NSYMTZ,RSYM)
!      
      return
      end
!-----------------------------------------------------------------------
      subroutine WriteMTZ_Express(stampaInfo, ifAsuCCP4, nomeMTZ, ier)
      USE General
      USE Symmetry
      USE Riflessi
      USE Molcom
      USE PhaseModule
      USE MtzMod
!
      implicit none
      integer :: ier, ix, iy, i, j, jj, k, kk, Lung, n1, n2, n3
      integer :: ierr, LabStep, j0, j00, j01, lsym, nSetsM, isegno
      integer :: isym, kfase, numbGenOld, ifAsuCCP4, hkli(3), hklo(3)
      real    :: yfl, flag, phase_in, phase_out
      integer :: ColonneStandard, LungMtzName, stampaInfo, ipher
      character (len=*)   :: nomeMTZ
      character (len=11) :: tirang
!
!
*     write(6,*) ' WriteMTZ_Express 1'
*     ifAsuCCP4 = 1
      ier = 0
      numbGenOld = numbGen
      numbGen = 0
!
!-- Calcolo Rfree
!
      iy = 67543
      ix = iy
      do i=1, nrefl
         call randu1(ix,iy,yfl)
         flag = real(int(yfl*20.0)+1)
         ix = iy
         rfree1(i) = nint(flag) - 1
      enddo
!
      kk = Lung(StructureName)
      CrystalName = StructureName(1:kk)
      call ucase(CrystalName)
!
      direttiva = nomeMTZ
      k = Lung(direttiva)
!
      LungMtzName = k
      MTZfile = direttiva
!
      HLA = 'HLA'
      HLB = 'HLB'
      HLC = 'HLC'
      HLD = 'HLD'
      Lab_Col(1) = 'H'
      Lab_Col(2) = 'K'
      Lab_Col(3) = 'L'
      j = 3
      Lab_Col(j+1) = 'F'
      Lab_Col(j+2) = 'SIGF'
      Lab_Col(j+3) = 'FreeR_flag'
      tipo_colonne(1:3) = 'H'
      tipo_colonne(j+1) = 'F'
      tipo_colonne(j+2) = 'Q'
      tipo_colonne(j+3) = 'I'
      j = j + 3
      ColonneStandard = j
      LabStep = 2
      LabCol1 = 'FC'
      LabCol2 = 'PHIC'
      LabCol3 = 'FOM'
      Col1 = 'F'
      Col2 = 'P'
      Col3 = 'W'
      n1 = 3
      n2 = 5
      n3 = 4
      LabStep = 3
      Lab_Col(j+1) = LabCol1(1:n1)
      Lab_Col(j+2) = LabCol2(1:n2)
      Lab_Col(j+3) = LabCol3(1:n3)
      jj = j + 3
      Lab_Col(jj+1) = HLA
      Lab_Col(jj+2) = HLB
      Lab_Col(jj+3) = HLC
      Lab_Col(jj+4) = HLD
      tipo_colonne(j+1) = Col1
      tipo_colonne(j+2) = Col2
      tipo_colonne(j+3) = Col3
      jj = j + 3
      tipo_colonne(jj+1) = 'A'
      tipo_colonne(jj+2) = 'A'
      tipo_colonne(jj+3) = 'A'
      tipo_colonne(jj+4) = 'A'
      nMTZlab = jj + 4
!
      ProjectName = ProgName(1:lProgName)
!
      DataSetName = ProgName(1:lProgName) // 
     *                  ': Ab Initio'
!
      call WriteMTZHeader(MTZOUT, direttiva, ProjectName, 
     *                    CrystalName, DataSetName, MaxLab, MaxSym,
     *                    SPGRNAM, NSPGR, PGNAM, NSYMTZ, RSYM, ier)
!
      if (ier < 0) then
          return
      endif

!-- Loop over reflections

      allocate (hkl(nMTZlab, nrefl), stat = ierr)
      hkl(:, :) = 0
      do i=1, nrefl
         hkl(1:3,i) = khl(1:3,i)
         j = 3
         hkl(j+1,i) = fovet(i)
         hkl(j+2,i) = sigmav(i)
         hkl(j+3,i) = rfree1(i)
         j = j + 3
         j = j + 1
         hkl(j,i) = fcvet(i)
         j = j + 1
         hkl(j,i) = real(iphaz(i))
         j = j + 1
         hkl(j,i) = wt(i)
      enddo
!

!
!-- Put reflections (h,k,l and phases) into asymmetric unit defined in ASUSET
!-- ASU
!
      isym = 0
      NAMSPG_MASTER = SPGRNAM
      NSPGRP_MASTER = NSPGR
      PGNAME_MASTER = PGNAM
      NSYM_MASTER   = NSYMTZ
      RSYM_MASTER   = RSYM
      CALL ASUSET(NAMSPG_MASTER,   NSPGRP_MASTER,
     +            PGNAME_MASTER,   NSYM_MASTER,
     +            RSYM_MASTER,     NSYMP_MASTER,
     +            NLAUE_MASTER,    .FALSE.)
      kfase = 1
      nSetsM = 1
      isegno = 1
      do i = 1, nrefl
         hkli(1:3) = nint(hkl(1:3,i))

         hklo(1:3) = hkli(1:3)

         lsym = (isym-1)/2 + 1
         hkl(1:3,i) = hklo(1:3)
         j00 = 6 + 2
         kfase = 0
         j01 = 7
         do k=1, nSetsM
            j0 = j00 + kfase
            phase_in = hkl(j0,i)
            phase_out = phase_in
            hkl(j0,i) = phase_out
            peso = hkl(j0+1,i)
            call hlconv1(mkang(i), peso, phase_out, ha,hb,hc,hd)
*           hkl(j0+1,i) = peso
            hkl(j0+2,i) = ha
            hkl(j0+3,i) = hb
            hkl(j0+4,i) = hc
            hkl(j0+5,i) = hd
            j00 = j0 + j01
         enddo
      enddo
!
!-- ASU
!
      nreflMTZ = nrefl
      call WriteMTZFooter(MTZOUT, hkl)
!
      deallocate (hkl, stat=ierr)
!
      numbGen = numbGenOld

      if (ifAsuCCP4 == 1) then
          call SortMTZ()
          direttiva = MTZfile
      endif

      if (stampaInfo > 0) then
          write(lo,100) direttiva(1:LungMtzName)
          if (knw == 1) 
     *    call examin(8888, mkang, khl, ipub, iphaz, nrefl, 0, wt, 1)
!
      endif
!
      return
  100 format(/,78('+'),/,' Fcalc, phases and and weights stored',
     *       ' (MTZ format) in file:',/,3x,a,/)
      end
!----------------------------------------------------------------------
      subroutine SortMTZ()
      USE General
!
      implicit none
      integer :: Lung, lcard, L, itype, iform, jop, jopen, ier
!
      cardDiret = nome(1:lnome) // '_cad.inp'
      lcard = Lung(cardDiret)
      itype = 0
      iform = 1
      jop = jopen(jhost,cardDiret(1:lcard),itype,iform,ier)
!H K L F SIGF FreeR_flag FC PHIC FOM HLA HLB HLC HLD
!         Lab_Ch(1) = 'H'
!         Lab_Ch(2) = 'K'
!         Lab_Ch(3) = 'L'
!         Lab_Ch(4) = 'F'
!         Lab_Ch(5) = 'SIGF'
!         Lab_Ch(7) = 'FOM'  ! peso
!         Lab_Ch(6) = 'PHIC' ! fase
!         Lab_Ch(8) = 'FC'   ! fc

      Msg = 'FILE 1 E1=' // Lab_Col(4) // ' E2=' // Lab_Col(5)
*    *   // ' E3=FreeR_flag E4=' // Lab_Ch(8) // ' E5=' // Lab_Ch(6)
*    *   // ' E6=' // Lab_Ch(7) 
     *   // ' E3=' // Lab_Col(6) // ' E4=' // Lab_Col(8) // ' E5=' 
     *   // Lab_Col(9) // ' E6=' // Lab_Col(7) 
     *   // ' E7=HLA E8=HLB E9=HLC E10=HLD'
      call OneBlankOnly(Msg)
      write(jhost,'(a)') 'LABIN  ' // Msg(1:Lung(Msg))
      write(jhost,'(a)') 'LABOUT ' // Msg(1:Lung(Msg))
      close (jhost)

      L = Lung(MTZfile)
      cardImage = nome(1:lnome) // '_CadOut.mtz'
      Msg = nome(1:lnome) // '_cad.sh'
      itype = 0
      iform = 1
      jop = jopen(jhost,Msg(1:Lung(Msg)),itype,iform,ier)
      Msg = 'cad  HKLIN1 ' // MTZfile(1:L) // ' HKLOUT ' 
     *    // cardImage(1:Lung(cardImage)) 
     *    // ' < ' // cardDiret(1:Lung(cardDiret))
     *    // ' > cad.log '
      write(jhost,'(a)') '#!/bin/sh'
      write(jhost,'(a)') Msg(1:Lung(Msg))
      close (jhost)
      Buffer1 = nome(1:lnome) // '_cad.sh'
      L = Lung(Buffer1)
      Buffer1(L+1:L+1) = char(0)
      call MakeExec(Buffer1)
      call Lanciash(Buffer1)
!     call cCopyFile(cardImage, MTZfile)
      MTZfile = cardImage
!
      return
      end
!----------------------------------------------------------------------
