 MODULE progtype

  integer, parameter  :: dblep = kind(1.0d0)
  integer, parameter  :: ALLOCA   = 1
  integer, parameter  :: DEALLOCA = 0

  type reflection_type
     integer, dimension(3) :: hkl    ! indici hkl
     integer               :: m      ! molteplicit� del riflesso
     real                  :: tthd   ! 2-theta del riflesso in gradi 
     real                  :: slaq   ! ((sen(theta) / lambda))**2 = rho**2
     real                  :: fo     ! fattori di struttura osservati 
     real                  :: fc     ! fattori di struttura calcolati dal modello 
     integer               :: ph     ! fasi calcolate dal modello
     real                  :: fv     ! fattori di struttura veri
     integer               :: phv    ! fase vera
     real                  :: lp     ! correzione di Lorentz-polarizzazione 
     real                  :: as     ! correzione di assorbimento 
     real                  :: fwhm   ! fwhm 
     real                  :: po     ! P.O. correction
     real                  :: pk     ! peak range
  end type reflection_type

  type bond_type
     integer :: n1,n2       ! atomi legati
     real    :: dist        ! distanza corrente
     real    :: sigma = 0.3 ! sigma
     integer :: ord = 0     ! bond order 0=undefined, 1=single, 2=double or aromatic, 3=triple
  end type

  type ref_limit_type
     real    :: ttmin     ! 2theta minimo usato
     real    :: ttmax     ! 2theta massimo usato
     integer :: nreftot   ! num. totale di riflessi
     integer :: nrefu     ! num. di riflessi usati
     integer :: nri       ! primo riflesso usato
     integer :: nrf       ! ultimo riflesso usato
     integer :: nctot     ! num. totale di conteggi
     integer :: ncu       ! num. di conteggi usati
     integer :: nci       ! primo conteggio usato
     integer :: ncf       ! ultimo conteggio usato
  end type ref_limit_type
  
  type refine_condition_type
     type(ref_limit_type) :: lim            ! info sui limiti di affinamento 
     integer              :: algo           ! tipo di algoritmo di l.sq.
     integer              :: typeback       ! tipo di background
     integer              :: typefun        ! tipo di funzione di profilo
     integer              :: nback          ! num. di coefficienti di background
     logical              :: auto_coefb     ! selezione automatica del numero di coeff. di backgr.?
     logical              :: auto_prof      ! affinamento automatico profilo
     logical              :: auto_stru      ! affinamento automatico struttura
     integer              :: ncauto         ! num. di cicli di affinamento automatico
     logical              :: winmess        ! messagistica attiva?
     integer              :: typewei        ! schema di pesaggio
     real                 :: eps            ! criterio di convergenza
     integer              :: maxcy          ! numero massimo di cicli di affinamento
     integer              :: decomp         ! 0=Rietveld 1=Le Bail 2=Pawley
     character(len=6)     :: sprint         ! V=verbose,S=small,C=correlation,E=st.dev.,M=lsq matrices,W=window per chi,B=statusbar
  end type refine_condition_type
  
  type refine_info_type
     logical :: refine_cell
     logical :: refine_structure
     logical :: refine_profile
     real    :: dw          ! Durbin Watson
     real    :: rp,rwp      ! Rp,Rwp
     integer :: npar
  end type refine_info_type

  type rparam_type
     real                  :: val  = 0  ! valore del parametro
     real                  :: sd = 0    ! sd
     integer               :: rcod = 0  ! posizione nella matrice jac. del parametro se affinato; 0 se non affinato
     character(len=10)     :: str = ' ' ! stringa associata al parametro
  end type rparam_type

!corr  type pearson_function_type
!corr     type(rparam_type) :: u,v,w
!corr     type(rparam_type) :: b0,b1,b2
!corr     type(rparam_type) :: asym
!corr  end type pearson_function_type

  type pvoigt_function_type
     type(rparam_type) :: u,v,w
     type(rparam_type) :: eta0,eta1,eta2
     type(rparam_type) :: asym
  end type pvoigt_function_type

  type tch_function_type
     type(rparam_type) :: u,v,w,z
     type(rparam_type) :: x,y
     type(rparam_type) :: asym
  end type tch_function_type

  type deratom
     real, dimension(3)    :: co
     real                  :: b
     real                  :: occ
  end type deratom

  type ipoint_type
     integer               :: x,y
  end type

  type point_type
     real               :: x,y
  end type

  type, public                         :: param_type
    real                               :: val        !Value of parameter
    integer, dimension(:), allocatable :: pat        !Atom(s) associated to parameter
    integer                            :: typepar    !Type of parameter
    real                               :: low        !Low-limit value of parameter
    real                               :: high       !High-limit value of parameter
    integer                            :: bound      !=0 fixed boundaries  1= periodic boundaries
    real                               :: stp        !The step length
    character(len=20)                  :: str        !Stringa associata al parametro 
  end type

  type opti_info_type
     integer           :: npar      ! num. totale di parametri
     integer           :: dofi      ! num. di DOF interni
     integer           :: dofe      ! num. di DOF esterni
     integer           :: nprot     ! num. di rotazioni affinate
     integer           :: npb       ! numeri di parametri termici affinati
     integer           :: npshift   ! numero di paramtri di shift affinati  
     integer           :: nreftot   ! num. totale riflessi 
     integer           :: nref      ! num. di riflessi usati
     integer           :: nco       ! numero di conteggi usati
     real              :: ttmin     ! 2theta min
     real              :: ttmax     ! 2theta max
     real              :: res       ! risoluzione
     integer           :: cftype    ! tipo di funzione costo
     logical           :: doc       ! Dynamical occupancy correction
     real              :: rwp_leb   ! Rwp(Le Bail) 
     integer           :: nrun      ! Number of run
     integer           :: nrunc     ! A quale run sono
     character(len=60) :: cost_function_name
     character(len=20) :: cost_function_name_short
     integer           :: seed      ! seme generatore random
     logical           :: autoalgo  ! selezione automatica del tipo di algo
     integer           :: algo      ! tipo di algoritmo di ottimizzazione
     real              :: sogcf     ! valore soglia per la funzione costo
     real              :: fmin      ! valore minimo della funzione costo
     integer           :: rmin      ! run corrispondente al valore minimo
     character(len=10) :: popt      ! Opzioni di output: !P = draw pattern su viewer principale
                                                         !M = draw molecule su viewer principale
                                                         !V = draw on secondary viewer
                                                         !E = export models on cif files
                                                         !S = stampa parametri
                                                         !B = manage status and progress bar
  end type

  integer, parameter :: NMAXSPECIE = 18  ! numero massimo di specie gestibili

  real, parameter :: CU_WAVE = 1.540560
  real, parameter :: DEF_WAVE = CU_WAVE

  type structure_info_type
    integer                        :: nspec
    integer, dimension(NMAXSPECIE) :: zspec,vspec
    real, dimension(6)             :: cell
    character(len=16)              :: spaceg
    real                           :: cell_volume
    real                           :: volume_per_atom
    real                           :: r_factor
    integer                        :: icent
    integer                        :: nsymmstr
  end type structure_info_type

!!corr  integer, parameter :: NLEN_LAB = 8

  type symminfo_type
     integer :: asym    ! corrispondente atomo nella unita' asimmetrica
     integer :: oper    ! operatore di simmetria
     integer :: opcode  ! codice operatore
  end type symminfo_type

!corr  integer, parameter :: NMAXINFO = 10

  type bond_setting_type
       real :: tolmin, tolmax ! legame se dmin+tolmin < d < dmax+tolmax
       real :: angle          ! legame se l'angolo < angle
  end type bond_setting_type
  type(bond_setting_type), parameter :: DEF_BONDSET = bond_setting_type(0.0,0.0,38)

!#define DISTRIB
#if defined(DISTRIB)
 logical, parameter :: DISTRIBUTED_VER = .true.
#else
 logical, parameter :: DISTRIBUTED_VER = .false.
#endif
 END MODULE progtype
