#include <cstdio>
#include <cstring>
#include <utils.h>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <sir_application.h>
//#include <gtkdefs.h>
#define GL_GLEXT_PROTOTYPES 1
#include "glext.h"
#include "models.h"
#include "jav_cb.h"


typedef struct {
                double v1[3], v2[3], v3[3];
                double vn[3];
                int nat;
               } Triangolo;

namespace STL {
std::vector<Triangolo> trigs;
int nTrigs = 0;

#define XC .525731112119133606 
#define ZC .850650808352039932
static GLdouble vdata[12][3] = {    
    {-XC, 0.0, ZC}, {XC, 0.0, ZC}, {-XC, 0.0, -ZC}, {XC, 0.0, -ZC},    
    {0.0, ZC, XC}, {0.0, ZC, -XC}, {0.0, -ZC, XC}, {0.0, -ZC, -XC},    
    {ZC, XC, 0.0}, {-ZC, XC, 0.0}, {ZC, -XC, 0.0}, {-ZC, -XC, 0.0} 
};
static GLuint tindices[20][3] = { 
    {0,4,1}, {0,9,4}, {9,5,4}, {4,5,8}, {4,8,1},    
    {8,10,1}, {8,3,10}, {5,3,8}, {5,2,3}, {2,7,3},    
    {7,10,3}, {7,6,10}, {7,11,6}, {11,0,6}, {0,1,6}, 
    {6,1,10}, {9,0,11}, {9,11,2}, {9,2,5}, {7,2,11} };
void normalize(GLdouble *a) {
    GLdouble d=sqrt(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]);
    a[0]/=d; a[1]/=d; a[2]/=d;
}

#define HEADER_SIZE 80
bool IsSTLBinary(std::string &filename)
{
    bool binary=false;
    std::ifstream is;
    int facenum;
    is.open(filename.c_str(), std::ios::in);
    is.seekg (0, is.end);
    int file_size = (int) is.tellg();
    is.seekg (0, is.beg);
    is.seekg(HEADER_SIZE);
    size_t sz = read_project(is, static_cast<char*>(static_cast<void*>(&facenum)), sizeof(int));
    if(sz != sizeof(int))
        binary = false;
    else
    {
        //int facet_size = sizeof(Triangolo) - sizeof(int) + sizeof(short);
        int facet_size = 12*sizeof(float) + sizeof(short);
        int expected_file_size = HEADER_SIZE + 4 + facet_size * facenum;
        if(file_size ==  expected_file_size)
           binary = true;
    }
    is.close();
    return binary;
}

void drawtri(GLdouble *a, GLdouble *b, GLdouble *c, int div, double r, int nat)
{
    if (div<=0) {
        
        Triangolo t;
        for(int j=0; j<3; j++)
        {
           t.v1[j] = a[j]*r;
           t.v2[j] = b[j]*r;
           t.v3[j] = c[j]*r;
        }
        t.nat = nat;
        trigs.push_back(t);
    } else {
        GLdouble ab[3], ac[3], bc[3];
        for (int i=0;i<3;i++) {
            ab[i]=(a[i]+b[i])/2;
            ac[i]=(a[i]+c[i])/2;
            bc[i]=(b[i]+c[i])/2;
        }
        normalize(ab); normalize(ac); normalize(bc);
        drawtri(a, ab, ac, div-1, r, nat);
        drawtri(b, bc, ab, div-1, r, nat);
        drawtri(c, ac, bc, div-1, r, nat);
        drawtri(ab, bc, ac, div-1, r, nat);
    }  
}

void PrintSferaSTL(int nat, double radius, int ndiv)
{
   ndiv = ndiv/50 + 2;
   if(ndiv > 4)
      ndiv = 4;
    for (int i=0;i<20;i++)
        drawtri(vdata[tindices[i][0]], vdata[tindices[i][1]], vdata[tindices[i][2]], ndiv, radius, nat);
}

void TFVert2Tri(std::vector<vet3> Vertici, std::vector<Triangolo>&trig)
{
  for(int i=0; i< (int) Vertici.size()-2; i++)
  {
       Triangolo trig;
       for(int j=0; j<3; j++)
       {
           trig.v1[j] = Vertici.at(0).v[j];
           trig.v2[j] = Vertici.at(i+1).v[j];
           trig.v3[j] = Vertici.at(i+2).v[j];
       }
       trig.nat = -1;
       trigs.push_back(trig);
  }
}

void QSVert2Tri(std::vector<vet3> Vertici, std::vector<Triangolo>&trig)
{
#if 0
  int nq = (int) Vertici.size()-1;
  for(int i=0; i< nq/2; i++)
  {
       int n1 = 2*i;
       int n2 = 2*i + 1;
       int n3 = 2*i + 3;
       int n4 = 2*i + 4;
       Triangolo trig;
       for(int j=0; j<3; j++)
       {
           trig.v1[j] = Vertici.at(n1).v[j];
           trig.v2[j] = Vertici.at(n2).v[j];
           trig.v3[j] = Vertici.at(n4).v[j];
       }
       trig.nat = -1;
       trigs.push_back(trig);
       for(int j=0; j<3; j++)
       {
           trig.v1[j] = Vertici.at(n4).v[j];
           trig.v2[j] = Vertici.at(n3).v[j];
           trig.v3[j] = Vertici.at(n1).v[j];
       }
       trig.nat = -1;
       trigs.push_back(trig);
  }

#else
  int nq = (int) Vertici.size();
  for(int i=0; i< nq/2-1; i++)
  {
       int n = 2*i;
       int n1 = (n+1) % nq;
       int n2 = (n+2) % nq;
       int n3 = (n+3) % nq;
       Triangolo trig;
       for(int j=0; j<3; j++)
       {
           trig.v1[j] = Vertici.at(n).v[j];
           trig.v2[j] = Vertici.at(n1).v[j];
           trig.v3[j] = Vertici.at(n3).v[j];
       }
       trig.nat = -1;
       trigs.push_back(trig);
       for(int j=0; j<3; j++)
       {
           trig.v1[j] = Vertici.at(n3).v[j];
           trig.v2[j] = Vertici.at(n2).v[j];
           trig.v3[j] = Vertici.at(n).v[j];
       }
       trig.nat = -1;
       trigs.push_back(trig);
  }
#endif
}

void PrintCilindroSTL(vet3 p1, vet3 p2, double raggio)
{
  vet3 p1p2N = MRVectorSub(p1, p2);
  vet3 firstPerpV = MRVectorFirstPerp(p1p2N);
  vet3 secondPerpV = MRVectorCrossProduct(p1p2N, firstPerpV);
  secondPerpV = MRVectorNormalize(secondPerpV);
  const int ANZ = 32;
  const float FULL = (float) (2.0f*M_PI);
  vet3 points[ANZ+1];

  for (int i=0; i<ANZ; i++)
  {
    float angle = FULL*((float) i/(float) ANZ);

    points[i].x = (float) (raggio*(cos(angle)*firstPerpV.x+sin(angle)*secondPerpV.x));
    points[i].y = (float) (raggio*(cos(angle)*firstPerpV.y+sin(angle)*secondPerpV.y));
    points[i].z = (float) (raggio*(cos(angle)*firstPerpV.z+sin(angle)*secondPerpV.z));
  }
  points[ANZ].x = points[0].x;
  points[ANZ].y = points[0].y;
  points[ANZ].z = points[0].z;
  std::vector<vet3> Vertici;
  Vertici.push_back(p1);
  for (int i=0; i<=ANZ; i++)
  {
       vet3  p = { p1.x+points[i].x, p1.y+points[i].y, p1.z+points[i].z };
       Vertici.push_back(p);
  }
  TFVert2Tri(Vertici, trigs);
  Vertici.clear();
  Vertici.push_back(p2);
  for (int i=0; i<=ANZ; i++)
  {
       vet3  p = { p2.x+points[i].x, p2.y+points[i].y, p2.z+points[i].z };
       Vertici.push_back(p);
  }
  TFVert2Tri(Vertici, trigs);
  Vertici.clear();
  for (int i=0; i<=ANZ; i++)
  {
       vet3  p = { p1.x+points[i].x, p1.y+points[i].y, p1.z+points[i].z };
       Vertici.push_back(p);
       vet3  pp = { p2.x+points[i].x, p2.y+points[i].y, p2.z+points[i].z };
       Vertici.push_back(pp);
  }
  QSVert2Tri(Vertici, trigs);
}

void writeSTL(std::string &fn, std::vector<vet3> &xyz)
{
  unsigned int ntrigs = (unsigned int) trigs.size();
  std::string f_ascii = fn;
  size_t n = f_ascii.find_last_of('.');
  if(n != std::string::npos)
     f_ascii = f_ascii.erase(n, std::string::npos);
  f_ascii.append("_ascii.stl");
  FILE *fp = fOpen(f_ascii.c_str(), "w");
  std::ofstream f;
  f.open(fn.c_str(), std::ios::binary | std::ios::out);
  std::string title = g_path_get_basename(fn.c_str());
  n = title.find_last_of('.');
  if(n != std::string::npos)
     title = title.erase(n, std::string::npos);

  fprintf(fp, "solid %s\n", title.c_str());
  const char header[81] =
"made by Jav                                                                     ";

  unsigned short int at =  0;
  f.write(header, HEADER_SIZE);
  f.write(static_cast<const char*>(static_cast<const void*>(&ntrigs)), 4);
  for (int i=0; i<(int)ntrigs; i++)
  {
       int nat = trigs.at(i).nat;
       float vc[3] = {2, 2, 2};
       if(nat > -1)
       {
          for(int j=0; j<3; j++)
              vc[j] += (float) xyz[nat].v[j];
       }
       float v1[3] = {(float)trigs.at(i).v1[0]+vc[0], (float)trigs.at(i).v1[1]+vc[1], (float)trigs.at(i).v1[2]+vc[2]};
       float v2[3] = {(float)trigs.at(i).v2[0]+vc[0], (float)trigs.at(i).v2[1]+vc[1], (float)trigs.at(i).v2[2]+vc[2]};
       float v3[3] = {(float)trigs.at(i).v3[0]+vc[0], (float)trigs.at(i).v3[1]+vc[1], (float)trigs.at(i).v3[2]+vc[2]};
       Vert p1 = { v1[0], v1[1], v1[2] };
       Vert p2 = { v2[0], v2[1], v2[2] };
       Vert p3 = { v3[0], v3[1], v3[2] };
       Vert n = norm(p1, p2, p3);
       float vet[3] =  {n.x, n.y, n.z};
       fprintf(fp, "facet normal %f %f %f\n", n.x, n.y, n.z);
       fprintf(fp, "    outer loop\n");
       fprintf(fp, "       vertex %f %f %f\n", v1[0], v1[1], v1[2]);
       fprintf(fp, "       vertex %f %f %f\n", v2[0], v2[1], v2[2]);
       fprintf(fp, "       vertex %f %f %f\n", v3[0], v3[1], v3[2]);
       fprintf(fp, "    endloop\n");
       fprintf(fp, "endfacet\n");
       f.write(static_cast<const char*>(static_cast<const void*>(vet)), 3*sizeof(float));
       f.write(static_cast<const char*>(static_cast<const void*>(v1)), 3*sizeof(float));
       f.write(static_cast<const char*>(static_cast<const void*>(v2)), 3*sizeof(float));
       f.write(static_cast<const char*>(static_cast<const void*>(v3)), 3*sizeof(float));
       f.write(static_cast<const char*>(static_cast<const void*>(&at)), sizeof(at));
  }
  f.close();
  fprintf(fp, "endsolid sfera\n");
  fclose(fp);
}

void readSTL_ASCII(std::string &fn, std::vector<float> &tvet, GtkWidget *win)
{
  using namespace gtkext;
  std::ifstream f;
  bool cicla = true;
  char line[256];
  Triangolo *trig = g_new0(Triangolo, 1);
  int nv = 0;

  f.open(fn.c_str(), std::ios::in);
  if(!f.is_open() || f.fail())
     goto call_error;
  while(cicla)
  {
     float v1[3];
     f.getline(line, 256);
     if ( (f.rdstate() & std::ifstream::eofbit ) != 0 )
         cicla = false;
     if(strstr(line, "solid ") || strstr(line, "endsolid"))
        continue;
     if(strstr(line, "facet normal"))
     {
        std::string linestr = line;
        size_t p = linestr.find("facet normal");
        linestr.erase(0, p+12);
        get_float_tokens(linestr.c_str(),  " ", 3,  v1);
        for (int j=0; j<3; j++)
             trig->vn[j] = v1[j];
        trig->nat = -1;
     }
     if(strstr(line, "vertex"))
     {
        std::string linestr = line;
        size_t p = linestr.find("vertex");
        linestr.erase(0, p+6);
        get_float_tokens(linestr.c_str(),  " ", 3,  v1);
        for (int j=0; j<3; j++)
        {
             if(nv == 1)
                trig->v1[j] = v1[j];
             else if(nv == 2)
                trig->v2[j] = v1[j];
             else if(nv == 3)
                trig->v3[j] = v1[j];
             tvet.push_back(v1[j]);
        }
        nv++;
        trig->nat = -1;
     }
     if(strstr(line, "outer loop"))
        nv = 1;
     if(strstr(line, "endloop"))
        nv = 0;
     if(strstr(line, "endfacet"))
        trigs.push_back(*trig);
  }
   
  f.close();
  return;
call_error:
  dlg_errmsg(GTK_WINDOW(win), g_strdup_printf("Error reading %s", fn.c_str()));
  if(f.is_open())
     f.close();
}

void readSTL_BIN(std::string &fn, std::vector<float> &tvet, GtkWidget *win)
{
  using namespace gtkext;
  char header[81];
  unsigned short int at =  0;
  unsigned int ntrigs;
  std::ifstream f;
  size_t nsz;

  f.open(fn.c_str(), std::ios::binary | std::ios::in);
  if(!f.is_open() || f.fail())
     goto call_error;
  nsz = read_project(f, header, HEADER_SIZE);
  if((nsz != HEADER_SIZE) || f.fail())
     goto call_error;
  header[HEADER_SIZE] = 0;
  nsz = read_project(f, static_cast<char*>(static_cast<void*>(&ntrigs)), 4);
  if((nsz != 4) || f.fail())
     goto call_error;
  for (int i=0; i<(int)ntrigs; i++)
  {
       float v1[3], v2[3], v3[3], vet[3];
       nsz = read_project(f, static_cast<char*>(static_cast<void*>(vet)), 3*sizeof(float));
       if((nsz != (3*sizeof(float)) || f.fail()))
          goto call_error;
       nsz = read_project(f, static_cast<char*>(static_cast<void*>(v1)), 3*sizeof(float));
       if((nsz != (3*sizeof(float)) || f.fail()))
          goto call_error;
       nsz = read_project(f, static_cast<char*>(static_cast<void*>(v2)), 3*sizeof(float));
       if((nsz != (3*sizeof(float)) || f.fail()))
          goto call_error;
       nsz = read_project(f, static_cast<char*>(static_cast<void*>(v3)), 3*sizeof(float));
       if((nsz != (3*sizeof(float)) || f.fail()))
          goto call_error;
       nsz = read_project(f, static_cast<char*>(static_cast<void*>(&at)), sizeof(at));
       if((nsz != (sizeof(short)) || f.fail()))
          goto call_error;
       Triangolo tri;
       for(int k=0; k<3; k++)
       {
           tri.v1[k] = v1[k];
           tri.v2[k] = v2[k];
           tri.v3[k] = v3[k];
           tri.vn[k] = vet[k];
           tvet.push_back(v1[k]);
           tvet.push_back(v2[k]);
           tvet.push_back(v3[k]);
       }
       tri.nat = -1;
       trigs.push_back(tri);
  }
  f.close();
  return;
call_error:
  dlg_errmsg(GTK_WINDOW(win), g_strdup_printf("Error reading %s", fn.c_str()));
  if(f.is_open())
     f.close();
}

void readSTL(std::string &fn, GtkWidget *win)
{
  std::vector<float> tvet;
  float mini, maxi, delta;

  trigs.clear();
  tvet.clear();
  if(!IsSTLBinary(fn))
     readSTL_ASCII(fn, tvet, win);
  else
     readSTL_BIN(fn, tvet, win);

  if(!tvet.empty())
  {
      mini = (*min_element(tvet.begin(), tvet.end()));
      maxi = (*max_element(tvet.begin(), tvet.end()));
      delta = maxi-mini;
      tvet.clear();
  }

  if(!trigs.empty())
     nTrigs = (int)trigs.size();
  else
     nTrigs = 0;

  if(delta)
  {
     float fact = 2 / delta;
     for (int i=0; i<(int)nTrigs; i++)
     {
          Triangolo *tri = &(trigs.at(i));
          for(int k=0; k<3; k++)
          {
              tri->v1[k] = -1 + (tri->v1[k] - mini)*fact;
              tri->v2[k] = -1 + (tri->v2[k] - mini)*fact;
              tri->v3[k] = -1 + (tri->v3[k] - mini)*fact;
          }
     }
  }
  
}

void DrawTriangles()
{
    float color1[4] = {1, 0, 0, 1};
    glEnable (GL_LIGHTING);
    glEnable (GL_NORMALIZE);
    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color1);
    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL ) ;

#if 0
    glBegin(GL_TRIANGLES);

    for(int i=0; i<nTrigs; i++)
    {
         Triangolo t = trigs.at(i);
         vet3 v1 = {t.v1[0], t.v1[1], t.v1[2]};
         vet3 v2 = {t.v2[0], t.v2[1], t.v2[2]};
         vet3 v3 = {t.v3[0], t.v3[1], t.v3[2]};
         vet3 vn = {t.vn[0], t.vn[1], t.vn[2]};
         glNormal3dv(vn.v);
         glVertex3dv(v1.v);
         glVertex3dv(v2.v);
         glVertex3dv(v3.v);
    }

    glEnd();

#endif
    GLenum mode = GL_TRIANGLES;
    GLdouble *vertices = g_new0(GLdouble, 9*nTrigs);
    GLdouble *normals  = g_new0(GLdouble, 9*nTrigs);

    int k = 0;
    for( int i = 0 ; i < nTrigs ; ++i )
    {
      Triangolo t = trigs.at(i);
      vet3 v1 = {t.v1[0], t.v1[1], t.v1[2]};
      vet3 v2 = {t.v2[0], t.v2[1], t.v2[2]};
      vet3 v3 = {t.v3[0], t.v3[1], t.v3[2]};
      vet3 vn = {t.vn[0], t.vn[1], t.vn[2]};

      vertices[k]   = v1.x;
      vertices[k+1] = v1.y;
      vertices[k+2] = v1.z;
      normals[k]    = vn.x;
      normals[k+1]  = vn.y;
      normals[k+2]  = vn.z;
      k += 3;
      vertices[k]   = v2.x;
      vertices[k+1] = v2.y;
      vertices[k+2] = v2.z;
      normals[k]    = vn.x;
      normals[k+1]  = vn.y;
      normals[k+2]  = vn.z;
      k += 3;
      vertices[k]   = v3.x;
      vertices[k+1] = v3.y;
      vertices[k+2] = v3.z;
      normals[k]    = vn.x;
      normals[k+1]  = vn.y;
      normals[k+2]  = vn.z;
      k += 3;
    }

    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
    glVertexPointer(3, GL_DOUBLE, 0, vertices);
    glNormalPointer(GL_DOUBLE, 0, normals);
    glDrawArrays(mode, 0, 3*nTrigs);
    glDisableClientState(GL_VERTEX_ARRAY);
    glDisableClientState(GL_NORMAL_ARRAY);
    g_free(vertices);
    g_free(normals);

}

}   // end namespace STL

namespace GLextmr
{
    extern Model *ModelInfo;
    extern GtkWidget *GLWINDOW;
    extern GLfloat sc_factor;
    extern float H[4][NKNOTS-2];
    extern int   nknots;

bool DrawTrigs()
{
using namespace STL;
    DrawTriangles();
    return(nTrigs > 0);
}

void on_importSTL(gpointer, gpointer data)
{
using namespace STL;
   GtkWidget *window = (GtkWidget *)data;
   std::string fn = "";
   fn =  gtkext::dlg_select_file(window, fn, gtkext::eSELECT_TO_READ,
           "STereoLithography file format (*.stl)", false, NULL, false);
   if(str_white(fn.c_str()))
       return;
   readSTL(fn, window);
}

void on_exportScene(gpointer, gpointer data)
{
using namespace STL;
   GtkWidget *window = (GtkWidget *)data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   if(gl->nModels <= 0)
      return;
   int whichModel = GetActiveModel() - 1;
   std::string fn;
   if(ModelInfo[whichModel].created)
   {
      fn = g_path_get_basename(ModelInfo[whichModel].modname);
      size_t n = fn.find_last_of('.');
      if(n != std::string::npos)
         fn = fn.erase(n, std::string::npos);
      fn.append(".stl");
   }
   else
      fn = "";
   fn =  gtkext::dlg_select_file(window, fn, gtkext::eSELECT_TO_WRITE,
           "STereoLithography file format (*.stl)", false, NULL, false);
   if(str_white(fn.c_str()))
       return;

   std::vector<vet3> xyzat;
   trigs.clear();
   xyzat.clear();
   for (int i=0; i<gl->nModels; i++)
        ModelInfo[i].exportScene(gl, xyzat);
   writeSTL(fn, xyzat);
   trigs.clear();
   xyzat.clear();
}

void PrintPoliedriSTL(Model *m)
{ 
using namespace STL;
  for (int i=0; i<m->nPoly; i++)
  {
       int nf = (m->Polyhedra[i].nvert == 4)? 4:8;
       for(int j=0; j<nf; j++)
       {
           Triangolo trig;
           for(int k=0; k<3; k++)
           {
               trig.v1[k] = m->atm[m->Polyhedra[i].face[j].n1].xyzN[k];
               trig.v2[k] = m->atm[m->Polyhedra[i].face[j].n2].xyzN[k];
               trig.v3[k] = m->atm[m->Polyhedra[i].face[j].n3].xyzN[k];
           }
           trig.nat = -1;
           trigs.push_back(trig);
       }
  }
}
void DrawTubesSTL(GLWindow *gl, int npoints, Knot knot[], double factor=1)
{
using namespace STL;
     for(int i=0; i<npoints-1; i++)
     {
          vet3 x1 = {{ knot[i].px, knot[i].py, knot[i].pz}};
          vet3 x2 = {{ knot[i+1].px, knot[i+1].py, knot[i+1].pz}};
          PrintCilindroSTL(x1, x2, factor*1.5*gl->cylinder);
     }
}

void Pol2Trigs(vet3 v[4])
{
using namespace STL;
    Triangolo trig;
    for(int j=0; j<3; j++)
    {
           trig.v1[j] = (float) v[0].v[j];
           trig.v2[j] = (float) v[1].v[j];
           trig.v3[j] = (float) v[2].v[j];
    }
    trig.nat = -1;
    trigs.push_back(trig);
    for(int j=0; j<3; j++)
    {
           trig.v1[j] = (float) v[2].v[j];
           trig.v2[j] = (float) v[3].v[j];
           trig.v3[j] = (float) v[0].v[j];
    }
    trig.nat = -1;
    trigs.push_back(trig);
}

void DrawSolidRibbonSTL(Knot src, Knot dst)
{
    vet3 v[4];
    
// Top Surface 
    v[0].x = src.px+src.wx+src.dx;
    v[0].y = src.py+src.wy+src.dy;
    v[0].z = src.pz+src.wz+src.dz;

    v[1].x = dst.px+dst.wx+dst.dx;
    v[1].y = dst.py+dst.wy+dst.dy;
    v[1].z = dst.pz+dst.wz+dst.dz;

    v[2].x = dst.px-dst.wx+dst.dx;
    v[2].y = dst.py-dst.wy+dst.dy;
    v[2].z = dst.pz-dst.wz+dst.dz;

    v[3].x = src.px-src.wx+src.dx;
    v[3].y = src.py-src.wy+src.dy;
    v[3].z = src.pz-src.wz+src.dz;
    Pol2Trigs(v);

// Bottom Surface 
    v[0].x = src.px+src.wx-src.dx;
    v[0].y = src.py+src.wy-src.dy;
    v[0].z = src.pz+src.wz-src.dz;

    v[1].x = dst.px+dst.wx-dst.dx;
    v[1].y = dst.py+dst.wy-dst.dy;
    v[1].z = dst.pz+dst.wz-dst.dz;

    v[2].x = dst.px-dst.wx-dst.dx;
    v[2].y = dst.py-dst.wy-dst.dy;
    v[2].z = dst.pz-dst.wz-dst.dz;

    v[3].x = src.px-src.wx-src.dx;
    v[3].y = src.py-src.wy-src.dy;
    v[3].z = src.pz-src.wz-src.dz;
    Pol2Trigs(v);

// Left Surface 
    v[0].x = src.px+src.wx+src.dx;
    v[0].y = src.py+src.wy+src.dy;
    v[0].z = src.pz+src.wz+src.dz;

    v[1].x = dst.px+dst.wx+dst.dx;
    v[1].y = dst.py+dst.wy+dst.dy;
    v[1].z = dst.pz+dst.wz+dst.dz;

    v[2].x = dst.px+dst.wx-dst.dx;
    v[2].y = dst.py+dst.wy-dst.dy;
    v[2].z = dst.pz+dst.wz-dst.dz;

    v[3].x = src.px+src.wx-src.dx;
    v[3].y = src.py+src.wy-src.dy;
    v[3].z = src.pz+src.wz-src.dz;
    Pol2Trigs(v);

// Right Surface 
    v[0].x = src.px-src.wx+src.dx;
    v[0].y = src.py-src.wy+src.dy;
    v[0].z = src.pz-src.wz+src.dz;

    v[1].x = dst.px-dst.wx+dst.dx;
    v[1].y = dst.py-dst.wy+dst.dy;
    v[1].z = dst.pz-dst.wz+dst.dz;

    v[2].x = dst.px-dst.wx-dst.dx;
    v[2].y = dst.py-dst.wy-dst.dy;
    v[2].z = dst.pz-dst.wz-dst.dz;

    v[3].x = src.px-src.wx-src.dx;
    v[3].y = src.py-src.wy-src.dy;
    v[3].z = src.pz-src.wz-src.dz;
    Pol2Trigs(v);
}

void DrawRibbonSTL(int kc,  Knot knot[], int npoints, Model *m)
{
    GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
    int ca = m->c_alphab[kc].atom;

    int plotStyleSav = m->plotStyle;

    if((m->plotStyle == RIBBONS) || (m->plotStyle == STRANDS))
    {
       for(int kk=0; kk<npoints-1; kk++)
           DrawSolidRibbonSTL(knot[kk], knot[kk+1]);
    }
    else
    {
        if(m->IsBetaSheet(ca))
        {
            for(int kk=0; kk<npoints-1; kk++)
                DrawSolidRibbonSTL(knot[kk], knot[kk+1]);
        }
        else if(m->IsHelix(ca))
        {
           for(int kk=0; kk<npoints-1; kk++)
              DrawSolidRibbonSTL(knot[kk], knot[kk+1]);
        }
        else if(m->IsDna(ca))
           DrawTubesSTL(gl, npoints, knot, 3);
        else
           DrawTubesSTL(gl, npoints, knot);
    }
    m->plotStyle = plotStyleSav;
}

static void GetSize(Knot *knot)
{
    if( !knot->vsize )
        knot->vsize = sqrt( knot->vnx*knot->vnx +
                            knot->vny*knot->vny +
                            knot->vnz*knot->vnz );

    if( !knot->hsize )
        knot->hsize = sqrt( knot->vnx*knot->hnx +
                            knot->vny*knot->hny +
                            knot->vnz*knot->hnz );
}

void DrawRibbonsSTL(GLWindow *gl, std::vector<C_Alpha> &c_alphab, Model *m)
{
    Knot knot[NKNOTS];
    float wide, hwide, size, kf = 1.5f;
    float ca[3], canext[3], o[3], o2[3];
    bool prev = false;
    bool ifArrow;

    memset(knot, 0, nknots*sizeof(Knot));
    int chain = -1;
    for(int i=0; i<(int)c_alphab.size(); i++)
    {
// Defining peptide planes
        if(!prev)
           memset(knot, 0, nknots*sizeof(Knot));

        if(c_alphab[i].chain != chain)
        {
           prev = false;
           chain = c_alphab[i].chain;
           memset(knot, 0, nknots*sizeof(Knot));
        }
        if(c_alphab[i].o == -1)
        {
           prev = false;
           continue;
        }
 
        for(int j=0; j<3; j++)
        {
            ca[j] = m->atm[c_alphab[i].atom].xyzN[j];
            if(c_alphab[i].isAmino)
               o[j] = m->atm[c_alphab[i].o].xyzN[j];
            else if(c_alphab[i].n == -1) // deoxyribonucleic acid
            {
               if(!c_alphab[i].breakF)
               {
                  o[j] = m->atm[c_alphab[i+1].o].xyzN[j];
                  if(c_alphab[i].o2 != -1)
                     o2[j] = m->atm[c_alphab[i].o2].xyzN[j];
               }
               else
                  o[j] = m->atm[c_alphab[i].o].xyzN[j];
            }
            else                          // ribonucleic acid
               o[j] = m->atm[c_alphab[i].o].xyzN[j];
            if(c_alphab[i].breakF)
               continue;
            canext[j] = m->atm[c_alphab[i+1].atom].xyzN[j];
        }
        if(c_alphab[i].breakF)            // siamo alla fine
        {
           if( m->IsBetaSheet(c_alphab[i].atom) && (m->plotStyle == CARTOONS))
           {
               wide = (float)(12*gl->cylinder);
               knot[nknots-1].px = ca[0] + (knot[nknots-1].tx/2);
               knot[nknots-1].py = ca[1] + (knot[nknots-1].ty/2);
               knot[nknots-1].pz = ca[2] + (knot[nknots-1].tz/2);
               ifArrow = true;
           } 
           else
           {
               wide = (float)(6*gl->cylinder);
               knot[nknots-1].px = ca[0];
               knot[nknots-1].py = ca[1];
               knot[nknots-1].pz = ca[2];
               ifArrow = false;
           }
           if( (knot[0].wide!=wide) && knot[0].hsize )
           {   
               size = knot[0].hsize;
               knot[0].wx = (wide*knot[0].hnx)/size;
               knot[0].wy = (wide*knot[0].hny)/size;
               knot[0].wz = (wide*knot[0].hnz)/size;

               if( !ifArrow )
               {   knot[nknots-1].wx = knot[0].wx;
                   knot[nknots-1].wy = knot[0].wy;
                   knot[nknots-1].wz = knot[0].wz;
               } 
               else
               {   
                   knot[nknots-1].wx = 0;
                   knot[nknots-1].wy = 0;
                   knot[nknots-1].wz = 0;
               }
            } 
            else if( ifArrow )
            {   
                wide = (float)(12*gl->cylinder);
                knot[nknots-1].wx = 0;
                knot[nknots-1].wy = 0;
                knot[nknots-1].wz = 0;
            }
            knot[1] = knot[nknots-1];
            if(!prev)
            {
            knot[0] = knot[1];
            knot[0].px = o[0];
            knot[0].py = o[1];
            knot[0].pz = o[2];
            }
            prev = false;
            DrawRibbonSTL(i, knot, 2, m);
            c_alphab[i].xyz[0] = knot[0].px;
            c_alphab[i].xyz[1] = knot[0].py;
            c_alphab[i].xyz[2] = knot[0].pz;
 //           if((IsDna(c_alphab[i].atom))  && (plotStyle == CARTOONS))
 //           {
               //drawSphere(knot[1].px, knot[1].py, knot[1].pz, 4.5f*(float)gl->cylinder,
//                         color1);
//            }
            continue;
        }
// Get vector A = Ca(i+l) - Ca(i)
        knot[nknots-1].tx = canext[0] - ca[0];
        knot[nknots-1].ty = canext[1] - ca[1];
        knot[nknots-1].tz = canext[2] - ca[2];
// Get vector B = O(i) - Ca(i) 
        float bx, by, bz;
        if(!c_alphab[i].isAmino && (c_alphab[i].o2 != -1) && (c_alphab[i].n == -1))
        {
           bx = (o[0]+o2[0])/2 - ca[0];
           by = (o[1]+o2[0])/2 - ca[1];
           bz = (o[2]+o2[0])/2 - ca[2];
        }
        else
        {
           bx = o[0] - ca[0];
           by = o[1] - ca[1];
           bz = o[2] - ca[2];
        }

// Form point P as the midpoint of Ca(i) and Ca(i+1)
        knot[nknots-1].px = (ca[0] + canext[0])/2;
        knot[nknots-1].py = (ca[1] + canext[1])/2;
        knot[nknots-1].pz = (ca[2] + canext[2])/2;

// Form vector C = A x B
// Vector C is normal to the peptide plane approximated by atoms Ca(i), O(i), and Ca(i+1
        knot[nknots-1].vnx = knot[nknots-1].ty*bz - knot[nknots-1].tz*by;
        knot[nknots-1].vny = knot[nknots-1].tz*bx - knot[nknots-1].tx*bz;
        knot[nknots-1].vnz = knot[nknots-1].tx*by - knot[nknots-1].ty*bx;

// If the peptide plane is part of a helix, translate in the direction of vector C, 
// away from the helix axis
        if(m->IsHelix(c_alphab[i].atom) && m->IsHelix(c_alphab[i+1].atom))
        {
           size = sqrt(knot[nknots-1].vnx*knot[nknots-1].vnx + 
                       knot[nknots-1].vny*knot[nknots-1].vny + 
                       knot[nknots-1].vnz*knot[nknots-1].vnz);
        
           knot[nknots-1].vsize = size;
           if( size )
           {   // 1.00 Angstrom Displacement
               wide = 0.7f/gl->deltav;
               knot[nknots-1].px += ((wide*knot[nknots-1].vnx)/size);
               knot[nknots-1].py += ((wide*knot[nknots-1].vny)/size);
               knot[nknots-1].pz += ((wide*knot[nknots-1].vnz)/size);
           } 
        }
        else 
            knot[nknots-1].vsize = 0;

// Form vector D = C x A
// Vector D lies parallel to the peptide plane and is perpendicular to vector A. 
        float dx = knot[nknots-1].vny*knot[nknots-1].tz - knot[nknots-1].vnz*knot[nknots-1].ty;
        float dy = knot[nknots-1].vnz*knot[nknots-1].tx - knot[nknots-1].vnx*knot[nknots-1].tz;
        float dz = knot[nknots-1].vnx*knot[nknots-1].ty - knot[nknots-1].vny*knot[nknots-1].tx;

// Scale vector D by one half the desired ribbon width
        dx /= (float)(3*gl->cylinder);
        dy /= (float)(3*gl->cylinder);
        dz /= (float)(3*gl->cylinder);
        knot[nknots-1].hsize = sqrt(dx*dx + dy*dy + dz*dz);

// Handle Carbonyl Oxygen Flip 
        if( prev && ((knot[0].hnx*dx + knot[0].hny*dy + knot[0].hnz*dz)<0) )
        {
            knot[nknots-1].hnx = -dx;   knot[nknots-1].vnx = -knot[nknots-1].vnx;
            knot[nknots-1].hny = -dy;   knot[nknots-1].vny = -knot[nknots-1].vny;
            knot[nknots-1].hnz = -dz;   knot[nknots-1].vnz = -knot[nknots-1].vnz;
        }
        else
        {
            knot[nknots-1].hnx = dx;
            knot[nknots-1].hny = dy;
            knot[nknots-1].hnz = dz;
        }

//  Set Ribbon Width 
        wide = (float)(6*gl->cylinder);

        ifArrow = false;
        if(m->plotStyle == CARTOONS)
        {
           if(m->IsBetaSheet(c_alphab[i].atom))
           {
              if(!m->IsBetaSheet(c_alphab[i+1].atom))
              {
                 ifArrow = true;
                 wide = (float)(12*gl->cylinder);
              }
           }
           else if(!m->IsHelix(c_alphab[i].atom))
                wide = (float)(kf*gl->cylinder);
        }
        if( knot[nknots-1].hsize && !ifArrow)
        {
            size = knot[nknots-1].hsize;
            knot[nknots-1].wx = (wide*knot[nknots-1].hnx)/size;
            knot[nknots-1].wy = (wide*knot[nknots-1].hny)/size;
            knot[nknots-1].wz = (wide*knot[nknots-1].hnz)/size;
            knot[nknots-1].wide = wide;
        }
        else
        {
            knot[nknots-1].wide = 0;
            knot[nknots-1].wx = 0;
            knot[nknots-1].wy = 0;
            knot[nknots-1].wz = 0;
        }

        if(prev && knot[0].hsize)
        {
           size = knot[0].hsize;
           knot[0].wx = (float)(wide*knot[0].hnx)/size;
           knot[0].wy = (float)(wide*knot[0].hny)/size;
           knot[0].wz = (float)(wide*knot[0].hnz)/size;
        }
        hwide = (float)(kf*gl->cylinder);
//240112        if(IsHelix(c_alphab[i].atom))
        if(m->IsHelix(c_alphab[i].atom) || (m->plotStyle == RIBBONS))
           hwide /= 3.;
        if(prev && knot[0].vsize)
        {
           size = knot[0].vsize;
           knot[0].dx = (float)(hwide*knot[0].vnx)/size;
           knot[0].dy = (float)(hwide*knot[0].vny)/size;
           knot[0].dz = (float)(hwide*knot[0].vnz)/size;
        }
        GetSize(&knot[nknots-1]);

        size = knot[nknots-1].vsize;
        knot[nknots-1].dx = (float)(hwide)*knot[nknots-1].vnx/size;
        knot[nknots-1].dy = (float)(hwide)*knot[nknots-1].vny/size;
        knot[nknots-1].dz = (float)(hwide)*knot[nknots-1].vnz/size;

        if(prev)
        {
           for(int k=0; k<nknots-2; k++)
           {
// Calculate Hermite Spline Points 
               knot[k+1].px = H[0][k]*knot[0].px + H[1][k]*knot[0].tx +
                              H[2][k]*knot[nknots-1].px + H[3][k]*knot[nknots-1].tx;
               knot[k+1].py = H[0][k]*knot[0].py + H[1][k]*knot[0].ty +
                              H[2][k]*knot[nknots-1].py + H[3][k]*knot[nknots-1].ty;
               knot[k+1].pz = H[0][k]*knot[0].pz + H[1][k]*knot[0].tz +
                              H[2][k]*knot[nknots-1].pz + H[3][k]*knot[nknots-1].tz;

//  Calculate Hermite Spline Widths 
               knot[k+1].wx = H[0][k]*knot[0].wx + H[2][k]*knot[nknots-1].wx;
               knot[k+1].wy = H[0][k]*knot[0].wy + H[2][k]*knot[nknots-1].wy;
               knot[k+1].wz = H[0][k]*knot[0].wz + H[2][k]*knot[nknots-1].wz;

// Calculate Surface Intensity 
               knot[k+1].vsize = 0;
               knot[k+1].vnx = H[0][k]*knot[0].vnx + H[2][k]*knot[nknots-1].vnx;
               knot[k+1].vny = H[0][k]*knot[0].vny + H[2][k]*knot[nknots-1].vny;
               knot[k+1].vnz = H[0][k]*knot[0].vnz + H[2][k]*knot[nknots-1].vnz;
               GetSize(&knot[k+1]);

//240112               if(plotStyle == CARTOONS)
               {
// Calculate Spline Heights 
                  knot[k+1].dx =  (float)(hwide*knot[k+1].vnx/knot[k+1].vsize); 
                  knot[k+1].dy =  (float)(hwide*knot[k+1].vny/knot[k+1].vsize);
                  knot[k+1].dz =  (float)(hwide*knot[k+1].vnz/knot[k+1].vsize);
// Calculate Surface Intensity 
                  knot[k+1].hsize = 0;
                  knot[k+1].hnx = H[0][k]*knot[0].hnx + H[2][k]*knot[nknots-1].hnx;
                  knot[k+1].hny = H[0][k]*knot[0].hny + H[2][k]*knot[nknots-1].hny;
                  knot[k+1].hnz = H[0][k]*knot[0].hnz + H[2][k]*knot[nknots-1].hnz;
                  GetSize(&knot[k+1]);
               }
           }

           DrawRibbonSTL(i, knot, nknots, m);
           knot[0] = knot[nknots-1];
            c_alphab[i].xyz[0] = knot[4].px;
            c_alphab[i].xyz[1] = knot[4].py;
            c_alphab[i].xyz[2] = knot[4].pz;
        }
        else                     // siamo all'inizio
        {
           knot[1] = knot[nknots-1];
           knot[0] = knot[nknots-1];
           knot[0].px = ca[0];
           knot[0].py = ca[1];
           knot[0].pz = ca[2];
           knot[0].hnx = dx;
           knot[0].hny = dy;
           knot[0].hnz = dz;
           c_alphab[i].xyz[0] = knot[0].px;
           c_alphab[i].xyz[1] = knot[0].py;
           c_alphab[i].xyz[2] = knot[0].pz;
//           if((IsDna(c_alphab[i].atom)) && (plotStyle == CARTOONS))
//           {
//               drawSphere(knot[0].px, knot[0].py, knot[0].pz, 4.5f*(float)gl->cylinder,
//                         color1);
//           }

        }
        prev = true;
    }
}

void Model::exportScene(GLWindow *gl, std::vector<vet3> &xyzat)
{
using namespace STL;
  if(plotStyle > C_ALPHA)
  {
     DrawRibbonsSTL(gl, c_alphab, this);
     return;
  }
     
  int subdiv;

  int ind = 0;
  if(plotStyle == VANDERWAALS)
     ind = 1;
  else if((plotStyle == RODS) || (plotStyle == C_ALPHA) || (plotStyle == POLYPEPTIDE))
     ind = 2;
  float subd = (float) (300. * maxRad[ind]) *sc_factor;
  if(ind != 2)
     subd *= (float) gl->sc_atoms;
  subdiv = (gl->Scale > 1.)? ((int) (subd * gl->Scale)) : ((int) subd);
  for(int i=0; i<nats;i++)
      atm[i].disegnato = ((plotStyle == STICK_AND_BALLS)||(plotStyle == ELLIPSOIDS));
  int i0=0, i1=0;
  jav_bond_type *legs=NULL;
  if(plotStyle < VANDERWAALS)
  {
     i0 = 0;
     i1 = nconn_prot;
     legs = bonds;
  }
  else if ( (plotStyle == POLYPEPTIDE) || (plotStyle == RIBBONS) )
  {
     i0 = 0;
     i1 = nconn_bb;
     legs = bonds_bb;
  }
  else if (plotStyle == C_ALPHA)
  {
     i0 = 0;
     i1 = nconn_ca;
     legs = bonds_ca;
  }


  double radius_c = gl->cylinder*gl->sc_bonds;
  for(int i=i0; i<i1;i++)
  //for(int i=0; i<1;i++)
  {
        int j1 = legs[i].n1 - 1;
        if(j1 >= nats)
            continue;
        if(j1 < 0)
           return;
         if(atm[j1].deleted || atm[j1].hidden)
            continue;
        int j2 = legs[i].n2 - 1;
        if(j2 >= nats)
            continue;
         if(atm[j2].deleted || atm[j2].hidden)
            continue;
        if( (!ifDrawH) &&  (IsHydrogen(j1) || IsHydrogen(j2)) )
            continue;
        if( (!ifDrawW) && ( atm_au[iEqui[j1]].isW  || atm_au[iEqui[j2]].isW ) )
            continue;
        vet3 p1 = MRVectorMake(atm[j1].xyzN);
        vet3 p2 = MRVectorMake(atm[j2].xyzN);
        PrintCilindroSTL(p1, p2, radius_c);
  }

  for(int i=0; i<nats;i++)
      atm[i].disegnato = 0;

  if(plotStyle < POLYPEPTIDE)
  {
  int np = 0;
  if(!xyzat.empty())
     np = (int) xyzat.size();
  for(int i=0; i<nats; i++)
  //for(int i=0; i<1; i++)
  {
      int z = atm[i].Z-1;
      if((z< 0) || (z > N_TRUE_ELEMENTS))
         z = N_TRUE_ELEMENTS;
      if( (!ifDrawH) &&  IsHydrogen(i) )
          continue;
      if( (!ifDrawW) &&  atm_au[iEqui[i]].isW )
          continue;
      if(atm[i].deleted || atm[i].hidden)
            continue;
      if(IsPeak(i))
         continue;
      double radius = atm[i].radN[ind];
      if(ind != 2)
         radius *=  gl->scale_atoms[z];
      else
         radius *=  gl->sc_bonds;
      PrintSferaSTL(i+np, radius, subdiv*5);
      xyzat.push_back(MRVectorMake(atm[i].xyzN));
  }
  }

  if(ifPoly && (nPoly > 0))
     PrintPoliedriSTL(this);
}

}   // end namespace GLextmr

