  module unit_cell

  USE trig_constants

  implicit none

  type cell_type
    private
    real, dimension(6)   :: par   = [10., 10., 10., 90., 90., 90.]
    real, dimension(3,3) :: ortom = reshape([1., 0., 0., 0., 1., 0., 0., 0., 1.],[3,3])
    real, dimension(3,3) :: ortoi = reshape([1., 0., 0., 0., 1., 0., 0., 0., 1.],[3,3])
    real, dimension(3,3) :: g     = reshape([1., 0., 0., 0., 1., 0., 0., 0., 1.],[3,3])
    real, dimension(6)   :: cod = 0

  contains
    procedure :: get_par
    procedure :: get_ortom
    procedure :: get_ortoi
    procedure :: get_g
    procedure :: set 
  end type cell_type

  CONTAINS

   function get_par(cell)
   class(cell_type), intent(in) :: cell
   real, dimension(6)           :: get_par
   get_par = cell%par
   end function get_par

!-----------------------------------------------------------------------

   function get_ortom(cell)
   class(cell_type), intent(in) :: cell
   real, dimension(3,3)         :: get_ortom
   get_ortom = cell%ortom
   end function get_ortom

!-----------------------------------------------------------------------

   function get_ortoi(cell)
   class(cell_type), intent(in) :: cell
   real, dimension(3,3)         :: get_ortoi
   get_ortoi = cell%ortoi
   end function get_ortoi

!-----------------------------------------------------------------------

   function get_g(cell)
   class(cell_type), intent(in) :: cell
   real, dimension(3,3)         :: get_g
   get_g = cell%g
   end function get_g

!-----------------------------------------------------------------------

   function set_cell_type(par) result(cell)
   real, dimension(6), intent(in) :: par
   type(cell_type)                :: cell               
   call cell%set(par)
   end function set_cell_type

!-----------------------------------------------------------------------

   subroutine set(cell,par)
   class(cell_type), intent(inout) :: cell
   real, dimension(6), intent(in)  :: par
!
   cell%par = par
   cell%g = matrice_metrica(par)
   cell%ortom = orthomatrix(par)
   cell%ortoi = orthomatrixi(par)
!
   end subroutine set

!-----------------------------------------------------------------------

   function matrice_metrica(cell)  result(gg)
   real, dimension(6), intent(in) :: cell
   real, dimension(3,3)           :: gg
!
   gg(1,1) = cell(1)**2
   gg(2,2) = cell(2)**2
   gg(3,3) = cell(3)**2
   gg(1,2) = cell(1) * cell(2) * cos(cell(6) * dtor)
   gg(1,3) = cell(1) * cell(3) * cos(cell(5) * dtor)
   gg(2,3) = cell(2) * cell(3) * cos(cell(4) * dtor)
   gg(3,1) = gg(1,3)
   gg(2,1) = gg(1,2)
   gg(3,2) = gg(2,3)
!
   end function matrice_metrica
!-----------------------------------------------------------------------
   real function determinante(mat)
!
   real, dimension(3,3) :: mat
!
   determinante = mat(1,1)*mat(2,2)*mat(3,3) + mat(1,2)*mat(2,3)*mat(3,1) + mat(1,3)*mat(2,1)*mat(3,2) &
                - mat(3,1)*mat(2,2)*mat(1,3) - mat(3,2)*mat(2,3)*mat(1,1) - mat(3,3)*mat(2,1)*mat(1,2)
!
   end function determinante
!------------------------------------------------------------------------
   function cella_reciproca(cell)  result(cells)
   real, dimension(6), intent(in) :: cell
   real, dimension(6)             :: cells
   real                           :: vol
   real                           :: s1,s2,s3
   real                           :: c1,c2,c3
!
   vol = sqrt(determinante(matrice_metrica(cell)))  ! volume cella
   s1 = sin(cell(4)*dtor);        c1 = cos(cell(4)*dtor)
   s2 = sin(cell(5)*dtor);        c2 = cos(cell(5)*dtor)
   s3 = sin(cell(6)*dtor);        c3 = cos(cell(6)*dtor)
   cells(1) = cell(2)*cell(3)*s1 / vol         ! a*
   cells(2) = cell(1)*cell(3)*s2 / vol         ! b*
   cells(3) = cell(1)*cell(2)*s3 / vol         ! c*
   cells(4) = (c2*c3-c1)/(s2*s3)               ! cos(alpha*)
   cells(5) = (c1*c3-c2)/(s1*s3)               ! cos(beta*)
   cells(6) = (c1*c2-c3)/(s1*s2)               ! cos(gamma*)
!
   end function cella_reciproca

!--------------------------------------------------------------------

   function orthomatrix(cell)  result(om)
!
!  Generate matrix to convert fractional coordinates in cartesian coordinates
!   
   real, dimension(6), intent(in) :: cell
   real, dimension(6)             :: cells
   real, dimension(3,3)           :: om
!
   cells = cella_reciproca(cell)
!
   om(1,1)=cell(1)*sin(cell(5)*dtor)                     ! a*sen(beta)
   om(1,2)=-cell(2)*sin(cell(4)*dtor)*cells(6)           ! -b*sen(alfa)*cos(gammas)
   om(1,3)=0
   om(2,1)=0
   om(2,2)=1.0/cells(2)                                  !b*sen(alfa)*sen(gammas) = 1.0/cells(2)
   om(2,3)=0
   om(3,1)=cell(1)*cos(cell(5)*dtor)                     !a*cos(beta)
   om(3,2)=cell(2)*cos(cell(4)*dtor)                     !b*cos(alfa)
   om(3,3)=cell(3)                                       !c
!
   end function orthomatrix
   
!--------------------------------------------------------------------

   function orthomatrixi(cell)  result(om)
!
!  Generate matrix to convert cartesian coordinates in fractionale coordinates
! 
   real, dimension(6), intent(in) :: cell
   real, dimension(6)             :: cells
   real, dimension(3,3)           :: om
   !real, dimension(3,3)           :: om1

   cells = cella_reciproca(cell)
!
   om(1,1)=cells(1)*sin(acos(cells(6)))
   om(1,2)=cell(2)*cells(2)*cells(6)*sin(cell(4)*dtor)/(cell(1)*sin(cell(5)*dtor))
   om(1,3)=0
   om(2,1)=0                                              
   om(2,2)=cells(2)                                       
   om(2,3)=0
   om(3,1)=-1.0/(cell(3)*tan(cell(5)*dtor))
   om(3,2) = -cell(2)*cells(2)*(cos(cell(4)*dtor) +       &
              cells(6)*sin(cell(4)*dtor)*cos(cell(5)*dtor)/sin(cell(5)*dtor))/cell(3)
   om(3,3)=1.0/cell(3)

   !write(6,*)'SENZA INVERSIONE'
   !do i=1,3
   !   write(6,*)om(i,:3)
   !enddo
!   
   !om1 = orthomatrix(cell)
   !call InvMat(om1,om,icod)
   !write(6,*)'CON INVERSIONE'
   !do i=1,3
   !   write(6,*)om(i,:3)
   !enddo
!
   end function orthomatrixi
   
!------------------------------------------------------------------------------------------

   function orthomatrix_std(cell)  result(om)
!
!  Generate the standard cartesian frame (x//a). use this reference frame for pdb file
! 
!  [ a     b*cos(gamma)   c*cos(beta)              ]
!  [ 0     b*sin(gamma)   -c*sin(beta)*cos(alpha*) ]
!  [ 0         0               1/c*                ]
!
   real, dimension(6), intent(in) :: cell
   real, dimension(6)             :: cells
   real, dimension(3,3)           :: om
!
   cells = cella_reciproca(cell)
!
   om(1,1)=cell(1)                              ! a
   om(1,2)=cell(2)*cos(cell(6)*dtor)            ! b*cos(gamma)
   om(1,3)=cell(3)*cos(cell(5)*dtor)            ! c*cos(beta)
   om(2,1)=0             
   om(2,2)=cell(2)*sin(cell(6)*dtor)            ! b*sin(gamma)
   om(2,3)=-cell(3)*sin(cell(5)*dtor)*cells(4)  ! -c*sin(beta)*cos(alpha*)
   om(3,1)=0
   om(3,2)=0
   om(3,3)=1/cells(3)                           ! 1/c* = c*sin(beta)*sin(gamma*); sin(gamma*) = sqrt(1-cos(gamma*)**2)
!
   end function orthomatrix_std

!------------------------------------------------------------------------------------------

   function orthomatrixi_std(cell)  result(om)
!  
!  Inverse matrix of the standard frame (x//a) for conversion fractional to cartesian
!
!  [ 1/a    -cos(gamma)/a*sin(gamma)   a**beta*  ]
!  [  0       1/b*sin(gamma)           b*alpha*  ]
!  [  0            0                   c*        ]
   real, dimension(6), intent(in) :: cell
   real, dimension(6)             :: cells
   real, dimension(3,3)           :: om
!
   cells = cella_reciproca(cell)
!
   om(1,1)=1/cell(1)                                             ! 1/a
   om(1,2)=-cos(cell(6)*dtor)/(cell(1)*sin(cell(6)*dtor))        ! -cos(gamma)
   om(1,3)=cells(1)*cells(5)                                     ! a**beta*
   om(2,1)=0                    
   om(2,2)=1.0/(cell(2)*sin(cell(6)*dtor))                       ! 1/b*sin(gamma)
   om(2,3)=cells(2)*cells(4)                                     ! b*alpha*
   om(3,1)=0
   om(3,2)=0
   om(3,3)=cells(3)                                              ! c*
!
   end function orthomatrixi_std

!------------------------------------------------------------------------------------------

   function deriv_orthomatrix_std(cell)   result(der)
!
!  Compute derivatve matrix to cartesian frame. Standard cartesian frame (x//a) is used.
!  This function is applied to compute the sd on distance and angle
!
   real, dimension(6), intent(in) :: cell
   real, dimension(3,3,6)         :: der
   real                           :: cosa,cosb,cosg,sina,sinb,sing
   real                           :: fval,gval
   real                           :: dfa,dfb,dfg
   real                           :: dga,dgb,dgg
                       !real, dimension(6)             :: cells
!
   der = 0
                       !            cells = cella_reciproca(cell)
!
!  Standard frame x//a
!  [ a     b*cos(gamma)   c*cos(beta)                    ]     [ a      b*cosg    c*cosb ]
!  [ 0     b*sin(gamma)   -c*sin(beta)*cos(alpha*)       ]  =  [ 0      b*sing    c*fval ]
!  [ 0         0         1/c* = c*sin(beta)*sin(gamma*)  ]     [ 0        0       c*gval ]
!  
   cosa = cos(cell(4)*dtor)
   cosb = cos(cell(5)*dtor)
   cosg = cos(cell(6)*dtor)
   sina = sin(cell(4)*dtor)
   sinb = sin(cell(5)*dtor)
   sing = sin(cell(6)*dtor)
!
!  Compute fval and his derivative
   fval = (cosa - cosb*cosg) / sing      ! sin(beta)*cos(alpha*)
   dfa = -sina/sing                      ! dfval/dalpha
   dfb = sinb*cosg/sing                  ! dfval/dbeta
   dfg =  (cosb - cosa*cosg) / sing**2   ! dfval/dgamma
    !write(0,*)'fval=',fval,cell(3)*fval,-cell(3)*sin(cell(5)*dtor)*cells(4)
!
!  Compute gval and his derivative
   gval = sqrt(sinb*sinb - fval*fval)    ! sin(beta)*sin(gamma*)
   dga = -fval*dfa/gval                  ! dgval/dalpha
   dgb = (cosb*sinb - fval*dfb) / gval   ! dgval/dbeta
   dgg = -fval*dfg/gval                  ! dgval/dgamma
    !write(0,*)'gval=',gval,cell(3)*gval,1/cells(3)
!
!  der(a) =      [ 1    0      0    0     0      0     ]
   der(1,1,1) = 1
!
!  der(b*cosg) = [ 0  cosg     0    0     0     b*sing ]
   der(1,2,2) = cosg
   der(1,2,6) = -cell(2)*sing
!
!  der(c*cosb) = [ 0     0   cosb   0   -c*sinb   0    ]  
   der(1,3,3) = cosb 
   der(1,3,5) = -cell(3)*sinb
!
!  der(b*sing) = [ 0   sing   0     0     0     b*cosg ]
   der(2,2,2) = sing
   der(2,2,6) = cell(2)*cosg
!
!  der(c*fval) = [ 0    0    fval   c*dfa  c*dfb c*dfg ]
   der(2,3,3) = fval
   der(2,3,4) = cell(3)*dfa
   der(2,3,5) = cell(3)*dfb
   der(2,3,6) = cell(3)*dfg
!
!  der(c*gval) = [ 0    0    gval   c*dga  c*dgb  c*dgg ]
   der(3,3,3) = gval
   der(3,3,4) = cell(3)*dga
   der(3,3,5) = cell(3)*dgb
   der(3,3,6) = cell(3)*dgg
!
   end function deriv_orthomatrix_std

!------------------------------------------------------------------------------------------

   subroutine distance_and_sd(sdcell,ortho,dermat,x1,x2,s1,s2,dis,sd)
   real, dimension(6), intent(in)     :: sdcell   ! standard deviations
   real, dimension(3,3), intent(in)   :: ortho         ! direct structure matrix
   real, dimension(3,3,6), intent(in) :: dermat        ! derivative of direct structure matrix 
   real, dimension(3), intent(in)     :: x1,x2         ! fractional coordinates of two points
   real, dimension(3), intent(in)     :: s1,s2         ! standard deviation on fractional coordinates
   real, intent(out)                  :: dis,sd        ! distance and sd
   real, dimension(3)                 :: xdiff
   real, dimension(3)                 :: xmat
   real, dimension(6)                 :: dxyz
   real, dimension(6)                 :: dcell
   integer                            :: i
!
   xdiff = x2 - x1
   xmat = matmul(ortho,xdiff) 
   dis = sqrt(dot_product(xmat,xmat))
!   
!  derivate of d^2 with rispect x,y,z, factor 2 is ignored
   dxyz(:3) = matmul(xmat,ortho)
   dxyz(4:6) = -dxyz(1:3)
!
!  derivate d^2 with rispect a,b,c,alpha,beta,gamma, factor 2 is ignored
   do i=1,6
      dcell(i) = dot_product(xmat,matmul(dermat(:,:,i),xdiff))
   enddo
!
!  compute sigma(d^2)**2 = sum(der**2)
   sd = 0
   do i=1,3
      sd = sd + (dxyz(i)*s2(i))**2 + (dxyz(i+3)*s1(i))**2    &
              + (dcell(i)*sdcell(i))**2                      &
              + (dcell(i+3)*sdcell(i+3)*dtor)**2    
   enddo
!
   sd = sqrt(sd)/dis
!
   end subroutine distance_and_sd

!------------------------------------------------------------------------------------------

   function matrice_mreciproca(cell)  result(ggr)
   real, dimension(6), intent(in) :: cell
   real, dimension(6)             :: cells
   real, dimension(3,3)           :: ggr
!
   cells = cella_reciproca(cell)
!
   ggr(1,1) = cells(1)**2
   ggr(2,2) = cells(2)**2
   ggr(3,3) = cells(3)**2
   ggr(1,2) = cells(1) * cells(2) * cells(6)
   ggr(1,3) = cells(1) * cells(3) * cells(5)
   ggr(2,3) = cells(2) * cells(3) * cells(4)
   ggr(3,1) = ggr(1,3)
   ggr(2,1) = ggr(1,2)
   ggr(3,2) = ggr(2,3)
!
   end function matrice_mreciproca
  
!--------------------------------------------------------------------

   real function cell_volume(cell)
   real, dimension(:), intent(in) :: cell
!
   cell_volume = sqrt(determinante(matrice_metrica(cell)))
!
   end function cell_volume

!--------------------------------------------------------------------

   real function cell_volume_std(vol,cell,cellstd) result(std)
   real, intent(in)               :: vol
   real, dimension(:), intent(in) :: cell
   real, dimension(:), intent(in) :: cellstd
   real, dimension(3)             :: a,b
   real, dimension(3)             :: cellrad
   real                           :: vol2
!
   a(:) = cellstd(:3) / cell(:3)
   cellrad(:) = cell(4:)*dtor
   b(1) = sin(cellrad(1))*(cos(cellrad(1)) - cos(cellrad(2))*cos(cellrad(3)))*cellstd(4)*dtor
   b(2) = sin(cellrad(2))*(cos(cellrad(2)) - cos(cellrad(1))*cos(cellrad(3)))*cellstd(5)*dtor
   b(3) = sin(cellrad(3))*(cos(cellrad(3)) - cos(cellrad(1))*cos(cellrad(2)))*cellstd(6)*dtor
   vol2 = vol**2
   std = vol2*sum(a(:)**2) + ((cell(1)*cell(2)*cell(3))**4)*sum(b(:)**2)/vol2
   std = sqrt(std)
!
   end function cell_volume_std

!--------------------------------------------------------------------

   subroutine print_cell(cell,sdcell,sys,kpr)
   USE strutil
   real, dimension(6), intent(in)            :: cell
   real, dimension(6), intent(in), optional  :: sdcell
   character(len=2), intent(in), optional    :: sys
   integer, intent(in)                       :: kpr
   character(len=*), dimension(6), parameter :: scell= ['a    ','b    ','c    ','alpha','beta ','gamma']
   integer                                   :: i
   character(len=2)                          :: csys
   real                                      :: vol, stdvol
!
   if (present(sys)) then
       csys = sys
   else
       csys = ' '
   endif
   if (present(sdcell)) then
       vol = cell_volume(cell)
       stdvol = cell_volume_std(vol,cell,sdcell)
       select case (csys)
         case ('CU')         ! Cubic
           write(kpr,10)trim(scell(1))//'= ',string_esd(cell(1),sdcell(1)),'Volume= ',string_esd(vol,stdvol)

         case ('TE','HE')    ! Tetragonal, hexagonal
           write(kpr,10)trim(scell(1))//'= ',string_esd(cell(1),sdcell(1)),    &
                        trim(scell(3))//'= ',string_esd(cell(3),sdcell(3)),'Volume= ',string_esd(vol,stdvol)

         case ('OR')         ! Orthorombic
           write(kpr,10)(trim(scell(i))//'= ',string_esd(cell(i),sdcell(i)), i=1,3)
           write(kpr,10)'Volume= ',string_esd(vol,stdvol)

         case ('MO')         ! Monoclinic
           write(kpr,10)(trim(scell(i))//'= ',string_esd(cell(i),sdcell(i)), i=1,3)
           write(kpr,10)trim(scell(5))//'= ',string_esd(cell(5),sdcell(5)),'Volume= ',string_esd(vol,stdvol)

         case default       
           write(kpr,10)(trim(scell(i))//'= ',string_esd(cell(i),sdcell(i)), i=1,3)
           write(kpr,10)(trim(scell(i))//'= ',string_esd(cell(i),sdcell(i)), i=4,6)
           write(kpr,10)'Volume= ',string_esd(vol,stdvol)

       end select
10     format(t5,a8,a,t30,a8,a,t50,a8,a,t70,a,a)
   else
       write(kpr,'(1x,a)')'Lattice parameters'
       do i=1,3
          write(kpr,'(9x,a,t20,f0.7)')trim(scell(i)),cell(i)
       enddo
       do i=4,6
          write(kpr,'(9x,a,t20,f0.4)')trim(scell(i)),cell(i)
       enddo
   endif
!
   end subroutine print_cell

   end module unit_cell

