  MODULE CCRYST

  USE trig_constants

  implicit none

  CONTAINS

!--------------------------------------------------------------------

   subroutine genera_2theta(ref,celp,lam)
   USE unit_cell
   USE progtype, only : reflection_type
!                                                   T *
!  2-theta = 2.0 * arcosen ( 0.5 * lambda * sqrt ( H G H ) ) in radianti
!
   type(reflection_type),dimension(:), intent(inout) :: ref
   real, dimension(6), intent(in)                    :: celp   ! cella
   real, intent(in)                                  :: lam    ! lambda
   integer                                           :: i
   real, dimension(3,3)                              :: ggr
!
   ggr = matrice_mreciproca(celp)   ! G*
!
   do i=1,size(ref) ! loop sui riflessi
      ref(i)%slaq = 0.25*DOT_PRODUCT(ref(i)%hkl,MATMUL(ggr,ref(i)%hkl))      ! (sentheta / lambda)**2
      ref(i)%tthd = 2.0*asin(min(lam*sqrt(ref(i)%slaq),1.0))*rtod  ! 2theta in gradi
   enddo
!
   end subroutine genera_2theta

!--------------------------------------------------------------------

   function beta_from_uij(uij,gr,cellr)   result(bij)
!
!  Calcola i termini bij del tensore beta da uij
!
   real, dimension(6), intent(in)   :: uij    ! u11, u22, u33, u12, u13, u23
   real, dimension(3,3), intent(in) :: gr     ! matrice metrica reciproca
   real, dimension(6), intent(in)   :: cellr  ! cella reciproca
   real, dimension(6)               :: bij
!
   bij(1) = twopis * uij(1) * gr(1,1)                  ! b11
   bij(2) = twopis * uij(2) * gr(2,2)                  ! b22
   bij(3) = twopis * uij(3) * gr(3,3)                  ! b33
   bij(4) = twopis * uij(4) * cellr(1) * cellr(2)      ! b12
   bij(5) = twopis * uij(5) * cellr(1) * cellr(3)      ! b13
   bij(6) = twopis * uij(6) * cellr(2) * cellr(3)      ! b23
!
   end function beta_from_uij

!--------------------------------------------------------------------

   function u_from_bij(bij,gr,cellr)   result(uij)
!
!  Calcola i termini uij  da bij
!
   real, dimension(6), intent(in)   :: bij    ! u11, u22, u33, u12, u13, u23
   real, dimension(3,3), intent(in) :: gr     ! matrice metrica reciproca
   real, dimension(6), intent(in)   :: cellr  ! cella reciproca
   real, dimension(6)               :: uij
!
   uij(1) = bij(1) / (twopis * gr(1,1))                  ! u11
   uij(2) = bij(2) / (twopis * gr(2,2))                  ! u22
   uij(3) = bij(3) / (twopis * gr(3,3))                  ! u33
   uij(4) = bij(4) / (twopis * cellr(1) * cellr(2))      ! u12
   uij(5) = bij(5) / (twopis * cellr(1) * cellr(3))      ! u13
   uij(6) = bij(6) / (twopis * cellr(2) * cellr(3))      ! u23
!
   end function u_from_bij

!--------------------------------------------------------------------

   real function bequiv_from_beta(bij,gg) result(bequiv)
!
!  Calcola b equivalente da beta Bequiv = (4/3)*Tr(beta*G)
!
   real, dimension(6), intent(in)   :: bij  ! tensore beta =  b11,b22,b33,b12,b13,b23
   real, dimension(3,3), intent(in) :: gg   ! matrice metrica
!
   bequiv = (4.0 / 3.0) * (bij(1)*gg(1,1) + bij(2)*gg(2,2) + bij(3)*gg(3,3) +  &
            2*bij(4)*gg(1,2) + 2*bij(5)*gg(1,3) + 2*bij(6)*gg(2,3))
!
   end function bequiv_from_beta

!--------------------------------------------------------------------

   logical function aniso_adp_is_ok(bij)  result(ok)
!
!  Check relationship between adp    
!
   real, dimension(6), intent(in) :: bij  ! tensore beta =  b11,b22,b33,b12,b13,b23
   real, dimension(3,3)           :: bmat,bmat2
   integer                        :: i,j
   !real, parameter                :: EPS = epsilon(1.0)
!
   !write(0,*)'bij=',bij(:)
   bmat = reshape([bij(1),bij(4),bij(5),bij(4),bij(2),bij(6),bij(5),bij(6),bij(3)],(/3,3/))
      !!!!FIXME - sir check for singular matrix on bmat
   !do i=1,3
   !   write(0,*)'BMAT: ',bmat(i,:3)
   !enddo
   bmat2 = bmat*bmat
!
!  Bii > 0
   do i=1,3
      ok = bmat(i,i) > 0
      if (.not.ok) return
      !if (.not.ok) then
      !        write(0,*)'RETURN WITH 1=',bmat(i,i)
      !    return
      !endif
   enddo
!
!  BiiBij > Bij^2
   do i=1,3
      do j=1,3
         if (i==j) cycle
         !ok = abs(bmat(i,i)*bmat(i,j) - bmat2(i,j)) >= 0
         ok = abs(bmat(i,i)*bmat(i,j) - bmat2(i,j)) >= 0
         if (.not.ok) return
         !if (.not.ok) then
         !     write(0,*)'RETURN WITH=',abs(bmat(i,i)*bmat(i,j) - bmat2(i,j))
         !     return
         !endif
      enddo
   enddo
!
   ok = abs(bmat(1,1)*bmat(2,2)*bmat(3,3) + bmat2(1,2)*bmat2(1,3)*bmat2(2,3) -   &
        bmat(1,1)*bmat2(2,3) + bmat(2,2)*bmat2(1,3) + bmat(3,3)*bmat2(1,2)) > 0
      !   if (.not.ok) then
      !        write(0,*)'RETURN WITH=',abs(bmat(1,1)*bmat(2,2)*bmat(3,3) + bmat2(1,2)*bmat2(1,3)*bmat2(2,3) -   &
      !  bmat(1,1)*bmat2(2,3) + bmat(2,2)*bmat2(1,3) + bmat(3,3)*bmat2(1,2))
      !        return
      !   endif
!
   end function aniso_adp_is_ok

!--------------------------------------------------------------------

   real function b_from_u(u) result(b)
!
!  Calcola b da u. b=8*pi^2*u
!
   real, intent(in) :: u
!
   b = 4.0*twopis*u   
!
   end function b_from_u

!--------------------------------------------------------------------

   real function u_from_b(b) result(u)
!
!  Calcola b da u. b=8*pi^2*u
!
   real, intent(in) :: b
!corr   real, parameter :: const=1/(4.0*twopis)
   real :: const
!
   const=1/(4.0*twopis)
   u = const*b
!
   end function u_from_b
  END MODULE CCRYST

!----------------------------------------------------------------------------------
!----------------------------------------------------------------------------------

 MODULE CALCTIME
!
! Questo modulo contiene procedure per il calcolo e la gestione dei tempi
!
!S init_timestep(nstepi,vetst)             Inizializza calcolo tempi
!S timestep(step)                          Chiama questa sub. all'inizio e alla fine dell'intervallo numero step 
!S print_timestep()                        Stampa tempi parziali
!S get_timestep(kstep,timer)               Prendi informazioni sul tempo kstep   
!F get_totaltime()                         Prendi il tempo totale
!S reset_totaltime()                       Azzera il tempo totale
!S end_timestep()                          Deallocazione
!S set_status_print_time(kstep,statusp)    Rendi non stampabile un determinato step temporale
!S function string_time(time)              Converte tempo(sec.) in stringa
 implicit none

 private 

 public :: init_timestep,timestep,print_timestep,end_timestep,string_time,    &
get_timestep,set_status_print_time,get_totaltime,reset_totaltime   

 type, public :: calctime_type
   real              :: tot      ! tempo totale dell'evento
   real              :: ini      ! momento iniziale dell'evento
   real              :: fin      ! momento finale dell'evento
   character(len=80) :: str      ! stringa associata all'evento
   integer           :: nv       ! quante volte misuro l'evento
   real              :: ave      ! media
   integer           :: state    ! 0=inizio, 1=fine
   logical           :: statp    ! Va stampato?
 end type calctime_type
 
 logical, public                                :: timeactive = .false.
 type(calctime_type), dimension(:), allocatable :: time
 integer, private                               :: nstep
 real, private                                  :: totaltime
     
 CONTAINS
 
   subroutine init_timestep(nstepi,vetst)
!   
!  Inizializza calcolo tempi
!
   integer, intent(in)                           :: nstepi  !num. di intervalli su cui misurare il tempo
   character(len=*), dimension(nstepi), optional :: vetst   !associa una stringa ad ogni intervallo
   character(len=6)                              :: strn
   integer                                       :: i
   
   nstep = nstepi
   allocate(time(nstep))
   time%tot = 0
   time%nv = 0
   time%state = 0
   time%statp = .true.
   if (present(vetst)) then
       do i=1,nstep
          time(i)%str = vetst(i)
       enddo   
   else
       do i=1,nstep
          write(strn,'(i6)')i
          time(i)%str = 'step n.'//trim(adjustl(strn))
       enddo
   endif    
   call cpu_time(totaltime)
   end subroutine init_timestep
   
 !----------------------------------------------------------------------------------
   
   subroutine timestep(step)
!
!  Chiama questa sub. all'inizio e alla fine dell'intervallo numero step
!
   integer, intent(in)   :: step
   !integer, dimension(8) :: time_array
!   
   if (.not.timeactive) return
   if (time(step)%state == 0) then
       call cpu_time(time(step)%ini)       
       !call date_and_time(values=time_array)
       !time(step)%ini = time_array(5) * 3600 + time_array(6) * 60 &
       !               + time_array(7) + 0.001 * time_array(8)
       time(step)%state = 1
   else
       call cpu_time(time(step)%fin)
       !call date_and_time(values=time_array)
       !time(step)%fin = time_array(5) * 3600 + time_array(6) * 60 &
       !               + time_array(7) + 0.001 * time_array(8)
       time(step)%tot = time(step)%tot + (time(step)%fin-time(step)%ini)
       time(step)%nv = time(step)%nv + 1
       time(step)%state = 0
   endif 
!   
   end subroutine timestep
   
  !----------------------------------------------------------------------------------
 
   subroutine print_timestep(kpri)
!
!  Stampa tempi parziali
!
   integer, optional, intent(in) :: kpri
   integer                       :: i
   integer                       :: maxl
   integer                       :: kpr
!   
   if (.not.timeactive) return   
   if (present(kpri)) then
       kpr = kpri
   else
       kpr = 6
   endif
   
!!!   if (any(time%statp == .true.)) then
   if (any(time%statp)) then
       write(kpr,'(/a)')'Summary of times'
       maxl = maxval(len_trim(time%str))
       do i=1,nstep
          if (time(i)%statp)write(kpr,'(a)')time(i)%str(1:maxl)//': '//trim(string_time(time(i)%tot))
       enddo 
   endif    
!       
   end subroutine print_timestep 

 !---------------------------------------------------------------------------------- 
 
   subroutine get_timestep(kstep,timer)
!
!  Prendi informazioni sul tempo kstep   
!
   integer, intent(in)              :: kstep
   type(calctime_type), intent(out) :: timer
!
   if (.not.timeactive) return
   time(kstep)%ave = time(kstep)%tot/time(kstep)%nv   ! calcola la media
   timer = time(kstep)   
!   
   end subroutine get_timestep
   
 !---------------------------------------------------------------------------------- 

   real function get_totaltime()  result(timet)
   real :: endtime
   call cpu_time(endtime)
   timet = endtime - totaltime
   end function get_totaltime

 !---------------------------------------------------------------------------------- 
 
   subroutine reset_totaltime()
   call cpu_time(totaltime)
   end subroutine reset_totaltime

 !---------------------------------------------------------------------------------- 
 
   subroutine end_timestep()
!
!  Deallocazione
!
!!!!!!   if (.not.timeactive) return
   if(allocated(time))deallocate(time)
       
   end subroutine end_timestep
   
 !----------------------------------------------------------------------------------
 
   subroutine set_status_print_time(kstep,statusp)
!
!  Rendi non stampabile un determinato step temporale
!   
   integer, intent(in) :: kstep
   logical, intent(in) :: statusp
!
   time(kstep)%statp = statusp
!   
   end subroutine set_status_print_time
   
 !---------------------------------------------------------------------------------- 
 
   function string_time(time)  result(str)
!
!  Converte tempo(sec.) in stringa
!
   USE strutil
   real, intent(in)  :: time
   character(len=80) :: str
   integer           :: hh,mm
   integer           :: total
!
   total = int(time)
   hh = total/3600
   mm = (total - 3600*hh) / 60
   str = trim(i_to_s(hh))//'h '//trim(i_to_s(mm))//'m '//trim(i_to_s(total - hh*3600 - mm*60))//'s '
!   
   end function string_time  
   
 END MODULE CALCTIME

!----------------------------------------------------------------------------------

MODULE PRNUTIL

 implicit none

 CONTAINS

   subroutine stampa_matrice(amatr,stringpar,ndigit,kpr,title,numcolo)
   USE progtype, only:dblep
   USE strutil
!
   real(dblep), dimension(:,:), intent(in)           :: amatr     ! matrice da stampare
   character(len=*), dimension(:), intent(in)        :: stringpar ! stringa associata alla riga
   integer, intent(in)                               :: ndigit    ! numero di cifre da stampare per numero
   integer,                          intent(in)      :: kpr       ! unita' di stampa
   character(len=*), intent(in), optional            :: title     ! eventuale titolo 
   integer,optional                                  :: numcolo   ! numero di colonne da usare per la stampa
   integer                                           :: numcol 
   integer                                           :: ndim    
   integer                                           :: num_elem    
   integer                                           :: i,j,is,iniz,ifin
   character(len=ndigit+7), dimension(:),allocatable :: stringv,stringp
   integer                                           :: ns
   character(len=20)                                 :: sform
   integer                                           :: wfield
!
   ndim = size(amatr,1)
   if(present(numcolo)) then
     numcol=numcolo
   else
     numcol = 70
   endif
   wfield  = ndigit + 7  !+3 perche:1 spazio di separazione+virgola+punto decimale+4 spazi per exp
!
!  Calcola il numero di elementi da stampare per blocco
   num_elem  = numcol / wfield
   allocate(stringv(num_elem),stringp(ndim))
!
!  centra tutta le stringhe nel campo wfield
   do i=1,ndim
      stringp(i) = centra_str(stringpar(i),wfield)
   enddo
!
!  Stampa
   if (present(title)) then
       write(6,'(/a)')centra_str(title,numcol+16) !perche 1x,t15
   endif
   i=0
   do 
      i=i+1
      iniz = 1 + (i - 1) * num_elem
      if (iniz > ndim) exit
      ifin = min (num_elem * i,ndim)
      ns = ifin-iniz+1
      write(sform,'(a,i0,a)')'(1x,t15,',ns,'a)'
      write(kpr,sform) stringp(iniz:ifin)(1:wfield)
      do j=1,ndim
         ns = 0
         do is=iniz,ifin
            ns = ns + 1
            stringv(ns) = string_sig(amatr(is,j),ndigit)
         enddo
         write(sform,'(a,i0,a)')'(1x,a,t15,',ns,'a)'
         write(kpr,sform)trim(stringpar(j)),stringv(1:ns)
      enddo
   enddo
!
   end subroutine stampa_matrice

!-------------------------------------------------------------------------------

   subroutine stampa_vettore(amatr,stringpar,ndigit,kpr,title)
   USE progtype, only:dblep
   USE strutil
!
   real(dblep), dimension(:),      intent(in) :: amatr    ! matrice da stampare
   character(len=*), dimension(:),       intent(in) :: stringpar! stringa associata alla riga
   integer, intent(in)                               :: ndigit    ! numero di cifre da stampare per numero
   integer,                          intent(in)      :: kpr       ! unita' di stampa
   character(len=*), intent(in), optional            :: title     ! eventuale titolo 
   integer :: numcol                                        ! larghezza massima della pagina di stampa
   integer :: ndim                                           ! dimensione della matrice
   integer :: num_elem                                      ! numero di elementi da stampare per riga
   integer :: i,is,iniz,ifin
   character(len=ndigit+7), dimension(:),allocatable :: stringv,stringp
   integer                                           :: ns
   character(len=20)                                 :: sform
   integer                                           :: wfield
!
   ndim = size(amatr,1)
   numcol = 70
   wfield  = ndigit + 7  !+3 perche:1 spazio di separazione+virgola+punto decimale+4 spazi per exp
!
!  Calcola il numero di elementi da stampare per blocco
   num_elem  = numcol / wfield
   allocate(stringv(num_elem),stringp(ndim))
!
!  centra tutta le stringhe nel campo wfield
   do i=1,ndim
      stringp(i) = centra_str(stringpar(i),wfield)
   enddo
!
!  Stampa
   if (present(title)) then
       write(6,'(/a)')centra_str(title,numcol+16) !perche 1x,t15
   endif
   i = 0
   do 
      i = i + 1
      iniz = 1 + (i - 1) * num_elem
      if (iniz > ndim) exit
      ifin = min (num_elem * i,ndim)
      ns = ifin-iniz+1
      write(sform,'(a,i0,a)')'(1x,t15,',ns,'a)'
      write(kpr,sform) stringp(iniz:ifin)(1:wfield)
      !write(kpr,'(9x,100(a8,2x))') stringpar(iniz:ifin)
      ns = 0
      do is=iniz,ifin
         ns = ns + 1
         stringv(ns) = string_sig(amatr(is),ndigit)
      enddo
      !write(kpr,'(8x,100e10.3)') amatr(iniz:ifin)
         !write(sform,'(a,i0,a)')'(1x,t15,',ns,'a)'
      write(kpr,sform)stringv(1:ns)
   enddo
!
   end subroutine stampa_vettore

END MODULE PRNUTIL

 MODULE STAT

 implicit none

 CONTAINS

   subroutine avevar(data,ave,var)
!
!  Calcola media e varianza
!
   real, dimension(:), intent(in) :: data
   real, intent(out)              :: ave,var
   integer                        :: n
   real, dimension(size(data))    :: s
!
   n=size(data)
   ave=sum(data(:))/n
   s(:)=data(:)-ave
   var=dot_product(s,s)
   var=(var-sum(s)**2/n)/(n-1)
!
   end subroutine avevar

  !--------------------------------------------------------------------

   subroutine avestd(data,ans,avef,nd)
!
!  Calcola la media eliminando i dati con |data(i) - ave| > ans*std
!
   real, dimension(:), intent(in) :: data
   real, intent(in)               :: ans
   real, intent(out)              :: avef
   integer, intent(out), optional :: nd
   real                           :: avei,var,stdn
   integer                        :: i,ni,nf
!
   call avevar(data,avei,var)
   stdn = ans*sqrt(var)
!
   ni = size(data)
   nf = 0
   avef = 0
   do i=1,ni
      if(abs(data(i) - avei) <= stdn) then
         avef = avef + data(i)
         nf = nf + 1
      endif
   enddo
   if (nf == 0) then  ! nf=0 se i dati sono tutti uguali e var = 0
       avef = avei
   else
       avef = avef / nf
   endif
   if (present(nd)) then
       nd = ni - nf
   endif
!
   end subroutine avestd

  !------------------------------------------------------------------------------------------

   real function Rvalue(fden,fval)
   real, dimension(:), intent(in) :: fden  ! grandezza a denominatore
   real, dimension(:), intent(in) :: fval
   real                           :: scaleR
   real                           :: sumFo
!
   sumFo = sum(fden)
   scaleR = sumFo / sum(fval)
   Rvalue = (sum(abs(fden-scaleR*fval)) / sumFo)*100.
!
   end function Rvalue

 END MODULE STAT

!--------------------------------------------------------------------

 MODULE rand_mod
 
 implicit none

 private :: randvalue_real,randvalue_int, randvaluev_real, randvaluev_int
 interface randvalue
   module procedure randvalue_real,randvalue_int, randvaluev_real, randvaluev_int
 end interface randvalue

 REAL, PRIVATE      :: half = 0.5

 CONTAINS
 
   Subroutine Init_Ran(Seed,jump)
   integer, intent (in)               :: seed
   integer, intent (in), optional     :: jump
   integer, dimension(:), allocatable :: seedv
   integer                            :: isize
   real                               :: xrand
   integer                            :: i
   call random_seed(size=isize)  ! isize is compiler dipendent
   allocate(seedv(isize))
   if (Seed == 0) then      ! seed selected by the system clock
       seedv(:) = int(seedgen(pid=1))
   else                     ! seed selected by the user
       seedv(:) = Seed
   endif
   call random_seed(put=seedv)   
   if (present(jump)) then
       do i=1,jump
          call random_number(xrand)
       enddo
   endif
   End Subroutine Init_Ran                                      

!--------------------------------------------------------------------

   function seedgen(pid)
   use iso_fortran_env
   implicit none
!! a Windows 32 bit NON PIACE
!!mr   integer(kind=int64) :: seedgen
   integer(kind=8) :: seedgen
   integer, intent(IN) :: pid
   integer :: s

   call system_clock(s)
   seedgen = abs( mod((s*181)*((pid-83)*359), 104729) ) 
   end function seedgen

!--------------------------------------------------------------------

   real function randvalue_real(minv,maxv) result(value)
   real, intent(in) :: minv,maxv
   real             :: xrand
!
   call random_number(xrand)
   value = xrand*(maxv-minv)+minv
!
   end function randvalue_real

!--------------------------------------------------------------------

   integer function randvalue_int(minv,maxv) result(value)
   integer, intent(in) :: minv,maxv
   real                :: xrand
!
   call random_number(xrand)
   value = int(xrand*(maxv-minv)+minv + 0.5)
!
   end function randvalue_int

!--------------------------------------------------------------------

   function randvaluev_real(n,minv,maxv)  result(xrand)
   integer, intent(in) :: n
   real, dimension(n)  :: xrand
   real, intent(in)    :: minv,maxv
!
   call random_number(xrand)
   xrand(:) = xrand(:)*(maxv-minv)+minv
!
   end function randvaluev_real

!--------------------------------------------------------------------

   function randvaluev_int(n,minv,maxv,unique)  result(xrand)
   integer, intent(in)           :: n
   integer, intent(in)           :: minv,maxv
   logical, intent(in), optional :: unique  
   integer, dimension(n)         :: xrand
   real, dimension(n)            :: xrandr
   integer                       :: xri, nrand, nmax
!
   if (present(unique)) then
       if (unique) then

           xrand(1) = randvalue(minv,maxv)
           if (n == 1) return
           nmax = min(n,maxv - minv + 1)  
           nrand = 1
           do 
              xri = randvalue(minv,maxv)
              if (any(xrand(:nrand) == xri)) cycle
              nrand = nrand + 1
              xrand(nrand) = xri
              if (nrand == nmax) exit
           enddo

           return
       endif
   endif
!
   call random_number(xrandr)
   xrand(:) = nint(xrandr(:)*(maxv-minv)+minv)
!
   end function randvaluev_int

!--------------------------------------------------------------------

   FUNCTION random_normal() RESULT(fn_val)

! Adapted from the following Fortran 77 code
!      ALGORITHM 712, COLLECTED ALGORITHMS FROM ACM.
!      THIS WORK PUBLISHED IN TRANSACTIONS ON MATHEMATICAL SOFTWARE,
!      VOL. 18, NO. 4, DECEMBER, 1992, PP. 434-435.

!  The function random_normal() returns a normally distributed pseudo-random
!  number with zero mean and unit variance.

!  The algorithm uses the ratio of uniforms method of A.J. Kinderman
!  and J.F. Monahan augmented with quadratic bounding curves.

   REAL :: fn_val

!     Local variables
   REAL     :: s = 0.449871, t = -0.386595, a = 0.19600, b = 0.25472,           &
            r1 = 0.27597, r2 = 0.27846, u, v, x, y, q

!     Generate P = (u,v) uniform in rectangle enclosing acceptance region

   DO
     CALL RANDOM_NUMBER(u)
     CALL RANDOM_NUMBER(v)
     v = 1.7156 * (v - half)

!     Evaluate the quadratic form
     x = u - s
     y = ABS(v) - t
     q = x**2 + y*(a*y - b*x)

!     Accept P if inside inner ellipse
     IF (q < r1) EXIT
!     Reject P if outside outer ellipse
     IF (q > r2) CYCLE
!     Reject P if outside acceptance region
     IF (v**2 < -4.0*LOG(u)*u**2) EXIT
   END DO

!     Return ratio of P's coordinates as the normal deviate
   fn_val = v/u
   RETURN

   END FUNCTION random_normal

 END MODULE rand_mod
