#ifdef HAVE_CONFIG_H
#  include <config.h>
#else
#  include <config_any.h>
#endif

#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <cerrno>
#include <cmath>
#include <cassert>

#include <string>
#include <sstream>
#include <deque>
#include <numeric>
#include <functional>
#include <exception>
#include <fstream>
#include <iomanip>
#include <locale>

#include <utils.h>
#include "ilm_defs.h"
#include "gtkproject.h"

#ifdef VERY_LOW_GVERSION
typedef gint GPid;
#endif
                                                                                                 
#include "util.h"
#include "g_ext.h"
#include "ilm_globals.h"
#include "application.h"

#include <cmtzlib.h>
#include <csymlib.h>
typedef std::vector<const CMtz::MTZCOL* > VMTZCOL;
typedef VMTZCOL::iterator VMTZCOL_it;
static VMTZCOL MtzValues, MtzEsds, MtzAllCols, MtzAllTypes;
static VMTZCOL MtzSelected;
static VMTZCOL DeaMtzValues, DeaMtzEsds, DeaMtzAllCols, DeaMtzAllTypes;
static VMTZCOL DeaMtzSelected;


double SolventFrac = 0;

namespace Ilm
{
  //! \class GtkProject
  //! The interaction between gtk+ and the project

  int (GtkProject::*GtkProject::ValidatePage[MAX_PAGES])(GtkProject::EValidate ) = 
  {
    &GtkProject::ValidateMTZ,
    &GtkProject::ValidateCont,
    &GtkProject::ValidateRun
  };

  class eq : public std::unary_function< const CMtz::MTZCOL * , bool > 
  {
  private:
    const char *label;
  public:
    eq(const char *nm) {label=nm;}
    result_type operator( ) ( argument_type col )
    {
        return ( result_type ) ( std::strcmp(col->label, label)==0 );
    }
  };  
   
  VMTZCOL_it get_mtz_col(VMTZCOL &v, const char *nm)
  {
    return 
    std::find_if(v.begin(), v.end(), eq(nm) );
  }

  //! Creates project window 
  void GtkProject::CreateWidget(const std::string &fn)
  { 
    //PangoFontDescription *font = DefaultFontMono();
    
    bSyncRunning = true; // disable calls to DataFromUI by callbacks
    poMainWnd = create_IlmProjectWnd ();

    //GtkWidget *w = LookupWidget("IlmInfoView");
    //widget_modify_font(w, font);
    //pango_font_description_free(font);

    TheApp.UpdateRecentMenu();
    // general
    std::stringstream strm; strm << gtk_window_get_title(MainWindow());
    strm << ": " << fn.c_str();

    switch (eType)
    {
    case MAD:
          strm << " (MAD/SAD)";
          gtk_widget_hide(LookupWidget("IlmMtzSpgFind"));
          gtk_widget_hide(LookupWidget("IlmContMirFrm"));
          gtk_widget_hide(LookupWidget("IlmContLigFrm"));
          gtk_widget_hide(LookupWidget("IlmTasksProFullBox"));
          gtk_widget_hide(LookupWidget("IlmContDerivsTbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv1Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDerivLbl"));
          gtk_widget_hide(LookupWidget("IlmContDerivSpin"));
          gtk_widget_hide(LookupWidget("IlmContDeriv2Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv3Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv4Lbl"));
          break;
    case MOLREP:
          strm << " (ReMo)";
          gtk_widget_hide(LookupWidget("IlmContMadFrm"));
          gtk_widget_hide(LookupWidget("IlmContMirFrm"));
          gtk_widget_hide(LookupWidget("IlmContNativeCheck"));
          gtk_widget_hide(LookupWidget("IlmContWavesLbl"));
          gtk_widget_hide(LookupWidget("IlmContWavesSpin"));
          gtk_widget_hide(LookupWidget("IlmContDerivsTbl"));
          gtk_widget_hide(LookupWidget("IlmContDerivLbl"));
          gtk_widget_hide(LookupWidget("IlmContDerivSpin"));
          gtk_widget_hide(LookupWidget("IlmContDeriv1Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv2Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv3Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv4Lbl"));
          break;
    case ABI:
          strm << " (Ab-initio)";
          gtk_widget_show(LookupWidget("IlmContSourceFrm"));
          gtk_widget_hide(LookupWidget("IlmContMadFrm"));
          gtk_widget_hide(LookupWidget("IlmContMirFrm"));
          gtk_widget_hide(LookupWidget("hboxNative"));
          gtk_widget_hide(LookupWidget("IlmContDeriv1Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv2Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv3Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv4Lbl"));
          break;
    case EDM:
          strm << " (EDM)";
          gtk_widget_hide(LookupWidget("IlmMtzSpgFind"));
          gtk_widget_hide(LookupWidget("IlmContMadFrm"));
          gtk_widget_hide(LookupWidget("IlmContMirFrm"));
          gtk_widget_hide(LookupWidget("hboxNative"));
          gtk_widget_hide(LookupWidget("IlmContDeriv1Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv2Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv3Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv4Lbl"));
          gtk_widget_hide(LookupWidget("IlmTasksProFullBox"));
          gtk_widget_hide(LookupWidget("IlmTasksMad1FullBox"));
          gtk_widget_hide(LookupWidget("IlmTasksMad2Box"));
          gtk_widget_hide(LookupWidget("IlmTasksMad2Check"));
          gtk_widget_hide(LookupWidget("IlmStrategyFrame"));
          break;
    case MIR:
    case MRS:
          if(eType == MIR)
          {
             strm << " (MIR/SIR)";
             gtk_widget_hide(LookupWidget("IlmContDerivsTbl"));
             gtk_widget_hide(LookupWidget("IlmContMadFrm"));
             button_set_label(GTK_BUTTON(LookupWidget("IlmTasksMad2Check")), "Task2: MIR phasing");
          }
          else
          {
             strm << " (MIRAS/SIRAS)";
             gtk_widget_hide(LookupWidget("IlmMtzSpgFind"));
             button_set_label(GTK_BUTTON(LookupWidget("IlmTasksMad2Check")), "Task2: MIRAS phasing");
             gtk_widget_hide(LookupWidget("IlmContMirFrm"));
          }
          gtk_widget_hide(LookupWidget("IlmMtzSpgFind"));
          gtk_widget_hide(LookupWidget("IlmContLigFrm"));
          gtk_widget_hide(LookupWidget("IlmContWavesLbl"));
          gtk_widget_hide(LookupWidget("IlmContWavesSpin"));
          gtk_widget_hide(LookupWidget("IlmContNativeCheck"));
          button_set_label(GTK_BUTTON(LookupWidget("IlmTasksMad1Check")), "Task1: Find heavy atom substructures");
          gtk_label_set_text(GTK_LABEL(LookupWidget("IlmTasksMad1SelectWavesLbl")), "Select derivatives:");
          for( int ii=1; ii<=MAX_DERIVATIVES; ii++ ) 
          {
                std::stringstream nm, val;
                nm << "IlmTasksMad1Wave" << ii << "Check";
                val << "Deriv." << ii;
                button_set_label(GTK_BUTTON(LookupWidget(nm.str().c_str())), val.str().c_str());
          }
          gtk_label_set_text(GTK_LABEL(LookupWidget("IlmTasksMad2SelectWavesLbl")), "Select derivatives:");
          for( int ii=1; ii<=MAX_DERIVATIVES; ii++ ) 
          {
                std::stringstream nm, val;
                nm << "IlmTasksMad2Wave" << ii << "Check";
                val << "Deriv." << ii;
                button_set_label(GTK_BUTTON(LookupWidget(nm.str().c_str())), val.str().c_str());
          }
          gtk_label_set_text(GTK_LABEL(LookupWidget("IlmTasksMad2FileLbl")), "Heavy atoms positions file: (PDB)");
          gtk_widget_hide(LookupWidget("IlmTasksMad2LsqLabel"));
          gtk_widget_hide(LookupWidget("IlmTasksMad2LsqF1Check"));
          gtk_widget_hide(LookupWidget("IlmTasksMad2LsqF2Check"));
          break;
    case DEA:
          strm << " (DEA)";
          gtk_widget_hide(LookupWidget("IlmMtzSpgFind"));
          gtk_widget_hide(LookupWidget("IlmContSourceFrm"));
          gtk_widget_hide(LookupWidget("IlmContLigFrm"));
          gtk_widget_hide(LookupWidget("IlmContMadFrm"));
          gtk_widget_hide(LookupWidget("IlmContMirFrm"));
          gtk_widget_hide(LookupWidget("hboxNative"));
          gtk_widget_hide(LookupWidget("IlmContDeriv1Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv2Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv3Lbl"));
          gtk_widget_hide(LookupWidget("IlmContDeriv4Lbl"));
          break;

    }
    const GtkAllocation  *alloc =  Ilm::TheApp.GetAllocation(); 
    int wW =  alloc->width;
    int hW =  alloc->height;
    int xP = alloc->x;
    int yP = alloc->y;
    GdkRectangle rect;
#if (GTK_MAJOR_VERSION > 2) && (GTK_MINOR_VERSION >= 22)
    GdkMonitor *mon = gdk_display_get_primary_monitor(gdk_display_get_default());
    gdk_monitor_get_geometry(mon, &rect);
#else
    GdkScreen *sc = gdk_screen_get_default();
    gdk_screen_get_monitor_geometry(sc, 0, &rect);
#endif
    int ww = rect.width;
    int hh = rect.height;

    if(hh < 1000) 
    {
       wW = ww-100;
       hW = hh-100;
    }
    gtk_window_set_title(MainWindow(), toUTF8(strm.str().c_str()));
    gtk_window_set_default_size (MainWindow(), wW, hW); 
    gtk_window_move (MainWindow(), xP, yP); 
    SetLastPage(-1);
    SetFirstErr(10);
    DisablePages(TRUE);

    gtk_widget_show (MainWidget()); 
    bSyncRunning = false; 
    FillInterface();
  }

  int GtkProject::ValidateMTZ(GtkProject::EValidate e)
  {
    const gchar *err_msg = NULL;
    int gotinfo = 0;
    
    dir_set_current(szWorkDir);
    DataFromUI(); 

    if( gext::str_is_white(szJobTitle) ) 
    {
        err_msg =  "Provide job title";
        goto lblFailure;
    }

    if( gext::str_is_white(szMtzFile) ) 
        err_msg =  "Provide MTZ filename";
    else  if( !g_file_test(szMtzFile, G_FILE_TEST_EXISTS) ) 
    {
          err_msg =  " ";
          if(e!=eSilent)
             err_msg = "MTZ file does not exist";
    }
    else
    {
      gotinfo = MtzGetInfo(NULL);
      if(gotinfo != 1)
      {
 
          err_msg =  " ";
          if(e!=eSilent)
             err_msg =  "Error reading MTZ file";
          goto lblFailure;
      }
      if(MtzSelected.size() < 2 )
      {
          err_msg =  "You have to select columns";
          goto lblFailure;
      }
      if(eType == EDM)                     // aggiunto controllo 021007
      {
         int countP = 0, countW = 0;
         for( int i=0; i<(int)MtzSelected.size(); i++ ) 
         {
              char typ = MtzSelected[i]->type[0];
              if(typ == 'P')
                 countP++;
              if(typ == 'W')
                 countW++;
         }
         if((countP != 1) || (countW != 1))
           err_msg = "You have to select one phase-weight pair!";
      }
      else                                            // 280409
      { 
         int countE = 0, countF = 0;
         for(unsigned int i=0; i< MtzSelected.size(); i++)
         {
             char typ = MtzSelected[i]->type[0];
             if( typ=='Q' || typ=='L' )
                 countE++;
             if( typ=='F' || typ=='G' || typ=='J')
                 countF++;
         }
         if((eType == ABI) || (eType == MOLREP) || (eType == DEA))
         {
             if((countF != 1) || (countE != 1))
                 err_msg = "You have to select one F-sigF pair!";
         }
         else                      // MIR, MAD, MIRAS
         {
             if( countF != countE )
                 err_msg = "You have to select F-sigF pairs!";
         }
      }
    }
lblFailure:
    if( err_msg!=NULL ) 
    {
      if( !gext::str_is_white(err_msg) && e!=eSilent  && gotinfo!=-1 ) 
        gtkext::dlg_errmsg(MainWindow(), err_msg);
    }
    else 
      FillInterface(); 

    return (err_msg == NULL);
  }

  int GtkProject::ValidateCont(GtkProject::EValidate e)
  {
    int i, j;
    const gchar *err_msg = NULL;
    bool anom_present = false;
    bool one_anom_present = false;
    nElements = 0;

    DataFromUI(); // ResiduesFromUI();
    if((iCont == 2) && dContent < 0.0)
    {
      err_msg = "Sequence file is incorrect or does not exist";
      e = eLoud;
      goto lblEnd;
    }
    else if( dContent<=0.0 ) {
      err_msg = "The molecular weight or number of residues must be above zero";
      goto lblEnd;
    }

    for( i=0; i<nElements; i++ ) {
      if( !is_valid_symbol(g_strstrip(szElement[i])) ) {
        err_msg = g_strdup_printf("Wrong element symbol %s", 
                       g_strstrip(szElement[i]));
        goto lblEnd;
      }
      else {
        if( nElementAmount[i]<=0 ) {
          err_msg = "Element amount has to be a positive number";
          goto lblEnd;
        }
        anom_present = false;              // 021006
        if( eType==MAD ) {
          for( j=0; j<nWaves; j++ ) {
            if( fabs(this->dElementF1F2[i][j][0])>0.0001 )
              anom_present = true;
            if( fabs(this->dElementF1F2[i][j][1])>0.0001 )
              anom_present = true;
          }
          one_anom_present |= anom_present;
          if(anom_present)
          {
             for( j=0; j<nWaves; j++ ) {
               if((fabs(this->dElementF1F2[i][j][0])<0.0001 )||(fabs(this->dElementF1F2[i][j][1])<0.0001))
               {
                   err_msg = "f' and/or f'' are not present";
                   goto lblEnd;
               }
             }
          }
        }
      }
    }
    if( !one_anom_present && eType==MAD )
      err_msg = "f' and f'' are not present";
      
    if( eType==MIR )
    {
      for( i=0; i<nDeriv; i++ )
      {
        if( nMirElements[i]==0 ) {
          err_msg = "At least one heavy element has to be present for every derivative";
          goto lblEnd;
        }
        for( j=0; j<nMirElements[i]; j++ )
        {
          if( !is_valid_symbol(g_strstrip(szMirElement[i][j])) ) 
          {
            err_msg = g_strdup_printf("Wrong element symbol %s", 
                       g_strstrip(szMirElement[i][j]));
            goto lblEnd;
          } 
          else 
          {
            if( nMirElementAmount[i][j]<=0 ) 
            {
               err_msg = "Element amount has to be a positive number";
               goto lblEnd;
            }
          }
          bMad2Waves[i] = true;
          bMad1Waves[i] = !bMad1Auto;
        }       
      }
    }
    if( eType==MRS )
    {
      if(nMirElements[0] < nDeriv)
      {
          err_msg = "At least one heavy element has to be present for every derivative";
          goto lblEnd;
      }
      for( i=0; i<nDeriv; i++ )
      {
          if( !is_valid_symbol(g_strstrip(szMirElement[i][0])) ) 
          {
            err_msg = g_strdup_printf("Wrong element symbol %s", 
                       g_strstrip(szMirElement[i][0]));
            goto lblEnd;
          } 
          else 
          {
            if( nMirElementAmount[i][0]<=0 ) 
            {
               err_msg = "Element amount has to be a positive number";
               goto lblEnd;
            }
          }
          bMad2Waves[i] = true;
          bMad1Waves[i] = !bMad1Auto;
          if(( fabs(this->dElementF1F2[i][0][0])>0.0001 ) && ( fabs(this->dElementF1F2[i][0][1])>0.0001 ))
              anom_present = true;
          else if(( fabs(this->dElementF1F2[i][0][0])<=0.0001 ) && ( fabs(this->dElementF1F2[i][0][1])<=0.0001 ))
              ;
          else
          {
              err_msg = "f' or f'' is not present";
              goto lblEnd;
          }
      }
    if( !anom_present  )
      err_msg = "f' and f'' are not present";
    }

lblEnd:
    if( err_msg!=NULL ) {
      if( e!=eSilent )
        gtkext::dlg_errmsg(MainWindow(), err_msg);
    }
    else {
      FillInterface(); // ?
    }

    return err_msg == NULL;
  }

  int GtkProject::ValidateRun(GtkProject::EValidate e)
  {
    const gchar *err_msg = NULL;
    std::stringstream str;
    SetReady(false);
    DataFromUI();

    if( eType==MOLREP )
    {
      for(int i=0; i<nMRModels; i++)
      {
          if( gext::str_is_white(szMRFile[i]) ) {
            err_msg = "Empty filename entry";
            goto lblEnd;
          }
          if( !g_file_test(szMRFile[i], G_FILE_TEST_EXISTS) ) {
            err_msg = "Model file does not exist";
            e = eLoud;
            goto lblEnd;
          }
          CheckPDBFile(szMRFile[i], i, e);
      }
    }
    else if (eType==ABI)
    {
      // inserire controlli per il caso Ab-initio
         if(bRecover)
         {
            std::string strcol;
            if( !g_file_test(strRecoverMTZ.c_str(), G_FILE_TEST_EXISTS) ) 
            {
                err_msg = "MTZ file does not exist";
                e = eLoud;
                goto lblEnd;
            }
            if( GetMtzCols(strRecoverMTZ, strcol))
            {
                err_msg = "Error reading MTZ file";
                e = eLoud;
                goto lblEnd;
            }
            if( gext::str_is_white(strRecoverData.c_str()))
            {
                err_msg = "You should select a trial";
                e = eLoud;
                goto lblEnd;
            }
         }
    }
    else if (eType==DEA)
    {
       if(nBuildPgm == 0)
       {
              err_msg =  "Select an Automatic Model Building Program";
              goto lblEnd;
       }
       if(bModel)
       {
          if( gext::str_is_white(szMRFile[0]) )
          {
              err_msg =  "Provide model file";
              goto lblEnd;
          }
          else  if( !g_file_test(szMRFile[0], G_FILE_TEST_EXISTS) )
          {
              err_msg = "Model file does not exist";
              e = eLoud;
              goto lblEnd;
          }
          CheckPDBFile(szMRFile[0], 0, e);
       }
       else
       {
          if( gext::str_is_white(szMtzFilePh) )
          {
              err_msg =  "Provide MTZ filename";
              goto lblEnd;
          }
          else  if( !g_file_test(toUTF8(szMtzFilePh), G_FILE_TEST_EXISTS) )
          {
              err_msg =  " ";
              if(e!=eSilent)
                 err_msg = "MTZ file does not exist";
              goto lblEnd;
          }
          else
          {
              if(DeaMtzGetInfo(NULL) != 1)
              {
                 err_msg =  " ";
                 if(e!=eSilent)
                    err_msg =  "Error reading MTZ file";
                 goto lblEnd;
              }
              if(DeaMtzSelected.size() < 2 )
              {
                  err_msg =  "You have to select columns";
                  goto lblEnd;
              }
              int countP = 0, countW = 0;
              for( int i=0; i<(int)DeaMtzSelected.size(); i++ )
              {
                 char typ = DeaMtzSelected[i]->type[0];
                 if(typ == 'P')
                    countP++;
                 if(typ == 'W')
                    countW++;
              }
              if((countP != 1) || (countW != 1))
              {
                 err_msg = "You have to select one phase-weight pair!";
                 goto lblEnd;
              }
          }
          if( !gext::str_is_white(szMRFile[1]) ) 
          {
             if( !g_file_test(szMRFile[1], G_FILE_TEST_EXISTS) )
             {
                 err_msg = "Heavy atoms file does not exist";
                 e = eLoud;
                 goto lblEnd;
             }
          }
          if( !gext::str_is_white(szMRFile[1]))
              CheckPDBFile(szMRFile[1], 1, e);
       }
    }
    else if (eType==EDM)
    {
      // inserire controlli per il caso EDM
    }
    else
    {
      if( !(bMad1 || bMad2 || bPro)) {
        err_msg = "Select at least one task";
        goto lblEnd;
      }
      if( bMad2 && !bMad1 && gext::str_is_white(szMad2File) && BoolFromUI("IlmTasksFilePositionRadio")) {
        err_msg = "Empty filename entry";
        goto lblEnd;
      }

      if( bMad2 && !bMad1 && BoolFromUI("IlmTasksBestTrialsRadio") && !g_file_test(GetFilename(Ilm::MAD1FRAC_EXT).c_str(), G_FILE_TEST_EXISTS) ) {
        err_msg = "Substructure file does not exist.\nPlease run Task1 before.";
        goto lblEnd;
      }
      if( bMad2 )
      {
        unsigned short sum = 0;
        for( unsigned i=0; i<MAX_WAVES; i++ ) sum += bMad2Waves[i];
        if(!sum ) {
          err_msg = "no waves were selected";
          goto lblEnd;
        }
      }
      if( bMad2 && !bMad1 )
      {
        if(gext::str_is_white(szMad2File))
           CheckAnomFile(str, GetFilename(Ilm::MAD1FRAC_EXT).c_str());
        else
           CheckAnomFilePDB(str, szMad2File);
        SetInfoText(str.str().c_str());
      }
      if( bMad2 && iMad2SetMin > iMad2SetMax  && iMad2SetMax!=-1 ) {
        err_msg = "Set Range wrong";
      }
      if( bMad2 && bMad2Lsq && !(bMad2LsqAdp || bMad2LsqPos || bMad2LsqSof || bMad2LsqF1 || bMad2LsqF2) )
        err_msg = "Select something to refine for LSQ";
    }
lblEnd:
    if( err_msg!=NULL ) { // neudacha 
      if( !gext::str_is_white(err_msg) && e!=eSilent )
        gtkext::dlg_errmsg(MainWindow(), err_msg);
    }
    else { // uspeh 
      FillInfo();
      SetReady(true);
      if( e==eActive ) {
        std::stringstream str;
        gint response=GTK_RESPONSE_REJECT;
        GtkWidget *dlg = create_IlmSubmitDlg();
        GtkWidget *w = lookup_widget(dlg, "IlmSubmitInfoLbl");
        gtk_window_set_transient_for(GTK_WINDOW(dlg), this->MainWindow());
        while( response==GTK_RESPONSE_REJECT )
        {
          str.str("");
          str << "The project is ready to go. \n \nThe following files will be overwritten if you proceed:\n";
          if(eType == DEA)
          {
             str.str("");
             str << "The project is ready to go. \n \nThe following files will be removed:\n";
            switch(nBuildPgm)
            {
            case 1:
               str << "dea_arp*.log\ndea_arp_dedm_edm*.out\n"
                   << "dea_arp_dedm_edm*.mtz\n\n";
               str << "The following files will be created (or overwritten):\n";
               str << "dea.sir\n"
                   << DEA_SH << "\n"  << CADMD_SH << "\n" ;
               if(bModel)
                   str << ARPMD_SH << "\n" ;
               else
                   str << CADPH_SH << "\n";
               str <<  ARPPH_SH << "\n" ;
               str << "refmac_arp" << SHELL_EXT << "\n";

               break;
            case 2:
               str << "dea_phenix*.log\ndea_phenix_dedm_edm*.out\n"
                   << "dea_phenix_dedm_edm*.mtz\n\n";
               str << "The following files will be created (or overwritten):\n";
               str << "dea.sir\n"
                   << DEA_SH << "\n"  << CADMD_SH << "\n" ;
               if(bModel)
                   str << PHEMD_SH << "\n" ;
               else
                   str << CADPH_SH << "\n";
               str <<  PHEPH_SH << "\n" ;
               str << "refmac_phenix" << SHELL_EXT << "\n";
               break;
            case 3:
               str << "dea_buccaneer*.log\ndea_buccaneer_dedm_edm*.out\n"
                   << "dea_buccaneer_dedm_edm*.mtz\n\n";
               str << "The following files will be created (or overwritten):\n";
               str << "dea.sir\n"
                   << DEA_SH << "\n"  << CADMD_SH << "\n" ;
               if(bModel)
                   str << BUCMD_SH << "\n" ;
               else
                   str << CADPH_SH << "\n";
               str <<  BUCPH_SH << "\n" ;
               str << "refmac_buccaneer" << SHELL_EXT << "\n";
               break;
            case 4:
               str << "dea_nautilus*.log\ndea_nautilus_dedm_edm*.out\n"
                   << "dea_nautilus_dedm_edm*.mtz\n\n";
               str << "The following files will be created (or overwritten):\n";
               str << "dea.sir\n"
                   << DEA_SH << "\n"  << CADMD_SH << "\n" ;
               if(bModel)
                   str << NAUMD_SH << "\n" ;
               else
                   str << CADPH_SH << "\n";
               str <<  NAUPH_SH << "\n" ;
               str << "refmac_nautilus" << SHELL_EXT << "\n";
               break;
            }
            str << "To run the DEA project, please check in the '" << szWorkDir 
                << "'\ndirectory for the script dea_run" << SHELL_EXT << ".\n"
                << "To change the " << ambp[nBuildPgm] << " condition, please check the script";
            if(bModel)
            {
               str << "s";
               if(nBuildPgm == 1)
                  str << " " << ARPMD_SH << " and";
               else if(nBuildPgm == 2)
                  str << " " << PHEMD_SH << " and";
               else if(nBuildPgm == 3)
                  str << " " << BUCMD_SH << " and";
               else if(nBuildPgm == 4)
                  str << " " << NAUMD_SH << " and";
            }
            if(nBuildPgm == 1)
               str << " " << ARPPH_SH << ".\n";
            else if(nBuildPgm == 2)
               str << " " << PHEPH_SH << ".\n";
            else if(nBuildPgm == 3)
               str << " " << BUCPH_SH << ".\n";
            else if(nBuildPgm == 4)
               str << " " << NAUPH_SH << ".\n";
            str << "\nPress 'OK' to  to continue \n";
          }
          if((eType == EDM) || (eType == ABI) || (eType == MOLREP))
          {
             str << GetFilename(PROJECT_EXT) << "\n"
             << GetFilename2(OUT_EXT, eType-1) << "\n"
             << GetFilename2(INP_EXT, eType-1) << "\n"
             << GetFilename2(BIN_EXT, eType-1) << "\n";
          }
          if((eType == MAD) || (eType == MIR) || (eType == MRS))
          {
             str << GetFilename(PROJECT_EXT) << "\n";
             if(bPro)
             {
                str << GetFilename2(OUT_EXT, 0) << "\n"
                << GetFilename2(INP_EXT, 0) << "\n"
                << GetFilename2(BIN_EXT, 0) << "\n";
             }
             if(bMad1)
             {
                   str << GetFilename2(OUT_EXT, 1) << "\n"
                    << GetFilename2(INP_EXT, 1) << "\n"
                    << GetFilename2(BIN_EXT, 1) << "\n";
             }
             if(bMad2)
             {
                str << GetFilename2(OUT_EXT, 2) << "\n"
                << GetFilename2(INP_EXT, 2) << "\n"
                << GetFilename2(BIN_EXT, 2) << "\n";
             }
          }
          if(eType != DEA)
             str << "\n \nPress 'Ok' to submit the job." << std::endl;
          set_label_text(w, str.str().c_str());
          char *bsnm = g_new0(char, MaxLenFilename);
          while( (response=gtk_dialog_run(GTK_DIALOG(dlg)))==GTK_RESPONSE_REJECT )
          { // change the name of the project
            std::string fn = gtkext::dlg_select_file(dlg, GetFilename(PROJECT_EXT), gtkext::eSELECT_TO_WRITE, "*.ilm");
            if( gext::str_is_white(fn.c_str()) )
              continue; 
            char *bsnms = g_path_get_basename(fn.c_str());
            CopiaStringa(bsnm, MaxLenFilename, str_cut(bsnms, ".ilm"));
            CopiaStringa(szWorkDir, PATH_LEN, g_path_get_dirname(fn.c_str()));
            if( !gext::str_is_white(bsnm) )
              break;
          }
          if( response==GTK_RESPONSE_REJECT )
          {
            assert(bsnm!=0);
            CopiaStringan(szStructName, sizeof(szStructName), bsnm, sizeof(szStructName)-1);
            szStructName[sizeof(szStructName)-1]= '\0';
            char *p = strrchr(szStructName, '.');
            if( p!=NULL ) *p = '\0';
            FillInterface();
          }
        }
        gtk_widget_destroy(dlg);
        if( response==GTK_RESPONSE_OK )
        {
          SaveAllFiles();
#ifndef NDEBUG
	  // the following is only for development version
          dlg = create_IlmSirEditDlg();
          //gtk_widget_show(dlg); - uzhasnaya funkciya dlya dialogov delayet ih topmost po kraynej mere v vindah
          gtk_window_set_transient_for(GTK_WINDOW(dlg), this->MainWindow());
          response = gtk_dialog_run(GTK_DIALOG(dlg));
          gtk_widget_destroy(dlg);
          if( response==GTK_RESPONSE_OK )
          {
#endif
           while(gtk_events_pending())
               gtk_main_iteration();

            SetReady( false );
            Go();
#ifndef NDEBUG
          }
#endif
        }
      }
    }
    return err_msg == NULL;
  }

  bool GtkProject::OnExit()
  {
    if( IsSubmitted() )
    {
      gtkext::dlg_errmsg(MainWindow(), "Terminate submitted job");
      return true; // do not quit
    }
    std::string filn = g_path_get_basename(GetFilename(PROJECT_EXT).c_str());
    std::string mesg = "Are you sure you want to quit project '" + filn + "'?";
    if( gtkext::dlg_yesno(MainWindow(), mesg.c_str()) ) {
      GtkAllocation alloc;
      gtk_widget_get_allocation(MainWidget(), &alloc);
      Ilm::TheApp.SetAllocation(alloc);
      Ilm::TheApp.WriteSettings();
      gtk_main_quit();
      return false; // quit
    }
    else {
      return true; // do not quit
    }
  }

  void GtkProject::DisablePages(bool disable)
  {
    int i, n;
    GtkWidget *page;
    gpointer f1, f2, f3;
    gboolean benable;
    GtkNotebook *ntb = GTK_NOTEBOOK( LookupWidget("IlmNotebook") );
    f1 = LookupObject("IlmMenuFileSave");
    f2 = LookupObject("IlmMenuFileSaveAs");
    f3 = LookupObject("IlmMenuFileSaveAscii");
    n = gtk_notebook_get_n_pages(ntb);
    for( i=0; i<n; i++ ) {
      page = gtk_notebook_get_nth_page(ntb, i);
      if( page==NULL ) break;
      benable = i<=LastPage()+1 ? disable : !disable;
//      gtk_widget_set_sensitive(page, benable);
      if(i == n-1)
      {
         obj_set_sensitive(f1, benable);
#ifndef NDEBUG
         obj_set_sensitive(f2, benable);
#endif
         obj_set_sensitive(f3, benable);
      }
      GtkWidget *lbl = gtk_notebook_get_tab_label(ntb, page);
      gtk_widget_set_sensitive(lbl, benable);
    }
  }

  void GtkProject::SetInfoText(const char *text) const
  {
    GtkWidget *view = LookupWidget("IlmInfoView");
    put_text(view, text, "big");
#if 0
    GtkTextView *view = GTK_TEXT_VIEW (LookupWidget("IlmInfoView"));
    GtkTextBuffer *buf =  gtk_text_view_get_buffer (view);
    GtkTextIter iter;
    GtkTextIter iter1, iter2;

    gtk_text_buffer_set_text(buf, " ", -1);
    gtk_text_buffer_set_text(buf, toUTF8(text), -1);
    gtk_widget_queue_draw( GTK_WIDGET(view) ); // invalidaterect
    gtk_text_buffer_get_end_iter(buf, &iter);
    gtk_text_view_scroll_to_iter(view, &iter, .0, TRUE, 0.9, .9);
    gtk_text_buffer_get_bounds(buf, &iter1, &iter2);
    gtk_text_buffer_apply_tag_by_name (buf, "big", &iter1, &iter2);
    // gtk_widget_show(GTK_WIDGET(view));
    //gtk_widget_queue_draw( GTK_WIDGET(view) ); // invalidaterect
#endif
  }

  int GtkProject::ApplyNthPage(int nth, EValidate e)
  {
    gint ipage;
    int result = FALSE;

    ipage = nth;
    if( ipage<0 )
      return FALSE;

    g_assert(ipage<MAX_PAGES);
    result = (this->*GtkProject::ValidatePage[ipage])(e); // validate_page[ipage](mw);

// questa parte andrebbe ricontrollata
    if( result ) {
      if( (LastPage()<ipage ) && ( FirstErr() > ipage) )
        SetLastPage(ipage);
      else
        SetFirstErr(10);
    }
    else {
      if( FirstErr() > ipage )
        SetFirstErr(ipage);     //mr 050208 - 1);
      if( LastPage()>=ipage )
        SetLastPage(ipage - 1);
    }

    DisablePages(TRUE);
    return result;
  }

  int GtkProject::CurrentPage()
  {
    return gtk_notebook_get_current_page(GTK_NOTEBOOK(LookupWidget("IlmNotebook")));
  }

  int GtkProject::ApplyCurrentPage(EValidate e)
  {
    DataFromUI();
    FillInfo();
    return ApplyNthPage(gtk_notebook_get_current_page(GTK_NOTEBOOK(LookupWidget("IlmNotebook"))), e);
  }

  void GtkProject::FillInfo(int aPage) const
  {
    gint page = aPage; //
    std::stringstream strm;
    
    if( page<0 )
      page = gtk_notebook_get_current_page(GTK_NOTEBOOK(LookupWidget("IlmNotebook")));
    strm << "";
    g_assert(page<MAX_PAGES);
    
    if( page==MTZ )
    {
      if(!gext::str_is_white(szSpgrName))
      {
        CSym::CCP4SPG *spg = CSym::ccp4spg_load_by_spgname(szSpgrName);
        if( spg!=0 ) 
        {
            strm << "Space group name:  " << spg->symbol_xHM << "\n";
            strm << "            Hall:  " << spg->symbol_Hall << "\n";
            strm << "            No:    " << spg->spg_num << "\n";
       	    strm << "            Order: " << spg->nsymop << "\n";
       	    strm << "Point group:       " << spg->point_group << "\n";
       	    strm << "Laue class:        " << spg->laue_name << "\n";
       	    strm << "Cell volume:       " << CellVolume(dCell) << "\n";
            CSym::ccp4spg_free(&spg);
        }      
        strm << "Number of Reflections: " << nNrRefl << "\n";
        strm << "Resolution Range: (" << dresMax <<" - " << dresMin << " A)" << "\n";
       }
    }

    if( page==CONT )
    {
      //std::stringstream strm;
      if(NNonHtot())
         strm << "Volume per non-hydrogen atom : " << CellVolume(dCell)/NNonHtot() << " A**3" << std::endl;
      if( dContent > 0.0 )
      {
         strm << "Estimated cell composition: ";
//old         OrganicContent(strm);
         Content(strm);
         strm << std::endl;
      }
      if(Vm(0))
      {
        std::ofstream x;
        std::stringstream fsc;
        if (!SolventFrac)
            SolventFrac = SolventFraction();
        if (MRabs(SolventFrac - SolventFraction()) > 1.e-6)
        {
            SolventFrac = SolventFraction();
//            DoubleToUI("IlmTasksEdmSolvFValue", SolventFraction() );
        }
        x << std::fixed; 
        strm.copyfmt(x);
        strm << std::setprecision (2);
        strm << "Matthews coefficient for target protein: " << Vm(0) << std::endl;
        strm << "Solvent fraction for target protein, assuming protein density equal to ";
        strm <<  DProteinVm <<  " : " << SolventFraction() << std::endl;
        fsc << std::setprecision(2);
        fsc << "(Computed value from cell content: " << SolventFraction() << ")";
//        LabelToUI("SolventFractionC", fsc.str().c_str());
      }
    }
    
    if( page==RUN )
    {
      if( bMad2 && !bMad1 )
      {
//mr270406        strm << "Information on PDB file (" << szMad2File << ") :\n";
        //CheckAnomFile(strm, szMad2File);
        if( gext::str_is_white(szMad2File))
        {
            int ntrials;
            try{ 
                Ilm::read_anom_file(GetFilename(MAD1FRAC_EXT).c_str(), strm, ntrials, eType); 
                if(ntrials > 0)
                {
                   if((eType != MIR) && (eType != MRS))
                      set_label_text(LookupWidget("IlmTasksMad2MaxLabel"),
                          g_strdup_printf("(max %d)", ntrials), MyBLUE, 1.0f);
                   else if (nDeriv > 0)
                      set_label_text(LookupWidget("IlmTasksMad2MaxLabel"),
                          g_strdup_printf("(max %d)", ntrials/nDeriv), MyBLUE, 1.0f);
                }
            } 
            catch( ... ) {strm<<" file is invalid";}
        }
        else
           try{ Ilm::read_anom_filePDB(szMad2File, strm, eType); } catch( ... ) {strm<<" file is invalid";}
        strm << "\n\n";
      }
      if(eType == MOLREP)
      {
         for(int kk=0; kk<nMRModels; kk++)
         {
             if(Vm(kk+1))
             {
                std::ofstream x;
                x << std::fixed; 
                strm.copyfmt(x);
                strm << std::setprecision (2);
                strm << "N.residues: " << NResiduesMod(kk) << std::endl;
                strm << "Matthews coefficient for model: " << Vm(kk+1) << std::endl;
                strm << "Solvent fraction for model: " << SolventFraction1() << std::endl;
                strm << "\n";
             }
         }
      }

      strm << "Commands to be executed:\n";
      if( eType == EDM) 
          strm << EDMCmd() << std::endl;
      else if( eType == DEA) 
      {
          strm.str("");
          strm << DEACmd() << std::endl;
      }
      else
      {
          if( bMad1 || eType==MOLREP || eType==ABI) 
              strm << Mad1Cmd() << std::endl;
//         if( bMad2 || bPro  && eType!=MOLREP && eType!=ABI)
          if( (bMad2 || bPro)  && (eType!=MOLREP) && (eType!=ABI) )
              strm << Mad2Cmd() << std::endl;
      }
    }
    
    SetInfoText( strm.str().c_str());
  }
  
  void GtkProject::SetAnomWavesState(int imad, bool aut) const
  {
    GtkWidget *wchk;
    int i;

    g_assert( imad == 1 || imad == 2 );

    for( i=0; i<MAX_WAVES; i++ ) {
      std::stringstream name;
      // name.seekp(0, std::ios::beg);
      name << "IlmTasksMad" << imad << "Wave" << i+1 << "Check";
      wchk = LookupWidget(name.str().c_str());
      if( i<nWaves )
      {
        if( aut )
          gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wchk), FALSE);
        gtk_widget_set_sensitive(wchk, !aut);
      }
      else
        gtk_widget_set_sensitive(wchk, FALSE);
    }
  }

  void GtkProject::SetMenuLast(const std::string &fn)
  {
    TheApp.AddLast(fn);
    TheApp.UpdateRecentMenu();
  }

  void GtkProject::CheckAnomFilePDB(std::ostream &str, const std::string &fn)
  {
    iMad2SetMax = 1;
    iMad2SetMin = 1;
    bSyncRunning = false;
    FillInterface();
    Synchronize();
  }
  
  void GtkProject::CheckAnomFile(std::ostream &str, const std::string &fn)
  {
    int ntrials;
    Ilm::read_anom_file(fn, str, ntrials, eType); // , minfom, maxfom);
    if(ntrials > 0)
    {
       if((eType != MIR) && (eType != MRS))
         set_label_text(LookupWidget("IlmTasksMad2MaxLabel"),
           g_strdup_printf("(max %d)", ntrials), MyBLUE, 1.0f);
       else  if (nDeriv > 0)
         set_label_text(LookupWidget("IlmTasksMad2MaxLabel"),
           g_strdup_printf("(max %d)", ntrials/nDeriv), MyBLUE, 1.0f);
    }
    DataFromUI();
    int imx = iMad2SetMax, imn=iMad2SetMin;
    if( imx > ntrials ) imx = ntrials;
    if( imx<1 ) imx = 1;
    iMad2SetMax = imx;
    if( imn > ntrials ) imn = ntrials;
    if( imn > imx ) imn=imx;
    if( imn<1 ) imn = 1;
    iMad2SetMin = imn;
    FillInterface();
    Synchronize();
  }

  void GtkProject::CheckPDBFile(const std::string &fn, int which, GtkProject::EValidate e)
  {
    int nresid0, nresid1, nresidui;
    Ilm::read_PDB_file(fn, nresid0, nresid1, nresidui);
    min_resid[which] = nresid0;
    max_resid[which] = nresid1;
    n_residui[which] = nresidui;
    if((nresidui == 0) && (e == eActive) && (eType != DEA))
      (void) MessageDialog(NULL, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE, "No residues found.");
    bSyncRunning = false;
    FillInterface();
    Synchronize();
  }
  
  bool GtkProject::DoWhileLock()
  {
	std::ifstream f(GetFilename2(LOCK_EXT, 1).c_str());
    if( f )
    {
       int stato = 0, nfrag = 0;
       std::string line;
       std::getline(f, line);
       f.close();
       stato = *(int *)( get_nth_field(line.c_str(), " ", 1, 'i')); 
       nfrag = *(int *)( get_nth_field(line.c_str(), " ", 2, 'i')); 
        
       if((stato >= 0) && (this->iStrategy == 3))
       {
          if((nfrag > numFragsMad) && !runSirprot)
          {
             if( bMad2 && eType!=MOLREP && eType!=ABI)
             {
                runSirprot = true;
                SyncExec(2);
                runSirprot = false;
            	f.open(GetFilename2(LOCK_EXT, 2).c_str());
                gint response, i;
                if(f)
                {
                   std::getline(f, line);
                   f.close();
                   response = *(int *)( get_nth_field(line.c_str(), " ", 1, 'i')); 
                   i = *(int *)( get_nth_field(line.c_str(), " ", 2, 'i')); 
                   switch(response)
                   {
                   case -1:          // Stop
                          return true;
                          break;
                   case 0:           // Close
                          return false;
                          break;
                   case 2:           // Cancel
                          f.open(GetFilename(STOP_EXT).c_str());
                          f.close();
                          return false;
                          break;
                   case 1:           // Forward
                          numFragsMad = i;
                          break;
                   }
                }
            }
         }
      }
      else if(nfrag <= 0)              // sir ends in error
      {
            if(eType >= EDM)
               gtkext::dlg_errmsg(MainWindow(), "Abnormal termination of Sir.\nCheck output file\n%s.", 
                  this->GetFilename2(OUT_EXT, eType-1).c_str());
            else
               gtkext::dlg_errmsg(MainWindow(), "Abnormal termination of Task1.\nCheck output file\n%s.", 
                  this->GetFilename2(OUT_EXT, 1).c_str());
            bMad2 = false;
            return false;
      }
    }

    return true; 
  }

  gint GtkProject::AsyncExec(const int what) const
  {
    gchar args[10][128], *pargs[10];
    int result = 0;
    std::string msg;
    gint pid = 0;
    GError *gerr = NULL;
#ifdef MACOSXX
    std::string cmd_line;
    std::string path_in_bundle = ".app/Contents/MacOS/";
#else
    GSpawnFlags flags = static_cast<GSpawnFlags>( G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD );
#endif
    assert( what==1 || what==2 || what==3 || what==4);
    if( what==1 ) 
    {
      CopiaStringa(args[0], 128, toUTF8(TheApp.Executables()[IExeSir].c_str()));
      CopiaStringa(args[1], 128, "--sub");
      if(eType >= EDM)
      {
         CopiaStringa(args[2], 128, toUTF8(GetFilename2(INP_EXT, eType-1).c_str()));
         CopiaStringa(args[3], 128, toUTF8(GetFilename2(OUT_EXT, eType-1).c_str()));
      }
      else
      {
         CopiaStringa(args[2], 128, toUTF8(GetFilename2(INP_EXT, 1).c_str()));
         CopiaStringa(args[3], 128, toUTF8(GetFilename2(OUT_EXT, 1).c_str()));
      }

/* Prepare command line for g_spawn_command_line_sync below */
#ifdef MACOSXX
      cmd_line = args[0];
      cmd_line += path_in_bundle;
      cmd_line += SZExecutables[IExeSir];
      cmd_line += ".sh ";
      cmd_line += g_shell_quote(args[2]);
      cmd_line += " ";
      cmd_line += g_shell_quote(args[3]);
#endif

      pargs[0] = args[0];
      pargs[1] = args[1];
      pargs[2] = args[2];
      pargs[3] = args[3];
      pargs[4] = NULL;
      args[4][0] = 0;
      if( eType==MIR )
        remove(GetFilename(MAD1FRAC_EXT).c_str());
//080706      if( eType==MAD )
        remove(GetFilename2(LOCK_EXT, 1).c_str());

/* Needed by MacOSX to properly call a bundle inside another bundle */
#ifdef MACOSXX
      result = g_spawn_command_line_sync(cmd_line.c_str(), NULL, NULL, NULL, &gerr);
#else
      result = g_spawn_async(NULL, pargs, NULL, flags, NULL, 0, (GPid *)(&pid), &gerr);
#endif
      if( gerr!=NULL || !result  ) 
      {
        msg = "Failed to execute: " + Mad1Cmd();
        if( gerr!=NULL )
        {
          msg += "\nDue to: ";
          msg += gerr->message;
          msg += "\nCHECK SETTINGS!";
          g_error_free(gerr);
        }
        std::string utf8msg = toUTF8(msg.c_str());
        throw UserError(utf8msg);
      } 
      return pid;
    }
    
    if( what==2 || what==3 || what==4)
    {
      gerr = NULL;
      CopiaStringa(args[0], 128, toUTF8(TheApp.Executables()[IExeSirProt].c_str()));
      //CopiaStringa(args[1], TheApp.SIRProtConfigDir().c_str());
      if( what==2 ) {
        CopiaStringa(args[1], 128, toUTF8(GetFilename2(INP_EXT, 2).c_str()));
        CopiaStringa(args[2], 128, toUTF8(GetFilename2(OUT_EXT, 2).c_str()));
      }
      else if( what==3 ) {
        CopiaStringa(args[1], 128, toUTF8(GetFilename2(INP_EXT, 0).c_str()));
        CopiaStringa(args[2], 128, toUTF8(GetFilename2(OUT_EXT, 0).c_str()));
      }
      else if( what==4 ) {
        CopiaStringa(args[1], 128, toUTF8(GetFilename2(INP_EXT, EDM-1).c_str()));
        CopiaStringa(args[2], 128, toUTF8(GetFilename2(OUT_EXT, EDM-1).c_str()));
      }

/* Prepare command line for g_spawn_command_line_sync below */
#ifdef MACOSXX
      cmd_line = args[0];
      cmd_line += path_in_bundle;
      cmd_line += SZExecutables[IExeSirProt];
      cmd_line += ".sh ";
      cmd_line += g_shell_quote(args[1]);
      cmd_line += " ";
      cmd_line += g_shell_quote(args[2]);
#endif
      
      pargs[0] = args[0];
      pargs[1] = args[1];
      pargs[2] = args[2];
      pargs[3] = NULL;
      args[3][0] = 0;
      pid = 0;
      if( eType==MIR && what==3 ) // PROFLEX
        remove(GetFilename(PROFLEXFRAC_EXT).c_str()); // delete fragments from previous runs
//mr      result = g_spawn_async(NULL, pargs, NULL, flags, NULL, 0, &pid, &gerr);

/* Needed by MacOSX to properly call a bundle inside another bundle */
#ifdef MACOSXX
      result = g_spawn_command_line_sync(cmd_line.c_str(), NULL, NULL, NULL, &gerr);
#else
      result = g_spawn_async(NULL, pargs, NULL, flags, NULL, 0, (GPid *)(&pid), &gerr);
#endif
      if( gerr!=NULL || !result  ) 
      {
        msg = "Failed to execute: " + Mad2Cmd();
        if( gerr!=NULL )
        {
          msg += "\nDue to: ";
          msg += gerr->message;
          msg += "\nCHECK SETTINGS!";
          g_error_free(gerr);
        }
        std::string utf8msg = toUTF8(msg.c_str());
        throw UserError(utf8msg);
      }
      return pid;
    }
    
    return pid;
  }


  gint GtkProject::SyncExec(const int what) const
  {
    gchar args[10][128], *pargs[10];
    int result = 0;
    std::string msg;
#ifdef MACOSXX
    std::string cmd_line;
    std::string path_in_bundle = ".app/Contents/MacOS/";
#endif
    gint pid = 0;
    GError *gerr = NULL;
    GSpawnFlags flags = static_cast<GSpawnFlags>( G_SPAWN_SEARCH_PATH );
    assert( what==1 || what==2 || what==3 || what==4);
    if( what==1 ) 
    {
      CopiaStringa(args[0], 128, toUTF8(TheApp.Executables()[IExeSir].c_str()));
      CopiaStringa(args[1], 128, "--sub");
      if(eType >= EDM)
      {
         CopiaStringa(args[2], 128, toUTF8(GetFilename2(INP_EXT, eType-1).c_str()));
         CopiaStringa(args[3], 128, toUTF8(GetFilename2(OUT_EXT, eType-1).c_str()));
      }
      else
      {
         CopiaStringa(args[2], 128, toUTF8(GetFilename2(INP_EXT, 1).c_str()));
         CopiaStringa(args[3], 128, toUTF8(GetFilename2(OUT_EXT, 1).c_str()));
      }

/* Prepare command line for g_spawn_command_line_sync below */
#ifdef MACOSXX
      cmd_line = args[0];
      cmd_line += path_in_bundle;
      cmd_line += SZExecutables[IExeSir];
      cmd_line += ".sh ";
      cmd_line += g_shell_quote(args[2]);
      cmd_line += " ";
      cmd_line += g_shell_quote(args[3]);
#endif

      pargs[0] = args[0];
      pargs[1] = args[1];
      pargs[2] = args[2];
      pargs[3] = args[3];
      pargs[4] = NULL;
      args[4][0] = 0;
      if( eType==MIR )
        remove(GetFilename(MAD1FRAC_EXT).c_str());
//080706      if( eType==MAD )
        remove(GetFilename2(LOCK_EXT, 1).c_str());

/* Needed by MacOSX to properly call a bundle inside another bundle */
#ifdef MACOSXX
      result = g_spawn_command_line_sync(cmd_line.c_str(), NULL, NULL, NULL, &gerr);
#else
      result = g_spawn_sync(NULL, pargs, NULL, flags, NULL, 0, NULL, NULL, NULL, &gerr);
#endif
      if( gerr!=NULL || !result  ) 
      {
        msg = "Failed to execute: " + Mad1Cmd();
        if( gerr!=NULL )
        {
          msg += "\nDue to: ";
          msg += gerr->message;
          msg += "\nCHECK SETTINGS!";
          g_error_free(gerr);
        }
        std::string utf8msg = toUTF8(msg.c_str());
        throw UserError(utf8msg);
      } 
      return pid;
    }
    
    if( what==2 || what==3 || what==4)
    {
      gerr = NULL;
      CopiaStringa(args[0], 128, toUTF8(TheApp.Executables()[IExeSirProt].c_str()));
      //CopiaStringa(args[1], TheApp.SIRProtConfigDir().c_str());
      if( what==2 ) {
        CopiaStringa(args[1], 128, toUTF8(GetFilename2(INP_EXT, 2).c_str()));
        CopiaStringa(args[2], 128, toUTF8(GetFilename2(OUT_EXT, 2).c_str()));
      }
      else if( what==3 ) {
        CopiaStringa(args[1], 128, toUTF8(GetFilename2(INP_EXT, 0).c_str()));
        CopiaStringa(args[2], 128, toUTF8(GetFilename2(OUT_EXT, 0).c_str()));
      }
      else if( what==4 ) {
        CopiaStringa(args[1], 128, toUTF8(GetFilename2(INP_EXT, EDM-1).c_str()));
        CopiaStringa(args[2], 128, toUTF8(GetFilename2(OUT_EXT, EDM-1).c_str()));
      }
      CopiaStringa(args[3], 128, " >");
      AppendiStringa(args[3], 128, toUTF8(GetFilename(OUT_EXT).c_str()));

/* Prepare command line for g_spawn_command_line_sync below */
#ifdef MACOSXX
      cmd_line = args[0];
      cmd_line += path_in_bundle;
      cmd_line += SZExecutables[IExeSirProt];
      cmd_line += ".sh ";
      cmd_line += g_shell_quote(args[1]);
      cmd_line += " ";
      cmd_line += g_shell_quote(args[2]);
#endif
      
      pargs[0] = args[0];
      pargs[1] = args[1];
      pargs[2] = args[2];
      pargs[3] = NULL;
      args[3][0] = 0;
      pid = 0;
      if( eType==MIR && what==3 ) // PROFLEX
        remove(GetFilename(PROFLEXFRAC_EXT).c_str()); // delete fragments from previous runs

/* Needed by MacOSX to properly call a bundle inside another bundle */
#ifdef MACOSXX
      result = g_spawn_command_line_sync(cmd_line.c_str(), NULL, NULL, NULL, &gerr);
#else
      result = g_spawn_sync(NULL, pargs, NULL, flags, NULL, 0, NULL, NULL, NULL, &gerr);
#endif
      if( gerr!=NULL || !result  ) 
      {
        msg = "Failed to execute: " + Mad2Cmd();
        if( gerr!=NULL )
        {
          msg += "\nDue to: ";
          msg += gerr->message;
          msg += "\nCHECK SETTINGS!";
          g_error_free(gerr);
        }
        std::string utf8msg = toUTF8(msg.c_str());
        throw UserError(utf8msg);
      }
      return pid;
    }
    
    return pid;
  }

  static gboolean check_lock(gpointer pproj)
  {
    GtkProject *proj = static_cast<GtkProject*>(pproj);

    assert(proj!=NULL);
    return proj->DoWhileLock();
  }

  void GtkProject::Go()
  {
    gboolean result = true;

    if( !(bMad1 || bMad2 || bPro) && eType!=MOLREP && eType!=ABI && eType != EDM)
      throw UserError("nothing selected for execution");

    SetSubmitError("");
    SetSubmitted(true);
      try
    {
      if(eType == DEA)
      {
         SetSubmitted(false);
         return;
      }
      gtk_widget_set_sensitive(LookupWidget("IlmProjectWnd"), FALSE);
      gtk_widget_hide(LookupWidget("IlmProjectWnd"));
      while(gtk_events_pending())
            gtk_main_iteration();
      if( bPro && eType!=MOLREP && eType!=ABI)
      {
        SyncExec(3);
      }
      if( ((eType != EDM) && bMad1) || eType==MOLREP || eType==ABI )
      {
        if (iStrategy == 3 && bMad2)
        {
            AsyncExec(1);
            g_timeout_add(100, check_lock,  static_cast<gpointer>(this)); // timer every 100 millisecs 
        }
        else
        {
            SyncExec(1);
            result = check_lock(static_cast<gpointer>(this));
            if(result && (eType==ABI))
               FillInterface();
        }
      }

      if( result && bMad2 && eType!=MOLREP && eType!=ABI && iStrategy!=3 && eType != EDM)
      {
        SyncExec(2);
        std::ifstream f(GetFilename2(LOCK_EXT, 2).c_str());
        gint response;
        if (f)
        {
            std::string line;
            std::getline(f, line);
            f.close();
            response = *(int *)( get_nth_field(line.c_str(), " ", 1, 'i')); 
            if(response == 1)
            {
               SaveAsSir(4, NULL);
               SetReady( false );
               Go();
            }
        }
      }

      // chiamare sirprot per EDM solo quando parte da file MTZ (bMad=false)
      if(eType == EDM)
      {
        SyncExec(4);
      }
    } 
    catch( ... ) 
    {
      gtk_widget_set_sensitive(LookupWidget("IlmProjectWnd"), TRUE);
      gtk_widget_show(LookupWidget("IlmProjectWnd"));
      SetSubmitted(false);
      throw;
    }
    gtk_widget_set_sensitive(LookupWidget("IlmProjectWnd"), TRUE);
    gtk_widget_show(LookupWidget("IlmProjectWnd"));
    SetSubmitted(false);
  }

  int GtkProject::ValidateAll(EValidate e)
  {
    int result = 1;
    for(int i=0; i<gtk_notebook_get_n_pages(GTK_NOTEBOOK(LookupWidget("IlmNotebook"))); i++ ) {
      result *= ApplyNthPage(i, e);
    }
    return result;
  }

  void GtkProject::AddCycles(const char *src, const char *dst)
  {
    FILE *fp1, *fp2;
    fp1 = fOpen(src, "r");
    fp2 = fOpen(dst, "w");
    if(fp1 && fp2)
    {
       char *riga1 = g_new0(char, 201);
       while(fgets(riga1,200,fp1) != NULL)
       {
             if(strstr(riga1, "cycdef"))
                (void) fputs(g_strdup_printf("let ncycles=%d\n",nCycles),fp2);
             else
                (void) fputs(riga1,fp2);
       }
       fclose(fp1);
       fclose(fp2);
       g_free(riga1);
    }
  }

  void GtkProject::WriteInte(std::ofstream &of) const
  {
#ifndef _WIN32
         of << "#! /bin/bash\n"
            << "let ncycles=" << nCycles <<"\n"
            << "if [ $# -ge 1 ]\n"
            << "then\n"
            << "let ncycles=$1\n"
            << "fi\n"
#else
         of << "@ECHO OFF\n"
            << "setlocal enabledelayedexpansion\n"
            << "SET /A ncycles=" << nCycles <<"\n"
            << "set argCount=0\n"
            << "for %%x in (%*) do set /A argCount+=1\n"
            << "if /I \"%argCount%\" GEQ \"1\" SET /A ncycles=%1\n"
#endif
            << COM_CHAR << " prepare the mtz file to be used in the loop (ILM/" 
            << ambp[nBuildPgm] << ")  below\n"
            << "echo \"DEA on " << szStructName << "\"" <<"\n"
            << CallShell("cad_obs") << " >dea_cad.log "
            << "\n";
  }

  std::string GtkProject::CallShell(const char *shellName) const
  {
     std::string result;
#ifdef _WIN32
     result.assign("CALL ");
#else
     result.assign("");
#endif
     result.append(g_build_filename(szWorkDir, shellName, NULL));
     result.append(SHELL_EXT);
     return  result;
  }

  void GtkProject::CreaShellFromModel(const char *nome)
  {  
    std::ofstream of(nome);
    WriteInte(of);
    switch(nBuildPgm)
    {
    case 1: // dea_arp_fromModel
         of << "# run arp with the model from ILMILIONE\n"
            << "echo\n" << "echo \"Running arp/Warp cycle 0\"\n"
            << "./arprun_model" << SHELL_EXT << " > dea_arp0.log\n"
            << " a=`fgrep 'Job ID is set to' dea_arp0.log | cut -c23-37`\n"
            << " cp $a/dea_obs_warpNtrace.pdb $a/dea_arp_dedm_edm0_warpNtrace.pdb\n"
            << "./refmac_arp" << SHELL_EXT << " $a 0 0\n"
            << "rm $a/dea_arp_dedm_edm0_warpNtrace.pdb\n"
            << "sirprot dea.sir >  dea_arp_dedm_edm0.out\n"
            << "cp dea_edm.mtz dea_arp_dedm_edm0.mtz\n"
            << "\n"
            << "# loop ARP + DDM/EDM\n"
            << "# in the first cycle the mtz (with phases) from ILMILIONE is used.\n"
            << "let i=1\n"
            << "while [ $i -le $ncycles ]\n"
            << "do\n"
            //<< "   echo $i\n"
            << "   echo\n" << "   echo \"Running arp/Warp cycle \"$i\n"
            << "    # ARP\n"
            << "    let ii=$i-1\n"
            << "   ./arprun_phases" << SHELL_EXT << " $i $ii > dea_arp$i.log\n"
            << "    a=`fgrep 'Job ID is set to' dea_arp$i.log | cut -c23-37`\n"
            << "   ./refmac_arp" << SHELL_EXT << " $a $i $ii\n"
            << "   # DEDM + EDM\n"
            << "   echo\n" << "    echo \"Running DEDM+EDM cycle \"$i\n"
            << "   sirprot dea.sir >  dea_arp_dedm_edm$i.out\n"
            << "   cp dea_edm.mtz dea_arp_dedm_edm$i.mtz\n"
            << "   let i++\n"
            << "done\n";
         break;
    case 2: // dea_phenix_fromModel
         of  << "# run phenix with the model from ILMILIONE\n"
             << "echo\n" << "echo \"Running Phenix cycle 0\"\n"
             << "./phenixrun_model" << SHELL_EXT << " \n"
             << "a=`tail -n1 dea_phenix0.log`\n"
             << "./refmac_phenix" << SHELL_EXT << " $a 0\n"
             << "less $a/AutoBuild_run_*.log  >> dea_phenix0.log\n"
             << "sirprot dea.sir >  dea_phenix_dedm_edm0.out\n"
             << "cp dea_edm.mtz dea_phenix_dedm_edm0.mtz\n"
             << "\n"
             << "# loop Phenix + DDM/EDM\n"
             << "let i=1\n"
             << "while [ $i -le $ncycles ]\n"
             << "do\n"
             //<< "   echo $i\n"
             << "echo\n" << "echo \"Running Phenix cycle \"$i\n"
             << "    # Phenix\n"
             << "    let ii=$i-1\n"
             << "   ./phenixrun_phases" << SHELL_EXT << " $i $ii\n"
             << "    a=`tail -n1 dea_phenix$i.log`\n"
             << "   ./refmac_phenix" << SHELL_EXT << " $a $i\n"
             << "   less $a/AutoBuild_run_*.log  >> dea_phenix$i.log\n"
             << "   # DEDM + EDM\n"
             << "   echo\n" << "    echo \"Running DEDM+EDM cycle \"$i\n"
             << "   sirprot dea.sir >  dea_phenix_dedm_edm$i.out\n"
             << "   cp dea_edm.mtz dea_phenix_dedm_edm$i.mtz\n"
             << "   let i++\n"
             << "done\n";
         break;
    case 3: // dea_buccaneer_fromModel
         of << COM_CHAR << " run buccaneer with the model from ILMILIONE\n"
            << "echo\n" << "echo \"Running Buccaneer cycle 0\"\n"
            << CallShell("buccaneerrun_model")
            << " > dea_buccaneer0.log\n"
            << CallShell("refmac_buccaneer")
            << " " << szWorkDir << " 0 0\n"
            << toUTF8(TheApp.Executables()[IExeSirProt].c_str()) << " dea.sir >  dea_buccaneer_dedm_edm0.out\n"
            << COPYF << " dea_edm.mtz dea_buccaneer_dedm_edm0.mtz\n"
            << "\n"
            << COM_CHAR << " loop Buccaneer + DDM/EDM\n"
            << COM_CHAR << " in the first cycle the mtz (with phases) from ILMILIONE is used.\n"
#ifndef _WIN32
            << "let i=1\n"
            << "while [ $i -le $ncycles ]\n"
            << "do\n"
            //<< "   echo $i\n"
            << "    echo\n" << "    echo \"Running Buccaneer cycle \"$i\n"
            << "    " << COM_CHAR << " Buccaneer\n"
            << "    let ii=$i-1\n"
            << "   " << CallShell("buccaneerrun_phases")
            << " $i $ii > dea_buccaneer$i.log\n"
            << "    " << CallShell("refmac_buccaneer")
            << " " << szWorkDir << " $i $ii\n"
            << "   " << COM_CHAR << " DEDM + EDM\n"
            << "   echo\n" << "    echo \"Running DEDM+EDM cycle \"$i\n"
            << "   " << toUTF8(TheApp.Executables()[IExeSirProt].c_str()) << " dea.sir >  dea_buccaneer_dedm_edm$i.out\n"
            << "   " << COPYF << " dea_edm.mtz dea_buccaneer_dedm_edm$i.mtz\n"
            << "   let i++\n"
            << "done\n";
#else
            << "FOR /L %%i IN (1,1,%ncycles%) DO (\n"
            //<< "   echo %%i\n"
            << "   echo\n" << "   echo \"Running Buccaneer cycle \"%%i\n"
            << "    " << COM_CHAR << " Buccaneer\n"
            << "SET /A ii=%%i-1\n"
            << "   " << CallShell("buccaneerrun_phases")
            << " %%i !ii! > dea_buccaneer%%i.log\n"
            << "    " << CallShell("refmac_buccaneer")
            << " " << szWorkDir << " %%i !ii!\n"
            << "   " << COM_CHAR << " DEDM + EDM\n"
            << "   echo\n" << "    echo \"Running DEDM+EDM cycle \"%%i\n"
            << "   " << toUTF8(TheApp.Executables()[IExeSirProt].c_str()) << " dea.sir >  dea_buccaneer_dedm_edm%%i.out\n"
            << "   " << COPYF << " dea_edm.mtz dea_buccaneer_dedm_edm%%i.mtz\n"
            << ")\n";
#endif
         break;
    case 4: // dea_nautilus_fromModel
         of << COM_CHAR << " run nautilus with the model from ILMILIONE\n"
            << "echo\n" << "echo \"Running Nautilus cycle 0\"\n"
            << CallShell("nautilusrun_model")
            << " > dea_nautilus0.log\n"
            << CallShell("refmac_nautilus")
            << " " << szWorkDir << " 0 0\n"
            << "   " << COM_CHAR << " DEDM + EDM\n"
            << "   echo\n" << "    echo \"Running DEDM+EDM cycle \"$i\n"
            << toUTF8(TheApp.Executables()[IExeSirProt].c_str()) << " dea.sir >  dea_nautilus_dedm_edm0.out\n"
            << COPYF << " dea_edm.mtz dea_nautilus_dedm_edm0.mtz\n"
            << "\n"
            << COM_CHAR << " loop Nautilus + DDM/EDM\n"
            << COM_CHAR << " in the first cycle the mtz (with phases) from ILMILIONE is used.\n"
#ifndef _WIN32
            << "let i=1\n"
            << "while [ $i -le $ncycles ]\n"
            << "do\n"
            //<< "   echo $i\n"
            << "    echo\n" << "    echo \"Running Nautilus cycle \"%%i\n"
            << "    " << COM_CHAR << " Nautilus\n"
            << "    let ii=$i-1\n"
            << "   "
            << CallShell("nautilusrun_phases")
            << " $i $ii > dea_nautilus$i.log\n"
            << "   " << CallShell("refmac_nautilus")
            << " " << szWorkDir << " $i $ii\n"
            << "   " << COM_CHAR << " DEDM + EDM\n"
            << "   echo\n" << "    echo \"Running DEDM+EDM cycle \"%%i\n"
            << "   " << toUTF8(TheApp.Executables()[IExeSirProt].c_str()) << " dea.sir >  dea_nautilus_dedm_edm$i.out\n"
            << "   " << COPYF << " dea_edm.mtz dea_nautilus_dedm_edm$i.mtz\n"
            << "   let i++\n"
            << "done\n";
#else
            << "FOR /L %%i IN (1,1,%ncycles%) DO (\n"
            << "   echo %%i\n"
            << "    " << COM_CHAR << " Nautilus\n"
            << "SET /A ii=%%i-1\n"
            << "   " << CallShell("nautilusrun_phases")
            << " %%i !ii! > dea_nautilus%%i.log\n"
            << "    " << CallShell("refmac_nautilus")
            << " " << szWorkDir << " %%i !ii!\n"
            << "   " << COM_CHAR << " DEDM + EDM\n"
            << "   " << toUTF8(TheApp.Executables()[IExeSirProt].c_str()) << " dea.sir >  dea_nautilus_dedm_edm%%i.out\n"
            << "   " << COPYF << " dea_edm.mtz dea_nautilus_dedm_edm%%i.mtz\n"
            << ")\n";

#endif
         break;
    }
    of.close();
    ChangeMode(nome, tXt);                 // chmod 777
  }

  void GtkProject::CreaShellFromPhases(const char *nome)
  {  
    std::ofstream of(nome);
    WriteInte(of);
    switch(nBuildPgm)
    {
    case 1: // dea_arp_fromPhases
         of  << "./cad_phases" << SHELL_EXT << "\n"
             << "\n"
             << "# loop ARP + DDM/EDM\n"
             << "# in the first cycle the mtz (with phases) from ILMILIONE is used.\n"
             << "let i=1\n"
             << "while [ $i -le $ncycles ]\n"
             << "do\n"
             << "   echo $i\n"
             << "    # ARP\n"
             << "    let ii=$i-1\n"
             << "   ./arprun_phases" << SHELL_EXT << " $i $ii > dea_arp$i.log\n"
             << "    a=`fgrep 'Job ID is set to' dea_arp$i.log | cut -c23-37`\n"
             << "   ./refmac_arp" << SHELL_EXT << " $a $i $ii\n"
             << "   # DEDM + EDM\n"
             << "   sirprot dea.sir >  dea_arp_dedm_edm$i.out\n"
             << "   cp dea_edm.mtz dea_arp_dedm_edm$i.mtz\n"
             << "   let i++\n"
             << "done\n";
         break;
    case 2: // dea_phenix_fromPhases
         of  << "./cad_phases" << SHELL_EXT << "\n"
             << "\n"
             << "# loop Phenix + DDM/EDM\n"
             << "# in the first cycle the mtz (with phases) from ILMILIONE is used.\n"
             << "let i=1\n"
             << "while [ $i -le $ncycles ]\n"
             << "do\n"
             << "   echo $i\n"
             << "    # Phenix\n"
             << "    let ii=$i-1\n"
             << "   ./phenixrun_phases" << SHELL_EXT << " $i $ii\n"
             << "    a=`tail -n1 dea_phenix$i.log`\n"
             << "   ./refmac_phenix" << SHELL_EXT << " $a $i\n"
             << "   less $a/AutoBuild_run_*.log  >> dea_phenix$i.log\n"
             << "   # DEDM + EDM\n"
             << "   sirprot dea.sir >  dea_phenix_dedm_edm$i.out\n"
             << "   cp dea_edm.mtz dea_phenix_dedm_edm$i.mtz\n"
             << "   let i++\n"
             << "done\n";
         break;
    case 3: // dea_buccaneer_fromPhases
         of  << CallShell("cad_phases") << " >dea_cadphases.log "
             << "\n"
             << "\n"
             << COM_CHAR << " loop Buccaneer + DDM/EDM\n"
             << COM_CHAR << " in the first cycle the mtz (with phases) from ILMILIONE is used.\n"
#ifndef _WIN32
             << "let i=1\n"
             << "while [ $i -le $ncycles ]\n"
             << "do\n"
             << "   echo $i\n"
             << "    " << COM_CHAR << " Buccaneer\n"
             << "    let ii=$i-1\n"
             << "   " << CallShell("buccaneerrun_phases")
             << " $i $ii > dea_buc$i.log\n"
             << "   " << CallShell("refmac_buccaneer")
             << " " << szWorkDir << " $i $ii\n"
             << "   " << COM_CHAR << " DEDM + EDM\n"
             << "   " << toUTF8(TheApp.Executables()[IExeSirProt].c_str()) 
             << " dea.sir >  dea_buccaneer_dedm_edm$i.out\n"
             << "   " << COPYF << " dea_edm.mtz dea_buccaneer_dedm_edm$i.mtz\n"
             << "   let i++\n"
             << "done\n";
#else
            << "FOR /L %%i IN (1,1,%ncycles%) DO (\n"
            << "   echo %%i\n"
            << "    " << COM_CHAR << " Buccaneer\n"
            << "SET /A ii=%%i-1\n"
            << "   " << CallShell("buccaneerrun_phases")
            << " %%i !ii! > dea_buc%%i.log\n"
            << "    " << CallShell("refmac_buccaneer")
            << " " << szWorkDir << " %%i !ii!\n"
            << "   " << COM_CHAR << " DEDM + EDM\n"
            << "   " << toUTF8(TheApp.Executables()[IExeSirProt].c_str()) << " dea.sir >  dea_buccaneer_dedm_edm%%i.out\n"
            << "   " << COPYF << " dea_edm.mtz dea_buccaneer_dedm_edm%%i.mtz\n"
            << ")\n";
#endif
         break;
    case 4: // dea_nautilus_fromPhases
         of  << CallShell("cad_phases")
             << "\n"
             << "\n"
             << COM_CHAR << " loop Nautilus + DDM/EDM\n"
             << COM_CHAR << " in the first cycle the mtz (with phases) from ILMILIONE is used.\n"
#ifndef _WIN32
             << "let i=1\n"
             << "while [ $i -le $ncycles ]\n"
             << "do\n"
             << "   echo $i\n"
             << "    " << COM_CHAR << " Nautilus\n"
             << "    let ii=$i-1\n"
             << "   " << CallShell("nautilusrun_phases")
             << " $i $ii > dea_nau$i.log\n"
             << "   " << CallShell("refmac_nautilus")
             << " " << szWorkDir << " $i $ii\n"
             << "   " << COM_CHAR << " DEDM + EDM\n"
             << "   " << toUTF8(TheApp.Executables()[IExeSirProt].c_str()) 
             << " dea.sir >  dea_nautilus_dedm_edm$i.out\n"
             << "   " << COPYF << " dea_edm.mtz dea_nautilus_dedm_edm$i.mtz\n"
             << "   let i++\n"
             << "done\n";
#else
            << "FOR /L %%i IN (1,1,%ncycles%) DO (\n"
            << "   echo %%i\n"
            << "    " << COM_CHAR << " Nautilus\n"
            << "SET /A ii=%%i-1\n"
            << "   " << CallShell("nautilusrun_phases")
            << " %%i !ii! > dea_nau%%i.log\n"
            << "    " << CallShell("refmac_nautilus")
            << " " << szWorkDir << " %%i !ii!\n"
            << "   " << COM_CHAR << " DEDM + EDM\n"
            << "   " << toUTF8(TheApp.Executables()[IExeSirProt].c_str()) << " dea.sir >  dea_nautilus_dedm_edm%%i.out\n"
            << "   " << COPYF << " dea_edm.mtz dea_nautilus_dedm_edm%%i.mtz\n"
            << ")\n";
#endif
         break;
    }
    of.close();
    ChangeMode(nome, tXt);                 // chmod 777
  }

  void GtkProject::CreaRefmacShell()
  {  
    std::string base("refmac_");
    base.append(ambpre[nBuildPgm]);
    base.append(SHELL_EXT);
    gchar *nome = g_build_filename(szWorkDir, base.c_str(), NULL);
    std::ofstream of(nome);
#ifndef _WIN32
     of << "#! /bin/bash\n";
#endif
     of << "echo '" << ambp[nBuildPgm] << " working directory: '"
        << REF_CHAR <<"1 " << REF_CHAR <<"2 " << REF_CHAR <<"3\n"
        << "refmac5 " << CONT_CHAR << "\n"
        << "hklin dea_obs.mtz " << CONT_CHAR << "\n";
    switch(nBuildPgm)
    {
    case 1: // ARP/WARP
             of << "xyzin "
                << REF_CHAR << "1/dea_arp_dedm_edm" << REF_CHAR << "3_warpNtrace.pdb " << CONT_CHAR << "\n";
         break;
    case 2: // PHENIX
             of << "xyzin "
                << REF_CHAR << "1/overall_best.pdb " << CONT_CHAR << "\n";
         break;
    case 3: // Buccaneer
             of << "xyzin buccaneer.pdb " << CONT_CHAR << "\n";
             break;
    case 4: // Nautilus
             of << "xyzin nautilus.pdb " << CONT_CHAR << "\n";
         break;
    }
#if 0
    of << "hklout dea_amb.mtz  <<END\n"
       << "labin FP=F_ILM SIGFP=SIGF_ILM FREE=FreeR_flag\n"
       << "refi -\n"
       << "    type UNRE -\n"
       << "    resi MLKF -\n"
       << "    meth CGMAT\n"
       << "ncyc 0\n"
       << "END\n"
#endif
    base.assign("refmac_");
    base.append(ambpre[nBuildPgm]);
    base.append(".inp");
    of << "hklout dea_amb.mtz  <" << base <<"\n";
#ifndef _WIN32
    of << "cp dea_amb.mtz dea_amb" << REF_CHAR << "2.mtz\n";
#else
    of << "copy dea_amb.mtz dea_amb" << REF_CHAR << "2.mtz\n";
#endif

    of.close();
    ChangeMode(nome, tXt);                 // chmod 777
    g_free(nome);
    nome = g_build_filename(szWorkDir, base.c_str(), NULL);
    of.open(nome);
    of << "labin FP=F_ILM SIGFP=SIGF_ILM FREE=FreeR_flag\n"
       << "refi -\n"
       << "    type UNRE -\n"
       << "    resi MLKF -\n"
       << "    meth CGMAT\n"
       << "ncyc 0\n";
    of.close();
  }

  void GtkProject::SaveAllFiles()
  {
    Save();
    SetMenuLast(GetFilename(PROJECT_EXT));

    TheApp.WriteSettings();
    if(eType == EDM)
    {
       remove(GetFilename2(OUT_EXT, EDM-1).c_str());
       remove(GetFilename2(LOG_EXT, EDM-1).c_str());
       remove(GetFilename2(GRAPH_EXT, EDM-1).c_str());
    }
    else if(eType == DEA)
    {
       std::string scriptname;
// Cancella dea.sir
       remove(GetFilename2(INP_EXT, DEA-1).c_str());
// Cancella tutti i files del tipo dea_phenix*.log
       mremove("dea_phenix*.log");
////////!!!!!       if(nBuildPgm == 1)      // Converte la sequenza da FASTA a PIR format
          FASTAToPIR();  

//NEW
       gchar *nome = g_build_filename(szWorkDir, DEA_SH, NULL);
       if(bModel)
          CreaShellFromModel(nome);
       else
          CreaShellFromPhases(nome);
       CreaRefmacShell();   
    }
    else
    {
       if (bMad1)
       {
           int tipo = 1;
           if(eType > EDM)
               tipo = eType-1;
           remove(GetFilename2(OUT_EXT, tipo).c_str());
           remove(GetFilename2(LOG_EXT, tipo).c_str());
           remove(GetFilename2(LOCK_EXT, 1).c_str());
           remove(GetFilename2(GRAPH_EXT, tipo).c_str());
           remove(GetFilename(STOP_EXT).c_str());
           remove(GetFilename2(BIN_EXT, tipo).c_str());
       }
      if (bMad2)
      {
           remove(GetFilename2(OUT_EXT, 2).c_str());
           remove(GetFilename2(LOG_EXT, 2).c_str());
           remove(GetFilename2(GRAPH_EXT, 2).c_str());
           remove(GetFilename2(LOCK_EXT, 2).c_str());
           remove(GetFilename2(BIN_EXT, 2).c_str());
       }
       remove(GetFilename2(OUT_EXT, 0).c_str());
       remove(GetFilename2(LOG_EXT, 0).c_str());
       remove(GetFilename2(GRAPH_EXT, 0).c_str());
    }

    tmLogModTime = 0;
    numFragsMad = 0;
    runSirprot = false;
    if(eType == EDM)
      SaveAsSirEdm();
    else if(eType == DEA)
    {
      SaveAsDea();
      CreateCadShell();
      if(nBuildPgm == 1)
         CreateScriptsArp();
      else if(nBuildPgm == 2)
         CreateScriptsPhenix();
      else if(nBuildPgm == 3)
         CreateScriptsBuccNau();
      else if(nBuildPgm == 4)
         CreateScriptsBuccNau();
    }
    else
    {
      if( bMad1 || eType==MOLREP || eType==ABI)
          SaveAsSir(1, NULL);
      if( bMad2 && eType!=MOLREP && eType!=ABI)
          SaveAsSir(2, NULL);
      if( bPro && eType!=MOLREP && eType!=ABI)
          SaveAsSir(3, NULL);
    }
  }

  void GtkProject::AddTextToMenu(const char *name, const std::string &text) const
  {
    gpointer item = LookupObject(name);
    std::string str = obj_get_label(item);
    std::string::size_type pos = str.find_first_of('(');
    if( pos == std::string::npos )
    { // not found
      str += " ( ";
      str += text + " )";
    }
    else
    {
      str.replace(pos+2, MaxLenFilename, text);
      str += " )";
    }
    obj_set_label(item, str.c_str());
  }

  void GtkProject::TextToOutMenu(const char *name, const char *ext, int ind) const
  {
    std::string fname = "IlmMenuOut";
    fname += name;
    AddTextToMenu(fname.c_str(), GetFilename2(ext, ind));
  }

  void GtkProject::TextToOutMenus(const char *name, const char *ext) const
  {
    std::string fname = "IlmMenuOut";
    fname += name;
    AddTextToMenu(fname.c_str(), GetFilename(ext));
  }

  void GtkProject::TextToInMenu(const char *name, const char *fn) const
  {
    std::string fname = "IlmMenuIn";
    fname += name;
    AddTextToMenu(fname.c_str(), fn);
  }

  void GtkProject::TextToUI(const char *name, const char *text) const
  {
    gtk_entry_set_text(GTK_ENTRY(LookupWidget(name)), toUTF8(text));
  }

  void GtkProject::LabelToUI(const char *name, const char *text) const
  {
    gtk_label_set_text(GTK_LABEL(LookupWidget(name)), toUTF8(text));
  }

  void GtkProject::TextWToUI(const char *name, const char *text) const
  {
    gtk_text_buffer_set_text(gtk_text_view_get_buffer (GTK_TEXT_VIEW (LookupWidget(name))), 
              toUTF8(text), -1);
  }

  void GtkProject::IntToUI(const char *name, int i) const
  {
    spin_button_set_value(LookupWidget(name), static_cast<double>(i));
  }

  void GtkProject::SetMaxSpinValue(const char *name, int i) const
  {
    spin_button_set_range(LookupWidget(name), 1., static_cast<double>(i));
  }

  void GtkProject::SetSpinValues(const char *name, double vmin, double vmax, double val) const
  {
    bSyncRunning = true;
    spin_button_set_range(LookupWidget(name), vmin, vmax);
    spin_button_set_value(LookupWidget(name), val);
  }

  void GtkProject::DoubleToUI(const char *name, double d) const
  {
    spin_button_set_value(LookupWidget(name), d);
  }

  void GtkProject::BoolToUI(const char *name, bool b) const
  {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(LookupWidget(name)), b?TRUE:FALSE);
  }

  void GtkProject::ListToUI(const char *name, int i) const
  {
    combo_set_val(LookupWidget(name), i+1);
  }
  int GtkProject::IntFromUI(const char *name) const
  {
    return spin_button_get_value_as_int(LookupWidget(name));
  }
  double GtkProject::DoubleFromUI(const char *name) const
  {
    return spin_button_get_value(LookupWidget(name));
  }
  bool GtkProject::BoolFromUI(const char *name) const
  {
    return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(LookupWidget(name)))!=0;
  }
  
  int GtkProject::ListFromUI(const char *name) const
  {
    return (combo_get_val(LookupWidget(name))-1);
  }

  const gchar *GtkProject::LabelFromUI(const char *name) const
  {
    return (gtk_label_get_text(GTK_LABEL(LookupWidget(name))));
  }

  void GtkProject::ClearSequence()
  {
    memset(szSeqFile, 0, sizeof(szSeqFile));
    szSeqFile[0] = ' ';
    ClearText(LookupWidget("IlmContSeq"));
    TextToUI("IlmContSeqFile", szSeqFile);
  }

  void GtkProject::WriteSequence() const
  {
      std::string fn = szSeqFile;
      if(!fn.empty())
      {
         std::string sequence = GetTextW(LookupWidget("IlmContSeq"));
         FILE *fp = fOpen(szSeqFile, "w");
         if(fp)
         {
            (void) fputs(sequence.c_str(), fp);
            (void) fclose(fp);
         }
      }
  }

  void GtkProject::ReadSequence() const
  {
      std::string fn = szSeqFile;

      TextToUI("IlmContSeqFile", szSeqFile);

      if(!fn.empty())
      {
         FILE *fp = fOpen(szSeqFile, "r");
         if( fp)
         {
            gchar *buf = g_new0(gchar, 80);
            std::string buffer = "";
            while(fgets(buf, 80, fp))
               buffer.append (buf);
            fclose(fp);
            TextWToUI("IlmContSeq", buffer.c_str ());
            g_free(buf);
         }
      }
  }

  void GtkProject::FillInterface() const
  {
    if( bSyncRunning )
      return;
    bSyncRunning = true;
    nomefile name;
    TextToUI("IlmJobEntry", szJobTitle);

    // MTZ
    
    TextToUI("IlmMtzFileEntry", szMtzFile);
    if(Ilm::PTheProject->FillFromMTZ())
       TextWToUI("IlmMtzColumnsEntry", szSelectedColumns);

    // Cell Content
    if(iCont == 1)
       BoolToUI("IlmContResiduesRadio", true);
    else if (iCont == 2)
       BoolToUI("IlmContSeqRadio", true);
    gtk_widget_set_sensitive(LookupWidget("IlmContResiduesSpin"), (iCont == 1));
    gtk_widget_set_sensitive(LookupWidget("IlmContSeqBox"), (iCont == 2));
    gtk_widget_set_sensitive(LookupWidget("IlmContSeqFrame"), (iCont == 2));
    gtk_widget_set_sensitive(LookupWidget("IlmContMolSpin"), (iCont == 0));
    if(iCont != 2)
        ClearText(LookupWidget("IlmContSeq"));
    else if(!gext::str_is_white (szSeqFile))
       ReadSequence();
    IntToUI("IlmContResiduesSpin", NResidues() );
    DoubleToUI("IlmContMolSpin", MolecularWeight());
    IntToUI("IlmContNmolsSpin", nMolecules);
    BoolToUI( "IlmContNativeCheck", bNativePresent);
    IntToUI("IlmContWavesSpin", nWaves);
    IntToUI("IlmContDerivSpin", nDeriv);
    int nNum, iFin;
    if((eType == MIR) || (eType == MRS))
    {
      iFin = MAX_DERIVATIVES;
      nNum = nDeriv;
    }
    else
    {
      iFin = MAX_WAVES;
      nNum = nWaves;
    }

// Ligands

    if( eType==MRS || eType==MOLREP || eType==ABI || eType==EDM)
    {
      for(int i=0; i<MAX_ELEMENTS; i++ ) {
        g_snprintf(name, sizeof(name), "IlmContLigSymbol%dEntry", i+1);
        TextToUI(name, szElement[i]);
        g_snprintf(name, sizeof(name), "IlmContLigAmount%dSpin", i+1);
        IntToUI(name, nElementAmount[i]);
      }
    }
    
    if(eType==MRS)    // Number of waves for each derivative
    {
        for( int i=1; i<=iFin; i++ ) 
        {
             std::stringstream strm;
             strm << "IlmContDeriv" << i << "Spin";
             gtk_widget_set_sensitive(LookupWidget(strm.str().c_str()), i<=nNum );
             IntToUI(strm.str().c_str(), nMrsWaves[i-1]);
        }
    }

// Additional elements (MAD e MIRAS)
    if( eType==MAD || eType==MRS)  
    {
      for(int i=1; i<=MAX_ELEMENTS; i++ ) {
        g_snprintf(name, sizeof(name), "IlmContSymbol%dEntry", i);
        if( eType==MAD )
          TextToUI(name, i<=nElements ? szElement[i-1] : "" );
        else
        {
          TextToUI(name, szMirElement[i-1][0]);
          gtk_widget_set_sensitive(LookupWidget(name), i<=nNum);
        }
        g_snprintf(name, sizeof(name), "IlmContAmount%dSpin", i);
        if( eType==MAD )
          IntToUI(name, i<=nElements ? nElementAmount[i-1] : 0);
        else
        {
          IntToUI(name, nMirElementAmount[i-1][0]);
          gtk_widget_set_sensitive(LookupWidget(name), i<=nNum);
        }
        for(int j=1; j<=iFin; j++ ) {
          for(int k=1; k<=2; k++ ) {
            g_snprintf(name, sizeof(name), "IlmContE%dW%dF%dSpin", i, j, k);
            if( eType==MAD )
              gtk_widget_set_sensitive(LookupWidget(name), j<=nNum);
            else
              gtk_widget_set_sensitive(LookupWidget(name), i<=nNum && j<=nMrsWaves[i-1]);
            DoubleToUI(name, dElementF1F2[i-1][j-1][k-1]);
          }
        }
      }
    } 
    else if( eType==MIR ) 
    {
      for(int i=0; i<=iFin; i++)
      {
        std::stringstream name;
        name << "IlmContMir" << i << "Tbl";
        gtk_widget_set_sensitive(LookupWidget(name.str().c_str()), i<=nNum);
        for( int j=0; j<MAX_ELEMENTS; j++ )
        {
          std::stringstream nm;
          nm << "IlmContMir" << i << "Amount" << j+1 << "Spin";
          if( i==0 )                                        //Native
            IntToUI(nm.str().c_str(), nElementAmount[j]);
          else                                              //Derivative
            IntToUI(nm.str().c_str(), nMirElementAmount[i-1][j]);
          nm.str("");
          nm << "IlmContMir" << i << "Symbol" << j+1 << "Entry";
          if( i==0 )                                        //Native
            TextToUI(nm.str().c_str(), szElement[j]);
          else                                              //Derivative
            TextToUI(nm.str().c_str(), szMirElement[i-1][j]);
        }
      }
    }
    g_snprintf(name, sizeof(name), "%s", Ilm::Sources[nSource]);
    BoolToUI(name, true);
    g_snprintf(name, sizeof(name), "%s", Ilm::Waves[nWaveLength]);
    BoolToUI(name, true);
    if( std::fabs( dWaveLength) > 0.0001 )
        TextToUI("IlmContWaveLEntry", g_strdup_printf("%lf", dWaveLength));
    
    
    // Tasks
    if( eType==MOLREP )
    {
      IntToUI("IlmTasksMRnModels", nMRModels);
      BoolToUI("IlmTasksMRPseudoCheck", bMRUsePseudo );
      for(int i=0; i<MAX_MODELS; i++)
      {
          TextToUI(g_strdup_printf("IlmTasksMRFileEntry%d",i), szMRFile[i]);
          BoolToUI(g_strdup_printf("IlmTasksMRHeteroCheck%d", i), bMRUseHetero[i] );
          BoolToUI(g_strdup_printf("IlmTasksMRPdbRadio%d", i), cMRAdp[i] == 'P'); 
          BoolToUI(g_strdup_printf("IlmTasksMRWilsonRadio%d", i), cMRAdp[i] == 'W'); 
          BoolToUI(g_strdup_printf("IlmTasksMR20Radio%d", i), cMRAdp[i] == '2');
          BoolToUI(g_strdup_printf("IlmTasksMRPolyalaCheck%d", i), bMRPolyalanine[i]);
          if((min_resid[i] != 0) && (max_resid[i] != 0))
          {
             gtk_widget_set_sensitive(LookupWidget(g_strdup_printf("IlmTasksMRCutCheck%d", i)), TRUE);
             BoolToUI(g_strdup_printf("IlmTasksMRCutCheck%d", i), bMRCutModel[i]);
             if(bMRCutModel[i])
             {
                IntToUI(g_strdup_printf("IlmTasksMRn1Spin%d", i), n1MR[i]);
                IntToUI(g_strdup_printf("IlmTasksMRn2Spin%d", i), n2MR[i]);
                if(n1MR[i] < min_resid[i])
                   IntToUI(g_strdup_printf("IlmTasksMRn1Spin%d", i), min_resid[i]);
                if(n2MR[i] > max_resid[i])
                   IntToUI(g_strdup_printf("IlmTasksMRn2Spin%d", i), max_resid[i]);
                if(n1MR[i] > n2MR[i])
                   IntToUI(g_strdup_printf("IlmTasksMRn1Spin%d", i), n2MR[i]);
             }
             else
             {
                IntToUI(g_strdup_printf("IlmTasksMRn1Spin%d", i), min_resid[i]);
                IntToUI(g_strdup_printf("IlmTasksMRn2Spin%d", i), max_resid[i]);
             }
          }
          else
             gtk_widget_set_sensitive(LookupWidget(g_strdup_printf("IlmTasksMRCutCheck%d", i)), FALSE);
          BoolToUI(g_strdup_printf("IlmTasksMRRotCheck%d", i), bMRRot[i]);
          IntToUI(g_strdup_printf("IlmTasksMRRot1Spin%d", i), iMRalfa[i]);
          IntToUI(g_strdup_printf("IlmTasksMRRot2Spin%d", i), iMRbeta[i]);
          IntToUI(g_strdup_printf("IlmTasksMRRot3Spin%d", i), iMRgamma[i]);
          BoolToUI(g_strdup_printf("IlmTasksMRTransCheck%d", i), bMRTrans[i]);
          DoubleToUI(g_strdup_printf("IlmTasksMRTrans1Spin%d", i), dMRxx[i]);
          DoubleToUI(g_strdup_printf("IlmTasksMRTrans2Spin%d", i), dMRyy[i]);
          DoubleToUI(g_strdup_printf("IlmTasksMRTrans3Spin%d", i), dMRzz[i]);
          IntToUI(g_strdup_printf("IlmTasksMRNmolSpin%d", i), nMRNmol[i]);
          BoolToUI(g_strdup_printf("IlmTasksMREdmCheck%d", i), bMREdm[i]);
          DoubleToUI(g_strdup_printf("IlmTasksMREdmSpin%d", i), dMREdmres[i]);
          gtk_widget_set_sensitive(LookupWidget(g_strdup_printf("IlmTasksMREdmSpin%d", i)), bMREdm[i]);
          set_tip ( LookupWidget(g_strdup_printf("IlmTasksMRn1Spin%d", i)), 
             g_strdup_printf("min = %d, max = %d", min_resid[i], max_resid[i]));
          set_tip ( LookupWidget(g_strdup_printf("IlmTasksMRn2Spin%d", i)), 
             g_strdup_printf("min = %d, max = %d", min_resid[i], max_resid[i]));
//190509          BoolToUI(g_strdup_printf("IlmTasksMRSeqRadio%d", i), bMRSeq[i]);
          DoubleToUI(g_strdup_printf("IlmTasksMRvalue%d", i), dMRvalue[i]);
          if(i < nMRModels)
             gtk_widget_show(LookupWidget(g_strdup_printf("IlmTasksMRModel%d",i)));
          else
             gtk_widget_hide(LookupWidget(g_strdup_printf("IlmTasksMRModel%d",i)));
      }
      BoolToUI("IlmTasksMRmRes", bMRRes);
      BoolToUI("IlmTasksMRaRes", !bMRRes);
      DoubleToUI("IlmTasksMRResMinSpin", dMRMinres);
      DoubleToUI("IlmTasksMRResMaxSpin", dMRMaxres);

      set_tip ( LookupWidget("IlmTasksMRResMinSpin"),
         g_strdup_printf("min = %.2f, max = %.2f", dresMax, dresMin));
      set_tip ( LookupWidget("IlmTasksMRResMaxSpin"),
         g_strdup_printf("min = %.2f, max = %.2f", dresMax, dresMin));

      BoolToUI("IlmTasksMRTSAuto", bMRTSAuto);
      BoolToUI("IlmTasksMRTSMan", !bMRTSAuto);
      DoubleToUI("IlmTasksMRTSRotSpin", dMRTSRot);
      DoubleToUI("IlmTasksMRTSRotLocSpin", dMRTSRotLoc);
      DoubleToUI("IlmTasksMRTSTraSpin", dMRTSTra);
      DoubleToUI("IlmTasksMRTSTraLocSpin", dMRTSTraLoc);

      FillInfo();
      bSyncRunning = false;
//      return;
    }
    else if( eType==ABI )
    {
      int nAbiBlocks1;

      BoolToUI("IlmTasksABIPattersonCheck", bAbiPatterson );
      BoolToUI("IlmTasksABIDMCheck", !bAbiPatterson );
      nAbiBlocks1 = nAbiBlocks;
      BoolToUI("IlmTasksABIDefaultsCheck", bAbiDefaults );
      if(bAbiDefaults)
         nAbiBlocks1 = 1;
      IntToUI("IlmTasksABIBlocks", nAbiBlocks1);
      gtk_widget_set_sensitive(LookupWidget("IlmTasksABIBlocks"), !bAbiDefaults);
      for(int i=0; i< nAbiBlocks1; i++ ) 
      {
        if(bAbiDefaults)
        {
           g_snprintf(name, sizeof(name), "IlmTaskABItG%d", i+1);
           IntToUI(name, 1);
           gtk_widget_set_sensitive(LookupWidget(name), FALSE);
           g_snprintf(name, sizeof(name), "IlmTaskABItS%d", i+1);
           IntToUI(name, 1);
           gtk_widget_set_sensitive(LookupWidget(name), FALSE);
           g_snprintf(name, sizeof(name), "IlmTaskABblock%dlabel", i+1);
           gtk_widget_set_sensitive(LookupWidget(name), FALSE);
        }
        else
        {
           g_snprintf(name, sizeof(name), "IlmTaskABItG%d", i+1);
           gtk_widget_set_sensitive(LookupWidget(name), TRUE);
           IntToUI(name, nAbiTrialG[i]);
           g_snprintf(name, sizeof(name), "IlmTaskABItS%d", i+1);
           gtk_widget_set_sensitive(LookupWidget(name), TRUE);
           if (nAbiTrialS[i] > nAbiTrialG[i])
               IntToUI(name, nAbiTrialG[i]);
           else
               IntToUI(name, nAbiTrialS[i]);
           g_snprintf(name, sizeof(name), "IlmTaskABblock%dlabel", i+1);
           gtk_widget_set_sensitive(LookupWidget(name), TRUE);
        }
        gtk_widget_set_sensitive(LookupWidget("IlmTasksABIImproveFrame"), dresMax > dresExtramin);
        if(dresMax < dresExtramin)
           BoolToUI("IlmTasksABIExtraCheck", 0);
        else
        {
           BoolToUI("IlmTasksABIExtraCheck", iMad2FlexDM[2] != 0);
           BoolToUI("IlmTasksABIFLRadio", iEdmFreeLunch != 0);
        }
        std::stringstream os;
        os << "Observed reflections resolution range: " << dresMax << " - " << dresMin;
        set_label_text(LookupWidget("IlmTasksABILabel"), (char *)os.str().c_str(), RED, 0.99f);
      }
      BoolToUI("IlmTasksEdmSFCheck", iMad2FlexDM[0] != 0);
      BoolToUI("IlmTasksEdmHistCheck", iMad2FlexDM[1] != 0);
      if(dSolventFraction)
         DoubleToUI("IlmTasksEdmSolvFValue", dSolventFraction );
      else
         DoubleToUI("IlmTasksEdmSolvFValue", SolventFraction() );
      std::stringstream fsc;
      fsc << std::setprecision(2);
      fsc << "(Computed value from cell content: " << SolventFraction() << ")";
      LabelToUI("SolventFractionC", fsc.str().c_str());
      ListToUI("IlmTasksEdmConvCombo", iEdmConv);
      ListToUI("IlmTasksEdmExtCombo", iEdmExt);

      
      gboolean ifRecov = g_file_test(GetFilename(REC_EXT).c_str(), G_FILE_TEST_EXISTS);
      gtk_widget_set_sensitive(LookupWidget("framerec"), ifRecov);
      if(ifRecov)
      {
           std::ifstream if1; 
           if1.open(GetFilename(REC_EXT).c_str()); 
           if( if1 )
           {
               char mtzname[PATH_LEN];
               std::string labstr;
               std::string colstr;
               int ntrials;
               VMTZCOL MtzCols;
               if1.getline(mtzname, sizeof(mtzname), '\n');
               labstr.append(mtzname);
               std::string pathname = g_build_filename(g_path_get_dirname(GetFilename(REC_EXT).c_str()),
                                                        mtzname, NULL);
               LabelToUI("IlmTasksABIMtzFileLab", pathname.c_str());
               if(GetMtzCols(pathname, colstr) == 0)
                  LabelToUI("IlmTasksABIMtzInfoLab", colstr.c_str());

               if1 >> ntrials;
               std::string s;
               getline(if1, s);
               GtkWidget *w = LookupWidget("IlmTasksABItrialsel");
               clear_list(w);
               for(int i=0; i< ntrials; i++)
               {
                   getline(if1, s);
                   add_gen_line_to_list(w, 3, get_nth_field(s.c_str(), ";", 1, 'p'),
                                              get_nth_field(s.c_str(), ";", 2, 'p'),
                                              get_nth_field(s.c_str(), ";", 3, 'p'));
               }
               if1.close();
           }

      }

      FillInfo();
      bSyncRunning = false;
      //return;
    }
//  DEA
    else if( eType == DEA)
    {
      IntToUI("IlmTasksDeaCycles", nCycles);
      if(find_program_in_path(amb_prg[nBuildPgm]))
         ListToUI("IlmTasksDeaCombo", nBuildPgm);
      else
         ListToUI("IlmTasksDeaCombo", 0);
      //ListToUI("IlmTasksDeaCombo", nBuildPgm);
      BoolToUI("IlmTasksDeaModelRadio", bModel);
      BoolToUI("IlmTasksDeaPhasesRadio", !bModel);
      TextToUI("IlmTasksDeaPDBModel", szMRFile[0]);
      TextToUI("IlmTasksDeaPDBPhases", szMRFile[1]);
      TextToUI("IlmDeaMtzFileEntry", szMtzFilePh);
      if(Ilm::PTheProject->FillDeaFromMTZ())
         TextWToUI("IlmDeaMtzColumnsEntry", szSelectedColumnsPh);
      FillInfo();
      bSyncRunning = false;
//      return;
    }
    else
    {
    
    //PROFLEX
    BoolToUI("IlmTasksProCheck", bPro);
    gtk_widget_set_sensitive(LookupWidget("IlmTasksProBox"), bPro);
    gtk_widget_set_sensitive(LookupWidget("IlmTasksMad1FullBox"), !bPro );
//    gtk_widget_set_sensitive(LookupWidget("IlmTasksMad2FullBox"), !bPro);
    gtk_widget_set_sensitive(LookupWidget("IlmTasksMad2Check"), !bPro);
    BoolToUI("IlmTasksMad1Check", bMad1 && !bPro);
    gtk_widget_set_sensitive(LookupWidget( "IlmTasksMad1Box"), bMad1 && !bPro);
    BoolToUI("IlmTasksMad2Check", bMad2 && !bPro);
    BoolToUI("IlmTasksEdmCheck", bEdm);
    gtk_widget_set_sensitive(LookupWidget( "IlmTasksEdmVBox"), bEdm);
    if (eType != EDM)
        gtk_widget_set_sensitive(LookupWidget( "IlmTasksImproveFrame"), bEdm & (bMad2 | bPro));
//021007        gtk_widget_set_sensitive(LookupWidget( "IlmTasksImproveFrame"), bEdm);
    gtk_widget_set_sensitive(LookupWidget( "IlmTasksEdmFrame"), bMad2 || bPro);
    gtk_widget_set_sensitive(LookupWidget( "IlmTasksMad2Box"), bMad2 && !bPro);
    BoolToUI("IlmTasksMad1AutoCheck", bMad1Auto);
    {
      GtkEntry *ent = GTK_ENTRY(LookupWidget( "IlmTasksMad2FileEntry"));
      if( bMad1 && BoolFromUI("IlmTasksBestTrialsRadio"))
        gtk_entry_set_text(ent, " "); //GetFilename(MAD1FRAC_EXT).c_str());
      else
        gtk_entry_set_text(ent, szMad2File);
      if(bMad1 || gext::str_is_white(szMad2File))
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(LookupWidget("IlmTasksBestTrialsRadio")), true);
      gtk_widget_set_sensitive(LookupWidget("IlmTasksFilePositionRadio"), !bMad1);
//      gtk_widget_set_sensitive(LookupWidget("IlmTasksMad2FileBox"), !bMad1);

//      gtk_widget_set_sensitive(GTK_WIDGET(ent), !bMad1);
      GtkWidget *w = LookupWidget("IlmTasksMad2BrowseBtn");
      gtk_widget_set_sensitive(w, !bMad1);
    }

    IntToUI("IlmTasksMad1NreflSpin", nMad1Refls);
    if (nMad1Refls > nNrRefl )
      IntToUI("IlmTasksMad1NreflSpin", nNrRefl);
    BoolToUI("IlmTasksMad1NreflCheck", nMad1Refls==0);
    //DoubleToUI("IlmTasksMad1MaxresSpin", dMad1Maxres>0.0?dMad1Maxres:0.001);
    DoubleToUI("IlmTasksMad1MaxresSpin", dMad1Maxres>0.0?dMad1Maxres:dresMin);

    if (dMad1Maxres > dresMin) 
      DoubleToUI("IlmTasksMad1MaxresSpin", dresMin);
    if (dMad1Maxres < dresMax) 
      DoubleToUI("IlmTasksMad1MaxresSpin", dresMax);

    BoolToUI("IlmTasksMad1MaxresCheck", dMad1Maxres<=0.0);
    gtk_widget_set_sensitive(LookupWidget("IlmTasksMad1MaxresSpin"), dMad1Maxres>0.0);
    IntToUI("IlmTasksMad1Ntrials1Spin", nMad1Trials1);
    IntToUI("IlmTasksMad1Ntrials2Spin", nMad1Trials2);
//    BoolToUI("IlmTasksMad1NtrialsCheck", nMad1Trials1<=0 && nMad1Trials2<=0);
    BoolToUI("IlmTasksMad1NtrialsCheck", bMad1Defaults);
    bool enab = !bMad1Defaults;                 //nMad1Trials1>0 && nMad1Trials2>0;
    BoolToUI("IlmTasksMad1PattersonCheck", bMad1Patterson);
    BoolToUI("IlmTasksMad1DMCheck", !bMad1Patterson);
    BoolToUI("IlmTasksMad1MorePowerCheck", bMad1MorePower);
    gtk_widget_set_sensitive(LookupWidget("IlmTasksMad1Ntrials1Spin"), enab);
    gtk_widget_set_sensitive(LookupWidget("IlmTasksMad1Ntrials2Spin"), enab);
    gtk_widget_set_sensitive(LookupWidget("IlmTasksBestTrialsRadio"), enab || !bMad1);
    gtk_widget_set_sensitive(LookupWidget("IlmTasksMad2MinSpin"), enab || !bMad1);
    gtk_widget_set_sensitive(LookupWidget("IlmTasksMad2MaxSpin"), enab || !bMad1);

    if(iMad2SetMax <= nMad1Trials2)
       IntToUI("IlmTasksMad2MaxSpin", iMad2SetMax);
    else
       IntToUI("IlmTasksMad2MaxSpin", nMad1Trials2);
    if(bMad1 && bMad2 && (nMad1Trials2 > 0))
    {
       if((eType != MIR) && (eType != MRS) )
          set_label_text(LookupWidget("IlmTasksMad2MaxLabel"),
              g_strdup_printf("(max %d)", nMad1Trials2), MyBLUE, 1.0f);
       else  if (nDeriv > 0)
          set_label_text(LookupWidget("IlmTasksMad2MaxLabel"),
              g_strdup_printf("(max %d)", nMad1Trials2/nDeriv), MyBLUE, 1.0f);

//        set_label_text(LookupWidget("IlmTasksMad2MaxLabel"),
//              g_strdup_printf("(max %d)", nMad1Trials2), MyBLUE, 1.0f);
    }
    IntToUI("IlmTasksMad2MinSpin", iMad2SetMin);
    {
      int nwaves = 0;
      nwaves = std::accumulate(bMad2Waves, bMad2Waves + sizeof(bMad2Waves), nwaves);
      gtk_widget_set_sensitive(LookupWidget("IlmTasksMad2LsqF1Check"), nwaves>1);
      // gtk_widget_set_sensitive(LookupWidget("IlmTasksMad2LsqF2Check"), nwaves>1);
    }
    gtk_widget_set_sensitive(LookupWidget("IlmTasksMad2LsqBox1"), bMad2Lsq);
//    gtk_widget_set_sensitive(LookupWidget("IlmTasksMad2LsqBox2"), bMad2Lsq);
    BoolToUI("IlmTasksMad2LsqCheck", bMad2Lsq);
    BoolToUI("IlmTasksMad2LsqPosCheck", bMad2LsqPos);
    BoolToUI("IlmTasksMad2LsqAdpCheck", bMad2LsqAdp);
    BoolToUI("IlmTasksMad2LsqSofCheck", bMad2LsqSof);
    BoolToUI("IlmTasksMad2LsqF1Check", bMad2LsqF1);
    BoolToUI("IlmTasksMad2LsqF2Check", bMad2LsqF2);
    // EDM
    BoolToUI("IlmTasksEdmSFCheck", iMad2FlexDM[0] != 0);
    BoolToUI("IlmTasksEdmHistCheck", iMad2FlexDM[1] != 0);
    if(dSolventFraction)
       DoubleToUI("IlmTasksEdmSolvFValue", dSolventFraction );
    else
       DoubleToUI("IlmTasksEdmSolvFValue", SolventFraction() );
    std::stringstream fsc;
    fsc << std::setprecision(2);
    fsc << "(Computed value from cell content: " << SolventFraction() << ")";
    LabelToUI("SolventFractionC", fsc.str().c_str());

    gtk_widget_set_sensitive(LookupWidget("IlmTasksEdmExCheck"), dresMax > dresExtramin);
    gtk_widget_set_sensitive(LookupWidget("IlmTasksEdmFLCheck"), dresMax > dresExtramin);
    if(dresMax < dresExtramin)
    {
       BoolToUI("IlmTasksEdmExCheck", 0);
       BoolToUI("IlmTasksEdmFLCheck", 0);
    }
    else
    {
       BoolToUI("IlmTasksEdmExCheck", iMad2FlexDM[2] != 0);
       BoolToUI("IlmTasksEdmFLCheck", iEdmFreeLunch  != 0);
    }
    BoolToUI("IlmTasksEdmFLDefaults", bFLDefaults);
    BoolToUI("IlmTasksEdmExtraDefaults", bExtraDefaults);
    SetSpinValues("IlmTasksEdmExtraSpin", MRmin(dresMax, 1), dresMax, dresExtra);
    SetSpinValues("IlmTasksEdmFLSpin", MRmin(dresMax, 1), dresMax, dresFL);
    BoolToUI("IlmTasksEdmFlexCheck", bProFlex);
    std::stringstream os;
    os << "Observed reflections resolution range: " << dresMax << " - " << dresMin;
    set_label_text(LookupWidget("IlmTasksEDMLabel"), (char *)os.str().c_str(), RED, 0.98f);



// new     Strategy

    IntToUI("IlmStrategySpinValue", nFrag);
    if(iStrategy)
    {
       const char *name = g_strdup_printf("IlmStrategy%1.1dRadio", iStrategy);
       BoolToUI(name, true);
    }

    for(int i=0; i<iFin; i++ ) {
      std::stringstream name;
      name << "IlmTasksProDeriv" << i+1 << "Check";
      BoolToUI(name.str().c_str(), bProDerivs[i]);
      gtk_widget_set_sensitive(LookupWidget(name.str().c_str()), i<nNum);
      name.str("");
      name << "IlmTasksMad1Wave" << i+1 << "Check";
      BoolToUI(name.str().c_str(), bMad1Waves[i]);
      gtk_widget_set_sensitive(LookupWidget(name.str().c_str()), i<nNum);
      name.str("");
      name << "IlmTasksMad2Wave" << i+1 << "Check";
      BoolToUI(name.str().c_str(), bMad2Waves[i]);
      gtk_widget_set_sensitive(LookupWidget(name.str().c_str()), i<nNum);
    }
    IntToUI("IlmTasksProNtrialsSpin", nProTrials);
    IntToUI("IlmTasksProNstoreSpin", nProStore);
    IntToUI("IlmTasksProNuseSpin", nProUse);
    gtk_widget_set_sensitive(LookupWidget("IlmTasksProNtrialsSpin"), nProTrials!=-1);
    gtk_widget_set_sensitive(LookupWidget("IlmTasksProNstoreSpin"), nProTrials!=-1);
    gtk_widget_set_sensitive(LookupWidget("IlmTasksProNuseSpin"), nProTrials!=-1);
    BoolToUI("IlmTasksProNtrialsCheck", nProTrials==-1);
    }
 
    // project menu
    switch (eType)
    {
    case ABI:
    case MOLREP:
      TextToOutMenu("Full", OUT_EXT, eType-1);
      TextToOutMenu("Info", INFO_EXT, eType-1);
      TextToInMenu("Task", GetFilename2(INP_EXT, eType-1).c_str());
      TextToOutMenu("Html", HTML_EXT, eType-1);
      TextToOutMenu("Mtz", MTZ_EXT, eType-1);
      break;
    case EDM:
      TextToOutMenu("Edm", MTZ_EXT, eType-1);
      TextToOutMenu("Full", OUT_EXT, eType-1);
      TextToOutMenu("Info", INFO_EXT, eType-1);
      TextToInMenu("Task", GetFilename2(INP_EXT, eType-1).c_str());
      TextToOutMenu("Html", HTML_EXT, eType-1);
      TextToOutMenus("Edm1", EDM_EXT1);
      if(!bEdm)
          obj_hide(LookupObject("IlmMenuOutEdm"));
       else
          obj_show(LookupObject("IlmMenuOutEdm"));
      if(iEdmFreeLunch)
         obj_show(LookupObject("IlmMenuOutEdm1"));
      else
         obj_hide(LookupObject("IlmMenuOutEdm1"));
      break;
    case MAD:
    case MRS:
    case MIR:
      for(int ind=0; ind<3; ind++)
      {
          char nome[60];
          ScriviStringa(nome, sizeof(nome), "Task%d", ind);
          TextToInMenu(nome, GetFilename2(INP_EXT, ind).c_str());
          ScriviStringa(nome, sizeof(nome), "Full%d", ind);
          TextToOutMenu(nome, OUT_EXT, ind);
      }
      if(!bPro)
      {
         obj_hide(LookupObject("IlmMenuOutFull0"));
         obj_hide(LookupObject("IlmMenuInTask0"));
      }
      else
      {
         obj_show(LookupObject("IlmMenuOutFull0"));
         obj_show(LookupObject("IlmMenuInTask0"));
      }
      if(!bMad1)
      {
          obj_hide(LookupObject("IlmMenuOutFull1"));
          obj_hide(LookupObject("IlmMenuInTask1"));
      }
      else
      {
          obj_show(LookupObject("IlmMenuOutFull1"));
          obj_show(LookupObject("IlmMenuInTask1"));
      }
      if(!bMad2)
      {
          obj_hide(LookupObject("IlmMenuOutFull2"));
          obj_hide(LookupObject("IlmMenuInTask2"));
      }
      else
      {
          obj_show(LookupObject("IlmMenuOutFull2"));
          obj_show(LookupObject("IlmMenuInTask2"));
      }
      if(eType == MAD)
      {
         obj_hide(LookupObject("IlmMenuOutFull0"));
         obj_hide(LookupObject("IlmMenuInTask0"));
      }
      TextToOutMenus("Phase", PHASE_EXT);
      if(!bEdm)
          obj_hide(LookupObject("IlmMenuOutEdm"));
      else
      {
          obj_show(LookupObject("IlmMenuOutEdm"));
          TextToOutMenus("Map1", EDM_MAP1);
      }
      TextToOutMenu("Edm", MTZ_EXT, EDM-1);
      TextToOutMenus("Html", HTML_EXT);
      TextToOutMenus("Info", INFO_EXT);
      if(iEdmFreeLunch)
      {
         obj_show(LookupObject("IlmMenuOutEdm1"));
         TextToOutMenus("Edm1", EDM_EXT1);
         TextToOutMenus("Map2", EDM_MAP2);
      }
      else
         obj_hide(LookupObject("IlmMenuOutEdm1"));
      break;
    case DEA:
      break;
    }

    FillInfo();
    bSyncRunning = false;
  }


  void GtkProject::DataFromUI()
  {
    if( bSyncRunning )
      return;
    bSyncRunning = true;
    nomefile name;
    char wavele[30];
    int i;

    TextFromUI(szJobTitle,"IlmJobEntry");
    std::locale cloc ( "C" );

//  MTZ
    TextFromUI(szMtzFile,"IlmMtzFileEntry");
    TextWFromUI(szSelectedColumns, "IlmMtzColumnsEntry");
    TextFromUI(szSpgrName,"IlmMtzSpgEntry");                         // 230107
    bFindSpace = BoolFromUI("IlmMtzSpgFind");

    // content 
    if( eType!=MIR )
      bNativePresent = BoolFromUI("IlmContNativeCheck");
    nWaves = IntFromUI("IlmContWavesSpin");
    nDeriv = IntFromUI("IlmContDerivSpin");
//  MRS
    for( i=1; i<=MAX_DERIVATIVES; i++ ) {
      std::stringstream strm;
      strm << "IlmContDeriv" << i << "Spin";
      nMrsWaves[i-1] = IntFromUI(strm.str().c_str());
    }

    iCont = 0;
    if(BoolFromUI("IlmContResiduesRadio"))
       iCont = 1;
    else if(BoolFromUI("IlmContSeqRadio"))
       iCont = 2;
    memset(szSeqFile, 0, sizeof(szSeqFile));
    szSeqFile[0] = ' ';
    nMolecules = IntFromUI("IlmContNmolsSpin");
    switch(iCont)
    {
    case 1:
      dContent = IntFromUI("IlmContResiduesSpin");
      break;
    case 2:
      TextFromUI(szSeqFile,"IlmContSeqFile");
      if(gext::str_is_white(szSeqFile))
      {
         CopiaStringa(szSeqFile, sizeof(szSeqFile), GetFilename(SEQ_EXT).c_str());
         WriteSequence();
      }
      {
         std::string sequence_txt = GetTextW(LookupWidget("IlmContSeq"));
         dContent = ContentFromSequenceStr(sequence_txt);
      }
      break;
    case 0:
      dContent = DoubleFromUI("IlmContMolSpin");
      break;
    }
    
    if( eType==MRS || eType==MOLREP || eType==ABI || eType==EDM)
    {
      nElements = 0;
      for(i=0; i<MAX_ELEMENTS; i++ ) {
        g_snprintf(name, sizeof(name), "IlmContLigSymbol%dEntry", i+1);
        TextFromUI(szElement[i], name);
        if( gext::str_is_white(szElement[i]) )
        {
          nElementAmount[i] = 0;                       // 070506
          continue; // skip empty strings
        }
        for (int kk=0; kk < (int) strlen(szElement[i]); kk++)
        {
             szElement[i][kk] = g_ascii_toupper( szElement[i][kk]);
             if(szElement[i][kk] != ' ')
                break;
        }
        g_snprintf(name, sizeof(name), "IlmContLigAmount%dSpin", i+1);
        nElementAmount[i] = IntFromUI(name);
        if( nElements < i ) 
        {
          g_strlcpy(szElement[nElements], szElement[i], MAX_SYMBOL_LEN);
          nElementAmount[nElements] = nElementAmount[i];
          szElement[i][0] = 0;
          nElementAmount[i] = 0;
        }
        nElements++;
      }
    }
    
    for(i=0; i< (int)(sizeof(Ilm::Sources)/sizeof(Ilm::Sources[0])); i++)
    {
        if (BoolFromUI(Ilm::Sources[i]))
        {
            nSource = i;
            break;
        }
    }
    for(i=0; i< (int)(sizeof(Ilm::Waves)/sizeof(Ilm::Waves[0])); i++)
    {
        if (BoolFromUI(Ilm::Waves[i]))
        {
            nWaveLength = i;
            break;
        }
    }
    
    TextFromUI(wavele, "IlmContWaveLEntry");
    if(gext::str_is_white(wavele))
       dWaveLength = 0;
    else
       dWaveLength = atof(wavele);

    if( eType==MAD || eType==MRS ) {
      int nel = 0;
      dMad1Maxres = DoubleFromUI("IlmTasksMad1MaxresSpin");
      nMad1Refls = IntFromUI("IlmTasksMad1NreflSpin");
      if( eType==MAD )
        nElements = nel;
      else
        nMirElements[0] = nel;
      for(i=0; i<MAX_ELEMENTS; i++ ) {
        g_snprintf(name, sizeof(name), "IlmContSymbol%dEntry", i+1);
        if( eType==MAD )
          TextFromUI(szElement[i], name);
        else
          TextFromUI(szMirElement[i][0], name);
        if( gext::str_is_white(szElement[i]) && eType==MAD)
          continue; // projti pustye strochki
        if( gext::str_is_white(szMirElement[i][0]) && eType==MRS )
          continue;
        if( eType==MAD )
        {
            for (int kk=0; kk < (int) strlen(szElement[i]); kk++)
            {
               szElement[i][kk] = g_ascii_toupper( szElement[i][kk]);
               if(szElement[i][kk] != ' ')
                  break;
            }
        }
        else
        {
            for (int kk=0; kk < (int) strlen(szMirElement[i][0]); kk++)
            {
               szMirElement[i][0][kk] = g_ascii_toupper( szMirElement[i][0][kk]);
               if(szMirElement[i][0][kk] != ' ')
                  break;
            }
        }
        g_snprintf(name, sizeof(name), "IlmContAmount%dSpin", i+1);
        if( eType==MAD )
          nElementAmount[i] = IntFromUI(name);
        else
          nMirElementAmount[i][0] = IntFromUI(name);
  
        for(int j=0; j<MAX_WAVES; j++ ) {
          for(int k=0; k<2; k++ ) {
            g_snprintf(name, sizeof(name), "IlmContE%dW%dF%dSpin", i+1, j+1, k+1);
            dElementF1F2[i][j][k] = DoubleFromUI(name);
            if( (eType==MAD && j>=nWaves) || (eType==MRS && (i>=nDeriv || j>=nMrsWaves[i])) )
              dElementF1F2[i][j][k] = 0.0;
          }
        }
  
        if( nel < i ) {
          if( eType==MAD ) {
            g_strlcpy(szElement[nel], szElement[i], MAX_SYMBOL_LEN);
            nElementAmount[nel] = nElementAmount[i];
          }
          else {
            g_strlcpy(szMirElement[nel][0], szMirElement[i][0], MAX_SYMBOL_LEN);
            szMirElement[i][0][0] = 0;
            nMirElementAmount[nel][0] = nMirElementAmount[i][0];
            nMirElementAmount[i][0] = 0;
          }
        }
        ++nel;
        if( eType==MAD )
          nElements = nel;
        else
          nMirElements[0] = nel;
      }
    } else if( eType==MIR ) {
      for(i=0; i<=MAX_DERIVATIVES; i++)
      {
        if( i==0 ) {
          nElements = 0;
          memset(nElementAmount, 0, sizeof(nElementAmount));
          memset(szElement, 0, sizeof(szElement));          
        } else {
          nMirElements[i-1] = 0;
          memset(nMirElementAmount[i-1], 0, sizeof(nMirElementAmount[i-1]));
          memset(szMirElement[i-1], 0, sizeof(szMirElement[i-1]));
        }
//mr        if( i<nWaves+1 )
        if( i<=nDeriv )
        {
          for( int j=0; j<MAX_ELEMENTS; j++ )
          {
            std::stringstream nm;
            nm << "IlmContMir" << i << "Symbol" << j+1 << "Entry";
            if( i==0 ) {
              TextFromUI(szElement[j], nm.str().c_str());
              if( gext::str_is_white(szElement[j]) )
                continue;
              for (int kk=0; kk < (int) strlen(szElement[j]); kk++)
              {
                   szElement[j][kk] = g_ascii_toupper( szElement[j][kk]);
                   if(szElement[j][kk] != ' ')
                      break;
              }
            } else {
              TextFromUI( szMirElement[i-1][j], nm.str().c_str());
              if( gext::str_is_white(szMirElement[i-1][j]) )
                continue;
              for (int kk=0; kk < (int) strlen(szMirElement[i-1][j]); kk++)
              {
                   szMirElement[i-1][j][kk] = g_ascii_toupper( szMirElement[i-1][j][kk]);
                   if(szMirElement[i-1][j][kk] != ' ')
                      break;
              }
            }
            nm.str("");
            nm << "IlmContMir" << i << "Amount" << j+1 << "Spin";
            if( i==0 ) {
              nElementAmount[j] = IntFromUI(nm.str().c_str());
              if( nElements<j ) {
                g_strlcpy(szElement[nElements], szElement[j], MAX_SYMBOL_LEN);
                nElementAmount[nElements] = nElementAmount[j];
                szElement[j][0] = '\0';
                nElementAmount[j] = 0;
              }
              ++nElements;
            } else {
              nMirElementAmount[i-1][j] = IntFromUI(nm.str().c_str());
              if( nMirElements[i-1]<j ) {
                g_strlcpy(szMirElement[i-1][nMirElements[i-1]], szMirElement[i-1][j], MAX_SYMBOL_LEN);
                nMirElementAmount[i-1][nMirElements[i-1]] = nMirElementAmount[i-1][j];
                szMirElement[i-1][j][0] = '\0';
                nMirElementAmount[i-1][j] = 0;
              }
              ++nMirElements[i-1];
            }
          }
        }
      }
    }
    
    // tasks
    if( eType==MOLREP )
    {
      bPro = bMad1 = bMad2 = false;
      // dritan 
      bEdm = false;
      //
      nMRModels = IntFromUI("IlmTasksMRnModels");
      bMRUsePseudo = BoolFromUI("IlmTasksMRPseudoCheck");
      for(i=0; i<nMRModels; i++)
      {
          TextFromUI(szMRFile[i], g_strdup_printf("IlmTasksMRFileEntry%d",i));
          bMRUseHetero[i] = BoolFromUI(g_strdup_printf("IlmTasksMRHeteroCheck%d", i));
          if( BoolFromUI(g_strdup_printf("IlmTasksMRPdbRadio%d", i)) )
            cMRAdp[i] = 'P';
          else if( BoolFromUI(g_strdup_printf("IlmTasksMRWilsonRadio%d", i)) )
            cMRAdp[i] = 'W';
          else
            cMRAdp[i] = '2';
          bMRPolyalanine[i] = BoolFromUI(g_strdup_printf("IlmTasksMRPolyalaCheck%d", i));
          bMRCutModel[i] = BoolFromUI(g_strdup_printf("IlmTasksMRCutCheck%d", i));
          if(bMRCutModel[i])
          {
             n1MR[i] = IntFromUI(g_strdup_printf("IlmTasksMRn1Spin%d", i));
             n2MR[i] = IntFromUI(g_strdup_printf("IlmTasksMRn2Spin%d", i));
             if(n1MR[i] < min_resid[i])
                n1MR[i] = min_resid[i];
             if(n2MR[i] > max_resid[i])
                n2MR[i] = max_resid[i];
             if(n2MR[i] < n1MR[i])
                n1MR[i] = n2MR[i];
          }
          bMRRot[i] = BoolFromUI(g_strdup_printf("IlmTasksMRRotCheck%d", i));
          if(bMRRot[i])
          {
             iMRalfa[i]  = IntFromUI(g_strdup_printf("IlmTasksMRRot1Spin%d", i));
             iMRbeta[i]  = IntFromUI(g_strdup_printf("IlmTasksMRRot2Spin%d", i));
             iMRgamma[i] = IntFromUI(g_strdup_printf("IlmTasksMRRot3Spin%d", i));
          }
          bMRTrans[i] = BoolFromUI(g_strdup_printf("IlmTasksMRTransCheck%d", i));
          if(bMRTrans[i])
          {
             dMRxx[i] = DoubleFromUI(g_strdup_printf("IlmTasksMRTrans1Spin%d", i));
             dMRyy[i] = DoubleFromUI(g_strdup_printf("IlmTasksMRTrans2Spin%d", i));
             dMRzz[i] = DoubleFromUI(g_strdup_printf("IlmTasksMRTrans3Spin%d", i));
          }
          nMRNmol[i] = IntFromUI(g_strdup_printf("IlmTasksMRNmolSpin%d", i));
          bMREdm[i] = BoolFromUI(g_strdup_printf("IlmTasksMREdmCheck%d", i));
          dMREdmres[i] = DoubleFromUI(g_strdup_printf("IlmTasksMREdmSpin%d", i));
          bMRSeq[i] = 1;  //190509 BoolFromUI(g_strdup_printf("IlmTasksMRSeqRadio%d", i));
          dMRvalue[i] = DoubleFromUI(g_strdup_printf("IlmTasksMRvalue%d", i));
      }

      bMRTSAuto = BoolFromUI("IlmTasksMRTSAuto");
      if(bMRTSAuto)
      {
         dMRTSRot = 0;
         dMRTSRotLoc = 0;
         dMRTSTra = 0;
         dMRTSTraLoc = 0;
      }
      else
      {
         dMRTSRot = DoubleFromUI("IlmTasksMRTSRotSpin");
         dMRTSRotLoc = DoubleFromUI("IlmTasksMRTSRotLocSpin");
         dMRTSTra = DoubleFromUI("IlmTasksMRTSTraSpin");
         dMRTSTraLoc = DoubleFromUI("IlmTasksMRTSTraLocSpin");
      }
      bMRRes = BoolFromUI("IlmTasksMRmRes");
      if(bMRRes)
      {
         dMRMinres = DoubleFromUI("IlmTasksMRResMinSpin");
         dMRMaxres = DoubleFromUI("IlmTasksMRResMaxSpin");
         if(dMRMinres < dresMax)
            dMRMinres = dresMax;
         if(dMRMaxres > dresMin)
            dMRMaxres = dresMin;
         if(dMRMaxres < dMRMinres)
            dMRMaxres = dMRMinres;
         
      }
      else
      {
         dMRMinres = dresMax;
         dMRMaxres = dresMin;
      }

      iStrategy = 0;
      bSyncRunning = false;
      return;
    }
    if( eType==ABI )
    {
      bPro = bMad1 = bMad2 = false;
      bEdm = false;
      bAbiPatterson = BoolFromUI("IlmTasksABIPattersonCheck");
//050208      if(bAbiPatterson)
//050208         nAbiBlocks=1;
//050208      else
         nAbiBlocks=IntFromUI("IlmTasksABIBlocks");
      bAbiDefaults = BoolFromUI("IlmTasksABIDefaultsCheck");
      if(bAbiDefaults)
      {
         nAbiBlocks=1;
         gtk_widget_show(LookupWidget("IlmTaskABItG1"));
         nAbiTrialG[0] = 1;
         gtk_widget_show(LookupWidget("IlmTaskABItS1"));
         nAbiTrialS[0] = 1;
         gtk_widget_show(LookupWidget("IlmTaskABblock1label"));
      }
      else
      {
         for(i=0; i< nAbiBlocks; i++ ) 
         {
            g_snprintf(name, sizeof(name), "IlmTaskABItG%d", i+1);
            gtk_widget_show(LookupWidget(name));
            nAbiTrialG[i] = IntFromUI(name);
            g_snprintf(name, sizeof(name), "IlmTaskABItS%d", i+1);
            gtk_widget_show(LookupWidget(name));
            nAbiTrialS[i] = IntFromUI(name);
            if (nAbiTrialS[i] > nAbiTrialG[i])
            {
               nAbiTrialS[i] = nAbiTrialG[i];
            }
            g_snprintf(name, sizeof(name), "IlmTaskABblock%dlabel", i+1);
            gtk_widget_show(LookupWidget(name));
         }
      }
      for(i=nAbiBlocks+1; i<= MAX_ABI_BLOCKS; i++ ) 
      {
        g_snprintf(name, sizeof(name), "IlmTaskABItG%d", i);
        gtk_widget_hide(LookupWidget(name));
        nAbiTrialG[i-1] = 0;
        g_snprintf(name, sizeof(name), "IlmTaskABItS%d", i);
        gtk_widget_hide(LookupWidget(name));
        nAbiTrialS[i-1] = 0;
        g_snprintf(name, sizeof(name), "IlmTaskABblock%dlabel", i);
        gtk_widget_hide(LookupWidget(name));
      }
      iStrategy = 0;
      if(dresMax > dresExtramin)
      {
         iMad2FlexDM[2] = BoolFromUI("IlmTasksABIExtraCheck");
         iEdmFreeLunch = BoolFromUI("IlmTasksABIFLRadio");
      }
      else
      {
         iMad2FlexDM[2] = 0;
         iEdmFreeLunch = 0;
      }

      bEdm = BoolFromUI("IlmTasksEdmCheck");
      iMad2FlexDM[0] = BoolFromUI("IlmTasksEdmSFCheck");
      iMad2FlexDM[1] = BoolFromUI("IlmTasksEdmHistCheck");
      iEdmConv = ListFromUI("IlmTasksEdmConvCombo");
      iEdmExt = ListFromUI("IlmTasksEdmExtCombo");
      dSolventFraction = DoubleFromUI("IlmTasksEdmSolvFValue");

      bRecover =  BoolFromUI("IlmTasksABIRadio2");
      if(bRecover)
      {
         SetRecoverMTZ(LabelFromUI("IlmTasksABIMtzFileLab"));
         if(get_selected_row_col(LookupWidget("IlmTasksABItrialsel"), 0))
         {
            std::string s = get_selected_row_col(LookupWidget("IlmTasksABItrialsel"), 0);
            s.append(" ");
            s.append(get_selected_row_col(LookupWidget("IlmTasksABItrialsel"), 1));
            SetRecoverData(s.c_str());
            s.clear();
            s = get_selected_row_col(LookupWidget("IlmTasksABItrialsel"), 2);
            SetRecoverMTZLab(s.c_str());
         }
         else
            SetRecoverData("");
      }

      bSyncRunning = false;
      return;
    }

    if(eType == DEA)
    {
      nCycles = IntFromUI("IlmTasksDeaCycles");
      nBuildPgm = ListFromUI("IlmTasksDeaCombo");
      bModel = BoolFromUI("IlmTasksDeaModelRadio");
      memset(szMRFile[0], 0, sizeof(szMRFile[0]));
      memset(szMRFile[1], 0, sizeof(szMRFile[1]));
      memset(szMtzFilePh, 0, sizeof(szMtzFilePh));
      memset(szSelectedColumnsPh, 0, sizeof(szSelectedColumnsPh));
      memset(szSpgrNamePh, 0, sizeof(szSpgrNamePh));
      if(bModel)
         TextFromUI(szMRFile[0], "IlmTasksDeaPDBModel");
      else
      {
         TextFromUI(szMRFile[1], "IlmTasksDeaPDBPhases");
         TextFromUI(szMtzFilePh,"IlmDeaMtzFileEntry");
         TextWFromUI(szSelectedColumnsPh, "IlmDeaMtzColumnsEntry");
         TextFromUI(szSpgrNamePh,"IlmDeaMtzSpgEntry"); 
      }

      bSyncRunning = false;
      return;
    }

    bPro = BoolFromUI("IlmTasksProCheck");
    if( !bPro ) {
      bMad1 = BoolFromUI("IlmTasksMad1Check");
      bMad2 = BoolFromUI("IlmTasksMad2Check");
    } else 
      bMad1=bMad2=false;
    if( bMad1 )
    {
      g_strlcpy(szMad2File, " ", sizeof(szMad2File)); //GetFilename(Ilm::MAD1FRAC_EXT).c_str(), sizeof(szMad2File));
      iMad2SetMin = 1;
      iMad2SetMax = 10;
    }
    
    bMad1Auto = BoolFromUI("IlmTasksMad1AutoCheck");
    for(i=0; i<MAX_WAVES; i++ ) {
      g_snprintf(name, sizeof(name), "IlmTasksMad1Wave%dCheck", i+1);
      bMad1Waves[i] = BoolFromUI(name);
      if( bMad1Auto )
        bMad1Waves[i] = false;
    }
    for(i=0; i<MAX_WAVES; i++ ) {
      g_snprintf(name, sizeof(name), "IlmTasksMad2Wave%dCheck", i+1);
      bMad2Waves[i] = BoolFromUI(name);
    }
    nMad1Refls = IntFromUI("IlmTasksMad1NreflSpin");
    if( BoolFromUI("IlmTasksMad1MaxresCheck") )
      dMad1Maxres = 0.0;
    else
      dMad1Maxres = DoubleFromUI("IlmTasksMad1MaxresSpin");
    bMad1Patterson = BoolFromUI("IlmTasksMad1PattersonCheck");
    bMad1Defaults = BoolFromUI("IlmTasksMad1NtrialsCheck");
    if( bMad1Defaults )
    {
        if(bMad1Patterson)
           nMad1Trials1 = nMad1Trials2 = 30;
        else
           nMad1Trials1 = nMad1Trials2 = 60;
    }
    else
    {
      nMad1Trials1 = IntFromUI("IlmTasksMad1Ntrials1Spin"),
      nMad1Trials2 = IntFromUI("IlmTasksMad1Ntrials2Spin");
    }

    if( nMad1Trials2>nMad1Trials1 ) 
      nMad1Trials2 = nMad1Trials1;
    bMad1MorePower = BoolFromUI("IlmTasksMad1MorePowerCheck");

    if( !bMad1 )
    {
      memset(szMad2File, 0, sizeof(szMad2File));
      if (BoolFromUI("IlmTasksFilePositionRadio"))
          TextFromUI(szMad2File, "IlmTasksMad2FileEntry");
    }
    iMad2SetMin = IntFromUI("IlmTasksMad2MinSpin");
    iMad2SetMax = IntFromUI("IlmTasksMad2MaxSpin");
//100606    if( iMad2SetMax>nMad1Trials2 && bMad1)
    if( iMad2SetMax>nMad1Trials2)
      iMad2SetMax = nMad1Trials2;
    if( iMad2SetMin>iMad2SetMax )	
      iMad2SetMin = iMad2SetMax;
    if( bMad1Defaults && bMad1)
      iMad2SetMin = 1, iMad2SetMax = -1;
    bMad2Lsq = BoolFromUI("IlmTasksMad2LsqCheck");
    //if( bMad2Lsq )
    //{
      bMad2LsqPos = BoolFromUI("IlmTasksMad2LsqPosCheck");
      bMad2LsqAdp = BoolFromUI("IlmTasksMad2LsqAdpCheck");
      bMad2LsqSof = BoolFromUI("IlmTasksMad2LsqSofCheck");
      bMad2LsqF1 = BoolFromUI("IlmTasksMad2LsqF1Check");
      bMad2LsqF2 = BoolFromUI("IlmTasksMad2LsqF2Check");
      int nwaves = 0;
      nwaves = std::accumulate(bMad2Waves, bMad2Waves + sizeof(bMad2Waves), nwaves);
      if( nwaves <= 1 )
        bMad2LsqF1 = false;
      gtk_widget_set_sensitive(LookupWidget("IlmTasksMad2LsqF1Check"), nwaves>1);
    //}
    //else
    //  bMad2LsqPos = bMad2LsqAdp = bMad2LsqSof = bMad2LsqF1 = bMad2LsqF2 = false;

    bEdm = BoolFromUI("IlmTasksEdmCheck");
//021007    if(eType == EDM)
//021007       bEdm = true;
    if( bMad1 && !bMad2 ) bEdm = false;
    iMad2FlexDM[0] = BoolFromUI("IlmTasksEdmSFCheck");
    iMad2FlexDM[1] = BoolFromUI("IlmTasksEdmHistCheck");
    dSolventFraction = DoubleFromUI("IlmTasksEdmSolvFValue");
    bProFlex = BoolFromUI("IlmTasksEdmFlexCheck");
    if(dresMax > dresExtramin)
    {
       iMad2FlexDM[2] = BoolFromUI("IlmTasksEdmExCheck");
       iEdmFreeLunch = BoolFromUI("IlmTasksEdmFLCheck");
//021007       if(!bEdm)
//021007          iEdmFreeLunch = 0;
       if(!bEdm && (eType != EDM))
          iEdmFreeLunch = 0;
    }
    else
    {
       iMad2FlexDM[2] = 0;
       iEdmFreeLunch = 0;
    }
    bFLDefaults = BoolFromUI("IlmTasksEdmFLDefaults");
    bExtraDefaults = BoolFromUI("IlmTasksEdmExtraDefaults");
    dresExtra = (float) DoubleFromUI("IlmTasksEdmExtraSpin");
    dresFL = (float) DoubleFromUI("IlmTasksEdmFLSpin");
    iEdmConv = ListFromUI("IlmTasksEdmConvCombo");
    iEdmExt = ListFromUI("IlmTasksEdmExtCombo");
    
    for(i=0; i<MAX_DERIVATIVES; i++ ) {
      std::stringstream name;
      name << "IlmTasksProDeriv" << i+1 << "Check";
      bProDerivs[i] = BoolFromUI(name.str().c_str());
      if( i>=nDeriv )
        bProDerivs[i] = false;
    }
    nProTrials = IntFromUI("IlmTasksProNtrialsSpin");
    nProStore = IntFromUI("IlmTasksProNstoreSpin");
    nProUse = IntFromUI("IlmTasksProNuseSpin");
    if( BoolFromUI("IlmTasksProNtrialsCheck") )
      nProTrials = nProStore = nProUse = -1;
    if( nProStore>nProTrials )
      nProStore = nProTrials;
    if( nProUse>nProStore )
      nProUse = nProStore;

// new

    if(gtk_widget_get_visible(LookupWidget("IlmStrategyFrame")))
    {
       nFrag = IntFromUI("IlmStrategySpinValue");
       if (BoolFromUI("IlmStrategy2Radio"))
           iStrategy = 2;
       else if (BoolFromUI("IlmStrategy3Radio"))
           iStrategy = 3;
       else
           iStrategy = 1;
    }
    else
       iStrategy = 0;

// mr 25/05/06
    if(bMad1 && bMad2 && (eType != EDM))
       iStrategy = 1;
// end

    bSyncRunning = false;
  }

  void GtkProject::Synchronize()
  {
      DataFromUI();
      FillInterface();
  }

// M T Z File

  void add_mtz_val(const char *val, int quale)
  {
    VMTZCOL_it it, vit, eit;
    if(val == NULL)
       return;
    if(quale)
    {
       it = get_mtz_col(DeaMtzAllCols, val);
       if( it==DeaMtzAllCols.end() )
         return;
       DeaMtzSelected.push_back(*it);
       DeaMtzAllCols.erase(it);
       vit = get_mtz_col(DeaMtzValues, val);
       if( vit!=DeaMtzValues.end()) 
          DeaMtzValues.erase(vit);
       eit = get_mtz_col(DeaMtzEsds, val);
       if( eit!=DeaMtzEsds.end()) 
            DeaMtzEsds.erase(eit);
    }
    else
    {
       it = get_mtz_col(MtzAllCols, val);
       if( it==MtzAllCols.end() )
         return;
       MtzSelected.push_back(*it);
       MtzAllCols.erase(it);
       vit = get_mtz_col(MtzValues, val);
       if( vit!=MtzValues.end()) 
          MtzValues.erase(vit);
       eit = get_mtz_col(MtzEsds, val);
       if( eit!=MtzEsds.end()) 
            MtzEsds.erase(eit);
    }
  }
    
  void add_mtz_col(const char *val, int quale)
  {
    VMTZCOL_it it, vit, eit;
    if(val == NULL)
       return;
    if(quale)
    {
       it = get_mtz_col(DeaMtzAllCols, val);
       if( it==DeaMtzAllCols.end() )
         return;
       if(strstr(Ilm::PTheProject->szSelectedColumnsPh, val) == NULL)
          DeaMtzSelected.push_back(*it);
       DeaMtzAllCols.erase(it);
       vit = get_mtz_col(DeaMtzValues, val);
       if( vit!=DeaMtzValues.end()) 
          DeaMtzValues.erase(vit);
       eit = get_mtz_col(DeaMtzEsds, val);
       if( eit!=DeaMtzEsds.end()) 
            DeaMtzEsds.erase(eit);
    }
    else
    {
       it = get_mtz_col(MtzAllCols, val);
       if( it==MtzAllCols.end() )
         return;
       if(strstr(Ilm::PTheProject->szSelectedColumns, val) == NULL)
          MtzSelected.push_back(*it);
       MtzAllCols.erase(it);
       vit = get_mtz_col(MtzValues, val);
       if( vit!=MtzValues.end()) 
          MtzValues.erase(vit);
       eit = get_mtz_col(MtzEsds, val);
       if( eit!=MtzEsds.end()) 
            MtzEsds.erase(eit);
    }
  }
    
  bool add_mtz_pair(const char *val, const char *esd, int quale)
  {
    VMTZCOL_it vit, eit;
    char SelC[PATH_LEN];
    if(!val || !esd)
       return false;

    if(quale)
    {
       Ilm::PTheProject->TextWFromUI(SelC, "IlmDeaMtzColumnsEntry");
       vit = get_mtz_col(DeaMtzValues, val);
       eit = get_mtz_col(DeaMtzEsds, esd);
       if( vit==DeaMtzValues.end() || eit==DeaMtzEsds.end() )
         return false;
       if(strstr(SelC, val) == NULL)
          DeaMtzSelected.push_back(*vit);
       if(strstr(SelC, esd) == NULL)
          DeaMtzSelected.push_back(*eit);
       DeaMtzValues.erase(vit);
       DeaMtzEsds.erase(eit);
       vit = get_mtz_col(DeaMtzAllCols, val);
       DeaMtzAllCols.erase(vit);
       eit = get_mtz_col(DeaMtzAllCols, esd);
       DeaMtzAllCols.erase(eit);
    }
    else
    {
       Ilm::PTheProject->TextWFromUI(SelC, "IlmMtzColumnsEntry");
       vit = get_mtz_col(MtzValues, val);
       eit = get_mtz_col(MtzEsds, esd);
       if( vit==MtzValues.end() || eit==MtzEsds.end() )
         return false;
       if(strstr(SelC, val) == NULL)
          MtzSelected.push_back(*vit);
       if(strstr(SelC, esd) == NULL)
          MtzSelected.push_back(*eit);
       MtzValues.erase(vit);
       MtzEsds.erase(eit);
       vit = get_mtz_col(MtzAllCols, val);
       MtzAllCols.erase(vit);
       eit = get_mtz_col(MtzAllCols, esd);
       MtzAllCols.erase(eit);
    }
    return true;

  }
  
  void clear_mtz(int flag)
  {
    MtzSelected.clear();
    if(flag)
    {
       memset(Ilm::PTheProject->szSelectedColumns, 0, PATH_LEN);
       Ilm::PTheProject->FillMtzVectors(NULL);
       add_mtz_val("H", 0);
       add_mtz_val("K", 0);
       add_mtz_val("L", 0);
       if(MtzEsds.size() == 1)
          gtk_button_clicked(GTK_BUTTON(Ilm::PTheProject->LookupWidget("IlmMtzAddBtn")));
    }
  }
    
  void GtkProject::MtzFill(const char *widname)
  {
    bSyncRunning = false;
    FillInterface();
    gtk_button_clicked(GTK_BUTTON(LookupWidget(widname)));
  }

  void clear_mtzdea(int flag)
  {
    DeaMtzSelected.clear();
    if(flag)
    {
       memset(Ilm::PTheProject->szSelectedColumnsPh, 0, PATH_LEN);
       Ilm::PTheProject->DeaFillMtzVectors(NULL);
       add_mtz_val("H", 1);
       add_mtz_val("K", 1);
       add_mtz_val("L", 1);
       if(DeaMtzEsds.size() == 1)
          gtk_button_clicked(GTK_BUTTON(Ilm::PTheProject->LookupWidget("IlmDeaMtzAddBtn")));
    }
  }

  void update_deamtzpage(GtkWidget *dlg)
  {
    GList *vitems=NULL, *sitems=NULL, *aitems=NULL, *bitems=NULL;
    unsigned i;
    GtkWidget *w;

    for( i=0; i<DeaMtzEsds.size(); i++ )
      sitems = g_list_append(sitems, (gpointer) DeaMtzEsds[i]->label);
    for( i=0; i<DeaMtzValues.size(); i++ )
      vitems = g_list_append(vitems, (gpointer) DeaMtzValues[i]->label );

    for( i=0; i<DeaMtzAllCols.size(); i++ )
    {
      aitems = g_list_append(aitems, (gpointer) DeaMtzAllCols[i]->label );
      bitems = g_list_append(bitems, (gpointer) DeaMtzAllCols[i]->type );
    }

    w = lookup_widget(dlg, "IlmDeaMtzValueCombo");
//    gtk_entry_set_text( GTK_ENTRY(get_widget(w,"entry")), "" );
//    if(g_list_length(vitems))
       set_popdown_strings(w, vitems);
    w = lookup_widget(dlg, "IlmDeaMtzEsdCombo");
//    gtk_entry_set_text( GTK_ENTRY(get_widget(w,"entry")), "" );
//    if(g_list_length(sitems))
       set_popdown_strings(w, sitems);
    w = lookup_widget(dlg, "IlmDeaMtzAllCols");
    populate_selection(GTK_TREE_VIEW(w), aitems, bitems);

    std::string str;
    for( i=0; i<DeaMtzSelected.size(); i++ ) {
      str += DeaMtzSelected[i]->label;
      str += ' ';
    }

    w = lookup_widget(dlg, "IlmDeaMtzColumnsEntry");
    gtk_text_buffer_set_text(gtk_text_view_get_buffer (GTK_TEXT_VIEW (w)), str.c_str(),-1);
 }


  void update_mtzpage(GtkWidget *dlg)
  {
    GList *vitems=NULL, *sitems=NULL, *aitems=NULL, *bitems=NULL;
    unsigned i;
    GtkWidget *w;

    for( i=0; i<MtzEsds.size(); i++ )
      sitems = g_list_append(sitems, (gpointer) MtzEsds[i]->label);
    for( i=0; i<MtzValues.size(); i++ )
      vitems = g_list_append(vitems, (gpointer) MtzValues[i]->label );
    
    for( i=0; i<MtzAllCols.size(); i++ )
    {
      aitems = g_list_append(aitems, (gpointer) MtzAllCols[i]->label );
      bitems = g_list_append(bitems, (gpointer) MtzAllCols[i]->type );
    }
    
    w = lookup_widget(dlg, "IlmMtzValueCombo");
//    gtk_entry_set_text( GTK_ENTRY(get_widget(w,"entry")), "" );
//    if(g_list_length(vitems))
       set_popdown_strings(w, vitems);
    w = lookup_widget(dlg, "IlmMtzEsdCombo");
//    gtk_entry_set_text( GTK_ENTRY(get_widget(w,"entry")), "" );
//    if(g_list_length(sitems))
       set_popdown_strings(w, sitems);
    w = lookup_widget(dlg, "IlmMtzAllCols");
//    populate_listg(GTK_TREE_VIEW(w), aitems);
    populate_selection(GTK_TREE_VIEW(w), aitems, bitems);

    std::string str;
    for( i=0; i<MtzSelected.size(); i++ ) {
      str += MtzSelected[i]->label;
      str += ' ';
    }
    
    w = lookup_widget(dlg, "IlmMtzColumnsEntry");
    gtk_text_buffer_set_text(gtk_text_view_get_buffer (GTK_TEXT_VIEW (w)), str.c_str(),-1);
 }
  
 void GtkProject::FillMtzVectors(gpointer mtzi)
 {
    char labels[100][31], types[100][3];
    int sets[100];
    int ncols;
    int i;
    CMtz::MTZ *mtzin;
    const std::string fn = fromUTF8(szMtzFile);

    MtzValues.clear();
    MtzEsds.clear();
    MtzAllCols.clear();

    if(mtzi == NULL)
       mtzin = CMtz::MtzGet(fn.c_str(), 0);
    else
       mtzin = (CMtz::MTZ* )mtzi;
    ncols = CMtz::MtzNcol(mtzin);
    CMtz::MtzListColumn(mtzin, labels, types, sets);

    for( i=0; i<ncols; i++ )
    {
      char typ = types[i][0];

      MtzAllCols.push_back( CMtz::MtzColLookup(mtzin, labels[i]) );
      if( typ=='Q' || typ=='L' || typ=='M')
        MtzEsds.push_back( CMtz::MtzColLookup(mtzin, labels[i]) );
      if((eType == EDM || eType == DEA) && typ == 'W')
        MtzEsds.push_back( CMtz::MtzColLookup(mtzin, labels[i]) );
      if( typ=='F' || typ=='G' || typ=='J' || typ=='K')
        MtzValues.push_back( CMtz::MtzColLookup(mtzin, labels[i]) );
      if((eType == EDM || eType == DEA) && typ == 'P')
        MtzValues.push_back( CMtz::MtzColLookup(mtzin, labels[i]) );
    }
  }

  const char *GtkProject::MtzGetSpgrName(const char *file)
  {
    CMtz::MTZ *mtzin;
    int nsympx, nspgrx;
    char ltypex[256], spgrnx[256], pgnamx[256];
    std::string fn = fromUTF8(file);

    mtzin = CMtz::MtzGet(fn.c_str(), 0);
    CMtz::ccp4_lrsymi(mtzin, &nsympx, ltypex, &nspgrx, 
       spgrnx, pgnamx);
    const std::string spgnam = g_strstrip(spgrnx);
    return DuplicaStringa(spgnam.c_str());
  }

  int GtkProject::MtzGetInfo(gpointer mtzi)
  {
    float resMin, resMax;
    float cell[6];
    int nsympx, nspgrx;
    char ltypex[256], spgrnx[256], pgnamx[256];
    CMtz::MTZ *mtzin;
    const std::string fn = fromUTF8(szMtzFile);
    int i;

    if(mtzi == NULL)
       mtzin = CMtz::MtzGet(fn.c_str(), 0);
    else
       mtzin = (CMtz::MTZ* )mtzi;

    if(mtzin == NULL)
       return 0;
    nNrRefl = CMtz::MtzNref(mtzin);
    CMtz::MtzResLimits(mtzin, &resMin, &resMax);
    try
    {
       dresMin = (float) (1.0 / sqrt((double)resMin));
       dresMax = (float) (1.0 / sqrt((double)resMax));
       if(dresExtra < 0.1)
          dresExtra = dresMax;
       if(dresFL < 0.1)
          dresFL = dresMax;
    }
    catch(...)
    {
    }
 
    CMtz::ccp4_lrsymi(mtzin, &nsympx, ltypex, &nspgrx, 
       spgrnx, pgnamx);
    if(gext::str_is_white(szSpgrName))                          // 230107
       CopiaStringa(szSpgrName, sizeof(szSpgrName), spgrnx);
    CSym::CCP4SPG *spg = CSym::ccp4spg_load_by_spgname(szSpgrName);
    if(!spg)
       return -1;
    nSpgrOrder = spg->nsymop;
    CSym::ccp4spg_free(&spg);


    CMtz::ccp4_lrcell(MtzIxtal(mtzin, 0), cell);
    for(i=0; i<6; i++) dCell[i] = cell[i];
    return 1;
 }

   int GtkProject::FillFromMTZ()
   {
    const std::string fn = fromUTF8(szMtzFile);
    GtkWidget *mainwnd = LookupWidget("IlmProjectWnd");

    if(gext::str_is_white(szMtzFile))
       return 0;
    if( !g_file_test(toUTF8(szMtzFile), G_FILE_TEST_EXISTS) ) 
       return 0;
    MtzSelected.clear();
    CMtz::MTZ *mtzin;
    mtzin = CMtz::MtzGet(fn.c_str(), 0);
    if( mtzin==0 )
        return 0;

    MtzGetInfo(mtzin);

    // filling in the dialog
    char titl[1024];
    CMtz::ccp4_lrtitl(mtzin, titl);
    GtkWidget *w = lookup_widget(mainwnd,"IlmMtzTitleEntry");
    gtk_entry_set_text(GTK_ENTRY(w), titl);

    std::stringstream tcell;
    tcell  << dCell[0] << " " << dCell[1] << " " 
             << dCell[2] << " " << dCell[3] << " " << dCell[4] << " " << dCell[5];
    w = lookup_widget(mainwnd, "IlmMtzSpgEntry");
    gtk_entry_set_text(GTK_ENTRY(w), szSpgrName);
    w = lookup_widget(mainwnd, "IlmMtzCellEntry");
    gtk_entry_set_text(GTK_ENTRY(w), tcell.str().c_str());
    w = lookup_widget(mainwnd, "IlmMtzHint");
    switch(eType)
    {
    case ABI:
         gtk_label_set_text(GTK_LABEL(w), 
                   "Ab-initio: select F, sigF for protein");
         break;
    case EDM:
         gtk_label_set_text(GTK_LABEL(w), 
                   "EDM: select F-sigF and phase-weight pair");
         break;
    case MAD:
         gtk_label_set_text(GTK_LABEL(w), 
                   "MAD/SAD: select F,sigF for protein if available and F+,sigF+, F-,sigF- for every wavelength");
         break;
    case MIR:
         gtk_label_set_text(GTK_LABEL(w), 
                   "MIR/SIR: select F,sigF for protein and all derivatives");
         break;
    case MRS:
         gtk_label_set_text(GTK_LABEL(w), 
                   "MIRAS/SIRAS: select F,sigF for protein if available and F+,sigF+, F-,sigF- for every wavelength and derivative");
         break;
    case MOLREP:
         gtk_label_set_text(GTK_LABEL(w), 
                  "Molecular Replacement: select F, sigF for protein");
         break;
    case DEA:
         gtk_label_set_text(GTK_LABEL(w), "DEA: select F, sigF for protein");
         break;
    }

    FillMtzVectors(mtzin);
    clear_mtz(0);
    if(strlen(szSelectedColumns) > 0)
    {
       GList *SelectedColumns = fill_list(szSelectedColumns, " ");
       if(SelectedColumns)
       {
          int nsc = (int) g_list_length(SelectedColumns);
          int i;
          for(i=0; i<nsc; i++)
          {
              char *pstr = (char *)g_list_nth_data(SelectedColumns, i);
              add_mtz_val(pstr, 0);
              g_free(pstr);
          }
          g_list_free(SelectedColumns);
       }
    }
    
    Ilm::update_mtzpage(mainwnd);
    return 1;
  }

  void GtkProject::UpdateDeaMTZInfo()
  {
       int gotinfo;
       if( !g_file_test(toUTF8(szMtzFilePh), G_FILE_TEST_EXISTS) )
       {
           gtkext::dlg_errmsg(MainWindow(), "MTZ file %s does not exist", toUTF8(szMtzFilePh));
           bSyncRunning = false;
           return;
       }
       else if((gotinfo = DeaMtzGetInfo(NULL)) != 1)
       {
           if(!gotinfo)
              gtkext::dlg_errmsg(MainWindow(), "Error reading MTZ file %s", toUTF8(szMtzFilePh));
           bSyncRunning = false;
           return;
       }
  }

  void GtkProject::UpdateMTZInfo()
  {
       int EsdCount = 0, FCount = 0;
       int gotinfo;
       if( !g_file_test(szMtzFile, G_FILE_TEST_EXISTS) ) 
       {
           gtkext::dlg_errmsg(MainWindow(), "MTZ file %s does not exist", szMtzFile);
           bSyncRunning = false;
           return;
       }
       else if((gotinfo = MtzGetInfo(NULL)) != 1)
       {
           if(!gotinfo)
              gtkext::dlg_errmsg(MainWindow(), "Error reading MTZ file %s", szMtzFile);
           bSyncRunning = false;
           return;
       }

       for(unsigned int i=0; i< MtzSelected.size(); i++)
       {
           char typ = MtzSelected[i]->type[0];
           if( typ=='Q' || typ=='L' )
               EsdCount++;
           if( typ=='F' || typ=='G' )
               FCount++;
       }

       bSyncRunning = true;
       nWaves = EsdCount / 2;
       SetMaxSpinValue("IlmContWavesSpin", nWaves);
       if(eType == MRS)
       {
          nDeriv = EsdCount / 3;
          for(int nd=0; nd <nDeriv; nd++)
              nMrsWaves[nd] = 1;
       }
       else                    //if(eType == MIR)
          nDeriv = EsdCount - 1;
       if(nDeriv > MAX_DERIVATIVES)
          nDeriv = MAX_DERIVATIVES;
       SetMaxSpinValue("IlmContDerivSpin", nDeriv);
       bNativePresent = ((EsdCount % 2) == 1);

       IntToUI("IlmContWavesSpin", nWaves);
       IntToUI("IlmContDerivSpin", nDeriv);
       BoolToUI( "IlmContNativeCheck", bNativePresent);
       if(eType >= MOLREP)
       {
           bSyncRunning = false;
           return;
       }
       int iFin, nNum;
       if((eType == MIR) || (eType == MRS))
       {
          iFin = MAX_DERIVATIVES;
          nNum = nDeriv;
          nWaves = 1;
       }
       else
       {
          iFin = MAX_WAVES;
          nNum = nWaves;
       }
       for(int i=0; i<iFin; i++)
       {
           nomefile name;
           g_snprintf(name, sizeof(name), "IlmTasksMad2Wave%dCheck", i+1);
//181006           bMad2Waves[i] = (i < nNum);
           BoolToUI(name, bMad2Waves[i]);
           gtk_widget_set_sensitive(LookupWidget(name), (i<nNum));
           g_snprintf(name, sizeof(name), "IlmTasksProDeriv%dCheck", i+1);
           bProDerivs[i] = (i < nNum);
           BoolToUI(name, bProDerivs[i]);
           gtk_widget_set_sensitive(LookupWidget(name), (i<nNum));
           g_snprintf(name, sizeof(name), "IlmTasksMad1Wave%dCheck", i+1);
           if(!bMad1Auto)
           {
//181006              bMad1Waves[i] = (i < nNum);
              BoolToUI(name, bMad1Waves[i]);
           }
           gtk_widget_set_sensitive(LookupWidget(name), (i<nNum));

           if(eType == MRS)
           {
              g_snprintf(name, sizeof(name), "IlmContDeriv%dSpin", i+1);
              IntToUI(name, nMrsWaves[i]);
           }
       }
       BoolToUI ("IlmTasksMad1AutoCheck", bMad1Auto);
       bSyncRunning = false;
   }

  void GtkProject::AppendPng(std::ofstream &of, const std::string &fn) const
  {
       if(g_file_test(fn.c_str(), G_FILE_TEST_EXISTS))
       {
          of << "<p align=\"center\">" 
             << "<font size=\"2\" style=\"font-family: courier new,courier,monospace;\"> "
             << std::endl
             << "<u><b>Image File: "<< fn << "</b></u></p>" 
             << std::endl
//220409             << "<img border=\"0\" src=\"" << fn 
             << "<img border=\"0\" src=\"" << g_path_get_basename(fn.c_str()) 
             << "\" >" << std::endl;
       }
  }

  int GtkProject::CreateHtml() const
  {
      std::string fn;
      if(eType >= EDM)
         fn  = GetFilename2(HTML_EXT, eType-1);
      else
         fn  = GetFilename(HTML_EXT);
      std::ofstream of(fn.c_str());
      int i0, i1;

      if( !of || !of.is_open() )
        throw UserError("Could not open file: " + fn);

                                 /*   header     */
      of << "<html>" << std::endl;
      of << "<head>" << std::endl;
      of << "<meta http-equiv=\"Content-Language\" content=\"en-us\">" << std::endl;
      of << "<meta name=\"GENERATOR\" content=\"Il Milione 1.0\">" << std::endl;
      of << "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=windows-1252\">" << std::endl;
      of << "<title>" << szStructName << "</title>" << std::endl;
      of << "</head>" << std::endl;
      of << "<body>" << std::endl;
      of << "<a name=\"home\"></a>" << std::endl;

                                 /*  body */

      std::stringstream os[3];
      switch(eType)
      {
      case ABI:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>Ab initio structure solution: summary for " 
              << szStructName << "</b></font></p>";
           i0 = eType - 1;
           i1 = eType - 1;
           break;
      case MOLREP:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>Molecular Replacement procedure: summary for " 
              << szStructName << "</b></font></p>";
           i0 = eType - 1;
           i1 = eType - 1;
           break;
      case EDM:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>Electron Density Modification procedure: summary for " 
              << szStructName << "</b></font></p>";
           i0 = eType - 1;
           i1 = eType - 1;
           break;
      case MAD:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>Mad/Sad project: summary for " << szStructName << "</b></font></p>";
           os[1] << "Task1: Finding the anomalous scatterers";
           os[2] << "Task2: Mad/Sad phasing";
           if(bMad1)
              i0 = 1;
           else
              i0 = 2;
           if(bMad2)
              i1 = 2;
           else
              i1 = 1;
           break;
      case MIR:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>Mir/Sir project: summary for " << szStructName << "</b></font></p>";
           os[0] << "Task0: Direct protein phasing";
           os[1] << "Task1: Finding the heavy atoms";
           os[2] << "Task2: Mir/Sir phasing";
           if(bPro)
           {
              i0 = 0;
              i1 = 0;
           }
           else
           {
              if(bMad1)
                 i0 = 1;
              else
                 i0 = 2;
              if(bMad2)
                 i1 = 2;
              else
                 i1 = 1;
           }
           break;
      case MRS:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>Miras/Siras project: summary for " << szStructName << "</b></font></p>";
           os[0] << "Task0: Direct protein phasing";
           os[1] << "Task1: Finding the heavy atoms";
           os[2] << "Task2: Miras/Siras phasing";
           if(bPro)
           {
              i0 = 0;
              i1 = 0;
           }
           else
           {
              if(bMad1)
                 i0 = 1;
              else
                 i0 = 2;
              if(bMad2)
                 i1 = 2;
              else
                 i1 = 1;
           }
           break;
      case DEA:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>DEA ?????: summary for " 
              << szStructName << "</b></font></p>";
           i0 = eType - 1;
           i1 = eType - 1;
           break;
      }
      of << "<pre>" << std::endl;
      int j = 0, k = 0;
      for (int i=i0; i<=i1; i++)
      {
           if(i1 > i0)
           {
              std::ifstream inf(GetFilename2(LOG_EXT, i).c_str());
              if( inf )
              {
                 of << "<align=\"left\"><font size=\"5\" "
                    << "style=\"font-family: times new roman,times,serif;\" color=\"#0000FF\"><b>"
                    << "<a href=\"#task" << i << "\">" << os[i].str()
                    << "</a></b></font>" << std::endl;
                inf.close();
              }
              j++;
           }
      }
      for (int i=i0; i<=i1; i++)
      {
           std::ifstream inf(GetFilename2(LOG_EXT, i).c_str());
           std::string line;
           if( inf )
           {
              of << " <hr color=\"#000080\" size=\"1\">";
              while( !inf.eof() )
              {
                 std::getline(inf, line);
                 of << line << std::endl;
              }
              inf.close();
              k++;
              AppendPng(of, GetFilename2(GRAPH_EXT, i));
           }
      }
      if(eType >= EDM)
         AppendPng(of, GetFilename2(FGRAPH_EXT, eType-1));
      else
         AppendPng(of, GetFilename2(FGRAPH_EXT, 1));

                                 /*   end */
      of << "</font>" << std::endl;
      of << "</pre>" << std::endl;
      of << "</body>" << std::endl;
      of << "</html>" << std::endl;
      of.close();
      return k;
  }

  int GtkProject::CreateInfoHtml() const
  {
      std::string fn;
      if(eType >= EDM)
         fn  = GetFilename2(INFO_EXT, eType-1);
      else
         fn  = GetFilename(INFO_EXT);
      std::ofstream of(fn.c_str());
      int i0, i1;

      if( !of || !of.is_open() )
        throw UserError("Could not open file: " + fn);

                                 /*   header     */
      of << "<html>" << std::endl;
      of << "<head>" << std::endl;
      of << "<meta http-equiv=\"Content-Language\" content=\"en-us\">" << std::endl;
      of << "<meta name=\"GENERATOR\" content=\"Il Milione 1.0\">" << std::endl;
      of << "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=windows-1252\">" << std::endl;
      of << "<title>" << szStructName << "</title>" << std::endl;
      of << "</head>" << std::endl;
      of << "<body>" << std::endl;
      of << "<a name=\"home\"></a>" << std::endl;

                                 /*  body */

      std::stringstream os[3];
      switch(eType)
      {
      case ABI:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>Ab initio structure solution: files info for " 
              << szStructName << "</b></font></p>";
           i0 = eType - 1;
           i1 = eType - 1;
           break;
      case MOLREP:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>Molecular Replacement procedure: files info for " 
              << szStructName << "</b></font></p>";
           i0 = eType - 1;
           i1 = eType - 1;
           break;
      case EDM:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>Electron Density Modification procedure: files info for " 
              << szStructName << "</b></font></p>";
           i0 = eType - 1;
           i1 = eType - 1;
           break;
      case MAD:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>Mad/Sad project: files info for " << szStructName << "</b></font></p>";
           os[1] << "Task1: Finding the anomalous scatterers";
           os[2] << "Task2: Mad/Sad phasing";
           if(bMad1)
              i0 = 1;
           else
              i0 = 2;
           if(bMad2)
              i1 = 2;
           else
              i1 = 1;
           break;
      case MIR:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>Mir/Sir project: files info for " << szStructName << "</b></font></p>";
           os[0] << "Task0: Direct protein phasing";
           os[1] << "Task1: Finding the heavy atoms";
           os[2] << "Task2: Mir/Sir phasing";
           if(bPro)
           {
              i0 = 0;
              i1 = 0;
           }
           else
           {
              if(bMad1)
                 i0 = 1;
              else
                 i0 = 2;
              if(bMad2)
                 i1 = 2;
              else
                 i1 = 1;
           }
           break;
      case MRS:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>Miras/Siras project: files info for " << szStructName << "</b></font></p>";
           os[0] << "Task0: Direct protein phasing";
           os[1] << "Task1: Finding the heavy atoms";
           os[2] << "Task2: Miras/Siras phasing";
           if(bPro)
           {
              i0 = 0;
              i1 = 0;
           }
           else
           {
              if(bMad1)
                 i0 = 1;
              else
                 i0 = 2;
              if(bMad2)
                 i1 = 2;
              else
                 i1 = 1;
           }
           break;
      case DEA:
           of << "<p align=\"left\"><font size=\"5\" style=\"font-family: times new roman,times,serif;\"";
           of << "color=\"#FF0000\"><b>DEA ????????????????????????? files info for " 
              << szStructName << "</b></font></p>";
           i0 = eType - 1;
           i1 = eType - 1;
           break;
      }
      of << "<pre>" << std::endl;
      int j = 0, k = 0;
      for (int i=i0; i<=i1; i++)
      {
           if(i1 > i0)
           {
              std::ifstream inf(GetFilename2(ILOG_EXT, i).c_str());
              if( inf )
              {
                 of << "<align=\"left\"><font size=\"5\" "
                    << "style=\"font-family: times new roman,times,serif;\" color=\"#0000FF\"><b>"
                    << "<a href=\"#task" << i << "\">" << os[i].str()
                    << "</a></b></font>" << std::endl;
                inf.close();
              }
              j++;
           }
      }
      for (int i=i0; i<=i1; i++)
      {
           std::ifstream inf(GetFilename2(ILOG_EXT, i).c_str());
           std::string line;
           if( inf )
           {
              of << " <hr color=\"#000080\" size=\"1\">";
              while( !inf.eof() )
              {
                 std::getline(inf, line);
                 of << line << std::endl;
              }
              inf.close();
              k++;
           }
      }
                                 /*   end */
      of << "</font>" << std::endl;
      of << "</pre>" << std::endl;
      of << "</body>" << std::endl;
      of << "</html>" << std::endl;
      of.close();
      return k;
  }

  int GtkProject::DeaMtzGetInfo(gpointer mtzi)
  {
    float resMin, resMax;
    float cell[6];
    int nsympx, nspgrx;
    char ltypex[256], spgrnx[256], pgnamx[256];
    CMtz::MTZ *mtzin;
    const std::string fn = fromUTF8(szMtzFilePh);
    int i;

    if(mtzi == NULL)
       mtzin = CMtz::MtzGet(fn.c_str(), 0);
    else
       mtzin = (CMtz::MTZ* )mtzi;

    if(mtzin == NULL)
       return 0;
    nNrReflPh = CMtz::MtzNref(mtzin);
    CMtz::MtzResLimits(mtzin, &resMin, &resMax);
    try
    {
       dresMinPh = (float) (1.0 / sqrt((double)resMin));
       dresMaxPh = (float) (1.0 / sqrt((double)resMax));
    }
    catch(...)
    {
    }

    CMtz::ccp4_lrsymi(mtzin, &nsympx, ltypex, &nspgrx,
       spgrnx, pgnamx);
    if(gext::str_is_white(szSpgrNamePh))                          // 230107
       CopiaStringa(szSpgrNamePh, sizeof(szSpgrNamePh), spgrnx);
    CSym::CCP4SPG *spg = CSym::ccp4spg_load_by_spgname(szSpgrNamePh);
    if(!spg)
       return -1;
    nSpgrOrderPh = spg->nsymop;
    CSym::ccp4spg_free(&spg);


    CMtz::ccp4_lrcell(MtzIxtal(mtzin, 0), cell);
    for(i=0; i<6; i++) dCellPh[i] = cell[i];
    return 1;
 }

   int GtkProject::FillDeaFromMTZ()
   {
    const std::string fn = fromUTF8(szMtzFilePh);
    GtkWidget *mainwnd = LookupWidget("IlmProjectWnd");

    if(gext::str_is_white(szMtzFilePh))
       return 0;
    if( !g_file_test(toUTF8(szMtzFilePh), G_FILE_TEST_EXISTS) ) 
       return 0;
    DeaMtzSelected.clear();
    CMtz::MTZ *mtzin;
    mtzin = CMtz::MtzGet(fn.c_str(), 0);
    if( mtzin==0 )
        return 0;

    DeaMtzGetInfo(mtzin);

    // filling in the dialog
    char titl[1024];
    CMtz::ccp4_lrtitl(mtzin, titl);
    GtkWidget *w = lookup_widget(mainwnd,"IlmDeaMtzTitleEntry");
    gtk_entry_set_text(GTK_ENTRY(w), titl);

    std::stringstream tcell;
    tcell  << dCellPh[0] << " " << dCellPh[1] << " " 
             << dCellPh[2] << " " << dCellPh[3] << " " << dCellPh[4] << " " << dCellPh[5];
    w = lookup_widget(mainwnd, "IlmDeaMtzSpgEntry");
    gtk_entry_set_text(GTK_ENTRY(w), szSpgrNamePh);
    w = lookup_widget(mainwnd, "IlmDeaMtzCellEntry");
    gtk_entry_set_text(GTK_ENTRY(w), tcell.str().c_str());
    w = lookup_widget(mainwnd, "IlmDeaMtzHint");
    set_label_text(w, 
                   "Select F-sigF and phase-weight pair", MyBLUE, 1.1f);

    DeaFillMtzVectors(mtzin);
    clear_mtzdea(0);
    if(strlen(szSelectedColumnsPh) > 0)
    {
       GList *SelectedColumns = fill_list(szSelectedColumnsPh, " ");
       if(SelectedColumns)
       {
          int nsc = (int) g_list_length(SelectedColumns);
          int i;
          for(i=0; i<nsc; i++)
          {
              char *pstr = (char *)g_list_nth_data(SelectedColumns, i);
              add_mtz_val(pstr, 1);
              g_free(pstr);
          }
          g_list_free(SelectedColumns);
       }
    }
    
    Ilm::update_deamtzpage(mainwnd);
    return 1;
  }

 void GtkProject::DeaFillMtzVectors(gpointer mtzi)
 {
    char labels[100][31], types[100][3];
    int sets[100];
    int ncols;
    int i;
    CMtz::MTZ *mtzin;
    const std::string fn = fromUTF8(szMtzFilePh);

    DeaMtzValues.clear();
    DeaMtzEsds.clear();
    DeaMtzAllCols.clear();

    if(mtzi == NULL)
       mtzin = CMtz::MtzGet(fn.c_str(), 0);
    else
       mtzin = (CMtz::MTZ* )mtzi;
    ncols = CMtz::MtzNcol(mtzin);
    CMtz::MtzListColumn(mtzin, labels, types, sets);

    for( i=0; i<ncols; i++ )
    {
      char typ = types[i][0];

      DeaMtzAllCols.push_back( CMtz::MtzColLookup(mtzin, labels[i]) );
      if( typ=='Q' || typ=='L' || typ=='M')
        DeaMtzEsds.push_back( CMtz::MtzColLookup(mtzin, labels[i]) );
      if((eType == EDM || eType == DEA) && typ == 'W')
        DeaMtzEsds.push_back( CMtz::MtzColLookup(mtzin, labels[i]) );
      if( typ=='F' || typ=='G' || typ=='J' || typ=='K')
        DeaMtzValues.push_back( CMtz::MtzColLookup(mtzin, labels[i]) );
      if((eType == EDM || eType == DEA) && typ == 'P')
        DeaMtzValues.push_back( CMtz::MtzColLookup(mtzin, labels[i]) );
    }
  }

} // namespace Ilm


