#ifndef __ILM_GTKPROJECT_H_mrt
#define __ILM_GTKPROJECT_H_mrt

#include "ilm_callbacks.h"
#include "ilm_interface.h"
#include <string.h>

#include "project.h"
#include "util.h"

namespace Ilm
{
  class GtkProject : public Project
  {
  public:
    enum EValidate { eSilent, eLoud, eActive };
    GtkProject(const std::string &fn, ProjectType typ) 
      : Project(fn, typ), poMainWnd(0), iLastPage(-1), bReady(false), bSyncRunning(false),
      bIsSubmitted(false), tmLogModTime(0), numFragsMad(0), runSirprot(false), iFirstErr(10) {}
    GtkProject(const std::string &fn) : Project(fn), poMainWnd(0), iLastPage(-1),  
      bReady(false), bSyncRunning(false), bIsSubmitted(false), tmLogModTime(0), 
      numFragsMad(0), runSirprot(false), iFirstErr(10) {}
    GtkProject(bool isasc, const std::string &fn) 
      : Project(isasc, fn), poMainWnd(0), iLastPage(-1),  bReady(false), bSyncRunning(false),
      bIsSubmitted(false), tmLogModTime(0), numFragsMad(0), runSirprot(false), iFirstErr(10) {}
      
    GtkWidget* MainWidget() const {return poMainWnd;}
    GtkWindow* MainWindow() const {return GTK_WINDOW( poMainWnd ) ; }
    void SetMainWindowDestroyed() {poMainWnd=NULL;}
    void CreateWidget(const std::string &fn);
    GtkWidget* LookupWidget(const char *name) const { return lookup_widget(poMainWnd, name); }
    GObject* LookupObject(const char *name) const { return (GObject *) get_object(poMainWnd, name); }
    void SetLastPage(int page) {iLastPage=page; }
    void SetFirstErr(int page) {iFirstErr=page; }
    int LastPage() const {return iLastPage;}
    int FirstErr() const {return iFirstErr;}
    void SetReady(bool ready) {bReady = ready;}
    bool IsReady() const {return bReady;}
    bool IsRecover() const {return bRecover;}
    void SetSubmitError(const std::string &msg) {strSubmitError = msg;}
    const std::string &GetSubmitError() const {return strSubmitError;}
    const std::string &GetRecoverData() const {return strRecoverData;}
    const std::string &GetRecoverMTZ() const {return strRecoverMTZ;}
    const std::string &GetRecoverMTZLab() const {return strRecoverMTZLab;}
    void SetRecoverData(const std::string &msg) {strRecoverData = msg;}
    void SetRecoverMTZ(const std::string &msg) {strRecoverMTZ = msg;}
    void SetRecoverMTZLab(const std::string &msg) {strRecoverMTZLab = msg;}
    bool IsSubmitted() const {return bIsSubmitted;}
    void SetSubmitted(bool toggle) { bIsSubmitted = toggle; }
    bool DoWhileLock(); // true - continue, false - stop
    void SystemExec() const;
    gint AsyncExec(int what) const;
    gint SyncExec(int what) const;
    void ClearSequence();
    void WriteSequence() const;
    void ReadSequence() const;
    void AddCycles(const char *src, const char *dst);

    void AddTextToMenu(const char *name, const std::string &text) const;
    void TextToInMenu(const char *name, const char *ext) const;
    void TextToOutMenu(const char *name, const char *ext, int ind) const;
    void TextToOutMenus(const char *name, const char *ext) const;
    void TextToUI(const char *name, const char *text) const;
    void LabelToUI(const char *name, const char *text) const;
    void TextWToUI(const char *name, const char *text) const;
    void IntToUI(const char *name, int i) const;
    void SetMaxSpinValue(const char *name, int i) const;
    void SetSpinValues(const char *name, double vmin, double  vmax, double val) const;
    void DoubleToUI(const char *name, double d) const;
    void BoolToUI(const char *name, bool b) const;
    void ListToUI(const char *name, int i) const;
    template< size_t N > void TextFromUI(char (&text)[N], const char *name) const
      {memset(text, 0, N); g_strlcpy(text, gtk_entry_get_text(GTK_ENTRY(LookupWidget(name))), N);}
    template< size_t N > void TextWFromUI(char (&text)[N], const char *name) const
      {memset(text, 0, N); g_strlcpy(text, get_text(GTK_TEXT_VIEW(LookupWidget(name))), N);}

    int IntFromUI(const char *name) const;
    double DoubleFromUI(const char *name) const;
    bool BoolFromUI(const char *name) const;
    int ListFromUI(const char *name) const;
    const gchar *LabelFromUI(const char *name) const;

    int FillFromMTZ();
    int FillDeaFromMTZ();
    bool OnExit();
    void Go();
    void FillInterface() const;
    void DataFromUI();
    void FillInfo(int aPage=-1) const;
    void Synchronize(); 
    int ValidateAll(EValidate e);
    void SaveAllFiles();
    void DisablePages(bool disable);
    void SetInfoText(const char *) const;
    int CurrentPage();
    int ApplyNthPage(int nth, EValidate e);
    int ApplyCurrentPage(EValidate e);
    void SetContState(bool elms);
    void SetAnomWavesState(int imad, bool aut) const;
    void SetMenuLast(const std::string &fn);
    void CheckAnomFile(std::ostream &str, const std::string &fn);
    void CheckAnomFilePDB(std::ostream &str, const std::string &fn);
    void CheckPDBFile(const std::string &fn, int which, EValidate e);
    void MtzFill(const char *widname);
    void FillMtzVectors(gpointer mtzi);
    void DeaFillMtzVectors(gpointer mtzi);
    int  MtzGetInfo(gpointer mtzi);
    int  DeaMtzGetInfo(gpointer mtzi);
    void UpdateMTZInfo();
    void UpdateDeaMTZInfo();
    const char *MtzGetSpgrName(const gchar *file);

    int ValidateMTZ(EValidate e);
    int ValidateCont(EValidate e);
    int ValidateRun(EValidate e);
    void AppendPng(std::ofstream &str,const std::string &fn) const;
    int CreateHtml() const;
    int CreateInfoHtml() const;
    void CreaRefmacShell();
    void CreaShellFromModel(const char *nome);
    void CreaShellFromPhases(const char *nome);
    void WriteInte(std::ofstream &str) const;
    std::string CallShell(const char *shellName) const;

    static int (GtkProject::*ValidatePage[MAX_PAGES]) (EValidate e );
  private:

    GtkWidget *poMainWnd;
    int iLastPage;
    bool bReady;
    mutable bool bSyncRunning;
    std::string strSubmitError;
    bool bIsSubmitted;
    time_t tmLogModTime;
    int numFragsMad;
    bool runSirprot;
    int iFirstErr;
    bool bRecover;
    std::string strRecoverMTZ;
    std::string strRecoverMTZLab;
    std::string strRecoverData;
  };

} // namespace Ilm

#endif // __ILM_GTKPROJECT_H_mrt

