#ifdef HAVE_CONFIG_H
#  include <config.h>
#else
#  include <config_any.h>
#endif

#include <sstream>
#include <cassert>
#include "ilm.h"
#include "g_ext.h"

#ifdef VERY_LOW_GVERSION
typedef gint GPid;
#endif

gboolean PasteOk;
void RunCoot(const std::string &fn, const char *opt)
{
    if( !g_file_test(fn.c_str(), G_FILE_TEST_EXISTS) )
    {
        gtkext::dlg_errmsg(NULL, "File does not exist: %s\n", fn.c_str());
        return;
    }
    int result = 0;
    gchar args[10][128], *pargs[10];
    gint pid = 0;
    GError *gerr = NULL;
    GSpawnFlags flags = static_cast<GSpawnFlags>( G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD );
    if( str_white(Ilm::TheApp.Executables()[Ilm::IExePDBViewer].c_str()) || (Ilm::TheApp.Executables()[Ilm::IExePDBViewer].find("unknown") != std::string::npos))
    {
        gtkext::dlg_errmsg(NULL, "Please configure Coot using Settings Tool.");
        return;
    }
    CopiaStringa(args[0], 128, Ilm::TheApp.Executables()[Ilm::IExePDBViewer].c_str());
    CopiaStringa(args[1], 128, opt);
    CopiaStringa(args[2], 128, fn.c_str());
    args[3][0] = 0;
    pargs[0] = args[0];
    pargs[1] = args[1];
    pargs[2] = args[2];
    pargs[3] = NULL;

    result = g_spawn_async(NULL, pargs, NULL, flags, NULL, 0, (GPid *)(&pid), &gerr);

    if( gerr!=NULL || !result  )
    {
        std::string msg = "Failed to execute: " + Ilm::TheApp.Executables()[Ilm::IExePDBViewer];
        if( gerr!=NULL )
        {
          msg += "\nDue to: ";
          msg += gerr->message;
          msg += "\nCHECK SETTINGS!";
          g_error_free(gerr);
        }
        gtk_widget_set_sensitive(Ilm::PTheProject->MainWidget(), FALSE);
        gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), msg.c_str());
        gtk_widget_set_sensitive(Ilm::PTheProject->MainWidget(), TRUE);
    }

}

void RunViewer(const std::string &fn, int nopts, nomefile *opts, int ifSync,
               char **buffer)
{
    if( !g_file_test(toUTF8(fn.c_str()), G_FILE_TEST_EXISTS) )
    {
        gtkext::dlg_errmsg(NULL, "File does not exist: %s\n", toUTF8(fn.c_str()));
        return;
    }
    if( str_white(Ilm::TheApp.Executables()[Ilm::IExePDBViewer].c_str()) )
    {
        gtkext::dlg_errmsg(NULL, "Please configure jav using Settings Tool.");
        return;
    }
    std::string exe = Ilm::TheApp.Executables()[Ilm::IExePDBViewer];
    std::vector<std::string> args;

    for(int j=0; j<nopts; j++)
        args.push_back(opts[j]);
    args.push_back(fn);
    std::string msg;
    int result = run_command(exe, args, msg, ifSync, buffer);
    if((result == -1) && !str_white(msg.c_str()))
    {
        std::string utf8msg = toUTF8(msg.c_str());
        gtk_widget_set_sensitive(Ilm::PTheProject->MainWidget(), FALSE);
        gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), utf8msg.c_str());
        gtk_widget_set_sensitive(Ilm::PTheProject->MainWidget(), TRUE);
    }
}

void RunViewer(const std::string &fn, const char *opts)
{
    nomefile optargs[1];
    CopiaStringa(optargs[0], MaxLenFilename, "-a");
    if( str_white(Ilm::TheApp.Executables()[Ilm::IExePDBViewer].c_str()) )
    {
        gtkext::dlg_errmsg(NULL, "Please configure jav or another viewer using Settings Tool.");
        return;
    }
    if(Ilm::TheApp.Executables()[Ilm::IExePDBViewer].find("jav") != std::string::npos)
       RunViewer(fn, 1, optargs, 0, NULL);
    else
       RunCoot(fn, opts);
}

extern "C" {

static GtkWidget *AboutDlg = NULL;
gboolean
on_IlmProjectWnd_delete_event          (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  return Ilm::PTheProject->OnExit();
}


void on_save_activate (gpointer, gpointer)
{
    if(Ilm::PTheProject->ValidateAll(Ilm::PTheProject->eLoud))
    {
       Ilm::PTheProject->Save();
       Ilm::PTheProject->SetMenuLast(Ilm::PTheProject->GetFilename(Ilm::PROJECT_EXT));
       Ilm::TheApp.WriteSettings();
    }
}


void on_saveascii_activate (gpointer, gpointer)
{
    if(Ilm::PTheProject->ValidateAll(Ilm::PTheProject->eLoud))
    {
       Ilm::PTheProject->SaveAscii();
    }
}

void on_save_as_activate (gpointer, gpointer)
{
    if(!Ilm::PTheProject->ValidateAll(Ilm::PTheProject->eLoud))
        return;
    char *bsnm = g_new0(char, MaxLenFilename);
    std::string fn = gtkext::dlg_select_file(Ilm::PTheProject->MainWidget(), Ilm::PTheProject->GetFilename(Ilm::PROJECT_EXT), gtkext::eSELECT_TO_WRITE, "*.ilm");
    if( !str_white(fn.c_str()) )
    {
      CopiaStringa(bsnm, MaxLenFilename, g_path_get_basename(fn.c_str()));
      CopiaStringa(Ilm::PTheProject->szWorkDir, sizeof(Ilm::PTheProject->szWorkDir), 
             g_path_get_dirname(fn.c_str()));
      CopiaStringa(Ilm::PTheProject->szStructName, sizeof(Ilm::PTheProject->szStructName),
             bsnm);
      char *p = strrchr(Ilm::PTheProject->szStructName, '.');
      if( p!=NULL ) *p = '\0';
      Ilm::PTheProject->FillInterface();
      Ilm::PTheProject->Save();
      Ilm::PTheProject->SetMenuLast(Ilm::PTheProject->GetFilename(Ilm::PROJECT_EXT));
      Ilm::TheApp.WriteSettings();
    }
}


void on_apply_activate (gpointer, gpointer wnd)
{
   GtkWidget *w = lookup_widget((GtkWidget *) wnd, "IlmOkBtn");
   gtk_button_clicked(GTK_BUTTON(w));
}

void on_setdefaults_activate (gpointer, gpointer wnd)
{
   GtkWidget *w = lookup_widget((GtkWidget *) wnd, "IlmDefaultsBtn");
   gtk_button_clicked(GTK_BUTTON(w));
}

void on_Quit(gpointer, gpointer)
{
  gboolean tmp = 100;
  GtkWidget *mw = Ilm::PTheProject->MainWidget();

  g_signal_emit_by_name(G_OBJECT(mw), "delete-event", NULL, &tmp);
}

void on_AboutBtn_clicked (gpointer, gpointer window)
{
  if(AboutDlg == NULL)
     AboutDlg = create_IlmAboutDlg();
  gtk_window_set_transient_for(GTK_WINDOW(AboutDlg), GTK_WINDOW(window));
  gtk_dialog_run(GTK_DIALOG(AboutDlg));
  gtk_widget_hide(AboutDlg);
}

void on_HelpBtn_clicked (gpointer, gpointer )
{
     std::string helpfile = Ilm::TheApp.PkgDataDir() + "ilmdir" + G_DIR_SEPARATOR_S +"ilm.html";
     Ilm::internet_browser(helpfile);
}

void
on_about_activate                      (gpointer, gpointer)
{
  on_AboutBtn_clicked(NULL, NULL);
}

void on_help_activate                  (gpointer, gpointer)
{
     std::string helpfile = Ilm::TheApp.PkgDataDir() + "ilmdir" + G_DIR_SEPARATOR_S +"ilm.html";
     Ilm::internet_browser(helpfile);
}

static gint IPage = -1;

void
on_IlmNotebook_switch_page             (GtkNotebook     *ntb,
//                                        GtkNotebookPage * ,
                                        gint             page_num,
                                        gpointer         ) 
{
  try
  {
    gint ipage = gtk_notebook_get_current_page(ntb); 
    if( ipage<0 || ipage==page_num || page_num==IPage )
      return; 
    if( !gtk_widget_get_sensitive(gtk_notebook_get_nth_page(ntb, page_num)) )
      goto lblFailure;
    if( ipage>=page_num ) {
      Ilm::PTheProject->ApplyNthPage(page_num, Ilm::GtkProject::eLoud);
      goto lblSuccess;
    }
    if( page_num>Ilm::PTheProject->LastPage()+1 ) 
      goto lblFailure;
    for( int i=ipage; i<page_num; i++ )
    {
      if( !Ilm::PTheProject->ApplyNthPage(i, Ilm::GtkProject::eLoud) ) 
        goto lblFailure;
    }
    if( page_num<=Ilm::PTheProject->LastPage() ) {
      Ilm::PTheProject->ApplyNthPage(page_num, Ilm::GtkProject::eLoud);
    }
  lblSuccess:
    Ilm::PTheProject->FillInfo(page_num);
    gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmDefaultsBtn"), page_num==Ilm::RUN);
    IPage = -1;
    goto lblEnd;
  lblFailure:
    IPage = ipage; 
  lblEnd: ;
  }
  catch( const Ilm::UserError &e )
  {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.GetMessage());
  }
}


void
on_IlmNotebook_switch_page_after       (GtkNotebook     *notebook,
//                                        GtkNotebookPage *,
                                        guint            page_num,
                                        gpointer         )
{
  if( IPage>=0 )
    gtk_notebook_set_current_page(notebook, IPage);
}


void
on_IlmMtzBrowseBtn_clicked            (GtkButton *button,
                                       gpointer user_data)
{
  try
  {
    std::string fn;
    _MyMTZ *mtz = (_MyMTZ *) user_data;
    Ilm::PTheProject->DataFromUI();
    fn = gtkext::dlg_select_file(Ilm::PTheProject->MainWidget(), mtz->mtzfile, 
        gtkext::eSELECT_TO_READ, "*.mtz");
    if( !fn.empty() ) {
      g_strlcpy(mtz->mtzfile, fn.c_str(), Ilm::PATH_LEN);
      Ilm::PTheProject->MtzFill(gtk_widget_get_name(mtz->del));
    }
  }
  catch( const Ilm::UserError &e )
  {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.GetMessage());
  }

}

void on_Ilm_text_view_clicked  (GtkButton    *,
                                        gpointer data)
{
  try
  {
    GtkWidget *entry = (GtkWidget *) data;
    const char *fil = gtk_entry_get_text(GTK_ENTRY(entry));
    std::string fn = fil;
    if( !fn.empty() )
      Ilm::view_file(fn);
  }
  catch( const Ilm::UserError &e )
  {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.GetMessage());
  }
}

void on_Ilm_pdb_modify_clicked  (GtkButton    *button,
                                        gpointer data)
{
using namespace std;
  try
  {
    int nmodel = GPOINTER_TO_INT(data);
    GtkWidget *entry = get_widget(GTK_WIDGET(button), "entry");
    const char *fil = gtk_entry_get_text(GTK_ENTRY(entry));
    nomefile opts[3];
    CopiaStringa(opts[0], MaxLenFilename, "-s=ct");
    CopiaStringa(opts[1], MaxLenFilename, "-c=re");
    CopiaStringa(opts[2], MaxLenFilename, "-M");
    std::string fn = fil;
    if( !fn.empty() )
    {
      char *buffer = g_new0(char, 1024);
      GtkWidget *dlg = gtk_widget_get_toplevel(GTK_WIDGET(button));
      gtk_widget_hide(dlg);
      while(gtk_events_pending())
            gtk_main_iteration();
      RunViewer(fn, 3, opts,1, &buffer);
      std::string outbuffer = buffer;
      string::size_type loc1 = outbuffer.find("OutFile: ", 0);
      if(loc1 != string::npos)
      {
         loc1 += 9;
         string::size_type loc2 = outbuffer.find("\n", loc1);
         if(loc2 != string::npos)
         {
            std::string filname = outbuffer.substr(loc1, loc2 - loc1);
            fn = filname;
            gtk_entry_set_text(GTK_ENTRY(entry), fn.c_str());
         }
      }
      gtk_widget_show(dlg);
      Ilm::PTheProject->DataFromUI();
      Ilm::PTheProject->CheckPDBFile(fn, nmodel, Ilm::PTheProject->eActive);
      Ilm::PTheProject->FillInterface();
    }
  }
  catch( const Ilm::UserError &e )
  {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.GetMessage());
  }
}

void on_Ilm_mtz_view_clicked  (GtkButton    *,
                                        gpointer data)
{
  try
  {
    GtkWidget *entry = (GtkWidget *) data;
    const char *fil = gtk_entry_get_text(GTK_ENTRY(entry));
    std::string fn = fil;
    if( !fn.empty() )
      myshow_mtz(fn, Ilm::PTheProject->MainWindow());
  }
  catch( const Ilm::UserError &e )
  {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.GetMessage());
  }
}


void
on_IlmTasksMad2BrowseBtn_clicked       (GtkButton       *btn,
                                        gpointer         )
{
  GtkWidget *mw = NULL;
  try
  {
    std::string fn;
    GtkEntry *ent = GTK_ENTRY(lookup_widget(GTK_WIDGET(btn), "IlmTasksMad2FileEntry"));
    mw = lookup_widget(GTK_WIDGET(btn), "IlmProjectWnd");
    fn = gtkext::dlg_select_file(mw, gtk_entry_get_text(ent), gtkext::eSELECT_TO_READ, "*.pdb,*.ent");
    if( !fn.empty() ) {
      std::stringstream str;
      g_strlcpy(Ilm::PTheProject->szMad2File, fn.c_str(), sizeof(Ilm::PTheProject->szMad2File));
      Ilm::PTheProject->CheckAnomFilePDB(str, fn);
    }
  }
  catch( const Ilm::UserError &e )
  {
    gtkext::dlg_errmsg(GTK_WINDOW(mw), e.GetMessage());
  }
}


void
on_IlmOk_clicked                       (GtkButton       *button,
                                        gpointer         )
{
  try
  {
    Ilm::PTheProject->ApplyCurrentPage(Ilm::GtkProject::eActive);
  }
  catch( const Ilm::UserError &e )
  {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.GetMessage());
  }
}

void
on_IlmSirEditBtn_clicked              (GtkButton       *,
                                        gpointer data)
{
  gint num = GPOINTER_TO_INT(data);
  if(Ilm::PTheProject->eType >= Ilm::EDM)
     Ilm::edit_file(Ilm::PTheProject->GetFilename2(Ilm::INP_EXT, Ilm::PTheProject->eType-1));
  else
     Ilm::edit_file(Ilm::PTheProject->GetFilename2(Ilm::INP_EXT, num));
}

void
on_IlmTasksMad1NreflCheck_toggled      (GtkToggleButton *togglebutton,
                                        gpointer         )
{
  GtkWidget *spin = Ilm::PTheProject->LookupWidget("IlmTasksMad1NreflSpin");
  if( ::gtk_toggle_button_get_active(togglebutton) )
  {
    ::gtk_widget_set_sensitive(spin, FALSE);
    Ilm::PTheProject->nMad1Refls = 0;
    Ilm::PTheProject->FillInterface(); 
  }
  else
  {
    ::gtk_widget_set_sensitive(spin, TRUE);
  }
}


void
on_IlmContMolRadio_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         )
{
  gboolean act = gtk_toggle_button_get_active(togglebutton);
  
  gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmContMolSpin"), act);
  gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmContResiduesSpin"), !act);
  gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmContSeqBox"), !act);
  gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmContSeqFrame"), !act);
  if(act)
     Ilm::PTheProject->ClearSequence();
}


void
on_IlmContResiduesRadio_toggled        (GtkToggleButton *togglebutton,
                                        gpointer         )
{
  gboolean act = gtk_toggle_button_get_active(togglebutton);
  
  gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmContResiduesSpin"), act);
  gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmContMolSpin"), !act);
  gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmContSeqBox"), !act);
  gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmContSeqFrame"), !act);
  if(act)
     Ilm::PTheProject->ClearSequence();
}

void
on_IlmContSeqRadio_toggled        (GtkToggleButton *togglebutton,
                                        gpointer         )
{
  gboolean act = gtk_toggle_button_get_active(togglebutton);
  
  gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmContSeqBox"), act);
  gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmContSeqFrame"), act);
  gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmContResiduesSpin"), !act);
  gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmContMolSpin"), !act);
}

void
on_IlmContSeqBrowse_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
  try
  {
    std::string fn;
    Ilm::PTheProject->TextFromUI(Ilm::PTheProject->szSeqFile,"IlmContSeqFile");

    fn = gtkext::dlg_select_file(Ilm::PTheProject->MainWidget(), Ilm::PTheProject->szSeqFile, 
        gtkext::eSELECT_TO_READ, "*.seq");
    if( !fn.empty() ) {
      g_strlcpy(Ilm::PTheProject->szSeqFile, fn.c_str(), sizeof(Ilm::PTheProject->szSeqFile));
        Ilm::PTheProject->ReadSequence();
      Ilm::PTheProject->ApplyNthPage(Ilm::CONT, Ilm::GtkProject::eSilent);
    }
  }
  catch( const Ilm::UserError &e )
  {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.GetMessage());
  }


}

void
on_IlmContSeqPaste                     (GtkTextView     *widget,
                                        gpointer         data)
{
      PasteOk = false;
      if(str_white(Ilm::PTheProject->szSeqFile))
      {
         CopiaStringa(Ilm::PTheProject->szSeqFile, sizeof(Ilm::PTheProject->szSeqFile), 
                      Ilm::PTheProject->GetFilename(".seq").c_str());
         Ilm::PTheProject->TextToUI("IlmContSeqFile", Ilm::PTheProject->szSeqFile);
         PasteOk = true;
      }
      else
      {
         if( gtkext::dlg_yesno(Ilm::PTheProject->MainWindow(), 
                "File already exists. Are you sure you want to overwrite?") ) 
            PasteOk = true;
         else
            g_signal_stop_emission_by_name ((gpointer) widget, "paste-clipboard");
       
      }
}

void
on_IlmContSeqPasteAfter                 (GtkTextView     *widget,
                                        gpointer         data)
{
      if(PasteOk)
      {
         while(gtk_events_pending())
               gtk_main_iteration();
         Ilm::PTheProject->WriteSequence();
      }
}

void
on_IlmContSolventBtn_clicked           (GtkButton       *,
                                        gpointer         )
{
  Ilm::PTheProject->DoubleToUI("IlmContSolventSpin", Ilm::PTheProject->SolventFraction());
}

static void on_setprg(GtkButton *button, const char *what)
{
  std::string fn;
  GtkWidget *w = lookup_widget(GTK_WIDGET(button), what);
  GtkWidget *dlg = gtk_widget_get_toplevel(GTK_WIDGET(button));
  gtk_window_set_modal(GTK_WINDOW(dlg), FALSE);
  const gchar *deffn = gtk_entry_get_text(GTK_ENTRY(w));
  fn = gtkext::dlg_select_file(lookup_widget(GTK_WIDGET(button),"IlmSettingsDlg"), deffn, gtkext::eSELECT_ALIEN, "*.*");
  gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);
  if( !fn.empty() ) {
    gtk_entry_set_text(GTK_ENTRY(w), fn.c_str());
  }
}

void
on_IlmSetProgBtn_clicked               (GtkButton       *button,
                                        const char *what         )
{
  on_setprg(button, what);
}

void on_IlmMenuOutHtml_activate (gpointer, gpointer)
{
   int result, tipo = Ilm::PTheProject->eType - 1;
   try{ result = Ilm::PTheProject->CreateHtml();}
   catch( const Ilm::UserError &e ) {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.GetMessage());
   }
   catch( const gext::exception &e ) {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.get_message());
   }
   if (result)
   {
       if(tipo >= (Ilm::EDM-1))
          Ilm::internet_browser(Ilm::PTheProject->GetFilename2(Ilm::HTML_EXT, tipo).c_str());
       else
          Ilm::internet_browser(Ilm::PTheProject->GetFilename(Ilm::HTML_EXT).c_str());
   }
   else
     gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), "Summary file(s) does not exist!");


}

void on_IlmMenuOutInfo_activate (gpointer, gpointer)
{
   int result, tipo = Ilm::PTheProject->eType - 1;
   try{ result = Ilm::PTheProject->CreateInfoHtml();}
   catch( const Ilm::UserError &e ) {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.GetMessage());
   }
   catch( const gext::exception &e ) {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.get_message());
   }
   if (result)
   {
       if(tipo >= (Ilm::EDM-1))
          Ilm::internet_browser(Ilm::PTheProject->GetFilename2(Ilm::INFO_EXT, tipo).c_str());
       else
          Ilm::internet_browser(Ilm::PTheProject->GetFilename(Ilm::INFO_EXT).c_str());
   }
   else
     gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), "Info file(s) does not exist!");


}

void on_IlmMenuOutFull_activate (gpointer, gpointer  data  )
{
  gint tipo = GPOINTER_TO_INT(data);
  if(tipo < 0)
     tipo = Ilm::PTheProject->eType - 1;
  Ilm::view_file(Ilm::PTheProject->GetFilename2(Ilm::OUT_EXT, tipo));
}


void on_IlmMenuOutPhase_activate (gpointer, gpointer )
{
  myshow_mtz(Ilm::PTheProject->GetFilename(Ilm::PHASE_EXT), Ilm::PTheProject->MainWindow());
}

void on_IlmMenuOutPhase_model_activate (gpointer, gpointer)
{
  RunViewer(Ilm::PTheProject->GetFilename(Ilm::PHASE_EXT), "--data");
}

void on_IlmMenuABIOutMtz_activate (gpointer, gpointer  data)
{
  gint tipo = Ilm::PTheProject->eType - 1;
  myshow_mtz(Ilm::PTheProject->GetFilename2(Ilm::MTZ_EXT, tipo), Ilm::PTheProject->MainWindow());
}

void on_IlmMenuABIOutMtz_model_activate (gpointer, gpointer data)
{
  gint tipo = Ilm::PTheProject->eType - 1;
  RunViewer(Ilm::PTheProject->GetFilename2(Ilm::MTZ_EXT, tipo), "--data");
}

void on_IlmMenuOutEdm_activate (gpointer, gpointer data )
{
  gint tipo = GPOINTER_TO_INT(data);
  if(tipo == 0) //After Edm
     myshow_mtz(Ilm::PTheProject->GetFilename(Ilm::EDM_EXT), Ilm::PTheProject->MainWindow());
  else
     myshow_mtz(Ilm::PTheProject->GetFilename(Ilm::EDM_EXT1), Ilm::PTheProject->MainWindow());
}

void on_IlmMenuOutEdm_model_activate (gpointer, gpointer  data)
{
  gint tipo = GPOINTER_TO_INT(data);
  if(tipo == 0) //After Edm
     RunViewer(Ilm::PTheProject->GetFilename(Ilm::EDM_EXT), "--data");
  else
     RunViewer(Ilm::PTheProject->GetFilename(Ilm::EDM_EXT1), "--data");
}

void on_IlmMenuOutMap_activate (gpointer, gpointer data)
{
  gint tipo = GPOINTER_TO_INT(data);
  if(tipo == 0) //After Edm
     RunViewer(Ilm::PTheProject->GetFilename(Ilm::EDM_MAP1), "--map");
  else
     RunViewer(Ilm::PTheProject->GetFilename(Ilm::EDM_MAP2), "--map");
}

void on_IlmMenuOutPdb_text_activate (gpointer, gpointer )
{
  try
  {
    std::string fn;
    fn = gtkext::dlg_select_file(Ilm::PTheProject->MainWidget(), "", gtkext::eSELECT_TO_READ, "*.pdb,*.ent");
    if( !fn.empty() )
      Ilm::view_file(fn);
  }
  catch( const Ilm::UserError &e )
  {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.GetMessage());
  }
}

void on_IlmMenuOutPdb_model_activate (gpointer, gpointer )
{
  try
  {
    std::string fn;
    fn = gtkext::dlg_select_file(Ilm::PTheProject->MainWidget(), "", gtkext::eSELECT_TO_READ, "*.pdb,*.ent");
    if( !fn.empty() )
       RunViewer(fn, "--pdb");
  }
  catch( const Ilm::UserError &e )
  {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.GetMessage());
  }
}


void on_IlmMenuOutFra_activate (gpointer, gpointer)
{
  Ilm::view_file(Ilm::PTheProject->GetFilename(Ilm::FRACLSQ_EXT));
}

void on_IlmMenuInFra_activate  (gpointer, gpointer)
{
  if(!str_white(Ilm::PTheProject->szMad2File))
     Ilm::view_file(Ilm::PTheProject->szMad2File);
  else
     Ilm::view_file(Ilm::PTheProject->GetFilename(Ilm::MAD1FRAC_EXT).c_str());
}


void on_IlmMenuInTask_activate (gpointer, gpointer data)
{
  int tipo = GPOINTER_TO_INT(data);
  if(tipo < 0)
     tipo = Ilm::PTheProject->eType - 1;
  Ilm::view_file(Ilm::PTheProject->GetFilename2(Ilm::INP_EXT, tipo));
}

void
on_Check_activated                      (GtkToggleButton *button,
                                        gpointer         )
{
  if(gtk_toggle_button_get_active(button))
     Ilm::PTheProject->Synchronize();
}

void
on_Check_toggled                        (GtkToggleButton *,
                                        gpointer         )
{
  Ilm::PTheProject->Synchronize();
}

gboolean on_IlmSplashWnd_delete_event(GtkWidget *widget,  GdkEvent *event, gpointer user_data)
{
  if( gtkext::dlg_yesno(GTK_WINDOW(widget), "Are you sure you want to quit?") ) {
    printf("Thanks for using  Il Milione\n");
    gtk_main_quit();
    return FALSE;
  }
  return TRUE;
}

static void on_new_prot(gpointer mw, Ilm::ProjectType typ)
{
  try {
    std::string fn = "";
    if( Ilm::TheApp.Last().size()!=0 )
      fn = Ilm::TheApp.Last().front();

    fn = gtkext::dlg_select_file(GTK_WIDGET(mw), fn, gtkext::eSELECT_TO_WRITE, "*.ilm");
    if( !fn.empty() ) { 
      if( Ilm::PTheProject!=NULL )
        delete Ilm::PTheProject;
      Ilm::PTheProject = NULL; 
      Ilm::PTheProject = new Ilm::GtkProject(fn, typ);
      gint x, y;
      gtk_window_get_position(Ilm::TheApp.SplashWindow(), &x, &y);
      gtk_widget_hide(Ilm::TheApp.SplashWidget());
      Ilm::PTheProject->CreateWidget(fn);
      gtk_window_move(Ilm::PTheProject->MainWindow(), x, y);
      Ilm::main_loop();
      if( Ilm::PTheProject!=NULL && Ilm::PTheProject->MainWidget()!=NULL )
      {
//        GtkAllocation alloc;
//        memcpy(&alloc, &(Ilm::PTheProject->MainWidget()->allocation), sizeof(GtkAllocation));
//        Ilm::TheApp.SetAllocation(alloc);
        gtk_widget_destroy(Ilm::PTheProject->MainWidget());
      }
      gtk_window_move(Ilm::TheApp.SplashWindow(), x, y);
      gtk_widget_show(Ilm::TheApp.SplashWidget());
    }
  } catch(const Ilm::UserError &e) {
    gtkext::dlg_errmsg(GTK_WINDOW(mw), "%s", e.GetMessage().c_str());
  } catch(const std::exception &e) {
    gtkext::dlg_errmsg(GTK_WINDOW(mw), "%s", e.what());
  } catch( ... ) {
    gtkext::dlg_errmsg(GTK_WINDOW(mw), "unknown exception");
  }

}

void on_SirBtn_clicked(gpointer, gpointer window)
{
    on_new_prot(window, Ilm::ABI);
}

void on_MadBtn_clicked(gpointer, gpointer window)
{
  on_new_prot(window, Ilm::MAD);
}

void on_MirBtn_clicked(gpointer, gpointer window)
{
  on_new_prot(window, Ilm::MIR);
}

void on_MrsBtn_clicked (gpointer, gpointer window)
{
  on_new_prot(window, Ilm::MRS);
}

void on_MolRepBtn_clicked(gpointer, gpointer window)
{
  on_new_prot(window, Ilm::MOLREP);
}

void on_EdmBtn_clicked(gpointer, gpointer window)
{
  on_new_prot(window, Ilm::EDM);
}

void on_DeaBtn_clicked(gpointer, gpointer window)
{
  on_new_prot(window, Ilm::DEA);
}
#include <ostream>
static void on_open(const std::string &fn1)
{
  if( fn1.empty() ) return;
  int lnb = (int) fn1.find_last_not_of(' ');
  std::string fn = fromUTF8(fn1.substr(0, lnb+1));

  try 
  {
    if( fn.length()>4 && fn.substr(fn.length()-4, fn.length()-1) == ".mtz" )
    {
       myshow_mtz(fn, Ilm::TheApp.SplashWindow());
       return;
    }
    else if((fn.length()>4 && fn.substr(fn.length()-4, fn.length()-1) == ".pdb")||
            (fn.length()>4 && fn.substr(fn.length()-4, fn.length()-1) == ".ent"))
    {
       RunViewer(fn, "--pdb");
       return;
    }
    else if( fn.length()>5 && fn.substr(fn.length()-5, fn.length()-1) == ".ailm" )
    {
      if( Ilm::PTheProject!=NULL )
        delete Ilm::PTheProject;
      Ilm::PTheProject = NULL; 
      Ilm::PTheProject = new Ilm::GtkProject(true, fn);
    }
    else
    {
      if( Ilm::PTheProject!=NULL )
        delete Ilm::PTheProject;
      Ilm::PTheProject = NULL; 
      Ilm::PTheProject = new Ilm::GtkProject(fn);
    }
    gint x, y;
    gtk_window_get_position(Ilm::TheApp.SplashWindow(), &x, &y);
    gtk_widget_hide(Ilm::TheApp.SplashWidget());
    Ilm::PTheProject->CreateWidget(fn);
    Ilm::PTheProject->SetMenuLast(Ilm::PTheProject->GetFilename(Ilm::PROJECT_EXT));
    gtk_window_move(Ilm::PTheProject->MainWindow(), x, y);
    Ilm::PTheProject->ValidateAll(Ilm::PTheProject->eSilent);
    Ilm::PTheProject->UpdateMTZInfo();
    Ilm::PTheProject->FillInterface();
    Ilm::main_loop();
    if( Ilm::PTheProject!=NULL && Ilm::PTheProject->MainWidget()!=NULL )
    {
      gtk_widget_destroy(Ilm::PTheProject->MainWidget());
    }
    gtk_window_move(Ilm::TheApp.SplashWindow(), x, y);
    gtk_widget_show(Ilm::TheApp.SplashWidget());
  } catch(const Ilm::UserError &e) {
    gtkext::dlg_errmsg(Ilm::TheApp.SplashWindow(), "%s", e.GetMessage().c_str());
  } catch(const std::exception &e) {
    gtkext::dlg_errmsg(Ilm::TheApp.SplashWindow(), "%s", e.what());
  } catch( ... ) {
    gtkext::dlg_errmsg(Ilm::TheApp.SplashWindow(), "unknown exception");
  }

}


void on_OpenBtn_clicked(gpointer, gpointer )
{
  GtkWidget *mw = Ilm::TheApp.SplashWidget();
  std::string fn = "";
  if( Ilm::TheApp.Last().size()!=0 )
    fn = Ilm::TheApp.Last().front();

  fn = gtkext::dlg_select_file(mw, fn, gtkext::eSELECT_TO_READ, "*.ilm");
  on_open(fn);
}

void on_SettingsBtn_clicked(gpointer , gpointer )
{
  GtkWidget *mw = Ilm::TheApp.SplashWidget();
  GtkWidget *dlg = create_IlmSettingsDlg();
  gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(mw)); 

  GtkWidget *w[Ilm::NExecutables];
  for(int i=0; i<(int)Ilm::NExecutables ; i++)
  {
      w[i] = lookup_widget(dlg, g_strdup_printf("IlmSetEntry%d", i+1));
      gtk_entry_set_text(GTK_ENTRY(w[i]), Ilm::TheApp.Executables()[i].c_str());
  }

  bool error = true;
  while(error)
  {
  gint response = ::gtk_dialog_run(GTK_DIALOG(dlg));
  error = false;
  if( response==GTK_RESPONSE_OK )
  {
    std::vector<std::string> exe;
    for( unsigned i=0; i<Ilm::NExecutables; i++ )
    {
         std::string str = gtk_entry_get_text(GTK_ENTRY(w[i]));
         if( (str_white(str.c_str()) ) && (i!= Ilm::IExePDBViewer))
         {
              gtk_widget_set_sensitive(dlg, FALSE);
              gtkext::dlg_errmsg(GTK_WINDOW(dlg), "You should set the filename for %s",
                            Ilm::SZLbExecutables[i]);
              gtk_widget_set_sensitive(dlg, TRUE);
              error = true;
              break;
         }
         else
              exe.push_back(g_strstrip(gtk_editable_get_chars(GTK_EDITABLE(w[i]), 0, -1)));
    }
    if (!error)
    {
       Ilm::TheApp.SetExecutables(exe);
       Ilm::TheApp.WriteSettings();
    }
  }
  }
  gtk_widget_destroy(dlg);
}

void
on_Spin_value_changed                  (GtkWidget   *,
                                        gpointer         )
{
  Ilm::PTheProject->Synchronize();
}

void
on_QuitBtn_clicked                     (gpointer        button,
                                        gpointer        window)
{
  static gboolean tmp = 0;
//  GtkWidget *mw = lookup_widget(GTK_WIDGET(button), "IlmSplashWnd");
  g_signal_emit_by_name(G_OBJECT(window), "delete-event", NULL, &tmp);
}

void
on_Entry_changed                       (GtkEditable     *,
                                        gpointer         )
{
  Ilm::PTheProject->Synchronize();
}

void on_IlmMtzAddPairBtn_clicked (GtkButton *button, gpointer user_data )
{
  GtkWidget *dlg = gtk_widget_get_toplevel(GTK_WIDGET(button));
  const gchar *vlb, *elb;
  _MyMTZ *mtz = (_MyMTZ *) user_data;

  vlb = combo_get_text( get_widget(mtz->vcombo,"entry") );
  elb = combo_get_text( get_widget(mtz->esdcombo,"entry") );
  if( Ilm::add_mtz_pair(vlb, elb, mtz->quale) )
  {
      if(mtz->quale)
         Ilm::update_deamtzpage(dlg);
      else
      {
         Ilm::update_mtzpage(dlg);
         Ilm::PTheProject->UpdateMTZInfo ();
      }
  }
}


void on_IlmMtzClearBtn_clicked (GtkButton *button, gpointer user_data)
{
    GtkWidget *dlg = gtk_widget_get_toplevel(GTK_WIDGET(button));
    _MyMTZ *mtz = (_MyMTZ *) user_data;

    if(Ilm::PTheProject->CurrentPage() == Ilm::MTZ)
    {
       Ilm::PTheProject->SetLastPage(Ilm::MTZ-1);
       Ilm::PTheProject->DisablePages(true);
    }

    if(mtz->quale)
    {
         Ilm::clear_mtzdea(1);
         Ilm::update_deamtzpage(dlg);
         Ilm::PTheProject->UpdateDeaMTZInfo ();
    }
    else
    {
         Ilm::clear_mtz(1);
         Ilm::update_mtzpage(dlg);
         Ilm::PTheProject->UpdateMTZInfo ();
    }
}

void  on_IlmMtzAddC_clicked(GtkButton *button, gpointer user_data)
{
  GtkWidget *dlg = gtk_widget_get_toplevel(GTK_WIDGET(button));
  _MyMTZ *mtz = (_MyMTZ *) user_data;
  const gchar *lb = get_selected_row((GtkWidget *) mtz->allcols);

  if(lb)
  {
      Ilm::add_mtz_col(lb,mtz->quale);
      if(mtz->quale)
      {
         Ilm::update_deamtzpage(dlg);
         Ilm::PTheProject->UpdateDeaMTZInfo ();
      }
      else
      {
         Ilm::update_mtzpage(dlg);
         Ilm::PTheProject->UpdateMTZInfo ();
      }
  }
}


void
on_recent_activate                     (gpointer menuitem,
                                        gpointer         )
{
  const gchar *fn = obj_get_label(menuitem);
  if(fn!=NULL)
    on_open(fn);
}

void
on_IlmDefaults_clicked                 (GtkButton       *,
                                        gpointer         )
{
  Ilm::PTheProject->SetDefaults();
  Ilm::PTheProject->FillInterface();
  Ilm::PTheProject->Synchronize();
}

static bool toggle_running = false;

void
on_IlmTasksMad1AutoCheck_toggled       (GtkToggleButton *btn,
                                        gpointer         )
{
  if( toggle_running ) return;
  toggle_running = true;
  if( gtk_toggle_button_get_active(btn) )
  {
    for( unsigned i=1; i<=4; i++ ) 
    {
      std::stringstream str;
      str << "IlmTasksMad1Wave" << i << "Check";
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Ilm::PTheProject->LookupWidget(str.str().c_str())), FALSE);
    }
  }
  else
  {
     int nNum;
     if((Ilm::PTheProject->eType == Ilm::MRS) || (Ilm::PTheProject->eType == Ilm::MIR))
        nNum = Ilm::PTheProject->nDeriv;
     else
        nNum = Ilm::PTheProject->nWaves;
    for( int i=1; i<=4; i++ ) 
    {
      std::stringstream str;
      str << "IlmTasksMad1Wave" << i << "Check";
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Ilm::PTheProject->LookupWidget(str.str().c_str())), 
              (i <= nNum));
    }
  }
  toggle_running = false;
}


void
on_IlmTasksMad1WaveCheck_toggled       (GtkToggleButton *,
                                        gpointer         )
{
  if( toggle_running ) return;
  toggle_running = true;
  gboolean aut; //  = true;
  for( unsigned i=1; i<=4; i++ )
  {
    std::stringstream str;
    str << "IlmTasksMad1Wave" << i << "Check";
    aut = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Ilm::PTheProject->LookupWidget(str.str().c_str())));
    if( aut ) 
      break;
  }
  GtkToggleButton *abtn = GTK_TOGGLE_BUTTON(Ilm::PTheProject->LookupWidget("IlmTasksMad1AutoCheck"));
  gtk_toggle_button_set_active( abtn, !aut );
  toggle_running = false;
}


void
on_IlmProjectWnd_destroy               (GObject       *,
                                        gpointer         )
{
  if( Ilm::PTheProject!=NULL )
    Ilm::PTheProject->SetMainWindowDestroyed();
}



void
on_IlmTasksMRBrowseBtn_clicked         (GtkButton       *,
                                        gpointer  data)
{
  try
  {
    int nmodel = GPOINTER_TO_INT(data);
    std::string fn;
    Ilm::PTheProject->DataFromUI();
    fn = gtkext::dlg_select_file(Ilm::PTheProject->MainWidget(), Ilm::PTheProject->szMRFile[nmodel], 
        gtkext::eSELECT_TO_READ, "*.pdb,*.ent");
    if( !fn.empty() ) {
      g_strlcpy(Ilm::PTheProject->szMRFile[nmodel], fn.c_str(), sizeof(Ilm::PTheProject->szMRFile[nmodel]));
      Ilm::PTheProject->CheckPDBFile(fn, nmodel, Ilm::PTheProject->eActive);
      Ilm::PTheProject->FillInterface();
    }
  }
  catch( const Ilm::UserError &e )
  {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.GetMessage());
  }

}

void on_IlmTasksDeaPDB         (GtkButton       *,
                                        gpointer  data)
{
  try
  {
    int quale = GPOINTER_TO_INT(data);
    std::string fn;
    Ilm::PTheProject->DataFromUI();
    fn = gtkext::dlg_select_file(Ilm::PTheProject->MainWidget(), Ilm::PTheProject->szMRFile[quale],
        gtkext::eSELECT_TO_READ, "*.pdb,*.ent");
    if( !fn.empty() ) {
      g_strlcpy(Ilm::PTheProject->szMRFile[quale], fn.c_str(), sizeof(Ilm::PTheProject->szMRFile[quale]));
      Ilm::PTheProject->CheckPDBFile(fn, quale, Ilm::PTheProject->eActive);
      Ilm::PTheProject->FillInterface();
    }
  }
  catch( const Ilm::UserError &e )
  {
    gtkext::dlg_errmsg(Ilm::PTheProject->MainWindow(), e.GetMessage());
  }

}


} // extern "C"

void
on_IlmStrategyRadio_toggled             (GtkToggleButton *togglebutton,
                                        gpointer data)
{
  gboolean act = gtk_toggle_button_get_active(togglebutton);
  gint quale = GPOINTER_TO_INT(data);
  
  gtk_widget_set_sensitive(Ilm::PTheProject->LookupWidget("IlmStrategySpin"), 
               act && (quale == 3));
}

void
open_file_from_cmd_line                 (const gchar *filnam)
{
       if(filnam!=NULL)
          on_open(filnam);
}

void
on_IlmSourceRadio_toggled             (GtkToggleButton *button,
                                        GtkWidget *dlg)
{
     if(gtk_toggle_button_get_active(button))
     {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget(dlg, Ilm::Waves[2])), TRUE);
        gtk_widget_set_sensitive(get_widget(dlg, Ilm::Waves[0]), FALSE);
        gtk_widget_set_sensitive(get_widget(dlg, Ilm::Waves[1]), FALSE);
     }
     else
     {
        gtk_widget_set_sensitive(get_widget(dlg, Ilm::Waves[0]), TRUE);
        gtk_widget_set_sensitive(get_widget(dlg, Ilm::Waves[1]), TRUE);
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget(dlg, Ilm::Waves[1])), TRUE);
     }

}

void get_examples(bool ifSetUp)
{
    std::string dist_ex_dir = Ilm::TheApp.PkgDataDir() + "examples" + G_DIR_SEPARATOR_S;
    std::string ex_dir = fromUTF8(g_get_home_dir());
    std::vector<std::string> Dirs;

    ex_dir.append(G_DIR_SEPARATOR_S);
    ex_dir.append("share"); 
    dir_create(ex_dir.c_str());
    ex_dir.append(G_DIR_SEPARATOR_S);
    ex_dir.append(PACKAGE); 
    dir_create(ex_dir.c_str());
    ex_dir.append(G_DIR_SEPARATOR_S);
    ex_dir.append("examples");
//    ex_dir.append(VERSION);
    ex_dir.append(G_DIR_SEPARATOR_S);
    dir_create(ex_dir.c_str());
    if(ex_dir.compare(dist_ex_dir) != 0)
       gtkext::mr_CopyDir((char *)dist_ex_dir.c_str(), (char *)ex_dir.c_str(), ifSetUp);

    dir_set_current(ex_dir.c_str());
    ChangeMode(ex_dir.c_str(), tXt);
    gtkext::clistfiles("*.dir", Dirs);
    for(unsigned int i=0; i < Dirs.size(); i++)
    {
       std::vector<std::string> Files;
       std::vector<std::string> SirFiles;
       dir_set_current(ex_dir.c_str());
       ChangeMode(Dirs[i].c_str(), tXt);
       dir_set_current(Dirs[i].c_str());
       gtkext::clistfiles("*.sir", SirFiles);
       for(unsigned int j=0; j < SirFiles.size(); j++)
       {
          char *filnam = g_build_filename(ex_dir.c_str(), Dirs[i].c_str(), SirFiles[j].c_str(), NULL);
          std::string fn = filnam;
          ChangeMode(fn.c_str(),  rwXt);
       }
       gtkext::clistfiles("*.ailm", Files);
       for(unsigned int j=0; j < Files.size(); j++)
       {
          char *filnam = g_build_filename(ex_dir.c_str(), Dirs[i].c_str(), Files[j].c_str(), NULL);
          std::string fn = filnam;
          ChangeMode(fn.c_str(),  rwXt);
          Ilm::GtkProject *prj;
          try {  prj = new Ilm::GtkProject(true, fn); }
          catch( const Ilm::UserError &e )
          {
            gtkext::dlg_errmsg("%s", e.GetMessage().c_str());
            continue;
          }
          catch( const gext::exception &e ) 
          {
            gtkext::dlg_errmsg("%s", e.get_message().c_str());
            continue;
          }

          if (prj == NULL)
              continue;
          std::string dir = g_path_get_dirname(filnam);
          CopiaStringa(prj->szWorkDir, sizeof(prj->szWorkDir), dir.c_str());
          std::string fil = g_path_get_basename(prj->szMtzFile);
          std::string filc = dir + G_DIR_SEPARATOR_S + fil;
          CopiaStringa(prj->szMtzFile, sizeof(prj->szMtzFile), filc.c_str()); 
          fil = g_path_get_basename(prj->szSeqFile);
          filc = dir + G_DIR_SEPARATOR_S + fil;
          CopiaStringa(prj->szSeqFile, sizeof(prj->szSeqFile), filc.c_str()); 
          fil = g_path_get_basename(prj->szMad2File);
          filc = dir + G_DIR_SEPARATOR_S + fil;
          CopiaStringa(prj->szMad2File, sizeof(prj->szMad2File), filc.c_str()); 
          for(int i=0; i<prj->nMRModels; i++)
          {
              fil = g_path_get_basename(prj->szMRFile[i]);
              filc = dir + G_DIR_SEPARATOR_S + fil;
              CopiaStringa(prj->szMRFile[i], sizeof(prj->szMRFile[i]), filc.c_str()); 
          }
          fil = g_path_get_basename(prj->szMtzFilePh);
          filc = dir + G_DIR_SEPARATOR_S + fil;
          CopiaStringa(prj->szMtzFilePh, sizeof(prj->szMtzFilePh), filc.c_str());
          prj->Save();
          prj->SaveAscii();
          Ilm::TheApp.AddExample(prj->GetFilename(Ilm::PROJECT_EXT));
          ChangeMode(prj->GetFilename(Ilm::PROJECT_EXT).c_str(),  rwXt);
          prj = NULL;
        }
    }
}

void ilmCheckForUpdates(GtkWidget *item, gpointer data)
{
      int n;
      int result = Ilm::TheApp.LastUpdate(false, &n);
      int flag = Ilm::TheApp.AutoUpdate();
      if(result > 0)
         UpdateCheck(Ilm::TheApp.SplashWidget(), result, &flag,
               g_strdup_printf("%d.%2.2d", n/100, n%100));
      else
         UpdateCheck(Ilm::TheApp.SplashWidget(), result, &flag, VERSION);
      Ilm::TheApp.SetAutoUpdate(flag);
}

