#ifndef __ILM_PROJECT_H_mrt
#define __ILM_PROJECT_H_mrt

// #include <iosfwd>
#include <string>
#include <exception>

#include "ilm_defs.h"

namespace Ilm
{
  const float dresExtramin = 1;

  class Project {
  public:
    Project(const std::string &fn, ProjectType typ);
    Project(const std::string &filename);
    Project(bool isasc, const std::string &fn);
    void SetDefaults();
    void Save() const;
    void SetType(ProjectType typ) { eType = typ; }
    int SaveAsSir(int mad, const gchar *fn);
    void SaveAsSirEdm() const;
    void SaveAscii() const;
    void SaveAsDea() const;
    void CreateScriptsPhenix() const;
    void CreateScriptsArp() const;
    void CreateScriptsBuccNau() const;
    void CreateCadShell() const;
    double SolventFraction() const {return 1.0 - DProteinVm/Vm(0);}
    double SolventFraction1() const {return 1.0 - DProteinVm/Vm(1);}
    double Vm(int which) const;
    int NResidues() const;
    int NResiduesMod(int i) const;
    int AnalizzaSequenza(const std::string &str) const;
    int ContentFromSequence() const;
    int ContentFromSequenceStr(const std::string &str) const;
    int NCellResidues() const {return NResidues()*nMolecules*nSpgrOrder;}
    int NCellResiduesMod(int i) const {return NResiduesMod(i)*nMRNmol[i]*nSpgrOrder;}
    double MolecularWeight() const;
    void OrganicContent(std::ostream &os) const;
    void Content(std::ostream &os) const;
    int HeteroAtoms(std::ostream &os) const;
    int HeteroAndAnom(std::ostream &os, std::ostream &os1, int *n_anom, int mad) const;
    void AddFlex(std::ostream &os) const;
    int NNonHProtein() const;
    int NNonHtot() const;
    std::string Mad1Cmd() const;
    std::string Mad2Cmd() const;
    std::string EDMCmd() const;
    std::string DEACmd() const;

    std::string GetFilename(const gchar *suffix) const;
    std::string GetFilename2(const gchar *suffix, int ind) const;

    void _ValidateGen(std::ostream &); // const; 
    void format_pir(std::string &s, std::string &t) const;
    void FASTAToPIR() const;

    char szSignature[256];
    double dPlatform[4];
    double iPlatform[4];

    ProjectType eType;

    // general
    gchar szWorkDir[PATH_LEN];
    gchar szStructName[128];
    gchar szJobTitle[128];

    // mtz
    gchar szMtzFile[PATH_LEN];
    gchar szSelectedColumns[PATH_LEN];
    int nSpgrOrder;
    double dCell[6];
    gchar szSpgrName[16];
    int nNrRefl;
    float dresMax, dresMin;

    // composition
    int iCont; // 0 = Molecular Weight, 1 = N. of residues, 2 = Sequence 
    double dSolventFraction; 
    double dContent;  // n resid or mol. weight
    int nMolecules;
    int nElements; // 1 <= nElements <= MAX_ELEMENTS
    char szElement[MAX_ELEMENTS][MAX_SYMBOL_LEN];
    int nElementAmount[MAX_ELEMENTS];
    double dElementF1F2[MAX_ELEMENTS][MAX_WAVES][2];
    // MIR/SIR additional elements
    int nMirElements[MAX_DERIVATIVES];
    char szMirElement[MAX_DERIVATIVES][MAX_ELEMENTS][MAX_SYMBOL_LEN];
    int nMirElementAmount[MAX_DERIVATIVES][MAX_ELEMENTS];

    bool bNativePresent; // true: present, false: net 
    int nWaves;
    int nDeriv;
    int nMrsWaves[MAX_WAVES];
    char szSeqFile[PATH_LEN];
    int nSource;
    int nWaveLength;
    double dWaveLength;

    // tasks
    bool bMad1, bMad2, bEdm;
    bool bMad1Auto;
    bool bMad1Waves[MAX_WAVES], bMad2Waves[MAX_WAVES];
    double dMad1Maxres;
    int nMad1Trials1, nMad1Trials2;
    // nr of Blocks 
    int nBlocks;

    gchar szMad2File[PATH_LEN];
    int nMad1Refls;
    int iMad2SetMin, iMad2SetMax;

    bool bMad1Patterson, bMad1MorePower, bMad1Defaults;
    bool bMad2Lsq, bMad2LsqPos, bMad2LsqAdp, bMad2LsqSof, bMad2LsqF1, bMad2LsqF2;

    // EDM
    int iMad2FlexDM[4], iEdmConv, iEdmExt, iEdmFreeLunch;
    float dresExtra, dresFL;
    bool bExtraDefaults, bFLDefaults;

    int iStrategy, nFrag;

    //PROFLEX
    bool bPro, bProFlex, bProDerivs[MAX_DERIVATIVES];
    int nProTrials, nProStore, nProUse;

    //MOLREP
    int nMRModels;
    gchar szMRFile[MAX_MODELS][PATH_LEN];
    bool bMRUsePseudo;
    bool bMRUseHetero[MAX_MODELS], bMRRot[MAX_MODELS], 
         bMRTrans[MAX_MODELS], bMREdm[MAX_MODELS], bMRSeq[MAX_MODELS];
    char cMRAdp[MAX_MODELS]; // 'P' - PDB, 'W' - Wilson, '2' - 20
    int nMRNmol[MAX_MODELS], iMRalfa[MAX_MODELS], iMRbeta[MAX_MODELS], iMRgamma[MAX_MODELS];
    double dMREdmres[MAX_MODELS], dMRxx[MAX_MODELS], dMRyy[MAX_MODELS], dMRzz[MAX_MODELS], dMRvalue[MAX_MODELS];
    bool bMRPolyalanine[MAX_MODELS], bMRCutModel[MAX_MODELS];
    int n1MR[MAX_MODELS], n2MR[MAX_MODELS];
    int min_resid[MAX_MODELS], max_resid[MAX_MODELS];
    int n_residui[MAX_MODELS];

    bool bMRRes, bMRTSAuto;
    double dMRMinres, dMRMaxres, dMRTSRot, dMRTSRotLoc, dMRTSTra, dMRTSTraLoc;

    //Ab-initio
    bool bAbiPatterson;
    bool bAbiDefaults;
    int nAbiBlocks;
    int nAbiTrialG[MAX_ABI_BLOCKS], nAbiTrialS[MAX_ABI_BLOCKS];

    bool bFindSpace;
   
    // DEA
    // gchar szMRFile[0][PATH_LEN]; usato per DEA Model file PDB
    // gchar szMRFile[1][PATH_LEN]; usato per DEA Phases Heavy atoms PDB
    int nBuildPgm;
    int nCycles;
    bool bModel;
    // DEA Phases: mtz
    gchar szMtzFilePh[PATH_LEN];
    gchar szSelectedColumnsPh[PATH_LEN];
    int nSpgrOrderPh;
    double dCellPh[6];
    gchar szSpgrNamePh[16];
    int nNrReflPh;
    float dresMaxPh, dresMinPh;
  };


} // namespace Ilm

#endif // __ILM_PROJECT_H_mrt

