#include "jav.h"
#include "glext.h"
#include "models.h"
#include "maps.h"
#include "callbacks.h"
#include <jav_cb.h>
#include <forcomm.h>
#include <gtkmrcomune.h>
#include <sstream>
#include <fstream>
#include <iomanip>

namespace main_app
{
    extern Application JavApp;
}

extern "C" void gesfile_(char *filenam, int *tipo, int *dum, size_t l);
extern "C" void exitilmv_();
extern "C" void salvapdb_(int *wm, int vmodi[], const char *savefile, size_t l);
extern _Dialogo *dialogo;

static const char *menus[] = {"Delete residue", "Delete Side Chain", "Delete Aromatic Ring"};
static const int NMENUS = (int)(sizeof(menus)/sizeof(menus[0]));

namespace main_app
{
    void printHelp(std::ostream &os);
}

namespace GLextmr
{
    extern Model *ModelInfo; //[5];
    extern Mappa *MapInfo;   //[5];
    extern std::vector<Legenda> Legend;
    extern GtkWidget *GLWINDOW;
    extern bool ilmilione;
    extern std::string defaultTitle;
    RESIDUE GetResiduo(int whichModel, int whichChain, int whichResidue);
    void SetLastRun(GtkWidget *w, bool flag);
    void sync_radios();
    void delete_model(gpointer, gpointer data);

#ifdef USE_ACTIONS
void on_action_ViewSequence(gpointer item, gpointer data)
{
     int kmodel = GPOINTER_TO_INT(data);
     ViewSequence(kmodel);
}
#else
void on_ViewSequence(gpointer item, gpointer data)
{
     bool active = (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item)) != 0);
     if(!active)
        return;
     int kmodel = GPOINTER_TO_INT(data);
     ViewSequence(kmodel);
}
#endif

void change_tip(GtkWidget *spin, gpointer data)
{
   int whichModel = GPOINTER_TO_INT(data);
   int n = spin_button_get_value_as_int(spin) - 1;
   if(n < 0)
      return;
   const char *hint = ModelInfo[whichModel].GetResName(n).c_str();
   set_tip( GTK_WIDGET(spin), hint);
}

void ok_delete(GtkWidget *button, gpointer data)
{
using namespace gtkext;
   int whichModel = GPOINTER_TO_INT(data);
   GtkWidget *dlg = gtk_widget_get_toplevel(button);
   GtkWidget *spin = get_widget(dlg, "from");
   int n1 = spin_button_get_value_as_int(spin) - 1;
   spin = get_widget(dlg, "to");
   int n2 = spin_button_get_value_as_int(spin) - 1;
   if(n1 > n2)
   {
      dlg_errmsg(GTK_WINDOW(dlg), "...");
      return;
   }
   int come = 0;
   for(int i=0; i<NMENUS; i++)
   {
       GtkWidget *radio = get_widget(dlg, menus[i]);
       if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio)))
       {
          come = i;
          break;
       }
   }
   switch(come)
   {
   case 0:              // delete residue
           for(int i=n1; i<=n2; i++)
              ModelInfo[whichModel].DeleteResidue(i);
           break;
   case 1:              // delete side chain
           for(int i=n1; i<=n2; i++)
              ModelInfo[whichModel].DeleteSideChain(i);
           break;
   case 2:              // delete aromatic ring
           for(int i=n1; i<=n2; i++)
              ModelInfo[whichModel].DeleteRing(i);
           break;
   }
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
   ModelInfo[whichModel].PreparaTabelle();
   gl->Redraw();
   gtk_widget_destroy(dlg);
}

void on_global_modify(gpointer,  gpointer )
{
   int whichModel = GetActiveModel() - 1;
   if(whichModel == -1)
      return;

   GtkWidget *dlg = gtk_dialog_new();
   gtk_container_set_border_width (GTK_CONTAINER (dlg), 8);
   gtk_window_set_title (GTK_WINDOW (dlg), "Modify Residues");
   const char *nameres = ModelInfo[whichModel].GetResName(0).c_str();

   GtkWidget *vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
   GtkWidget *table = table_new(NMENUS/2+2, 2, FALSE);
   gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 2);
   GSList *group = NULL;
   for(int kn=0; kn<NMENUS; kn++)
   {
      GtkWidget *radio = RADIO_BUTTON_WITH_LABEL(group, menus[kn], NULL);
      std::string name1 = menus[kn];
      group =  gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
      g_object_set_data(G_OBJECT(dlg), name1.c_str(), radio);
      table_attach(table, radio, kn%2, kn%2 +1, kn/2, kn/2+1, TRUE);
   }
   int nmax = ModelInfo[whichModel].nResAmino;
   GtkWidget *spin  = SPIN_WITH_LABEL("From Residue", NULL, 1, nmax, 1, 1, 0);
   g_object_set_data(G_OBJECT(dlg), "from", get_widget(spin, "spin"));
   gtk_container_set_border_width(GTK_CONTAINER(spin), 4);
   table_attach(table, spin, 0, 1, NMENUS+1, NMENUS+2,TRUE);
   g_signal_connect(G_OBJECT(get_widget(spin, "spin")), "value-changed",
       G_CALLBACK(change_tip), GINT_TO_POINTER(whichModel));
   set_tip( spin, nameres);
   spin  = SPIN_WITH_LABEL("To Residue", NULL, 1, nmax, 1, 1, 0);
   g_object_set_data(G_OBJECT(dlg), "to", get_widget(spin, "spin"));
   gtk_container_set_border_width(GTK_CONTAINER(spin), 4);
   table_attach(table, spin, 1, 2, NMENUS+1, NMENUS+2, TRUE);
   g_signal_connect(G_OBJECT(get_widget(spin, "spin")), "value-changed",
       G_CALLBACK(change_tip), GINT_TO_POINTER(whichModel));
   set_tip( spin, nameres);
   GtkWidget *button = OK_button();
   gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_OK);
   g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(ok_delete),
        GINT_TO_POINTER(whichModel));
   button = CANCEL_button();
   gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_CANCEL);
   g_signal_connect_swapped(G_OBJECT(button), "clicked",
       G_CALLBACK(gtk_widget_destroy), dlg);
   gtk_widget_show_all(dlg);
}

void on_delete_residue(gpointer, gpointer )
{
    GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
    int whichModel = GetActiveModel() - 1;
    int nsel = (int) ModelInfo[whichModel].resid_sel.size();
    for(int i=0; i<nsel; i++)
    {
        int j = ModelInfo[whichModel].resid_sel.at(i) - 1;
        ModelInfo[whichModel].DeleteResidue(j);
    }
    endModify(whichModel);
    ModelInfo[whichModel].PreparaTabelle();
    gl->Redraw();
}

void on_delete_sidechain(gpointer, gpointer )
{
    GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
    int whichModel = GetActiveModel() - 1;
    int nsel = (int) ModelInfo[whichModel].resid_sel.size();
    for(int i=0; i<nsel; i++)
    {
        int j = ModelInfo[whichModel].resid_sel.at(i) - 1;
        ModelInfo[whichModel].DeleteSideChain(j);
    }
    endModify(whichModel);
    ModelInfo[whichModel].PreparaTabelle();
    gl->Redraw();
}

void on_delete_aromatic(gpointer, gpointer )
{
    GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
    int whichModel = GetActiveModel() - 1;
    int nsel = (int) ModelInfo[whichModel].resid_sel.size();
    for(int i=0; i<nsel; i++)
    {
        int j = ModelInfo[whichModel].resid_sel.at(i) - 1;
        ModelInfo[whichModel].DeleteRing(j);
    }
    endModify(whichModel);
    ModelInfo[whichModel].PreparaTabelle();
    gl->Redraw();
}

void check_save_model(int whichModel)
{
using namespace gtkext;
   if(!ilmilione)
      return;
   if( dlg_yesno(GTK_WINDOW(GLWINDOW), "Do you want to save model?") )
   {
       FilselItem local_item;

       local_item = saveas(GLWINDOW, "*.pdb", NULL);
       if(local_item.exitCode != -1)
       {
          int wM = whichModel + 1;
          int *vmodi = g_new0(int, ModelInfo[whichModel].nats_au);
          for(int i=0; i<ModelInfo[whichModel].nats_au; i++)
              vmodi[i] = ModelInfo[whichModel].atm[i].deleted?1:0;
          std::string savefile = local_item.cwd;
          savefile.append(str_cut(local_item.fileinp, "."));
          savefile.append(".pdb");
          size_t l = savefile.length();
          salvapdb_(&wM, vmodi, savefile.c_str(), l);
          if(ilmilione)
             g_print("OutFile: %s\n", savefile.c_str());
       }
   }
}

void GLWindow::UpdateTitle()
{
//     if(!str_white(defaultTitle.c_str()))
//       return;
//     std::string titolo = "Jav";
     std::string titolo = defaultTitle;
     int whichModel = GetActiveModel() - 1;
     if(nModels && (whichModel != -1))
     {
        titolo.append(": Active model = ");
        titolo.append(g_path_get_basename(ModelInfo[whichModel].modname));
     }
     if(nMaps && (activeMap != -1))
     {
        titolo.append(", Active map = ");
        titolo.append(MapInfo[activeMap].name);
        UpdateSigma();
     }
     gtk_window_set_title(GTK_WINDOW(GLWINDOW), toUTF8(titolo.c_str()));
}

void update_title(GLWindow *gl, int ifModel)
{
     if(!str_white(defaultTitle.c_str()))
       return;
     if(strlen(gtk_window_get_title(GTK_WINDOW(GLWINDOW))) >3)
        return;
     std::string title = "Jav";
     if(gl->nModels)
     {
        title.append(": ");
        title.append(g_path_get_basename(ModelInfo[0].modname));
     }
     else if(ifModel)
        gtk_window_set_title(GTK_WINDOW(GLWINDOW), toUTF8(title.c_str()));
     else if(gl->nMaps)
     {
        title.append(": ");
        title.append(MapInfo[0].name);
     }
     gtk_window_set_title(GTK_WINDOW(GLWINDOW), toUTF8(title.c_str()));
}

#ifdef USE_ACTIONS

void on_action_modify_model(GtkAction *item)
{
     GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
     int whichModel = gtk_radio_action_get_current_value(GTK_RADIO_ACTION(item));
     int inModify = GetActiveModel() - 1;
     if(inModify >= 0)
        check_save_model(inModify);
     inModify = whichModel;

     GtkAction *wid = (GtkAction *)get_object(GLWINDOW, "Modify");
     if(wid)
        gtk_action_set_sensitive(wid, TRUE);
     wid = (GtkAction *)get_object(GLWINDOW, "Polyhedra");
     if(wid)
        gtk_action_set_sensitive(wid, !ModelInfo[inModify].isPDB);
     wid = (GtkAction *)get_object(GLWINDOW, "Polyhedra Properties");
     if(wid)
        gtk_action_set_sensitive(wid, !ModelInfo[inModify].isPDB);
     wid = (GtkAction *)get_object(GLWINDOW, "Contacts");
     if(wid)
        gtk_action_set_sensitive(wid, TRUE);
//     update_items_modify(true);
     updateModify(1);
     wid = (GtkAction *)get_object(GLWINDOW, "Tools");
     if(wid)
        gtk_action_set_sensitive(wid, !ModelInfo[inModify].isPDB);
     gl->UpdateTitle();
     int wm = inModify + 1;
     SetActiveModel(wm);
     SetActive(inModify);
     aggiornaMenuSym();
     if(get_object(GLWINDOW, "Polyhedra") != NULL)
     {
          gboolean ifp = ModelInfo[inModify].ifPoly;
          gpointer item1 = get_object(GLWINDOW, "Polyhedra");
          obj_set_active(item1, ifp, true);
     }
     sync_menubar();
}

void update_model_selector()
{
     GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
//    if(!gl->nModels)
//       return;
     GtkUIManager *merge = (GtkUIManager *) get_object(GLWINDOW, "ui-manager");
     GtkActionGroup *ag = (GtkActionGroup *) get_object(GLWINDOW, "action_group");
     GError *error = NULL;
     std::string buf = "<ui>"
                       "  <menubar name='MenuBar'>"
                       "    <menu action='File'>";
     for(int i=0; i<gl->nModels; i++)
         buf.append(g_strdup_printf("    <menuitem action='model%d'/>", i));
     buf.append("    </menu>"
                "  </menubar>"
                "</ui>");

     GtkRadioActionEntry act[MAX_NUM_MODELS];
     guint actModels = (guint) GPOINTER_TO_INT(g_object_get_data(G_OBJECT(merge), "nModels"));
     if(actModels)
     {
        gtk_ui_manager_remove_ui(merge, actModels);
        for(int i=0; i<MAX_NUM_MODELS; i++)
        {
            const char *name = g_strdup_printf("model%d",i);
            GtkAction *act = gtk_action_group_get_action(ag, name);
            if(act != NULL)
               gtk_action_group_remove_action(ag, act);
        }       
     }
    if(!gl->nModels)
       return;
     for(int i=0; i<gl->nModels; i++)
     {
         act[i].name = g_strdup_printf("model%d",i);
         act[i].stock_id = NULL; 
         act[i].label = g_path_get_basename(ModelInfo[i].modname);
         act[i].accelerator = NULL;
         act[i].tooltip = NULL;
         act[i].value = i;
     }
     gtk_action_group_add_radio_actions(ag, act, gl->nModels, 0, 
              G_CALLBACK(on_action_modify_model), NULL);
     g_object_set_data(G_OBJECT(merge), "nModels", GINT_TO_POINTER(0));
     actModels = gtk_ui_manager_add_ui_from_string(merge, buf.c_str(), -1, &error);
     if(!actModels)
     {
          g_message ("building menus failed: %s", error->message);
          g_error_free (error);
     }
     else
        g_object_set_data(G_OBJECT(merge), "nModels", GINT_TO_POINTER(actModels));
     for(int i=0; i<gl->nModels; i++)
     {
         const char *name = g_strdup_printf("model%d",i);
         GtkAction *act = gtk_action_group_get_action(ag, name);
         if(act != NULL)
         {
            if(gl->nModels > 1)
               gtk_action_activate(act);
            else
               on_action_modify_model(act);
         }
     }
}
#else

void on_modify_model(gpointer item, gpointer data)
{
     bool active = (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item)) != 0);
     if(!active)
        return;
     GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
     int whichModel = GPOINTER_TO_INT(data);
     int inModify = GetActiveModel() - 1;
     if(inModify >= 0)
        check_save_model(inModify);
     inModify = whichModel;

     GtkWidget *wid = get_widget(GLWINDOW, "Modify");
     if(wid)
        gtk_widget_set_sensitive(wid, TRUE);
     wid = get_widget(GLWINDOW, "Polyhedra");
     if(wid)
        gtk_widget_set_sensitive(wid, !ModelInfo[inModify].isPDB);
     wid = get_widget(GLWINDOW, "Polyhedra Properties");
     if(wid)
        gtk_widget_set_sensitive(wid, !ModelInfo[inModify].isPDB);
     wid = get_widget(GLWINDOW, "Contacts");
     if(wid)
        gtk_widget_set_sensitive(wid, TRUE);
//     update_items_modify(true);
     updateModify(1);
     wid = get_widget(GLWINDOW, "Tools");
     if(wid)
        gtk_widget_set_sensitive(wid, !ModelInfo[inModify].isPDB);
     gl->UpdateTitle();
     int wm = inModify + 1;
     SetActiveModel(wm);
     SetActive(inModify);
     aggiornaMenuSym();
     if(get_object(GLWINDOW, "Polyhedra") != NULL)
     {
          gboolean ifp = ModelInfo[inModify].ifPoly;
          gpointer item1 = get_object(GLWINDOW, "Polyhedra");
          obj_set_active(item1, ifp, true);
     }
     sync_radios();
}

void update_model_selector()
{
     GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
     GtkWidget *item = get_widget(GLWINDOW, "File");
     GtkWidget *menuitem = gtk_menu_item_get_submenu(GTK_MENU_ITEM(item));
     int i0 = -1;
     if(menuitem != NULL)
     {
        GList *children = gtk_container_get_children(GTK_CONTAINER(menuitem));
        for(int i=0; i<(int)g_list_length(children); i++)
        {
            item = (GtkWidget *)g_list_nth_data(children, i);
            if(get_widget(GLWINDOW, "Exit") == item)
            {
               i0 = i + 1;
               break;
            }
        }
        if(i0 > 0)
        {
           for(int i=i0; i<(int)g_list_length(children); i++)
           {
               item = (GtkWidget *)g_list_nth_data(children, i);
               gtk_widget_destroy(item);
           }
        }
        if(!gl->nModels)
           return;
        GSList *bg_group = NULL;
        const char *fontname = (const char*)get_string(GLWINDOW, "fontU");
        int kp = 0;
        for(int i=0; i<gl->nModels; i++)
        {
           kp++;
           item = gtk_radio_menu_item_new_with_label(bg_group, ModelInfo[i].modname);
           bg_group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(item));
           GtkWidget *label = gtk_bin_get_child(GTK_BIN(item));
           gtk_widget_show(item);
           set_widget_font(label, fontname);
           gtk_menu_shell_append(GTK_MENU_SHELL(menuitem),item );
           gtk_widget_set_name(item, ModelInfo[i].modname);
           gtk_widget_show(item);
           int hnd = g_signal_connect( G_OBJECT(item), "activate",
                        G_CALLBACK(on_modify_model), GINT_TO_POINTER(i));
           g_object_set_data(G_OBJECT(item), "hnd", GINT_TO_POINTER(hnd));
//           if((gl->nModels == 1))// && (gl->nMaps == 0))
              gtk_menu_item_activate(GTK_MENU_ITEM(item));
        }
     }
}
#endif

#ifdef USE_ACTIONS
void update_modify_menu(bool ifNewModel)
{
     if(!ifNewModel)
        return;
     GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
     GtkAction *wid = (GtkAction *)get_object(GLWINDOW, "Modify");
     if(wid)
        gtk_action_set_sensitive(wid, FALSE);
     wid = (GtkAction *)get_object(GLWINDOW, "Tools");
//     if(wid)
//        gtk_action_set_sensitive(wid, FALSE);
     wid = (GtkAction *)get_object(GLWINDOW, "Contacts");
     if(wid)
        gtk_action_set_sensitive(wid, FALSE);
     wid = (GtkAction *)get_object(GLWINDOW, "Polyhedra");
     if(wid)
        gtk_action_set_sensitive(wid, FALSE);
     wid = (GtkAction *)get_object(GLWINDOW, "Polyhedra Properties");
     if(wid)
        gtk_action_set_sensitive(wid, FALSE);
     SetActiveModel(1);
//     SetActiveModel(0);
     update_model_selector();

     GtkAction *item = (GtkAction *)get_object(GLWINDOW, "Sequence");
     if(item == NULL)
        return;
     
     GtkUIManager *merge = (GtkUIManager *) get_object(GLWINDOW, "ui-manager");
     GtkActionGroup *ag = (GtkActionGroup *) get_object(GLWINDOW, "action_group");
     GError *error = NULL;
     std::string buf = "<ui>"
                       "  <menubar name='MenuBar'>"
                       "    <menu action='ViewMenu'>"
                       "       <menu action='Sequence'>";
     for(int i=0; i<gl->nModels; i++)
         buf.append(g_strdup_printf("    <menuitem action='models%d'/>", i));
     buf.append("       </menu>"
                "    </menu>"
                "  </menubar>"
                "</ui>");

//     GtkRadioActionEntry act[MAX_NUM_MODELS];
     guint actModels = (guint) GPOINTER_TO_INT(g_object_get_data(G_OBJECT(merge), "nModelsSeq"));
     if(actModels)
     {
        gtk_ui_manager_remove_ui(merge, actModels);
        for(int i=0; i<MAX_NUM_MODELS; i++)
        {
            const char *name = g_strdup_printf("models%d",i);
            GtkAction *act = gtk_action_group_get_action(ag, name);
            if(act != NULL)
               gtk_action_group_remove_action(ag, act);
        }       
     }
     int kp = 0;
     GSList *r_group = NULL;
     for(int i=0; i<gl->nModels; i++)
     {
        if(ModelInfo[i].IsProtein())
        {
           kp++;
           GtkRadioAction *act =
              gtk_radio_action_new(g_strdup_printf("models%d",i),
                                   g_path_get_basename(ModelInfo[i].modname),
                                   NULL, NULL, i);
           gtk_radio_action_set_group(act, r_group);
           r_group = gtk_radio_action_get_group(act);
           gtk_action_group_add_action(ag, GTK_ACTION(act));
           int hnd = g_signal_connect( G_OBJECT(act), "activate",
                        G_CALLBACK(on_action_ViewSequence), GINT_TO_POINTER(i));
           g_object_set_data(G_OBJECT(act), "hnd", GINT_TO_POINTER(hnd));
        }
     }
     if(!kp)
        return;
     g_object_set_data(G_OBJECT(merge), "nModelsSeq", GINT_TO_POINTER(0));
     actModels = gtk_ui_manager_add_ui_from_string(merge, buf.c_str(), -1, &error);
     if(!actModels)
     {
          g_message ("building menus failed: %s", error->message);
          g_error_free (error);
     }
     else
        g_object_set_data(G_OBJECT(merge), "nModelsSeq", GINT_TO_POINTER(actModels));
     gtk_action_set_sensitive(item, kp);
     sync_menubar();
}
#else
void update_modify_menu(bool ifNewModel)
{
     if(!ifNewModel)
        return;
     GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
     GtkWidget *wid = get_widget(GLWINDOW, "Modify");
     if(wid)
     {
        gtk_widget_set_sensitive(wid, FALSE);
//        if(!ilmilione)
//           gtk_widget_hide(wid);
     }
     wid = get_widget(GLWINDOW, "Tools");
     if(wid)
        gtk_widget_set_sensitive(wid, FALSE);
     wid = get_widget(GLWINDOW, "Contacts");
     if(wid)
        gtk_widget_set_sensitive(wid, FALSE);
     wid = get_widget(GLWINDOW, "Polyhedra");
     if(wid)
        gtk_widget_set_sensitive(wid, FALSE);
     wid = get_widget(GLWINDOW, "Polyhedra Properties");
     if(wid)
        gtk_widget_set_sensitive(wid, FALSE);
     SetActiveModel(0);
     update_model_selector();

     GtkWidget *item = get_widget(GLWINDOW, "Sequence");
     if(item == NULL)
        return;
     
     GtkWidget *menuitem = gtk_menu_new();
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menuitem);
     gtk_widget_show(menuitem);
     GSList *bg_group = NULL;
     const char *fontname = (const char*)get_string(GLWINDOW, "fontU");
     int kp = 0;
     for(int i=0; i<gl->nModels; i++)
     {
        if(ModelInfo[i].IsProtein())
        {
           kp++;
           item = gtk_radio_menu_item_new_with_label(bg_group, ModelInfo[i].modname);
           bg_group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(item));
           GtkWidget *label = gtk_bin_get_child(GTK_BIN(item));
           gtk_widget_show(item);
           set_widget_font(label, fontname);
           gtk_menu_shell_append(GTK_MENU_SHELL(menuitem),item );
           gtk_widget_set_name(item, ModelInfo[i].modname);
           gtk_widget_show(item);
           int hnd = g_signal_connect( G_OBJECT(item), "activate",
                        G_CALLBACK(on_ViewSequence), GINT_TO_POINTER(i));
           g_object_set_data(G_OBJECT(item), "hnd", GINT_TO_POINTER(hnd));
        }
     }
     item = get_widget(GLWINDOW, "Sequence");
     gtk_widget_set_sensitive(item, kp);
     sync_menubar();
//     update_title(gl,0);
}
#endif

void ImportFile(std::string fn, int t)
{
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");

   size_t sz = strlen(fromUTF8(fn).c_str());
   int tipo = t;
   int dum = 0;
   char *filenam = g_new0(char, sz+1);
   CopiaStringa(filenam, sz+1, fromUTF8(fn).c_str());
   gl->DistAng_vet.clear();

   gesfile_(filenam, &tipo, &dum, sz);
   dir_set_current( g_path_get_dirname(filenam) );
}

bool ifPol(GLWindow *gl)
{
   for(int i=0; i<gl->nModels; i++)
       if(ModelInfo[i].nPoly)
          return true;
   return false;
}

void on_Open(GtkWidget *window, const char *filter, int tipo)
{
using namespace gtkext;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");

   if( ( (tipo < 11) || (tipo > 20) ) && (gl->nMaps == MAX_NUM_MAPS) )
   {
      dlg_errmsg(GTK_WINDOW(window), "Sorry. Maximum number of reflections/map files already loaded.");
      return;
   }
   if( ( (tipo > 10) || (tipo < 21) ) && (gl->nModels == MAX_NUM_MODELS) )
   {
      dlg_errmsg(GTK_WINDOW(window), "Sorry. Maximum number of model files already loaded.");
      return;
   }
//   if( ( (tipo > 10) || (tipo < 21) ) && ifPol(gl) )
//   {
//      dlg_errmsg(GTK_WINDOW(window), "Sorry. Maximum number of model files already loaded.");
//      return;
//   }
   std::string fn;
   fn =  gtkext::dlg_select_file(window, fn, gtkext::eSELECT_TO_READ, filter);
   if(str_white(fn.c_str()))
       return;
   aggiorna_cfg(fn);
   ImportFile(fn, tipo);
}

void on_SmartOpen(GtkWidget *window, const char *filter)
{
using namespace gtkext;
   int tipo = 0;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   const char *valid_exts[] = {"mtz", "fcf", "phs", "hkl", 
                               "pdb", "ent", "cif", "ins", "res",
                               "map", "xyz", "dmp"}; 
   int valid_types[] =        {  1,      2,     3,     4,
                                11,     11,    12,    13,   13,
                                21,     22,    23};

   std::string fn;
   fn =  gtkext::dlg_select_file(window, fn, gtkext::eSELECT_TO_READ, filter);
   if(str_white(fn.c_str()))
       return;
   size_t next = fn.find_last_of(".");
   if (next == std::string::npos)
   {
       dlg_errmsg(GTK_WINDOW(window), "Sorry. Unknown file type.");
       return;
   }
   std::string extens = fn.substr(next+1);
   for(int i=0; i<(int)(sizeof(valid_exts)/sizeof(valid_exts[0])); i++)
   {
       if(extens.compare(valid_exts[i]) == 0)
       {
          tipo = valid_types[i];
          break;
       }
   }
   if(tipo == 0) 
   {
      dlg_errmsg(GTK_WINDOW(window), "Sorry. Unknown extension.");
      return;
   }

   if( ( (tipo < 11) || (tipo > 20) ) && (gl->nMaps == MAX_NUM_MAPS) )
   {
      dlg_errmsg(GTK_WINDOW(window), "Sorry. Maximum number of reflections/map files already loaded.");
      return;
   }
   if( ( (tipo > 10) || (tipo < 21) ) && (gl->nModels == MAX_NUM_MODELS) )
   {
      dlg_errmsg(GTK_WINDOW(window), "Sorry. Maximum number of model files already loaded.");
      return;
   }
//   if( ( (tipo > 10) || (tipo < 21) ) && ifPol(gl) )
//   {
//      dlg_errmsg(GTK_WINDOW(window), "Sorry. Maximum number of model files already loaded.");
//      return;
//   }
   aggiorna_cfg(fn);
   ImportFile(fn, tipo);
}

void on_OpenGeneral(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;

   on_SmartOpen(window, "Coordinates files (*.pdb,*.ent,*.cif,*.ins,*.res)@"
                        "Reflections files (*.mtz,*.fcf,*.phs,*.hkl)@"
                        "Map files (*.map,*.xyz,*.dmp)");
}

void DeleteAll(GLWindow *gl)
{
       int model_to_delete[MAX_NUM_MODELS], map_to_delete[MAX_NUM_MAPS];
       for(int i=0; i<MAX_NUM_MODELS; i++)
       {
           model_to_delete[i] = 1;
           map_to_delete[i]   = 1;
       }

#if 0
      for(int i=0; i<gl->nModels; i++)
      {
          ModelInfo[i].freeModel();
          ModelInfo[i].created = 0;
      }
      gl->nModels = 0;
#else
      for(int i=gl->nModels; i>0; i--)
          delete_model(NULL, GINT_TO_POINTER(i));
#endif
      for(int i=0; i<gl->nMaps; i++)
      {
             MapInfo[i].freeMap();
             if((MapInfo[i].ifDifference <= 1))
                gl->usedC[i] = -1;
      }
      
      deletemapsmodels_(map_to_delete, model_to_delete);
      gl->nMaps = 0;
      update_infos_model(-1);
      update_infos_map(1);
      update_infos_cell(0);
      gl->UpdateTitle();
      gl->ResetView(1);
      //gl->on_rotate(0, -3);
      //gl->on_rotate(1, -3);
      gl->RiNormalizza();
      gl->Redraw();
}

void on_LastRun(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");

   if(!gl->Restore())
      DeleteAll(gl);
}

bool loadconfirm(GLWindow *gl, const char *msg)
{
   if((gl->nMaps+gl->nModels) == 0)
      return true;
   bool result = (gtkext::dlg_yesno(GTK_WINDOW(GLWINDOW), msg) != 0);
   if(result)
       DeleteAll(gl);
   return result;
}

void LoadExample(GLWindow *gl, const char *name)
{
   std::string fn = g_get_home_dir();
   fn.append(G_DIR_SEPARATOR_S);
   fn.append("share");
   fn.append(G_DIR_SEPARATOR_S);
   fn.append(PACKAGE);
   fn.append(G_DIR_SEPARATOR_S);
   fn.append("examples");
//   fn.append(VERSION);
   fn.append(G_DIR_SEPARATOR_S);
   fn.append("viewer");
   (void) dir_set_current(fn.c_str());
   main_app::JavApp.UpdateWD(fn.c_str());
   fn.append(G_DIR_SEPARATOR_S);
   fn.append(name);

   if(!g_file_test(fn.c_str(), G_FILE_TEST_EXISTS))
   {
      gtkext::dlg_errmsg(GTK_WINDOW(GLWINDOW), "Sorry. File %s does not exist.", fn.c_str());
      return;
   }
   bool result = loadconfirm(gl, 
          "If you load an example, the data you are working with will be dismitted. Do you want to proceed ?");

   if(result)
   {
       if(!gl->Restore(fn))
//       if(!gl->Restore(fn, true))
          DeleteAll(gl);
   }
}

void on_LoadSmall(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");

   LoadExample(gl, "small4.ex");
}

void on_LoadMacro(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");

   LoadExample(gl, "macro4.ex");
}

void on_ClearLabels(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   if(!gl->nModels)
      return;
   for(int k=0; k<gl->nModels; k++)
       for(int i=0; i<ModelInfo[k].nats; i++)
           ModelInfo[k].atm[i].drawLabel = 0;
   gl->Redraw();
}

void on_Close(gpointer, gpointer data)
{
using namespace gtkext;
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");

   if( (gl->nModels == 0) && (gl->nMaps == 0) )
   {
//      dlg_errmsg(GTK_WINDOW(window), "Sorry. No model/map loaded.");
      return;
   }

   GtkWidget *dlg = gtk_dialog_new();
   gtk_container_set_border_width (GTK_CONTAINER (dlg), 8);
   gtk_window_set_title (GTK_WINDOW (dlg), "Remove Model(s)/Map(s)");

   GtkWidget *vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
   GtkWidget *label = LABEL_BOLD("Remove Model(s)/Map(s) corresponding to:",
                                 "red", 1.1f);
   set_alignment(label, 0.5, 0.);
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 4);

   GtkWidget *hbox = hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 8);
   GtkWidget *frame = FRAME_NEW("Models", "blue");
   gtk_container_set_border_width (GTK_CONTAINER (frame), 8);
   if(gl->nModels)
      gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
   GtkWidget *frame2 = FRAME_NEW("Maps", "blue");
   gtk_container_set_border_width (GTK_CONTAINER (frame2), 8);
   if(gl->nMaps)
      gtk_box_pack_start(GTK_BOX(hbox), frame2, FALSE, FALSE, 0);
   if(gl->nModels)
   {
      GtkWidget *table = table_new(gl->nModels, 2, FALSE);
      gtk_container_set_border_width (GTK_CONTAINER (table), 4);
      gtk_container_add(GTK_CONTAINER(frame), table);
      for(int i=0; i<gl->nModels; i++)
      {
          std::stringstream stri;
          stri << "mod" << i;
          GtkWidget *label =
             gtk_label_new(g_path_get_basename(ModelInfo[i].modname));
          set_alignment(label, 0., 0.5);
          set_margins(label, 10, 10, 10, 10);
          GtkWidget *check = gtk_check_button_new();
          g_object_set_data(G_OBJECT(dlg), stri.str().c_str(), check);
          table_attach_exp(table, label, 0, 1, i, i+1);
          table_attach_all(table, check, 1, 2, i, i+1);
      }
   }
   if(gl->nMaps)
   {
      GtkWidget *table = table_new(gl->nMaps, 2, FALSE);
      gtk_container_add(GTK_CONTAINER(frame2), table);
      for(int i=0; i<gl->nMaps; i++)
      {
          std::stringstream stri;
          stri <<"map" << i;
          GtkWidget *label = gtk_label_new(MapInfo[i].name);
          set_alignment(label, 0., 0.5);
          set_margins(label, 10, 10, 10, 10);
          GtkWidget *check = gtk_check_button_new();
          g_object_set_data(G_OBJECT(dlg), stri.str().c_str(), check);
          table_attach_exp(table, label, 0, 1, i, i+1);
          table_attach_all(table, check, 1, 2, i, i+1);
      }
   }
   GtkWidget *button = OK_button();
   gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_OK);
   button = CANCEL_button();
   gtk_dialog_add_action_widget(GTK_DIALOG(dlg), button, GTK_RESPONSE_CANCEL);
   gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
         GTK_BUTTONBOX_SPREAD);
   gtk_widget_show_all(dlg);
   int result = gtk_dialog_run(GTK_DIALOG(dlg));
   int map_to_scale[MAX_NUM_MAPS];
   int model_to_delete[MAX_NUM_MODELS], map_to_delete[MAX_NUM_MAPS];
   for(int i=0; i<MAX_NUM_MODELS; i++)
       model_to_delete[i] = 0;
   for(int i=0; i<MAX_NUM_MAPS; i++)
   {
       map_to_scale[i] = 0;
       map_to_delete[i] = 0;
   }
   if(result == GTK_RESPONSE_OK)
   {
      int nmaps_to_delete = 0;
      for(int i=0; i<gl->nModels; i++)
      {
          std::stringstream stri;
          stri << "mod" << i;
          GtkWidget *but = get_widget(dlg, stri.str().c_str());
          model_to_delete[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(but));
      }
      for(int i=0; i<gl->nMaps; i++)
      {
          std::stringstream stri;
          stri << "map" << i;
          GtkWidget *but = get_widget(dlg, stri.str().c_str());
          map_to_delete[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(but));
          if(map_to_delete[i])
             nmaps_to_delete++;
          else
             map_to_scale[i] = nmaps_to_delete;
      }

      for(int i=gl->nModels; i>0; i--)
      {
          if(model_to_delete[i-1])
             delete_model(NULL, GINT_TO_POINTER(i));
      }

      int num = gl->nMaps;
      for(int i=0; i<gl->nMaps; i++)
      {
          if(map_to_delete[i])
          {
             MapInfo[i].freeMap();
             if((MapInfo[i].ifDifference <= 1))
                gl->usedC[i] = -1;
          }
          else if(map_to_scale[i])
          {
              MapInfo[i-map_to_scale[i]] = MapInfo[i];
              if(MapInfo[i].ifDifference > 1)
                 MapInfo[i+MAX_NUM_MAPS-map_to_scale[i]] = MapInfo[i+MAX_NUM_MAPS];
          }
      }
      num = gl->nMaps - nmaps_to_delete;
      gl->nMaps = num;
      SetLastRun(window, (gl->nMaps + gl->nModels) == 0);

      gl->AggiornaListaSpecie();
      update_infos_model(-1);
      update_infos_map(1);
      update_infos_cell(gl->nMaps + gl->nModels);
      if((gl->nMaps + gl->nModels) == 0)
      {
         gl->ResetView(1);
//         gl->on_rotate(0, 3);
//         gl->on_rotate(1, -3);
//         gl->on_rotate(2, 18);
      }
      gl->RiNormalizza();
      gl->Redraw();

      deletemapsmodels_(map_to_delete, model_to_delete);
   }
   gtk_widget_destroy(dlg);
}

void on_Quit(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");

/*
   if(inModify >= 0)
      check_save_model(inModify);
*/
   glDisable(GL_LIGHT0);
   if( gtkext::dlg_yesno(GTK_WINDOW(window), "Are you sure you want to quit?") )
   {
       exitilmv_();
       printf(" Thanks for using %s\n", main_app::JavApp.PackageName());
       if(gl)
          gl->Destroy(0);
   }
   glEnable(GL_LIGHT0);
}

/*
void update_center()
{
     if(g_object_get_data(G_OBJECT(GLWINDOW), SettingsWindow))
     {
        GtkWidget *dlg = get_widget(GLWINDOW, SettingsWindow);
        set_values_for_centre(dlg, GLWINDOW);
     }
}
*/
void open_chelp(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   std::string hlpfn = g_strdup_printf( "%s%c.tmp",
       g_path_get_dirname(main_app::JavApp.ConfDir().c_str()),
       G_DIR_SEPARATOR);
   std::ofstream os;
   os.open(fromUTF8(hlpfn).c_str());
   if(!os)
      return;
   main_app::printHelp(os);
   os.close();

   gtkext::text_window(GTK_WINDOW(window), hlpfn.c_str(), "*.*", 0, "Command options");
   Cancella( hlpfn.c_str());
}

#if 0
void change_residue(GtkWidget *button, gpointer data)
{
     int displacement = GPOINTER_TO_INT(data);
     GtkWidget *dlg = gtk_widget_get_toplevel(button);
     GtkWidget *store = get_widget(dlg, "Chains");
     GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(store));
     GList *rowsel = NULL;
     rowsel = get_selections(store);
     if(rowsel)
     {
        GtkTreePath *path = (GtkTreePath *) g_list_nth_data (rowsel, 0);
        if(displacement)
           gtk_tree_path_next (path);
        else
           gtk_tree_path_prev (path);
        gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW(store), path, NULL, FALSE,
           0, 0);
        gtk_tree_selection_select_path(selection, path);
        char *linea = gtk_tree_path_to_string(path);
        int nodi[3];
        get_int_tokens(linea, ":", 2, nodi);
        int whichModel = get_int(dlg, "whichModel") - 1;
        if(whichModel >= 0)
        {
           if(ModelInfo[whichModel].Chains[nodi[0]].nResidues > (nodi[1]+1))
           {
              RESIDUE r0 = GetResiduo(whichModel, nodi[0], nodi[1]);
              int Calpha = 0;
              int k0 = r0.atoms[0];
              int kk = ModelInfo[whichModel].FindAtom("CA", r0);
              if(kk != -1)
                 Calpha = kk - k0;
              store = get_widget(dlg, "atoms_PDB");
              select_row(GTK_WIDGET(store), Calpha);
              gtk_button_clicked(GTK_BUTTON(get_widget(dlg, "gobutt")));
           }
        }
        g_list_foreach (rowsel, (GFunc)gtk_tree_path_free, NULL);
        g_list_free (rowsel);
     }
}

static void selected_residue (GtkTreeSelection *selection, gpointer data)
{
    GtkTreeIter iter;
    GtkTreeModel *model;
    GtkTreeView *treeview = gtk_tree_selection_get_tree_view(selection);
    GtkWidget *dlg = (GtkWidget *)data;
    int wm = get_int(dlg, "whichModel") - 1;

    if (gtk_tree_selection_get_selected (selection, &model, &iter))
    {
       char *linea = DuplicaStringa(get_selected_row_path(GTK_WIDGET(treeview)));
       GtkTreePath *path = gtk_tree_path_new_from_string(linea);
       if(gtk_tree_view_expand_row (treeview, path, 0) == 0)
       {
          int nodi[3];
          get_int_tokens(linea, ":", 2, nodi);
//          g_print ("chain %d residue %d\n", nodi[0]+1, nodi[1]+1);
          ModelInfo[wm].FillListaAtomi(dlg, nodi);
          g_object_set_data(G_OBJECT(dlg), "whichChain", GINT_TO_POINTER(nodi[0]));
          g_object_set_data(G_OBJECT(dlg), "whichResidue", GINT_TO_POINTER(nodi[1]));
          g_object_set_data(G_OBJECT(dlg), "whichAtom", GINT_TO_POINTER(-1));
       }
    }
}

void gotoatom(GtkWidget *button, gpointer data)
{
using namespace gtkext;
     GtkWidget *window = (GtkWidget *) data;
     GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
     GtkWidget *dlg = gtk_widget_get_toplevel(button);
     int whichModel = get_int(dlg, "whichModel") - 1;
     int whichAtom = get_int(dlg, "whichAtom");
     if(whichAtom < 0)
     {
        dlg_errmsg(GTK_WINDOW(window), "Sorry. No atom selected");
        return;
     }
     int katom = whichAtom;
     if( ModelInfo[whichModel].isPDB)
     {
          int whichChain = get_int(dlg, "whichChain");
          int whichResidue = get_int(dlg, "whichResidue");
          RESIDUE rr = GetResiduo(whichModel, whichChain, whichResidue);
          katom = rr.atoms[whichAtom];
      }

     float posi[3];
     for (int i=0; i<3; i++)
          posi[i] = ModelInfo[whichModel].atm[katom].xyzc[i];
     memcpy(gl->CentreColor, ModelInfo[whichModel].labelcolor, 3*sizeof(float));
     move_center(posi, whichModel, katom, false);  //true);
}

static void selected_model(GtkWidget *combo, gpointer )
{
    GtkWidget *dlg = gtk_widget_get_toplevel(combo);
    int whichModel = combo_get_val(combo);
    if(whichModel > 0)
    {
       g_object_set_data(G_OBJECT(dlg), "whichModel", GINT_TO_POINTER(whichModel));
       ModelInfo[whichModel-1].FillLists(dlg);
    }
}

static void selected_atom(GtkTreeSelection  *sele, gpointer data)
{
     int row;
     GtkWidget *dlg = (GtkWidget *)data;

     row = row_selected(sele);
     if(row != -1)
        g_object_set_data(G_OBJECT(dlg), "whichAtom", GINT_TO_POINTER(row));
}

GtkWidget *create_gotoatom (gpointer data, GList *listModels, int ifGoto)
{
  GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");

  GtkWidget *window = (GtkWidget *) data;
  GtkWidget *dialog1;
  GtkWidget *button;
  GtkWidget *label;
  GtkWidget *frame;
  GtkWidget *combo=NULL;

  dialog1 = gtk_dialog_new ();
  if(ifGoto)
     gtk_window_set_title (GTK_WINDOW (dialog1), "Go to atom");
  else
     gtk_window_set_title (GTK_WINDOW (dialog1), "Atoms List");
  gtk_container_set_border_width(GTK_CONTAINER(dialog1), 8);
//  gtk_widget_set_size_request(dialog1, -1, 300);
  GtkWidget *vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dialog1));
  gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog1), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(dialog1), GTK_WINDOW(GLWINDOW));
 
  label = LABEL_BOLD("Select an atom for Centering", "Red", 1.1f);
  if(ifGoto)
     gtk_box_pack_start(GTK_BOX(vbox1), label, FALSE, FALSE, 10);

  if(gl->nModels > 1)
  {
     frame = FRAME_BOLD("Select model", "blue", 1.1f);
     gtk_box_pack_start(GTK_BOX(vbox1), frame, FALSE, FALSE, 10);
     combo = create_combo (NULL, NULL, 0, 0, NULL, G_CALLBACK(selected_model));
     gtk_editable_set_editable(GTK_EDITABLE(get_widget(combo, "entry")), FALSE);
     block_handler(combo);
     set_popdown_strings(combo, listModels);
     unblock_handler(combo);
     gtk_container_add(GTK_CONTAINER(frame), get_widget(combo, "parent"));
     gtk_container_set_border_width(GTK_CONTAINER(get_widget(combo, "parent")), 8);
  }
  else
     g_object_set_data(G_OBJECT(dialog1), "whichModel", GINT_TO_POINTER(1));

// PDB files

  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
  g_object_set_data(G_OBJECT(dialog1), "frame_PDB", frame);
  gtk_box_pack_start (GTK_BOX (vbox1), frame, TRUE, TRUE, 4);

  GtkWidget *vbox2 = vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(frame), vbox2);
  if(ifGoto)
  {
      GtkWidget *hbox = hbutton_box_new();
      gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_SPREAD);
      gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 4);
      button = myimage_button ("_Previous Residue", "previousgr.png");
      gtk_container_add(GTK_CONTAINER(hbox), button);
      (void) g_signal_connect(G_OBJECT (button), "clicked",
         G_CALLBACK(change_residue), GINT_TO_POINTER(0));
      button = myimage_button ("  _Next Residue", "nextgr.png");
      gtk_container_add(GTK_CONTAINER(hbox), button);
      (void) g_signal_connect(G_OBJECT (button), "clicked",
         G_CALLBACK(change_residue), GINT_TO_POINTER(1));
  }

  GtkWidget *hbox1 = hpaned_new();
  gtk_box_pack_start (GTK_BOX (vbox2), hbox1, TRUE, TRUE, 4);

  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
  gtk_paned_pack1((GtkPaned *)hbox1, frame, TRUE, FALSE);
  create_tree_store (dialog1, frame, "Chains", G_CALLBACK(selected_residue), dialog1);

  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
  gtk_paned_pack2((GtkPaned *)hbox1, frame, TRUE, FALSE);
  GtkWidget *list = createSelection(dialog1, frame, 
    "Atoms    (    x       y      z   )    occ.    B ",
                      "atoms_PDB", 0, "blue");
  set_widget_fontN(list);
  gtk_widget_set_size_request(get_widget(list, "scrollwin"), -1, 300);
  GtkTreeViewColumn *column = gtk_tree_view_get_column(GTK_TREE_VIEW(list), 0);
  label = gtk_tree_view_column_get_widget(column);
  if(label)
     set_widget_fontN(label);
  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
  (void) g_signal_connect (G_OBJECT (selection), "changed",
                      G_CALLBACK (selected_atom), dialog1);

// CIF, SHELX

  list = createSelection(dialog1, vbox1, 
    "Atoms            (    x       y      z   )    occ.    B ",
                "atoms_o", 0, "blue");
  g_object_set_data(G_OBJECT(dialog1), "frame_noPDB", get_widget(list, "frame"));
  set_widget_fontN(list);
  column = gtk_tree_view_get_column(GTK_TREE_VIEW(list), 0);
  label = gtk_tree_view_column_get_widget(column);
  if(label)
     set_widget_fontN(label);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
  (void) g_signal_connect (G_OBJECT (selection), "changed",
                      G_CALLBACK (selected_atom), dialog1);

  if(ifGoto)
  {
     button = myimage_button("", "go.png");
     gtk_dialog_add_action_widget(GTK_DIALOG (dialog1), button, 1);
     g_object_set_data(G_OBJECT(dialog1), "gobutt", button);
     (void) g_signal_connect(G_OBJECT (button), "clicked",
         G_CALLBACK(gotoatom), data);
  }
  button = myimage_button("  _Close", "cancel.png");
  gtk_dialog_add_action_widget(GTK_DIALOG (dialog1), button, 2);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
         GTK_BUTTONBOX_SPREAD);

  chiudi_dlg_data *st = g_new0(chiudi_dlg_data, 1);
  st->window = window;
  if(ifGoto)
     st->name = "gotoatom";
  else
     st->name = "listatoms";

  (void) g_signal_connect(G_OBJECT (button), "clicked",
         G_CALLBACK(chiudi_dlg), (gpointer) st);

  (void) g_signal_connect_swapped(G_OBJECT(dialog1), "delete_event",
        G_CALLBACK(gtk_button_clicked), G_OBJECT(button));
   (void) g_signal_connect (G_OBJECT (dialog1), "delete_event",
                      G_CALLBACK(true_cb), NULL);

  gtk_widget_show_all(dialog1);
//  g_object_set_data(G_OBJECT(dialog1), "whichModel", GINT_TO_POINTER(1));
//  ModelInfo[0].FillLists(dialog1);
  if(combo && (gl->nModels > 1))
     combo_set_val(combo, 1);

  g_object_set_data(G_OBJECT(dialog1), "whichAtom", GINT_TO_POINTER(-1));
  return dialog1;
}

void GoToAtom(gpointer, gpointer data)
{
using namespace gtkext;
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   GtkWidget *dlg;
   GList *listModels = NULL;

   if(!gl->nModels)
   {
      dlg_errmsg(GTK_WINDOW(window), "Sorry. No models loaded.");
      return;
   }
   for(int i=0; i<gl->nModels; i++)
       listModels = g_list_append(listModels,
             g_path_get_basename(ModelInfo[i].modname));

   if(g_object_get_data(G_OBJECT(window), "gotoatom"))
   {
      dlg = get_widget(window, "gotoatom");
      gdk_window_raise(gtk_widget_get_window(dlg));
   }
   else
   {
      dlg = create_gotoatom(data, listModels, 1);
      g_object_set_data(G_OBJECT(window), "gotoatom", dlg);
      g_object_set_data(G_OBJECT(dlg), "whichModel", GINT_TO_POINTER(1));
   }
   int whichModel = get_int(dlg, "whichModel") - 1;
   if(whichModel < 0)
      return;
    ModelInfo[whichModel].FillLists(dlg);
}
#endif
void on_movecxplus(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   gl->on_move_center(0, 1);
}

void on_movecxminus(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   gl->on_move_center(0, -1);
}

void on_movecyplus(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   gl->on_move_center(1, 1);
}

void on_movecyminus(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   gl->on_move_center(1, -1);
}

void on_moveczplus(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   gl->on_move_center(2, 1);
}

void on_moveczminus(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   gl->on_move_center(2, -1);
}

void on_clip(gpointer button, gpointer data)
{
     GtkWidget *window = (GtkWidget *) data;
     GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");

     gl->ClipFactor = gtk_range_get_value(GTK_RANGE(button));
     gl->change_clip_value();
     gl->Redraw();
}

/*
void on_clickselect(gpointer, gpointer data)
{
using namespace gtkext;
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   bool visibleModel = false;
   for(int i=0; i<gl->nModels; i++)
         if(ModelInfo[i].ifVis)
            visibleModel = true;
   if(!visibleModel)
   {
        dlg_errmsg(GTK_WINDOW(window), "Sorry. No visible atoms.");
        return;
   }
   set_motion_type(SELECT_CENTRE, 1);
   gl->PrintMess("Click on atom to define the center of the plot.");
}
*/

// L e g e n d

void ShowHideLegend(gpointer item, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   gl->ToggleLegend(obj_get_active(item));
}

void DeleteLegend(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   gl->DestroyLegend();
}

void on_subscript(GtkToggleButton *widget, GtkWidget *text)
{
    GtkTextIter iter;
    GtkTextBuffer *tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
    GtkTextMark *mark = gtk_text_buffer_get_insert(tb);
    gtk_text_buffer_get_iter_at_mark(tb, &iter, mark);
    if(gtk_toggle_button_get_active(widget))
       gtk_text_buffer_insert(tb, &iter, "<sub>", -1);
    else
       gtk_text_buffer_insert(tb, &iter, "</sub>", -1);
    gtk_text_buffer_place_cursor(tb, &iter);
    gtk_widget_grab_focus(text);
}

void on_superscript(GtkToggleButton *widget, GtkWidget *text)
{
    GtkTextIter iter;
    GtkTextBuffer *tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
    GtkTextMark *mark = gtk_text_buffer_get_insert(tb);
    gtk_text_buffer_get_iter_at_mark(tb, &iter, mark);
    if(gtk_toggle_button_get_active(widget))
       gtk_text_buffer_insert(tb, &iter, "<sup>", -1);
    else
       gtk_text_buffer_insert(tb, &iter, "</sup>", -1);
    gtk_text_buffer_place_cursor(tb, &iter);
    gtk_widget_grab_focus(text);
}
  
void on_bold(GtkToggleButton *widget, GtkWidget *text)
{
    GtkTextIter iter;
    GtkTextBuffer *tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
    GtkTextMark *mark = gtk_text_buffer_get_insert(tb);
    gtk_text_buffer_get_iter_at_mark(tb, &iter, mark);
    if(gtk_toggle_button_get_active(widget))
       gtk_text_buffer_insert(tb, &iter, "<b>", -1);
    else
       gtk_text_buffer_insert(tb, &iter, "</b>", -1);
    gtk_text_buffer_place_cursor(tb, &iter);
    gtk_widget_grab_focus(text);
}

void on_italic(GtkToggleButton *widget, GtkWidget *text)
{
    GtkTextIter iter;
    GtkTextBuffer *tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
    GtkTextMark *mark = gtk_text_buffer_get_insert(tb);
    gtk_text_buffer_get_iter_at_mark(tb, &iter, mark);
    if(gtk_toggle_button_get_active(widget))
       gtk_text_buffer_insert(tb, &iter, "<i>", -1);
    else
       gtk_text_buffer_insert(tb, &iter, "</i>", -1);
    gtk_text_buffer_place_cursor(tb, &iter);
    gtk_widget_grab_focus(text);
}

void on_underline(GtkToggleButton *widget, GtkWidget *text)
{
    GtkTextIter iter;
    GtkTextBuffer *tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
    GtkTextMark *mark = gtk_text_buffer_get_insert(tb);
    gtk_text_buffer_get_iter_at_mark(tb, &iter, mark);
    if(gtk_toggle_button_get_active(widget))
       gtk_text_buffer_insert(tb, &iter, "<u>", -1);
    else
       gtk_text_buffer_insert(tb, &iter, "</u>", -1);
    gtk_text_buffer_place_cursor(tb, &iter);
    gtk_widget_grab_focus(text);
}

void on_greek(GtkToggleButton *widget, GtkWidget *text)
{
    GtkTextIter iter;
    GtkTextBuffer *tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
    GtkTextMark *mark = gtk_text_buffer_get_insert(tb);
    gtk_text_buffer_get_iter_at_mark(tb, &iter, mark);
    if(gtk_toggle_button_get_active(widget))
       gtk_text_buffer_insert(tb, &iter, "<span lang=\"Greek\">", -1);
    else
       gtk_text_buffer_insert(tb, &iter, "</span>", -1);
    gtk_text_buffer_place_cursor(tb, &iter);
    gtk_widget_grab_focus(text);
}

GtkWidget  *create_legend_dialog(GtkWidget *window)
{
using namespace gtkext;
     GtkWidget *dialog = gtk_dialog_new();
     gtk_window_set_title(GTK_WINDOW(dialog), "Legend Editor");
     gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(window));
     gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog), TRUE);
     gtk_widget_set_size_request(dialog, -1, 250);

     GtkWidget *frame = gtk_frame_new(NULL);
     gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG(dialog))), frame, FALSE, FALSE, 0);
     GtkWidget *hbox = hbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER (frame), hbox);

     frame = gtk_frame_new(NULL);
     gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG(dialog))), frame, TRUE, TRUE, 0);
     gtk_container_set_border_width (GTK_CONTAINER (frame), 6);

     GtkWidget *scrollwin = gtk_scrolled_window_new( NULL, NULL);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);
     gtk_widget_show (scrollwin);
     gtk_container_add(GTK_CONTAINER (frame), scrollwin);

     GtkWidget *text = gtk_text_view_new ();
     g_object_set_data(G_OBJECT(dialog), "testo", text);

     GtkWidget *button = image_button("cancel.png");
     gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
     set_tip( button, "Cancel");
     g_signal_connect((gpointer) button, "clicked", G_CALLBACK(on_click), GINT_TO_POINTER(EXIT));

     button = image_button("apply.png");
     gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 4);
     set_tip( button, "Apply");
     g_signal_connect((gpointer) button, "clicked", G_CALLBACK(on_click), GINT_TO_POINTER(SAVE));

     button = font_button("monospace 8");
     gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 4);
     g_object_set_data(G_OBJECT(dialog), "font", button);
     set_tip( button, "Select font for legend");

     float col[] = {1, 1, 1, 1};
     button = color_button(col, "Pick a colour for legend");
     g_object_set_data(G_OBJECT(dialog), "color", button);
     gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 4);
     set_tip( button, "Select colour for legend");

     button = myimage_togbutton("subscript.png");
     gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 4);
     set_tip( button, "Subscript start/end");
     g_signal_connect(G_OBJECT(button), "toggled", G_CALLBACK(on_subscript), text);

     button = myimage_togbutton("superscript.png");
     gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 4);
     set_tip( button, "Superscript start/end");
     g_signal_connect(G_OBJECT(button), "toggled", G_CALLBACK(on_superscript), text);

     button = myimage_togbutton("greek.png");
     gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 4);
     set_tip( button, "Greek letters start/end");
     g_signal_connect(G_OBJECT(button), "toggled", G_CALLBACK(on_greek), text);

     button = myimage_togbutton("bold.png");
     gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 4);
     set_tip( button, "Bold start/end");
     g_signal_connect(G_OBJECT(button), "toggled", G_CALLBACK(on_bold), text);
     g_object_set_data(G_OBJECT(dialog), "boldb", button);

     button = myimage_togbutton("italic.png");
     gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 4);
     set_tip( button, "Italic start/end");
     g_signal_connect(G_OBJECT(button), "toggled", G_CALLBACK(on_italic), text);

     button = myimage_togbutton("underline.png");
     gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 4);
     set_tip( button, "Underline start/end");
     g_signal_connect(G_OBJECT(button), "toggled", G_CALLBACK(on_underline), text);

     GtkTextBuffer *text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
     gtk_text_view_set_right_margin  (GTK_TEXT_VIEW(text), 4);
     gtk_text_view_set_left_margin  (GTK_TEXT_VIEW(text), 4);
     g_object_set_data(G_OBJECT(dialog), "buffer", text_buffer);
     gtk_text_view_set_editable(GTK_TEXT_VIEW(text), TRUE);
     gtk_container_add(GTK_CONTAINER(scrollwin), text);
     gtk_text_buffer_create_tag (text_buffer, "BOLD", "weight", PANGO_WEIGHT_BOLD, NULL);
     gtk_text_buffer_create_tag (text_buffer, "NORMAL", "weight", PANGO_WEIGHT_NORMAL, NULL);
     return dialog;
}

void Legenda::LegendDialog(GtkWidget *window)
{
using namespace gtkext;
     GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
     GtkWidget *dialog;

     if(get_object(window, "LegendDialog") == NULL)
     {
        dialog = create_legend_dialog(window);
        g_object_set_data(G_OBJECT(window), "LegendDialog", dialog);
     }
     else
     {
        dialog = get_widget(window, "LegendDialog");
        gdk_window_raise(gtk_widget_get_window(dialog));
     }
     GtkWidget *button = get_widget(dialog, "font");
     font_button_set_font_name(button, fontname);
     button = get_widget(dialog, "color");
     color_button_set_color(button, colore);
     GtkTextBuffer *text_buffer = (GtkTextBuffer *)get_object(dialog, "buffer");
     GtkTextIter iter1, iter2;
     gtk_text_buffer_get_bounds (text_buffer, &iter1, &iter2);
     gtk_text_buffer_delete(text_buffer, &iter1, &iter2);
     if(testo != NULL)
     {
        GtkTextIter iter;
        gtk_text_buffer_get_start_iter(text_buffer, &iter);
        char *str = (DuplicaStringa(toUTF8(testo)));
        gtk_text_buffer_insert(text_buffer, &iter,
            g_strchomp(str), (gint) strlen(g_strchomp(str)));
     }
     gtk_widget_show_all(dialog);
     gboolean do_loop = TRUE;
     int response;
     while (do_loop)
     {
          response = gtk_dialog_run(GTK_DIALOG(dialog));
          if(GTK_IS_WIDGET(dialog))
             gtk_widget_hide(dialog);
          switch(response)
          {
          case GTK_RESPONSE_DELETE_EVENT:
          case EXIT:
          case GTK_RESPONSE_NONE:
               do_loop = FALSE;
               break;
          case SAVE:
               GtkTextIter i1, i2;
               gtk_text_buffer_get_bounds(text_buffer, &i1, &i2);
               const char *buffer = gtk_text_buffer_get_text(
                   text_buffer, &i1, &i2, FALSE);
               if(pango_parse_markup(buffer, -1, 0, NULL, NULL, NULL, NULL))
               {
                  if(testo)
                     g_free(testo);
                  testo = DuplicaStringa(buffer);
               }
               color_button_get_color(get_widget(dialog, "color"), colore);
               const gchar *fontn = font_button_get_font_name 
                        (get_widget(dialog, "font"));
               g_free(fontname);
               fontname = DuplicaStringa(fontn);
               gl->Redraw();
               break;
          }
     }
     g_object_set_data(G_OBJECT(window), "LegendDialog", NULL);
     if(GTK_IS_WIDGET(dialog))
        gtk_widget_destroy(dialog);
}

void DrawFlag()
{
    glPushMatrix();
    glLoadIdentity();

    glRasterPos3f (-1.48f, -1.2f, 0.f);
    GdkPixbuf *pix = create_pixbuf("bandiera.png");
    if(pix == NULL)
       return;
    GdkPixbuf *dest = gdk_pixbuf_flip(pix, FALSE);
    if(dest != NULL)
    {
       unsigned char *p1 = gdk_pixbuf_get_pixels(dest);
       if(p1 != NULL)
       {
          int w = gdk_pixbuf_get_width(pix);
          int h = gdk_pixbuf_get_height(pix);
          glDisable (GL_DEPTH_TEST);
          glEnable (GL_BLEND);
          glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
          glDrawPixels (w, h, GL_RGBA, GL_UNSIGNED_BYTE, p1);
       }
       g_object_unref(dest);
    }
    g_object_unref(pix);

    glDisable (GL_BLEND);
    glEnable (GL_DEPTH_TEST);
    glColor3f(1.f, 1.f, 1.f);
    glPopMatrix();
}

void AddLegend(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   gl->nLegends++;
   Legenda leg = Legenda();
   Legend.push_back(leg);
   Legend[gl->nLegends-1].LegendDialog(window);
   if(Legend[gl->nLegends-1].GetText() == NULL)
   {
      gl->nLegends--;
      Legend.erase(Legend.begin()+gl->nLegends);
   }
   if(gl != NULL)
      gl->Redraw();
}

void ModifyLegend(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   if(gl->activeLegend != -1)
      Legend[gl->activeLegend].LegendDialog(window);
}

bool parse_string(const char *str_in, std::string &str_out)
{
   gchar *text;
   PangoAttrList *attrs;
   PangoAttrIterator *iterator;
   PangoAttribute *attr;
   gint start, end;

   gboolean result = pango_parse_markup (str_in, -1, 0, &attrs, &text, NULL, NULL);
   if(!result)
      return false;

  iterator = pango_attr_list_get_iterator(attrs);
  str_out.clear();
  do
  {
      pango_attr_iterator_range(iterator, &start, &end);
      if(end-start > (int)strlen(text))
         end = start+(int)strlen(text);

      std::string temp_str = text;
      if((attr = pango_attr_iterator_get(iterator, PANGO_ATTR_LANGUAGE))!=NULL)
      {
          PangoAttrLanguage *language = (PangoAttrLanguage *)attr;
          const char *strlang = pango_language_to_string(language->value);
          if(strstr(strlang, "greek"))
          {
             for(int i=(int) start; i<(int)end; i++)
                 str_out.append(special_car_greek(text[i]));
          }
      }
      else if ((attr = pango_attr_iterator_get(iterator, PANGO_ATTR_UNDERLINE))!=NULL)
      {
          str_out.append("<u>");
          str_out.append(temp_str.substr(start, end-start));
          str_out.append("</u>");
      }
      else if ((attr = pango_attr_iterator_get(iterator, PANGO_ATTR_WEIGHT))!=NULL)
      {
          str_out.append("<b>");
          str_out.append(temp_str.substr(start, end-start));
          str_out.append("</b>");
      }
      else if ((attr = pango_attr_iterator_get(iterator, PANGO_ATTR_STYLE))!=NULL)
      {
          str_out.append("<i>");
          str_out.append(temp_str.substr(start, end-start));
          str_out.append("</i>");
      }
      else if ((attr = pango_attr_iterator_get(iterator, PANGO_ATTR_RISE))!=NULL)
      {
          PangoAttrInt *bal = (PangoAttrInt *)attr;
          if(bal->value > 0)
             str_out.append("<sup><small>");
          else
             str_out.append("<sub><small>");
          str_out.append(temp_str.substr(start, end-start));
          if(bal->value > 0)
             str_out.append("</small></sup>");
          else
             str_out.append("</small></sub>");
      }
      else
          str_out.append(temp_str.substr(start, end-start));
   }
   while (pango_attr_iterator_next(iterator));
   pango_attr_iterator_destroy (iterator);
   return true;
}

void on_Help(gpointer, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   std::string urlname = main_app::JavApp.PkgDataDir() + G_DIR_SEPARATOR +  main_app::JavApp.PkName() + ".htm";
   URLOpen(urlname, main_app::JavApp.InternetBrowser(), window);

}

void on_SaveRun (gpointer, gpointer data)
{   
using namespace gtkext; 
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   FilselItem local_item;
    
   std::string curdir = getPWD();
   local_item = saveas(window, "*.jbk", NULL);
   dir_set_current(curdir.c_str());
   if(local_item.exitCode > 0)
   { 
       std::string bckfile = local_item.cwd;
       bckfile.append(str_cut(local_item.fileinp, "."));
       bckfile.append(".jbk");
       gl->Save(bckfile, true);
       DeleteAll(gl);
       if(!gl->Restore(bckfile, true))
          DeleteAll(gl);
   }      
}    

void on_SaveAscii (gpointer, gpointer data)
{   
using namespace gtkext; 
   GtkWidget *window = (GtkWidget *) data;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   FilselItem local_item;
    
   std::string curdir = getPWD();
   local_item = saveas(window, "*.ex", NULL);
   dir_set_current(curdir.c_str());
   if(local_item.exitCode > 0)
   { 
       std::string bckfile = local_item.cwd;
       bckfile.append(str_cut(local_item.fileinp, "."));
       bckfile.append(".ex");
       gl->Save(bckfile);
//       gl->Save(bckfile, true);
       DeleteAll(gl);
       if(!gl->Restore(bckfile))
//       if(!gl->Restore(bckfile, true))
          DeleteAll(gl);
   }      
}    

void on_LoadRun (gpointer, gpointer data)
{
using namespace gtkext;
   GtkWidget *window = (GtkWidget *) data;
   std::string fn;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");

   std::string curdir = getPWD();
   fn =  dlg_select_file(window, fn, eSELECT_TO_READ, "*.jbk");
   dir_set_current(curdir.c_str());
   if(str_white(fn.c_str()))
       return;
   bool result = loadconfirm(gl,
          "If you load an old run, the data you are working with will be dismitted. Do you want to proceed ?");
   if(result)
   {
      if(!gl->Restore(fn, true))
         DeleteAll(gl);
   }
}

void on_LoadAscii (gpointer, gpointer data)
{   
using namespace gtkext; 
   GtkWidget *window = (GtkWidget *) data;
   std::string fn;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");

   std::string curdir = getPWD();
   fn =  dlg_select_file(window, fn, eSELECT_TO_READ, "*.ex");
   dir_set_current(curdir.c_str());
   if(str_white(fn.c_str()))
       return;
   bool result = loadconfirm(gl,
          "If you load an example, the data you are working with will be dismitted. Do you want to proceed ?");
   if(result)
   {
      if(!gl->Restore(fn))
         DeleteAll(gl);
   }
}

/*
#ifdef GTK_TOGGLE_TOOL_BUTTON
void on_selectMode(GtkWidget *item, gpointer )
{
   if(gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(item)))
      set_motion_type(ON_SELECT, 1);
}

void on_manipulationMode(GtkWidget *item, gpointer )
{
   if(gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(item)))
      set_motion_type(NO_MOVE, 0);
}

void on_zoomMode(GtkWidget *item, gpointer )
{
   if(gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(item)))
      set_motion_type(ZOOM_MODE, 4);
}
#endif
*/
}
