#include <cstdio>
#include <sstream>
#include <string>
#include <curl/curl.h>
#include <glib.h>
#include <utils.h>
#include <string.h>

#define CURL_MINORV   (LIBCURL_VERSION_NUM /256) % 256

struct FtpFile {
  const char *filename;
  FILE *stream;
};

static size_t my_fwrite(void *buffer, size_t size, size_t nmemb, void *stream)
{
  struct FtpFile *out=(struct FtpFile *)stream;
  if(out && !out->stream) {
    out->stream=fOpen(out->filename, "wb");
    if(!out->stream)
      return 0;
  }
  return fwrite(buffer, size, nmemb, out->stream);
}

bool GetRemoteFile(std::string &url, std::string &localfile, bool ifSilent)
{
  CURL *curl;
  CURLcode res;
  double c_length;
  struct FtpFile ftpfile={
    localfile.c_str(), 
    NULL
  };
  bool result = true;
  curl_global_init(CURL_GLOBAL_DEFAULT);

  curl = curl_easy_init();
  if(curl) {
    curl_easy_setopt(curl, CURLOPT_URL, url.c_str());
    curl_easy_setopt(curl, CURLOPT_NOBODY, 1L);
    curl_easy_setopt(curl, CURLOPT_VERBOSE, 0L);

    res = curl_easy_perform(curl);

    if(CURLE_OK != res) {
       std::stringstream msg;
       msg  << "Error from CURL: "
#if CURL_MINORV > 12
            << curl_easy_strerror(res) 
#else
               << "Number " << res
#endif
            << std::endl << "Check your internet connection" << std::endl;
       if(!ifSilent)
          MessageDialog(NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, msg.str().c_str());
      result = false;
    }
    else
    {
        res = curl_easy_getinfo(curl, CURLINFO_CONTENT_LENGTH_DOWNLOAD, &c_length);
        if(CURLE_OK != res) {
           std::stringstream msg;
           msg << "Error from CURL: "
#if CURL_MINORV > 12
               << curl_easy_strerror(res) 
#else
               << "Number " << res
#endif
               << std::endl ;
          if(!ifSilent)
             MessageDialog(NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, msg.str().c_str());

          result = false;
        }
        if(c_length > 0)
        {
#if CURL_MINORV > 12
            curl_easy_reset(curl);
#else
            curl_easy_setopt(curl, CURLOPT_NOBODY, 0L);
            curl_easy_setopt(curl, CURLOPT_VERBOSE, 0L);
#endif
            curl_easy_setopt(curl, CURLOPT_URL, url.c_str());
            curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, my_fwrite);
            curl_easy_setopt(curl, CURLOPT_WRITEDATA, &ftpfile);
            res = curl_easy_perform(curl);
        }
        else
            result = false;
     }
    curl_easy_cleanup(curl);
  }

  if(ftpfile.stream)
    fclose(ftpfile.stream); 

  curl_global_cleanup();
  return result;
}

int CurrentVersionInfo(const char *filinfo, int *versionNumber=NULL)
{
   char rigar[256];
   int result = 0;
   FILE *fp = fOpen(filinfo, "r");
   if(fp != NULL)
   {
      if((fgets(rigar,256,fp) != NULL) && (strlen(rigar) > 1))
      {
         int nums[2];
         if((fgets(rigar,256,fp) != NULL) && (strlen(rigar) > 1))
         {
            get_int_tokens(rigar, " ", 2, nums);
            result = nums[0]*400 + nums[1];
         }
         if(versionNumber != NULL)
         {
            if((fgets(rigar,256,fp) != NULL) && (strlen(rigar) > 1))
            {
               get_int_tokens(rigar, ".", 2, nums);
               *versionNumber = nums[0]*100 + nums[1];
            }
         }
      }
      fclose(fp);
   }
   return result;
}

int ServerVersionInfo(bool ifSilent, int *ver)
{
   std::string local_file  = g_build_filename(getHomeDir(), "tmp.tmp", NULL);
   std::string remote_file;
#ifdef SIR_PGM
   remote_file.assign("http://wwwba.ic.cnr.it/sites/default/files/CheckUpdatesSir.txt");
#else
   remote_file.assign("http://wwwba.ic.cnr.it/sites/default/files/CheckUpdatesExpo.txt");
#endif
// absolute path on srvic /var/www/html/drupal/sites/default/files/CheckUpdates.txt
   bool result = GetRemoteFile(remote_file, local_file, ifSilent);
   int retval = 0;
   if(result)
      retval = CurrentVersionInfo(local_file.c_str(), ver);
   Cancella(local_file.c_str());
   return retval;
}

int IsUpdateAvailable(const char *filinfo, bool ifSilent, int *versionNumber)
{
   int myversion = CurrentVersionInfo(filinfo);
   int srvversion = ServerVersionInfo(ifSilent, versionNumber);
   if(srvversion == 0)
      return -1;
   if (srvversion > myversion)
      return 1;
   else
      return 0;
}
