MODULE bb_bc


implicit none

type bb_bc_condition
   integer :: nump     ! size of population
   integer :: niter    ! number of iterations
   real    :: tol      ! Stopping criterion
   integer :: nlocmax  ! numero di local search
   real    :: beta     ! parametro di controllo
   logical :: auto     ! automatic setting of niter
end type bb_bc_condition

type pop_type
   real                            :: cfval
   real, dimension(:), allocatable :: x
end type pop_type

CONTAINS

   subroutine bb_bc_initialize(cond)
   type(bb_bc_condition) :: cond
!
   cond%nump = 100
  !cond%nump = 5
   cond%niter = 20
   cond%tol = 0.1  
   cond%nlocmax = 3
   cond%beta = 0.3
   cond%auto = .true.

   !cond%nump = 160
   !cond%niter = 200
!
   end subroutine bb_bc_initialize

!-------------------------------------------------------------------------------

   subroutine bb_bc_algo(par,cond,ginfo_ini,bestcfvalue,func,funcs,sc_ini,neval,kevent)
   USE rand_mod
   USE simulated_anneal
   USE nr
   USE stat
   USE SA_INTERFACE
   USE realloc
   USE progtype, only: param_type

!  USE sa_type

   INTERFACE
!
!     Function per il calcolo della funzione costo
      real function func(xpar,hpar)
      real, intent(in), dimension(:) :: xpar
      integer, intent(in)            :: hpar
      end function func
!
      subroutine funcs(xp,fval,grad,icod)
      real, dimension(:), intent(in)            :: xp
      real, intent(out)                         :: fval
      real, dimension(:), intent(out), optional :: grad
      integer, intent(in)                       :: icod
      end subroutine funcs

   END INTERFACE
   type(param_type), dimension(:), intent(inout) :: par         ! parametri
   type(bb_bc_condition), intent(in)             :: cond
   type(opti_info_type), intent(in)              :: ginfo_ini
   real, intent(out)                             :: bestcfvalue ! valore finale della funzione costo
   type(SimAnn_Conditions_type), intent(in)      :: sc_ini
   integer, intent(out)                          :: kevent
   integer                                       :: neval 
   type(opti_info_type)                          :: ginfo
   type(SimAnn_Conditions_type)                  :: sc
   integer                                       :: npar
   type(pop_type), dimension(:), allocatable     :: pop

!  type(replica_type), dimension(10)              :: fstar

   integer                                       :: i,j,k   !!!,jj
   real, dimension(size(par))                    :: xc,xcm1,xcm2
   integer                                       :: kmethod
   real                                          :: xrand
   real                                          :: val,rangev
   real                                          :: alpha
   integer                                       :: it
   real                                          :: cfxc
   real                                          :: alpha_min,alpha_max
   real                                          :: coef
  ! real                                          :: sacfvalue
  !real                                          :: popcfvalue
   integer                                       :: nloc,nlocc
   type(param_type), dimension(size(par)) :: sapar
    real, dimension(size(par)) :: xopt
    real :: fopt,tfin
    integer :: nacc,nobdss
   logical :: bestfound
   integer, dimension(:), allocatable :: iord
   integer :: jord    !!!!,jord1
   real :: cfave,cfvar,stopc
   integer                         :: nacf
   integer                         :: ngen
   real                            :: fmin    !!,num,den
   integer                         :: kpr = 0
!
   npar = size(par)   ! numero di parametri
   kmethod = 2
   !kmethod = 1
!
!  Starting population is generating
   ngen = 1*cond%nump    ! numero di elementi da generare
   allocate(pop(ngen),iord(ngen))
   do i=1,ngen
      allocate(pop(i)%x(npar))
   enddo
   pop(1)%x = par(:)%val
   pop(1)%cfval = func(pop(1)%x,0)
!CORO
! Applica un fast SA sul modello importato
        !write(6,*)'FCVAL_P',pop(1)%cfval
         fmin = func(pop(1)%x,0)
         neval = 1
         ginfo = ginfo_ini
         sc = sc_ini
         sapar(:) = par(:)
         call sa(sapar,sc,ginfo,tfin,xopt,fopt,nacc,neval,nobdss,kevent,func,funcs,fmin)

         if((fopt/pop(1)%cfval)<0.1) then
            pop(1)%cfval = fopt
            pop(1)%x = xopt
            par(:)%val=xopt
         endif
         if (kevent == 1 .or. kevent == 2) then ! gestione eventi grafica
             bestcfvalue = fopt
             return    
         endif

        !write(6,*)'FCVAL_D',pop(1)%cfval
!CORO
   do i=2,ngen
      do j=1,npar
         pop(i)%x(j) = randvalue(par(j)%low,par(j)%high)
      enddo
      pop(i)%cfval = func(pop(i)%x,0)
   enddo
   call indexx(pop%cfval,iord)
   pop(:) = pop(iord)
   call reallocate_pop(pop,cond%nump)
   call reallocate(iord,cond%nump)

!
!  Calcola il centro di massa della popolazione iniziale
   xcm1 = centerpoint(pop,kmethod,npar)
   bestcfvalue = func(xcm1,0)
   par(:)%val = xcm1
   xcm2(:) = xcm1(:)
!
!  New population is generated
!corr   neval = 1
   ginfo = ginfo_ini
   call fcnprint(ginfo,ginfo%popt,xcm1,bestcfvalue,0.0,neval,0)
   alpha_min = 0.0001
  !alpha_max = 5.0
   alpha_max = 1.0
   sc = sc_ini
   coef = log(alpha_min/alpha_max) / cond%niter
      nacf = 0
       stopc = 1.0
       alpha = 1
!CE
   do it=1,cond%niter      ! loop sul numero di iterazioni
      alpha = alpha_max * exp(coef*it)
!
! Big bang phase
             xc(:) = xcm1(:)
      do j=1,cond%nump    ! loop sulla popolazione
         do k=1,npar      ! loop sui parametri della popolazione

           !xrand=(random_normal()+4)/4       ! genero numeri random con distribuzione gaussiana
           !val = xc(k) + (par(k)%high*(xrand-1))/it

            call random_number(xrand)               ! da 0 a 1
           !val = xc(k) + (xrand*2 -1) * alpha      ! +-1
            
!CROS
           if(ginfo%dofi>10) then
            if(it<=(0.5*cond%niter)) then
               if(j<=nint(0.7*cond%nump)) then
                   val = xc(k) + (xrand*2 -1) * alpha      ! +-1
               else
                   val = xc(k) + (xrand*2 -1)
               endif
            else
               val = xc(k) + (xrand*2 -1) * alpha      ! +-1
            endif
           else
               val = xc(k) + (xrand*2 -1) * alpha      ! +-1
           endif
!CROS
           rangev = par(k)%high - par(k)%low
           if (par(k)%bound == 0) then     ! Fixed boundaries
                if(val < par(k)%low .or. val > par(k)%high) then
                    call random_number(xrand)
                    val = par(k)%low + rangev*xrand
                endif
           else                            ! Periodic boundaries
                if(val < par(k)%low) then
                   val = val + rangev
                endif
                if(val > par(k)%high) then
                   val = val - rangev
                endif
            endif
            pop(j)%x(k) = val
           !if(val < par(k)%low.or.val > par(k)%high) write(6,*)'FUORI DAL RANGE=',k,j,val,par(k)%low,par(k)%high 
         enddo       ! do su k, num di parametri

!
! Calcola funzione costo per ogni elemento della popolazione
         pop(j)%cfval = func(pop(j)%x,0)

      enddo    ! fine do su j, popolazione
!
!     Big crunch phase

      call indexx(pop(:)%cfval,iord)

      iord(:) = iord(cond%nump:1:-1)    ! dal piu grande al piu piccolo
!
      nloc = 0
      nlocc = 0
      fmin = bestcfvalue
      call avevar(pop(:)%cfval,cfave,cfvar)
      do j=1,cond%nump
         jord = iord(j)      ! solo su al max i 3 modelli peggiori

         nloc = nloc + 1
         if (nloc > cond%nlocmax) exit
                         if (jord < 0) cycle
!
! Applica un fast SA
         sapar(:) = par(:)
         sapar(:)%val = pop(jord)%x
          !!write(6,*)'SA_IN',nlocc,jord,fopt
         call sa(sapar,sc,ginfo,tfin,xopt,fopt,nacc,neval,nobdss,kevent,func,funcs,fmin)
         nlocc = nlocc + 1
          !!write(6,*)'SA_OUT',nlocc,jord,fopt
         if (fopt < fmin) then
             fmin = fopt
         endif
         if (kpr > 0) then
             write(6,'(a,i0,a,i0,a,i0,2x,f0.3,a,f0.3)')'n.',it,'/',cond%niter, &
                ' local search on pop. n.',jord,pop(jord)%cfval,'-->',fopt
         endif
         pop(jord)%cfval = fopt
         pop(jord)%x = xopt
      !! write(6,*)'VALORI SA',nlocc,jord,xopt,fopt
         if (kevent == 1 .or. kevent == 2) exit    ! gestione eventi grafica
         if(fmin<=nint(0.15*cfave)) then      ! uscite *.out_1sa_alpha5
          !!write(6,*)'ESCO DA SA=',nlocc,fmin,cfave,nint(0.15*cfave)
            exit
         endif
      enddo       ! fine if su SA
!
!CROS
     !if(nlocc>1) then
!calcola il cm come la media tra i modelli che vengono affinati da SA
     !   den=0.0
     !   do j=1,nlocc
     !      jord = iord(j)      ! i 3 modelli peggiori
     !      den = den+(1/pop(jord)%cfval)**2
     !   end do

     !   do i=1,npar
     !    num = 0
     !    do j=1,nlocc
     !       jord = iord(j)      ! i 3 modelli peggiori
     !       num = num + pop(jord)%x(i) / (pop(jord)%cfval**2.0)
     !    enddo
     !       xcm1(i) = num / den
     !   enddo
     !   sacfvalue = func(xcm1,0)
     ! ! popcfvalue = func(pop(1:ngen)%x,0)
     !   write(6,*)'CFOM CM',nlocc,sacfvalue,cfave
     !!! write(6,*)'VALORI CM',nlocc,xcm1(:)
!!!!!!!!!if(sacfvalue>(1.05*cfave)) then   ! uscite *.out_bb_gen13
     !   if(sacfvalue>(0.5*cfave)) then    ! uscite *.mod_cm
     !      xcm1(:) = centerpoint(pop,kmethod,npar)
     !      write(6,*)'CM CALCOLATO COME MINIMO',nlocc,xcm1(:)
     !     !xcmcfvalue = func(xcm1,0)
     !   else
     !      write(6,*)'CM CALCOLATO COME MEDIA',nlocc,xcm1(:)
     !   endif
     !else
         xcm1(:) = centerpoint(pop,kmethod,npar)  ! prende il mod con cfval minimo
     !endif
!CROS
!
      cfxc = func(xcm1,0)
      neval = neval + 1
      bestfound = cfxc < bestcfvalue
      if (bestfound) then
          bestcfvalue = cfxc
          par(:)%val = xcm1(:)
          xcm2(:) = xcm1(:)
          call fcnprint(ginfo,ginfo_ini%popt,xcm1,bestcfvalue,0.0,neval,it)
          if (kpr > 0) then
              write(6,*)it,'cfval=',cfxc,' alpha=',alpha
          endif
      endif
! va normalizzata la popolazione per fare questo
! Check stopping criterion
      call avevar(pop(:)%cfval,cfave,cfvar)
      stopc = sqrt(cfvar)/cfave
      if (kpr > 0) then
          write(6,*)'Stopping criterion. Tol:',stopc
      endif
      if (stopc < cond%tol .or. kevent == 1 .or. kevent == 2) then
          if (kpr > 0) then
              write(6,*)'Stopping criterion verified. Tol:',stopc,' neval=',neval
          endif
          exit
      endif

   enddo
   bestcfvalue = func(par(:)%val,0)
!
   end subroutine bb_bc_algo

!-------------------------------------------------------------------------------

   real function pop_distance(pop1,pop2,par)  result(pdist)
   USE progtype, only: param_type
   type(pop_type), intent(in)   :: pop1,pop2
   type(param_type), dimension(:),intent(in) :: par
   real, dimension(size(pop1%x)) :: pdiff
   integer                       :: nd
   real                          :: dist
   real                          :: cfdiff
   integer                       :: i
   integer                       :: npar
!
   pdist = 1
   !pdiff(:) = (pop1%x - pop2%x) / max(pop1%x,1.0)
   !pdist = sqrt(dot_product(pdiff,pdiff))
   npar = size(pop1%x)
   nd = 0
   do i=1,npar
      if (par(i)%typepar == 5) then
          nd = nd + 1
          pdiff(nd) = (pop1%x(i) - pop2%x(i)) / max(pop1%x(i),1.0)
      endif
   enddo
   if (nd > 0) then
       dist = sqrt(dot_product(pdiff(:nd),pdiff(:nd)))
   else
       dist = 0.0
   endif
   cfdiff = abs(pop1%cfval - pop2%cfval)
       !write(0,*)'diff=',dist,cfdiff
   if (dist < 0.1 .and. cfdiff < 0.5) then
       pdist = -1
   endif
!
   end function pop_distance

!-------------------------------------------------------------------------------

   function centerpoint(pop,kmethod,np)  result(xc)
!
!  Trova il centro di massa della popolazione
!
   type(pop_type), dimension(:), intent(in) :: pop
   integer, intent(in)                      :: kmethod   
   integer, intent(in) :: np
!corr   real, dimension(size(pop(1)%x))          :: xc
   real, dimension(np) :: xc
   integer                                  :: npop
   integer, dimension(1)                    :: loc
   real                                     :: num,den
   integer                                  :: i,j
!
   if (kmethod == 1) then
       !np = size(pop(1)%x)
       npop = size(pop)
       den = sum(1/pop(:)%cfval)
       do i=1,np
          num = 0
          do j=1,npop
             num = num + pop(j)%x(i) / pop(j)%cfval
          enddo
          xc(i) = num / den
       enddo
   else
       loc = minloc(pop%cfval)
      !write(6,*)'MINIMO',loc
       xc(:) = pop(loc(1))%x
   endif 
!
   end function centerpoint

!--------------------------------------------------------------------------------------------------

   subroutine bb_bc_print_conditions(bcond,unt)
   type(bb_bc_condition), intent(in) :: bcond
   integer, intent(in)               :: unt
!
   write(unt,"(/8x,a)")'Global optimization by Hybrid Big Bang Big Crunch (HBB-BC)'
   write(unt,'(1x,a,i0)')  'Polulation size:              ',bcond%nump
   write(unt,'(1x,a,i0)')  'Number of iterations:         ',bcond%niter
   write(unt,'(1x,a,f0.4)')'Stopping criterion:           ',bcond%tol
   write(unt,'(1x,a,i0)')  'Number of local search:       ',bcond%nlocmax
!corr   write(unt,'(1x,a,i0)')'Control parameter alfa: ',bcond%beta
!
   end subroutine bb_bc_print_conditions

!--------------------------------------------------------------------------------------------------

   subroutine reallocate_pop(vetr,n,savevet)
!
!  Rialloca ad n un vettore di tipo model
!  Se savevet = .true. o non esiste si salva il suo contenuto.
!
   type(pop_type), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)                     :: n
   logical, optional, intent(in)           :: savevet
   logical                                 :: savev
   integer                                 :: nv
   type(pop_type), allocatable                :: vsav(:)
   integer                                 :: nsav
!
!  se n = 0 (riallocazione a 0): dealloca ed esci
   if (n == 0) then
       if (allocated(vetr)) deallocate(vetr)
       return
   endif
!
   if (.not.allocated(vetr)) then
       allocate(vetr(n))
   else
!
       nv = size(vetr)
       if (present(savevet)) then
           savev = savevet
       else
           savev = .true.
       endif
!
       if (savev) then
!
!          nsav contiene qual è la porzione di vetr da salvare
           select case(nv-n)
             case (1:)       ! compatta x ad n
               nsav = n
             case (:-1)      ! espandi x ad n
               nsav = nv
             case (0)
               return        ! n=nv non fare niente
           end select
!
!          salva vetr fino a nsav
           allocate(vsav(nsav))
           vsav = vetr(:nsav)
!
!          dealloca vetr
           deallocate(vetr)
!
!          rialloca vetr su n
           allocate(vetr(n))
!
!          ripristina vetr fino a nsav
           vetr(:nsav) = vsav(:nsav)
       else
           if (nv /= n) then
               deallocate(vetr)
               allocate(vetr(n))
           endif
       endif
   endif
!
   end subroutine reallocate_pop
END MODULE bb_bc
