MODULE commandsmod

   implicit none 

!  common /charas/ beg,commen,allcom
   character(len=1)                  :: beg = '%'
   character(len=1)                  :: commen = '>'
   character(len=14), dimension(80)  :: allcom = (/        &
       'normal        ','              ','invariants    ', & ! 3
       'phase         ','fourier       ','menu          ', & ! 6
       'end           ','continue      ','job           ', & ! 9
       'graphics      ','cancel        ','initialize    ', & !12
       'path          ','export        ','              ', & !15
       'remo          ','polpo         ','fcaos         ', & !18
       'use           ','return        ','automatic     ', & !21
       'structure     ','data          ','nowindow      ', & !24
       'window        ','hidden        ','lsq           ', & !27
       'changelab     ','hydrogen      ','geometry      ', & !30
       'agreement     ','weight        ','group         ', & !33
       'resid         ','plot          ','debug         ', & !36
       'nodraw        ','ilmilione     ','develop       ', & !39
       'memory        ','coordinates   ','saved         ', & !42
       'build         ','difference    ','recover       ', & !45
       'ligand        ','dedm          ','gedm          ', & !48
       'edm           ','import        ','analyze       ', & !51
       'biascorrect   ','helix         ','sannel        ', & !54
       'amb           ','set           ','phantom       ', & !57
       '              ','              ','              ', & !60
       '              ','              ','              ', & !63
       '              ','              ','              ', & !66
       '              ','              ','              ', & !69
       '              ','              ','              ', & !72
       '              ','              ','              ', & !75
       '              ','              ','              ', & !78
!!gpu  '              ','              '/)                   !80
       '              ','cuda          '/)                   !80
!
!   kit       = 0 Powders Kit
!-- kitvet(i) = 0 if the corresponding command in allcom is a general one
!-- kitvet(i) = 4 if the corresponding command in allcom is an eXtra one
!-- kitvet(i) = 2 if the corresponding command in allcom is a SirPow one
!-- kitvet(i) = 5 if the corresponding command in allcom is a Rietveld one
!
   common /pkitc/ kit,kitvet
   integer                :: kit = 0
   integer, dimension(36) :: kitvet = (/  &
                0 , 0 , 0 ,               &  !  3
                0 , 4 , 2 ,               &  !  6
                2 , 2 , 2 ,               &  !  9
                2 , 2 , 0 ,               &  ! 12
                0 , 0 , 0 ,               &  ! 15
                0 , 4 , 0 ,               &  ! 18
                0 , 0 , 0 ,               &  ! 21
                2 , 4 , 2 ,               &  ! 24
                0 , 0 , 0 ,               &  ! 27
                0 , 0 , 0 ,               &  ! 30
                0 , 0 , 0 ,               &  ! 33
                0 , 0 , 0/)                  ! 36

   integer, parameter :: NLENDIR = 780
   type command_type
        integer                                           :: code   !  codice comando
        character(len=NLENDIR), dimension(:), allocatable :: strdir !  stringa con direttiva
        integer                                           :: ndir   !  num. di direttive
   end type command_type

   type(command_type), dimension(:), allocatable :: commands

CONTAINS
   
   subroutine create_commands(commd,ipcom,icomqq,cardq)
   type(command_type), dimension(:), allocatable, intent(inout) :: commd
   integer, intent(in)                                          :: ipcom
   integer, dimension(:,:), intent(in)                          :: icomqq
   character(len=*), dimension(:)                               :: cardq
   integer                                                      :: i,j
   integer                                                      :: icd
!
   if (allocated(commd)) deallocate(commd)
   allocate(commd(ipcom))
   icd = 0
   do i=1,ipcom
      commd(i)%code = icomqq(i,1)
      commd(i)%ndir =icomqq(i,2)
      if (commd(i)%ndir /= 0) then
          allocate(commd(i)%strdir(commd(i)%ndir))
          do j=1,commd(i)%ndir
             icd = icd + 1
             commd(i)%strdir(j) = cardq(icd)
          enddo
      endif
   enddo
!
   end subroutine create_commands

!-----------------------------------------------------------------------------------------

   integer function ncommands(commd)
!
!  Numero di comandi
!
   type(command_type), dimension(:), allocatable, intent(in) :: commd
!
   if (allocated(commd)) then
       ncommands = size(commd)
   else
       ncommands = 0
   endif
!
   end function ncommands

!-----------------------------------------------------------------------------------------

   integer function command_position(commd,stringcomm)  result(nposc)
!
!  Cerca un comando nella lista e ne restituisce la posizione
!  il risultato è 0 se non esiste
!
   USE strutil
   type(command_type), dimension(:), allocatable, intent(in) :: commd
   character(len=*), intent(in)                              :: stringcomm
!
   nposc = position_stringv(stringcomm,allcom(commd(:)%code))
!
   end function command_position

!-----------------------------------------------------------------------------------------

   logical function is_directive(commd,dirstring)
!
!  controlla l'esistenza di una direttiva nel comando commd
!
   USE strutil
   type(command_type), intent(in)                    :: commd
   character(len=*), intent(in)                      :: dirstring
   character(len=NLENDIR), dimension(:), allocatable :: commdir
   character(len=NLENDIR)                            :: strd
   integer                                           :: i
   integer                                           :: nlend
!
   if (commd%ndir > 0) then
!
!      Isola la prima stringa per ogni direttiva
       allocate(commdir(commd%ndir))
       do i=1,commd%ndir
          strd = commd%strdir(i)
          call cutst(strd,line2=commdir(i))
          nlend = min(4,len_trim(commdir(i))) 
          commdir(i) = commdir(i)(1:nlend)  ! prendi non + di 4 caratteri
       enddo
!
       nlend = min(4,len_trim(dirstring))
       is_directive = position_stringv(dirstring(1:nlend),commdir) > 0
   else
       is_directive = .false.
   endif
!
   end function is_directive

!-----------------------------------------------------------------------------------------

   logical function is_directive_cmd(commd,dirstring)  result(isdir)
!
!  Controlla se esiste una direttiva
!
   type(command_type), dimension(:), intent(in) :: commd
   character(len=*), intent(in)                 :: dirstring
   integer                                      :: i
!
   isdir = .false.
   do i=1,size(commd)  
      isdir = is_directive(commd(i),dirstring)
      if (isdir) exit
   enddo
!
   end function is_directive_cmd

!-----------------------------------------------------------------------------------------

    logical function is_directive_in_command(commd,strcomm,strdir)   result(isdir)
!
!   Controlla se esiste la direttiva strdir nel comando strcomm
!
    type(command_type), dimension(:), allocatable, intent(in) :: commd
    character(len=*), intent(in)                              :: strcomm,strdir
    integer                                                   :: nposc
!
!   Locazione del comando strcomm
    nposc = command_position(commd,strcomm)
!
    if (nposc > 0) then
        isdir = is_directive_cmd(commd(nposc:nposc),strdir)
    else
        isdir = .false.
    endif
!
    end function is_directive_in_command

!-----------------------------------------------------------------------------------------

   subroutine print_commands(commd)
!
!  Stampa i comandi
!
   type(command_type), dimension(:), allocatable, intent(in) :: commd
   integer                                                   :: i,j
!
   do i=1,ncommands(commd)
      write(0,*)'%'//trim(allcom(commd(i)%code))
      do j=1,commd(i)%ndir
         write(0,*)trim(commd(i)%strdir(j))
      enddo
   enddo
!
   end subroutine print_commands

END MODULE commandsmod
