#include <utils.h>
#include <gtkdefs.h>
#include <sir_application.h>
#include <settings.h>

extern _Dialogo *dialogo;
namespace main_app {
extern Application TheApp;
                   }

Settings *mySettings;
static GtkWidget *settings_dlg = NULL;
#ifdef COLORSET
static int nrows = 4, ncols = 4;
#endif

gchar *font_selection_get_font_name(GtkWidget *fsel)
{
#ifdef GTK_FONT_CHOOSER
   PangoFontDescription *fdesc = gtk_font_chooser_get_font_desc(GTK_FONT_CHOOSER(fsel));
   return pango_font_description_to_string(fdesc);
#else
   return gtk_font_selection_get_font_name(GTK_FONT_SELECTION(fsel));
#endif
}

void font_selection_set_font_name(GtkWidget *fsel, const char *fontnam)
{
#ifdef GTK_FONT_CHOOSER
   PangoFontDescription *fdesc = pango_font_description_from_string(fontnam);
   gtk_font_chooser_set_font_desc(GTK_FONT_CHOOSER(fsel), fdesc);
#else
   gtk_font_selection_set_font_name(GTK_FONT_SELECTION(fsel), fontnam);
#endif
}

void font_selection_set_preview_text(GtkWidget *fsel, const char *riga)
{
#ifdef GTK_FONT_CHOOSER
   gtk_font_chooser_set_preview_text(GTK_FONT_CHOOSER(fsel), riga);
#else
   gtk_font_selection_set_preview_text(GTK_FONT_SELECTION(fsel), riga);
#endif
}

int OnAccept(int npag, GtkWidget *fsel, GtkWidget *dlg, _SettingsInfo *info)
{
//   GtkWidget *dlg = gtk_widget_get_toplevel(csel);
   if(npag == 0)                                      //Fonts
   {
      gchar *fontnam = font_selection_get_font_name(fsel);
      if(!str_white(fontnam))
      {
          mySettings->fontname = DuplicaStringa(fontnam);
          GtkToggleButton *sel_p = (GtkToggleButton *) get_widget(GTK_WIDGET(fsel),"fspacingp");
          GtkToggleButton *sel_f = (GtkToggleButton *) get_widget(GTK_WIDGET(fsel),"fspacingf");
          if(gtk_toggle_button_get_active(sel_p))
          {
             mySettings->whichFont = 0;
             gtk_label_set_text(GTK_LABEL(get_widget(dlg, "fnamep")), fontnam);
          }
          else if(gtk_toggle_button_get_active(sel_f))
          {
             mySettings->whichFont = 1;
             if(gtkext::is_monospace(fontnam))
                gtk_label_set_text(GTK_LABEL(get_widget(dlg, "fnamen")), fontnam);
             else
             {
                (void) MessageDialog(dlg, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
                  "You must specify a fixed width font!");
                return -1;
             }
          }
          else
             mySettings->whichFont = 2;
          info->cbFonts(mySettings);
       }
       else
       {
          (void) MessageDialog(dlg, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
            "You must specify a font name!");
          return -1;
       }
   }
#ifdef COLORSET
   else                                               //Colors
       info->cbColors(mySettings);
#endif
   return 0;
}

#ifdef COLORSET
void Paint(GtkWidget *dlg)
{
    if(!gtk_widget_get_visible(dlg))
       return;
    GtkWidget *drawingarea1 = get_widget(dlg, "drawarea");
    if(!gtk_widget_get_window(drawingarea1))
       return;
    _SettingsInfo *info = (_SettingsInfo *) g_object_get_data (G_OBJECT(dlg), "info");
    gpointer context = get_context(gtk_widget_get_window(drawingarea1));
    g_object_set_data(G_OBJECT(drawingarea1), "gc", context);
    set_foreground( context, &(mySettings->CurrentColor));
    GtkAllocation alloc;
    gtk_widget_get_allocation(drawingarea1, &alloc);
    if (mySettings->tWin == 0)
         draw_rectangle (gtk_widget_get_window(drawingarea1), context,
                         TRUE, 2, 2,
                         (int)alloc.width - 4,
                         (int)alloc.height - 4);
    else
    {
         GdkColor bgcol[2] = { 
                              {0, 0, 0, 0},
                              {0, 65535, 65535, 65535}};
         set_foreground( context, &(bgcol[mySettings->jbg]));
         draw_rectangle (gtk_widget_get_window(drawingarea1), context,
                         TRUE, 2, 2,
                         (int)alloc.width - 4,
                         (int)alloc.height - 4);
         info->cbDraw(drawingarea1, context, mySettings);
    }
}

gboolean OnExpose (GtkWidget *widget, gpointer ob,  GtkWidget *dlg)
{
    Paint(dlg);
    return FALSE;
}
                                                                                                                      
/*
void OnButton (GtkToggleButton *button, gpointer data)
{
    GdkColor col;
    int ipos;
    GtkWidget *frame;
    GtkColorSelection *gtkColsel = GTK_COLOR_SELECTION(get_widget(
       gtk_widget_get_toplevel(GTK_WIDGET(button)), "gtkcsel"));
    GtkWidget *dlg = gtk_widget_get_toplevel(GTK_WIDGET(button));
                                                                                                                       
    frame = (GtkWidget *) data;
    if(gtk_toggle_button_get_active(button))
    {
       gtk_frame_set_shadow_type(GTK_FRAME(frame),GTK_SHADOW_IN);
       ipos = get_int(GTK_WIDGET(button),"id");
       col = mySettings->colori[ipos];
       if(mySettings->old_selected && (mySettings->old_selected != button))
          gtk_toggle_button_set_active(mySettings->old_selected, FALSE);
       mySettings->old_selected = button;
       gtk_color_selection_set_current_color(gtkColsel, &col);
                                                                                                                       
       if(mySettings->ind >= 0)
       {
          mySettings->indcolor[mySettings->ind] = ipos;
          Paint(dlg);
       }
    }
    else
       gtk_frame_set_shadow_type(GTK_FRAME(frame),GTK_SHADOW_NONE);
}
*/

void UpdateColorSelection (GtkWidget *csel)
{
   GtkWidget *gtklist = get_widget(csel, "objtab");
   int nobj = get_int(gtklist, "nobj");
   for(int i=0; i< nobj; i++)
   {
       char *nome = g_strdup_printf("%d", i);
       GtkWidget *cb = get_widget(gtklist, nome);
       int real_ind = get_int(cb, "ind"), ind;
       if(real_ind<N_OBJS)
          ind = i + 2;
       else
          ind = mySettings->nat[real_ind-N_OBJS]+N_OBJS-1;
       float *fcol = main_app::TheApp.getfCol(ind);
       GdkColor *colore = ToGdkColor(fcol);
       mySettings->colori[real_ind] = *colore;
       mycolor_button_set_color(cb, fcol);
   }
}

void ResetPalette (GtkButton *button, gpointer data)
{
      GtkWidget *dlg = gtk_widget_get_toplevel(GTK_WIDGET(button));
      int i;
      for (i=0; i<16; i++)
           mySettings->BaseColors[i] = main_app::bcolors[i];
      main_app::TheApp.ResettaPalette();
      UpdateColorSelection(get_widget(dlg, "colsel"));
     
      Paint(dlg);
}

GtkWidget* create_colorselection (GtkWidget *framef, GtkWidget *dlg)
{
  GtkWidget *dialog1 = vbox_new (FALSE, 4);
  gtk_widget_show (dialog1);
  gtk_container_add (GTK_CONTAINER (framef), dialog1);
  gtk_container_set_border_width (GTK_CONTAINER (dialog1), 4);
  g_object_set_data(G_OBJECT (dlg), "colsel", dialog1);

  GtkWidget *hbox1 = hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (dialog1), hbox1, TRUE, TRUE, 0);

  GtkWidget *vbox1 = vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
#ifdef IN_GTK3
  gtk_box_pack_start (GTK_BOX (hbox1), vbox1, TRUE, TRUE, 0);
#else
  GtkWidget *align = gtk_alignment_new(0, 0, 0, 0);
  gtk_widget_show (align);
  gtk_box_pack_start (GTK_BOX (hbox1), align, TRUE, TRUE, 0);
  gtk_container_add (GTK_CONTAINER (align), vbox1);
#endif

  GtkWidget *frame = NEW_FRAME ("Objects and Colours", true);
  gtk_container_set_border_width(GTK_CONTAINER(frame), 4);
  gtk_widget_show (frame);
//  gtk_container_add (GTK_CONTAINER (align), frame);
  gtk_box_pack_start (GTK_BOX (vbox1), frame, FALSE, FALSE, 0);

  GtkWidget *table = table_new (N_OBJS-2+NUMSPECIES, 2, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), 8);
  gtk_widget_show (table);
  gtk_container_add (GTK_CONTAINER (get_widget(frame, "align")), table);
  g_object_set_data(G_OBJECT(dialog1), "objtab", table);
  table_set_row_spacings(table, 6);
  table_set_col_spacings(table, 15);

  GtkWidget *radio = myimage_button("_Reset Colours", "resetpalette.png");
  gtk_widget_show (radio);
  gtk_box_pack_end (GTK_BOX (vbox1), radio, FALSE, FALSE, 0);
  (void) g_signal_connect(G_OBJECT(radio), "clicked",
                    G_CALLBACK(ResetPalette), NULL);

  frame = NEW_FRAME ("Preview", true);
  gtk_container_set_border_width(GTK_CONTAINER(frame), 4);
  gtk_widget_show (frame);

#ifdef IN_GTK3
  gtk_box_pack_end (GTK_BOX (hbox1), frame, TRUE, TRUE, 0);
#else
  align = gtk_alignment_new(1, 0, 0, 0);
  gtk_widget_show (align);
  gtk_box_pack_end (GTK_BOX (hbox1), align, TRUE, TRUE, 0);
  gtk_container_add (GTK_CONTAINER (align), frame);
#endif

  GtkWidget *drawingarea1 = gtk_drawing_area_new ();
  gtk_widget_set_size_request(drawingarea1, 225, 225);
  g_object_set_data(G_OBJECT (dlg), "drawarea", drawingarea1);
  gtk_widget_show (drawingarea1);
  gtk_container_add (GTK_CONTAINER (get_widget(frame, "align")), drawingarea1);

  (void) g_signal_connect (G_OBJECT (drawingarea1), DRAW_EVENT,
                      G_CALLBACK (OnExpose),
                      dlg);

  return dialog1;
}
#endif

void SetFilter (GtkToggleButton *button, _SettingsInfo *info)
{
    GtkWidget *fontsel;
    GtkWidget *label;
    int scelta;
    
    scelta = get_int(GTK_WIDGET(button), "which");
    fontsel = lookup_widget(GTK_WIDGET(button), "fontselection");
    if(gtk_toggle_button_get_active(button))
    {
      switch (scelta)
      {
       case 0:                          /* proporzionale */
            label = lookup_widget(GTK_WIDGET(button), "fnamep");
            if(strlen(info->PropFont) > 2)
            {
                font_selection_set_font_name(fontsel, info->PropFont);
                gtk_label_set_text(GTK_LABEL(label), info->PropFont);
            }
            else
                gtk_label_set_text(GTK_LABEL(label), "to be set");
            break;
 
       case 1:                        /* larghezza fissa */
            label = lookup_widget(GTK_WIDGET(button), "fnamen");
            if(strlen(info->FixedFont) > 2)
            {
                font_selection_set_font_name(fontsel, info->FixedFont);
                gtk_label_set_text(GTK_LABEL(label), info->FixedFont);
            }
            else
                gtk_label_set_text(GTK_LABEL(label), "to be set");
            break;
 
        default:                        /* user font */
            label = lookup_widget(GTK_WIDGET(button), "fnameu");
            if(strlen(info->AppFont) > 2)
            {
                font_selection_set_font_name(fontsel, info->AppFont);
                gtk_label_set_text(GTK_LABEL(label), info->AppFont);
            }
            else
                gtk_label_set_text(GTK_LABEL(label), "to be set");
            break;
 
       }
    }
}

void create_fontselection(GtkWidget *frame, GtkWidget *dlg, 
                   _SettingsInfo *info)
{
   GtkWidget *fsel;
   GtkWidget *vbox1, *sel_button, *box, *vbox, *label;
   char riga[120];

   vbox1 = vbox_new(FALSE, 0);
   gtk_widget_show(vbox1);
   gtk_container_add(GTK_CONTAINER(frame), vbox1);
#ifdef GTK_FONT_CHOOSER
   fsel = gtk_font_chooser_widget_new();
#else
   fsel = gtk_font_selection_new();
#endif
   gtk_container_set_border_width(GTK_CONTAINER(fsel), 8);
   ScriviStringa(riga,  sizeof(riga), "Selected font for %s", info->pkgnam);
   font_selection_set_preview_text(fsel, riga);
   g_object_set_data(G_OBJECT(dlg), "fontselection", fsel);
   gtk_box_pack_start(GTK_BOX(vbox1), GTK_WIDGET(fsel),
                      TRUE, TRUE,0);

   box = hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox1), box,
                      FALSE, FALSE,0);
   gtk_widget_show(box);

   vbox = vbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(box), vbox,TRUE, TRUE,0);
   gtk_widget_show(vbox);

   sel_button = RADIO_BUTTON_WITH_LABEL(NULL, "Proportional Font", MyBLUE);
   gtk_container_add(GTK_CONTAINER(vbox), sel_button);
   g_object_set_data(G_OBJECT(fsel), "fspacingp", sel_button);
   g_object_set_data(G_OBJECT(sel_button), "which", GINT_TO_POINTER(0));
   (void) g_signal_connect (G_OBJECT (sel_button), "toggled",
                      G_CALLBACK (SetFilter), info);
   gtk_widget_show(sel_button);

   sel_button = RADIO_BUTTON_WITH_LABELW( (GtkRadioButton *)sel_button, 
               "Non proportional Font", MyBLUE);
   gtk_container_add(GTK_CONTAINER(vbox), sel_button);
   g_object_set_data(G_OBJECT(fsel), "fspacingf", sel_button);
   g_object_set_data(G_OBJECT(sel_button), "which", GINT_TO_POINTER(1));

   (void) g_signal_connect (G_OBJECT (sel_button), "toggled",
                      G_CALLBACK (SetFilter), info);
   gtk_widget_show(sel_button);

   sel_button = RADIO_BUTTON_WITH_LABELW( (GtkRadioButton *)sel_button, 
                "Menu and Windows Font", MyBLUE);
   gtk_container_add(GTK_CONTAINER(vbox), sel_button);
   g_object_set_data(G_OBJECT(fsel), "fspacingu", sel_button);
   g_object_set_data(G_OBJECT(sel_button), "which", GINT_TO_POINTER(2));
   (void) g_signal_connect (G_OBJECT (sel_button), "toggled",
                      G_CALLBACK (SetFilter), info);
   gtk_widget_show(sel_button);
   
   vbox = vbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(box), vbox,TRUE, TRUE,0);
   gtk_widget_show(vbox);

   label = label_boxed(" ");
   gtk_container_add(GTK_CONTAINER(vbox), label);
   g_object_set_data(G_OBJECT(dlg), "fnamep", gtk_bin_get_child(GTK_BIN(label)));
   gtk_widget_show(label);

   label = label_boxed(" ");
   gtk_container_add(GTK_CONTAINER(vbox), label);
   g_object_set_data(G_OBJECT(dlg), "fnamen", gtk_bin_get_child(GTK_BIN(label)));
   gtk_widget_show(label);

   label = label_boxed(" ");
   gtk_container_add(GTK_CONTAINER(vbox), label);
   g_object_set_data(G_OBJECT(dlg), "fnameu", gtk_bin_get_child(GTK_BIN(label)));
   gtk_widget_show(label);

//   return fsel;
}

void open_font_selection(GtkWidget *fsel, _SettingsInfo *info)
{
     GtkToggleButton *prop, *fix;
     GtkWidget *label;

     prop = (GtkToggleButton *)get_widget(GTK_WIDGET(fsel), "fspacingp");
     fix  = (GtkToggleButton *)get_widget(GTK_WIDGET(fsel), "fspacingf");
     if(gtk_toggle_button_get_active(prop))
                (void) font_selection_set_font_name(fsel, info->PropFont); 
     else if(gtk_toggle_button_get_active(fix))
                (void) font_selection_set_font_name(fsel, info->FixedFont);
     else
                (void) font_selection_set_font_name(fsel, info->AppFont);

     label = lookup_widget(settings_dlg, "fnamep");
     if(strlen(info->PropFont) > 2)
        gtk_label_set_text(GTK_LABEL(label), info->PropFont);
     else
        gtk_label_set_text(GTK_LABEL(label), "to be set");
     label = lookup_widget(settings_dlg, "fnamen");
     if(strlen(info->FixedFont) > 2)
        gtk_label_set_text(GTK_LABEL(label), info->FixedFont);
     else
        gtk_label_set_text(GTK_LABEL(label), "to be set");
     label = lookup_widget(settings_dlg, "fnameu");
     if(strlen(info->AppFont) > 2)
        gtk_label_set_text(GTK_LABEL(label), info->AppFont);
     else
        gtk_label_set_text(GTK_LABEL(label), "to be set");
     gtk_widget_hide(label);
     label = get_widget(GTK_WIDGET(fsel), "fspacingu");
     gtk_widget_hide(label);
}

GtkWidget *create_settings_dlg(_SettingsInfo *info)
{
   GtkWidget *dlg, *okay_button, *cancel_button, *apply_button, *vbox1, 
             *notebook;

   dlg = gtk_dialog_new();
   gtk_window_set_title(GTK_WINDOW(dlg), g_strdup_printf("Settings for %s",
                        info->pkgnam));
//   gtk_widget_set_size_request(dlg, 650, 500);
   gtk_container_set_border_width
       (GTK_CONTAINER (gtk_dialog_get_content_area(GTK_DIALOG(dlg))), 4);
   g_object_set_data(G_OBJECT (dlg), "info", info);
   notebook = gtk_notebook_new ();
   g_object_set_data(G_OBJECT (dlg), "notebook", notebook);
   gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG(dlg))),
         notebook, TRUE, TRUE, 0);
   gtk_widget_show (notebook);
   gtk_widget_set_can_focus (GTK_WIDGET(notebook), FALSE);


   vbox1 = gtk_frame_new(NULL);
   gtk_widget_show (vbox1);
   gtk_container_add (GTK_CONTAINER (notebook), vbox1);
   create_fontselection(vbox1, dlg, info);
   set_tab_label (GTK_NOTEBOOK (notebook), 0, "Fonts");


#ifdef COLORSET
   vbox1 = gtk_frame_new(NULL);
   gtk_widget_show (vbox1);
   create_colorselection(vbox1, dlg);
   if(dialogo->JAV_VIEWER == 0)
   {
      gtk_container_add (GTK_CONTAINER (notebook), vbox1);
      set_tab_label (GTK_NOTEBOOK (notebook),1, "Colours");
   }
#endif

   okay_button = myimage_button("   _OK", "ok.png");
   gtk_widget_set_name(okay_button, "okay");
   gtk_dialog_add_action_widget(GTK_DIALOG(dlg), okay_button, GTK_RESPONSE_OK);

   apply_button = myimage_button ("_Accept", "apply.png");
   gtk_widget_set_name(apply_button, "apply");
   gtk_dialog_add_action_widget(GTK_DIALOG(dlg), apply_button, GTK_RESPONSE_APPLY);

   cancel_button = myimage_button("  _Cancel", "cancel.png");
   gtk_widget_set_name(cancel_button, "cancel");
   gtk_dialog_add_action_widget(GTK_DIALOG(dlg), cancel_button, GTK_RESPONSE_CANCEL);

   gtk_button_box_set_layout(GTK_BUTTON_BOX(
      gtk_widget_get_parent(okay_button)), GTK_BUTTONBOX_SPREAD);
   return dlg;
}

#ifdef COLORSET
void UpdateFrameColors (GtkWidget *win, GdkColor Palette[])
{
   int i;
   GtkWidget *but, *pix;
   char num[3];


   for(i=0; i<nrows*ncols;i++)
   {
      ScriviStringa(num, sizeof(num), "%d", i);
      but = get_widget(win, num);
      pix = get_widget(but, "pix");
      set_widget_color(pix, Palette[i]);
   }
}

GtkWidget *create_frame_colors(GtkWidget *dialog1, GCallback cb_func)
{
  GtkWidget *table1;
  GtkWidget *togglebutton;
  GtkWidget *frame;
  GtkWidget *framec;
  GtkWidget *align;
  int i, j, k;
  char num[3];

  frame = NEW_FRAME ("Colours", true);
  gtk_container_set_border_width(GTK_CONTAINER(frame), 4);
  align = get_widget(frame, "align");

  table1 = table_new (nrows, ncols, TRUE);
  gtk_container_set_border_width(GTK_CONTAINER(table1), 2);  //10);
  table_set_row_spacings(table1, 0);          //10);
  table_set_col_spacings(table1, 0);          //10);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (align), table1);

  for(j=0; j<nrows;j++)
  {
     for(k=0; k<ncols; k++)
     {
         i = j*ncols + k;
         framec = gtk_frame_new(NULL);
         gtk_widget_show(framec);
         gtk_frame_set_shadow_type(GTK_FRAME(framec),GTK_SHADOW_NONE);
         togglebutton = mr_toggle_button_new_with_color();
         ScriviStringa(num, sizeof(num), "%d", i);
         gtk_container_set_border_width(GTK_CONTAINER(togglebutton), 4);
/* ogni bottone conosce la sua posizione (indice nel vettore dei colori) */
         g_object_set_data(G_OBJECT(togglebutton), "id", GINT_TO_POINTER(i));
         g_object_set_data(G_OBJECT(togglebutton), "frame", framec);
         g_object_set_data(G_OBJECT(togglebutton), "father", frame);
/* salvo l'handler dei bottoni nella dialog */
         g_object_set_data(G_OBJECT(dialog1), num, togglebutton);
         gtk_container_set_border_width(GTK_CONTAINER(togglebutton), 0);
         gtk_button_set_relief(GTK_BUTTON(togglebutton), GTK_RELIEF_NONE);
         gtk_widget_set_can_focus (togglebutton, FALSE);
         gtk_widget_set_size_request(togglebutton, 30, 30);
         gtk_widget_show (togglebutton);
         gtk_container_add(GTK_CONTAINER(framec), togglebutton);
         table_attach (table1, framec, k, k+1, j, j+1);
         (void) g_signal_connect(G_OBJECT(togglebutton), "toggled",
                    G_CALLBACK(cb_func), framec);
      }
  }
  return frame;
}

void ToggleButtons (GtkWidget *win, int colore_attivo)
{
   int i;
   GtkWidget *but, *frame;
   char num[3];
                                                                                                                            
   for(i=0; i<nrows*ncols;i++)
   {
      ScriviStringa(num, sizeof(num), "%d", i);
      but = get_widget(win, num);
      if(i != colore_attivo)
      {
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(but), FALSE);
         frame = get_widget(but, "frame");
         gtk_frame_set_shadow_type(GTK_FRAME(frame),GTK_SHADOW_NONE);
     }
   }
}

void ResetButtons (GtkWidget *win)
{
   int i;
   GtkWidget *but, *frame;
   char num[3];

   for(i=0; i<nrows*ncols;i++)
   {
      ScriviStringa(num, sizeof(num), "%d", i);
      but = get_widget(win, num);
      frame = get_widget(but, "frame");
      gtk_frame_set_shadow_type(GTK_FRAME(frame),GTK_SHADOW_NONE);
   }
}

void onSetDefaults(GtkWidget *button, gpointer data)
{
}

#ifdef IN_GTK3
void onSetColor(GtkWidget *cb, gpointer data)
#else
void onSetColor(GtkWidget *cb, GtkStyle *style, gpointer data)
#endif
{
   float color[4];
   GtkWidget *dlg = gtk_widget_get_toplevel(cb);
   mycolor_button_get_color(cb, color);
   GdkColor *colore = ToGdkColor(color);
   int which = GPOINTER_TO_INT(data);
   memcpy(&mySettings->colori[which], colore, sizeof(GdkColor));
   Paint(dlg);
}


void open_color_selection(GtkWidget *csel, int param, _SettingsInfo *info)
{
   GtkWidget *gtklist;

   mySettings->tWin = param;
   gtklist = get_widget(csel, "objtab");
   GList *children = gtk_container_get_children(GTK_CONTAINER(gtklist));
   for(int i=0; i< (int)g_list_length(children); i++)
      gtk_container_remove(GTK_CONTAINER(gtklist), GTK_WIDGET(g_list_nth_data(children, i)));
   g_object_set_data(G_OBJECT(gtklist), "nobj", GINT_TO_POINTER((int)info->oggetti.size()));
   for(int i=0; i< (int) info->oggetti.size(); i++)
   {
       mySettings->ind = info->oggetti[i].indc;
       GtkWidget *label = LABEL_NEW(info->oggetti[i].label.c_str(), NULL);
       set_alignment(label, 0., 0.5);
       gtk_widget_show(label);
       table_attach(gtklist, label, 0, 1, i, i+1, TRUE);
       const GdkColor colore = mySettings->colori[mySettings->ind];
       float *fcol = ToColor3f(&colore);
       float color1[] = {fcol[0], fcol[1], fcol[2], 1};
       GtkWidget *cb = mycolor_button(color1, "Pick a color to be used for draw atoms");
       gtk_widget_show_all(cb);
       g_signal_connect(G_OBJECT(cb), COLOR_CHANGED, G_CALLBACK(onSetColor),
           GINT_TO_POINTER(mySettings->ind));
       char *nome = g_strdup_printf("%d", i);
       g_object_set_data(G_OBJECT(gtklist), nome, cb);
       g_object_set_data(G_OBJECT(cb), "ind", GINT_TO_POINTER(mySettings->ind));
       table_attach_all(gtklist, cb, 1, 2, i, i+1);
   }
   gtk_widget_show(csel);
   mySettings->ind = 0;
}
#endif

Settings::Settings(const _SettingsInfo info, int twin)
{
     int i;
     pkgnam = info.pkgnam;
     whichFont = 0;
     AppFont = info.AppFont;
     PropFont = info.PropFont;
     FixedFont = info.FixedFont;
     jbg = info.bg;
     for(i=0; i<50; i++)
         indcolor[i] = info.indcolor[i];
     for(i=0; i<16; i++)
         colorb[i] = info.colorb[i];
     for(i=0; i<N_OBJS+N_ELEMENTS; i++)
         colori[i] = info.colorb[i];
     CurrentColor.red = 0; 
     CurrentColor.blue = 0;
     CurrentColor.green = 0;
     CurrentColor.pixel = 0;
     old_selected = NULL;
     ind = -1;
     tWin = twin;
     NumSpec = info.nSpec;
     MaxSpecies = info.maxspecies;
     for(i=0; i<MaxSpecies; i++)
         nat[i] = info.nat[i];
}

void open_settings_notebook (GtkWidget* mainw, _SettingsInfo *info, rgbCol col[])
{
     GtkWidget *fsel;
     GtkWidget *notebook;
     int result, npag, j;

     mySettings = new Settings(*info, 1);
     if (settings_dlg == NULL)
          settings_dlg = create_settings_dlg (info);

     for(j=0; j<16; j++)
          mySettings->BaseColors[j] = col[j];
     notebook = get_widget(settings_dlg, "notebook");
#ifdef COLORSET
     GtkWidget *colsel;
     if(dialogo->JAV_VIEWER == 0)
     {
     colsel = get_widget(settings_dlg, "colsel");
     open_color_selection(colsel, 1, info);
     gtk_widget_show (colsel);
     }
#endif
     fsel = get_widget(settings_dlg, "fontselection");

     open_font_selection(fsel, info);
     gtk_widget_show (GTK_WIDGET(fsel));
     gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);

     gtk_window_set_transient_for(GTK_WINDOW(settings_dlg), GTK_WINDOW(mainw));
     gtk_widget_show (settings_dlg);
     gdk_window_raise (gtk_widget_get_window(settings_dlg));
     setSmiley(mainw, "smil5.gif");
     gtk_widget_set_sensitive(get_widget(mainw, "menubar"), FALSE);
     gtk_widget_set_sensitive(get_widget(mainw, "toolbar_main"), FALSE);

     bool ciclo = true;
     while(ciclo)
     {
        result = gtk_dialog_run(GTK_DIALOG(settings_dlg));
        npag = gtk_notebook_get_current_page (GTK_NOTEBOOK(notebook));
        switch(result)
        {
        case GTK_RESPONSE_OK:
             ciclo = false;
        case GTK_RESPONSE_APPLY:
             if(OnAccept(npag, fsel, settings_dlg, info) < 0)
                ciclo = true;
             break;
        case GTK_RESPONSE_CANCEL:
        case GTK_RESPONSE_DELETE_EVENT:
             ciclo = false;
            break;
        }
     }
     gtk_widget_hide(settings_dlg);
     gtk_widget_set_sensitive(get_widget(mainw, "menubar"), TRUE);
     gtk_widget_set_sensitive(get_widget(mainw, "toolbar_main"), TRUE);
     resetSmiley(mainw);
}
