!--------------------------------------------------------------
!--------------------------------------------------------------
!
!  SUBROUTINES SUI RESTRAINTS
!
!--------------------------------------------------------------
!--------------------------------------------------------------
 MODULE RRESTR
 USE progtype, only:dblep
 USE atom_basic, only: atom_type

 implicit none

 integer, parameter :: RESDIST=1, RESANGLE=2, RESPLANE=3, ABUMP=4

 type restraint_type
   integer                            :: code = 0  ! indica il tipo di restraint: 1=distanza,2=angolo,3=piano,4=anti-bump
   integer, dimension(:), allocatable :: na        ! puntatori agli atomi coinvolti nel restraint
   real                               :: val       ! valore corrente del restraint
   real                               :: targ      ! valore target del restraint
   real                               :: sigma     ! deviazione standard
   real                               :: wei = 1.0 ! weight
   integer, dimension(3)              :: sym = 0   ! if /= 0 consider symmetry equivalent atoms
 end type 

 type constraint_type
   integer                            :: code     ! codice identificativo del tipo di constraint (1=B,2=co.,3=sof,4=BH)
   integer, dimension(:), allocatable :: vpar     ! i parametri coinvolti
   integer, dimension(:), allocatable :: vat      ! gli atomi coinvolti
   real, dimension(:), allocatable    :: coef     ! coefficienti
   integer                            :: npar     ! il numero totale di parametri coinvolti
   integer                            :: npari    ! numero di parametri indipendenti
   integer                            :: ref      ! codice di affinamento
   integer                            :: typec    ! settato da 1=user 2=programma
 end type constraint_type

 real, dimension(3)                               :: wqq         ! pesi per ogni classe di restraint
 integer, dimension(3)                            :: ktargettype ! tipo di target: 1 da table, 2 uguale al target 
 integer                                          :: autowei = 0 ! pesi in automatico se vale 1
 real(dblep)                                      :: chi2func    ! contributo al chiquadro delle funzioni non di restraints
 real(dblep), dimension(3)                        :: chi2res     ! contributo al chiquadro per ogni classe di restraint
 
!corr type(constraint_type), dimension(:), allocatable :: constrv     ! i constraints

! S resapp(reslist,numj,kcalcder,coderwi)                    Calcola jacobiano e contributo al chiquadro dei restraints
! S weigthrestr(reslist,coderw,numj,kcalcder)                Gestione pesi 
! S aresd(nrj,rest,kcalcder)                                 Jacobiano dei restraints su distanza
! S aresa(nrj,rest,kcalcder)                                 Jacobiano dei restraints su angoli
! S arespnew(nrj,rest,kcalcder)                              Jacobiano dei restraints su piani
! F plane_condition(at)                                      Controlla planarita degli atomi
! F mdistance_lsqplane(ati)                                  Calcola la distanza media degli atomi at dal piano dei minimi quadrati
! F nrestraints(reslist)                                     Restituisce il numero di restraints
! S reset_restraints(reslist)                                Azzera restraints
! S save_restraints(unitbin,res)                             Write restraints on binary file
! S read_restraints(unitbin,res,err)                         Read restraints from binary file
! S add_restraint_to_list(reslist,res)                       Aggiungi un restraint alla lista
! S remove_restraint_from_list(reslist,pos)                  Rimuovi dalla lista il restraint in posizione pos 
! F restraint_position(reslist,res)                          Cerca la posizione di un restraint all'interno della lista
! S stampa_restraints(kpr,reslist)                           Stampa i restraints
! S read_restraints_from_file(filename,reslist)              Legge restraints da file esterno
! S write_restraints_on_file(filename,reslist,atom)          Scrive i restraints su un file esterno
! S get_restraint_from_string(string,atom,res,ier)           Legge il restraint dalla stringa del tipo:
! S set_value_for_restraint(res,atom)                        Calcola valore corrente per il restraint
! S set_equivalent_for_angle(resv,legm)                      For angle n1-n2-n3 assign negative value to atom for which is not necessary compute equivalent
! S set_weight(res,code,wei)                                 Assign weight to all restraints of type code
! F iconn_to_restraints(iconn,atom,code,reslist)             Converte tutte le distanze e/o angoli in restraints
! S update_restraints(reslist,rmvet)                         Aggiorna la lista dei restraints se ho eliminato degli atomi
! S reset_constraints()                                      Azzera constraints
! S set_restr(reslist,code,atr,targ,sig)                     Definisce il restraint e lo aggiunge alla lista 
! S add_constraint(code,vpar,vat,atom,coefi)                 Aggiunge un nuovo constraint alla lista
! F constr_aveb(constr,atom,bmin,bmax)  result(bmed)         Compute an average b for atom in constr
! S add_constraint_on_biso()                                 Imponi che atomi della stessa specia abbiano lo stesso fattore termico
! F par_in_constraint(kat,npar) result(kconstr)              Controlla se un atomo e' implicato in un constraint
! F coef_of_constraint(kconstr,nat,npar) result(coef)        Prende il coefficente del parametro npar del constraint
! S refine_constraint(constr,atom,code)                      Chiedo di affinare (code=1) o non affinare (code=0) un constraint
! S delete_constraint(code)                                  Elimina tutti i constrain di codice uguale a code
! S reallocate_restr(vetr,n,savevet)                         Rialloca ad n un vettore di tipo restraint
! S reallocate_constr(vetr,n,savevet)                        Rialloca ad n un vettore di tipo restraint
! S calc_chi2res(reslist,npf)                                Chi2 dei restraints
! S ordina_res(reslist)                                      Ordina i restraints per classe
! S set_constr_on_position()                                 Cerca atomi in posizione speciale generando nconstr constraints
! S print_lsqcond()                                          Stampa informazioni sulle posizioni speciali dai constraint

 CONTAINS
 
   subroutine resapp(reslist,numj,kcalcder,coderwi)
!
!  Calcola jacobiano e contributo al chiquadro dei restraints
!
   type(restraint_type), dimension(:), allocatable, intent(inout) :: reslist
   integer, intent(in)           :: numj     ! puntatore allo jacobiano
   integer, intent(in)           :: kcalcder ! = 0/1 -> non calcolo / calcolo anche le derivate dei restraints
   integer, intent(in), optional :: coderwi  ! codici per differente gestione dei pesi
   integer                       :: coderw
   integer                       :: i
   integer                       :: numres
!
   numres = nrestraints(reslist)  !!!!size(reslist)
   if(numres==0) return
!
   do i=1,numres

      select case(reslist(i)%code)

         case(1)                                !distances
          call aresd(i+numj,reslist(i),kcalcder)
          
         case(2)                                !angles
          call aresa(i+numj,reslist(i),kcalcder)

         case(3)                                !planes
          call arespnew(i+numj,reslist(i),kcalcder)

      end select
   enddo
!   
!  Integra funcls e jacob con il contributo dei pesi
   if (present(coderwi)) then
       coderw = coderwi
   else
       coderw = 0
   endif
   call weigthrestr(reslist,coderw,numj,kcalcder)
!
   end subroutine resapp

!-----------------------------------------------------------------------

   subroutine weigthrestr(reslist,coderw,numj,kcalcder)
!
!  Questa subroutine richiede che i restraints siano ordinati per classi (usa ordina_res)
!
   USE genlsq, only:funcls,jacob
   type(restraint_type), dimension(:), intent(in) :: reslist
   integer, intent(in)             :: coderw
   integer, intent(in)             :: numj
   integer, intent(in)             :: kcalcder
   integer                         :: posi,posf
   real(dblep), dimension(3), save :: weires
   integer                         :: i
   integer, dimension(3)           :: nrr
   logical                         :: lpr = .false.
!
!  Calcola num. di restraints per classe
   do i=1,3
      nrr(i) = count(reslist(:)%code==i)
   enddo
!  
!  Calcola in weires la scala tra restraints e funzioni non restraints
   if (coderw >  0) then
       call calc_chi2res(reslist,numj)
       do i=1,3
          if (nrr(i) == 0) then
              weires(i) = 0.0
          else
              weires(i) = chi2func / chi2res(i)
              if(lpr) write(6,'(a,i5,g20.6)')'weilog=',int(log10(weires(i))),weires(i),chi2func,chi2res(i)
          endif
       enddo
   endif
!
   select case (coderw)
       case (0)
!
!        usa il peso fornito esternamente 
         weires = wqq

       case (1)
!
!        wqq diventa il fattore di scala 
!        Viene aggiornato per un'eventuale stampa
         wqq = weires

       case (2)
!
!        wqq diventa un peso dei restraints dopo la scala
!        Non viene aggiornato perchè va mantenuto il valore fornito esternamente
         weires = wqq*weires
   end select
   if (lpr) then
       write(6,*)'WEIRES=',weires
       write(6,*)'WQQ   =',wqq
       write(6,*)'CHI2  =',chi2func,chi2res(1:3)
       write(6,*)'CHI2*W=',chi2func,weires(1:3)*chi2res(1:3)
   endif
!
!  Applica il peso calcolato ai restraints
   posf = numj
   do i=1,3
      if (nrr(i) == 0) cycle
      posi = posf + 1
      posf = posf + nrr(i)
      funcls(posi:posf) = sqrt(weires(i))*funcls(posi:posf)
      if(kcalcder == 1) jacob(posi:posf,:) = sqrt(weires(i))*jacob(posi:posf,:)
   enddo
!
   end subroutine weigthrestr

!-----------------------------------------------------------------------

   subroutine aresd(nrj,rest,kcalcder)
!
!  jacobiano dei restraints su distanza
!
   USE variables, only:atm
   USE GENLSQ,       only:jacob,funcls,weilsq
   USE General,    only:xgg
   USE CGEOM,        only:distanzaC
!
   integer, intent(in)       :: nrj         !puntatore del restraints nello jacobiano
   type(restraint_type), intent(inout) :: rest
   integer, intent(in)       :: kcalcder    !se =1 calcolo il contributo allo jacobiano
   type(atom_type), dimension(2) :: ra          !gli atomi coinvolti nel restraints
   real                      :: diste,sigma !distanza attesa e sigma
   integer                   :: i
   real                      :: coef,dista,dd
   real,  dimension(2,3)     :: de
   integer                   :: kpar
!
   ra(1) = atm(rest%na(1))
   ra(2) = atm(rest%na(2))
   diste = rest%targ
   sigma = rest%sigma
   coef = 1.0 / sigma**2
!
!  Calcola distanza tra gli atomi
   dista  = distanzaC(ra(1)%xc,ra(2)%xc,xgg)
   rest%val = dista
   dd = diste - dista
!
   weilsq(nrj) = sqrt(coef)
   funcls(nrj) = weilsq(nrj)*dd
!
!  calcolo derivate della distanza rispetto ad x,y,z
!  de(1,:) = derivate della distanza ristetto alle coord x1,y1,z1 di at1
!  de(2,:) = derivate della distanza ristetto alle coord x2,y2,z2 di at2
   if (kcalcder == 1) then
       de(1,:) = MATMUL (xgg,ra(1)%xc - ra(2)%xc) / dista
       de(2,:) = -de(1,:)
!
       jacob(nrj,:) = 0.0  !azzero su tutti i parametri
       do i=1,2
          do kpar=1,3
             if (ra(i)%rcod(kpar) > 0) then
                 !jacob(nrj,ra(i)%rcod(kpar)) = -sqrt(wqq(1)*coef) * de(i,kpar)
                 jacob(nrj,ra(i)%rcod(kpar)) = -weilsq(nrj) * de(i,kpar)
             endif
          enddo
       enddo
   endif
!
   end subroutine aresd
   
!-----------------------------------------------------------------------

   subroutine aresa(nrj,rest,kcalcder)
!
!  Jacobiano dei restraints su angoli
!
   USE variables, only:atm
   USE General,    only:xgg,rtod,dtor
   USE GENLSQ,       only:jacob,funcls,weilsq
   USE CGEOM,        only:distanzaC
!
   type(atom_type), dimension(3) :: ra           !info sui parametri coinvolti nel restraints
   real                      :: angE,sigma   !distanza attesa e sigma
   integer, intent(in)       :: nrj          !puntatore del restraints nello jacobiano
   type(restraint_type), intent(inout) :: rest
   integer, intent(in)       :: kcalcder     !se=1 calcolo il contributo allo jacobiano
   real,     dimension(3)    :: XGX1
   real, dimension(3,3)      :: de
   integer                   :: i
   real                      :: coef
   real                      :: dis1,dis2,dis11,dis22,dis12,dd,XGX
   real                      :: costhA,angA,costhE
   integer                   :: kpar
!
   ra(1) = atm(rest%na(1))
   ra(2) = atm(rest%na(2))
   ra(3) = atm(rest%na(3))
   angE = rest%targ
   sigma = rest%sigma
   coef = 1.0 / sigma**2
   dis1 = distanzaC(ra(1)%xc,ra(2)%xc,xgg)
   dis2 = distanzaC(ra(2)%xc,ra(3)%xc,xgg)
   dis11 = dis1 * dis1
   dis22 = dis2 * dis2
   dis12 = dis1 * dis2
   XGX = DOT_PRODUCT(ra(1)%xc-ra(2)%xc,MATMUL(xgg,ra(3)%xc-ra(2)%xc))
   costhA = XGX / dis12
   angA = acos ( costhA ) * rtod
   rest%val = angA
   costhE = cos ( angE*dtor )
!
   dd = costhE - costhA
   weilsq(nrj) = sqrt(coef)
   funcls(nrj) = weilsq(nrj)*dd
   if (kcalcder == 1) then
!
!      derivate del coseno dell'angolo rispetto ad x,y,z di n1
       de(1,1:3) = MATMUL(xgg,((ra(3)%xc-ra(2)%xc) - XGX * (ra(1)%xc-ra(2)%xc)/dis11)/dis12)
!
!      derivate del coseno dell'angolo rispetto ad x,y,z di n3
       de(3,1:3) = MATMUL(xgg,((ra(1)%xc-ra(2)%xc) - XGX * (ra(3)%xc-ra(2)%xc)/dis22)/dis12)
!
!      derivate del coseno dell'angolo rispetto ad x,y,z di n2
       XGX1 = 2*ra(2)%xc - ra(3)%xc - ra(1)%xc + XGX * ((ra(1)%xc-ra(2)%xc)/dis11 + (ra(3)%xc-ra(2)%xc)/dis22)
       de(2,1:3) = MATMUL(xgg,XGX1) / dis12
!
       jacob(nrj,:) = 0.0
       do i=1,3
          do kpar=1,3
             if (ra(i)%rcod(kpar) > 0) then
                 !jacob(nrj,ra(i)%rcod(kpar)) = -sqrt(wqq(2)*coef) * de(i,kpar)
                 jacob(nrj,ra(i)%rcod(kpar)) = -weilsq(nrj) * de(i,kpar)
             endif
          enddo
       enddo
   endif
!
   end subroutine aresa

!-----------------------------------------------------------------------

   subroutine arespnew(nrj,rest,kcalcder)
!
!  Jacobiano dei restraints su piani
!
   USE progtype, only:dblep
   USE variables, only:atm
   USE GENLSQ, only:jacob,funcls,weilsq
!
   real                  :: sigma         !sigma
   integer, intent(in)                  :: nrj           !puntatore del restraints nello jacobiano
   type(restraint_type), intent(in) :: rest
   integer, intent(in)                  :: kcalcder      !se=1 calcolo il contributo allo jacobiano
   type(atom_type), dimension(size(rest%na))        :: rat           !gli atomi coinvolti nel restraint
   integer                              :: i,k,l,m
   real                                 :: detV,dd,coef
   real,          dimension(3)          :: xc
   real,          dimension(3)          :: ddetV
   real,          dimension(3)          :: dbard
   real(dblep), dimension(3,3)        :: s,s2
   integer                              :: kpar
   integer                              :: numat
!
!  calcolo piano degli atomi
   sigma = rest%sigma
   numat = size(rest%na)
    do i=1,numat
       rat(i) = atm(rest%na(i))
    enddo
!
!  trova il baricentro degli atomi da planarizzare
   do i=1,3
      dbard(i) = sum(rat%xc(i)) / numat
   enddo
!
!  calcolo elementi della matrice V 
   s = 0.0
   do i = 1 , numat
      xc = rat(i)%xc - dbard   ! shift rispetto al baricentro
      do k = 1 , 3
         do l = 1 , 3
            s(k,l) =s(k,l)+xc(l)*xc(k)
         enddo
      enddo
   enddo
!
!  calcolo determinante della matrice V
   s2(1,3) = s(1,3)**2
   s2(1,2) = s(1,2)**2
   s2(2,3) = s(2,3)**2
   detV = -s2(1,3)*s(2,2)+2*s(1,2)*s(1,3)*s(2,3)-s(1,1)*s2(2,3)      &
          -s2(1,2)*s(3,3)+s(1,1)*s(2,2)*s(3,3)
!
   dd = detV
   coef = 1.0 / sigma**2
   weilsq(nrj) = sqrt(coef)
   funcls(nrj) = weilsq(nrj)*dd
   if (kcalcder == 1) then
!
       jacob(nrj,:) = 0.0
       do m=1,numat
!
!         calcolo derivate di V
!         ddetV(1) = derivata di detV rispetto a x
!         ddetV(2) = derivata di detV rispetto a y
!         ddetV(3) = derivata di detV rispetto a z
          xc = rat(m)%xc
          ddetV(1) = 2.0*xc(1)*(s(2,2)*s(3,3) - s2(2,3))         &
                   + 2.0*xc(2)*(s(1,3)*s(2,3) - s(1,2)*s(3,3))   &
                   + 2.0*xc(3)*(s(1,2)*s(2,3) - s(1,3)*s(2,2))

          ddetV(2) = 2.0*xc(2)*(s(1,1)*s(3,3) - s2(1,3))         &
                   + 2.0*xc(1)*(s(1,3)*s(2,3) - s(1,2)*s(3,3))   &
                   + 2.0*xc(3)*(s(1,2)*s(1,3) - s(1,1)*s(2,3))

          ddetV(3) = 2.0*xc(3)*(s(1,1)*s(2,2) - s2(1,2))         &
                   + 2.0*xc(1)*(s(1,2)*s(2,3) - s(1,3)*s(2,2))   &
                   + 2.0*xc(2)*(s(1,2)*s(1,3) - s(1,1)*s(2,3))
!
          do kpar=1,3
             if (rat(m)%rcod(kpar) > 0) then
                 jacob(nrj,rat(m)%rcod(kpar)) = weilsq(nrj) * ddetV(kpar)
             endif
          enddo
       enddo
   endif
!
   end subroutine arespnew 
   
!--------------------------------------------------------------------------

   real function plane_condition(at)   result(detV)
!
!  Calcola a partire da atomi in coord. cartesiane detV.
!  detV = 0 se gli atomi sono su un piano 
!   
   type(atom_type), dimension(:), intent(in) :: at      !atomi in coordinate cartesiane  
   integer                               :: numat
   integer                               :: i,k,l
   real, dimension(3)                    :: dbard
   real, dimension(3,3)                  :: s,s2
   real, dimension(3)                    :: xc
!   
!  trova il baricentro degli atomi da planarizzare
   numat = size(at)
   do i=1,3
      dbard(i) = sum(at%xc(i)) / numat
   enddo
!
!  calcolo elementi della matrice V 
   s = 0.0
   do i = 1 , numat
      xc = at(i)%xc - dbard   ! shift rispetto al baricentro
      do k = 1 , 3
         do l = 1 , 3
            s(k,l) =s(k,l)+xc(l)*xc(k)
         enddo
      enddo
   enddo
!
!  calcolo determinante della matrice V
   s2(1,3) = s(1,3)**2
   s2(1,2) = s(1,2)**2
   s2(2,3) = s(2,3)**2   
   detV = -s2(1,3)*s(2,2)+2*s(1,2)*s(1,3)*s(2,3)-s(1,1)*s2(2,3)      &
          -s2(1,2)*s(3,3)+s(1,1)*s(2,2)*s(3,3)
!   
   end function plane_condition
      
!--------------------------------------------------------------------------

   real function mdistance_lsqplane(ati)  result(distam)
!
!  Calcola la distanza media degli atomi at dal piano dei minimi quadrati
!
   USE CGEOM, only:lsqplane
   USE variables, only:ortom
   type(atom_type), dimension(:), intent(in) :: ati
   real, dimension(3,size(ati))   :: cat
   real, dimension(4)             :: pvet
   integer                        :: nap
   integer                        :: i
!
   nap = size(ati)
!
!  Converti in coordinate cartesiane
   do i=1,nap
      cat(:,i) = MATMUL(ortom,ati(i)%xc)
   enddo
!
!  Calcola normale al piano lsq
   call lsqplane(cat,pvet)
!
!  Calcolo distanza media dal piano
   distam = 0.0
   do i=1,nap
      distam = distam + abs(SUM(pvet(:3)*cat(:,i)))
   enddo
   distam = distam/nap
!
   end function mdistance_lsqplane

!--------------------------------------------------------------------------

   integer function nrestraints(reslist)
!
!  Restituisce il numero di restraints
!
   type(restraint_type), dimension(:), allocatable, intent(in) :: reslist
!   
   if(allocated(reslist)) then 
      nrestraints = size(reslist)
   else
      nrestraints = 0
   endif
!
   end function nrestraints

!--------------------------------------------------------------------------

   subroutine reset_restraints(reslist)
!
!  Azzera restraints
!
   type(restraint_type), dimension(:), allocatable, intent(inout) :: reslist
   autowei = 0
   wqq(:3) = (/10000.,10000.,10000./)
   chi2res(:) = 0
!
!  Cancella restraints già esistenti
   if (allocated(reslist)) deallocate(reslist)
!
   end subroutine reset_restraints

!--------------------------------------------------------------------------

   subroutine add_restraint_to_list(reslist,res)
!
!  Aggiungi un restraint alla lista 
!
   type(restraint_type), dimension(:), allocatable, intent(inout) :: reslist
   type(restraint_type), intent(in)                               :: res                                           
   integer                                                        :: n
   integer                                                        :: pos
   integer                                                        :: code
   integer                                                        :: i
!
   n = nrestraints(reslist)
!
!  Calcola la posizione in cui inserire il restrain
   pos = 1
   if (n /= 0) then
       code = res%code
       do i=1,code
          pos = pos + count(reslist(:)%code == i)
       enddo
   endif
!
   call reallocate_restr(reslist,n+1) 
   reslist(pos+1:) = reslist(pos:n)   ! libera la posizione
   reslist(pos) = res
!
   end subroutine add_restraint_to_list

!--------------------------------------------------------------------------

   subroutine remove_restraint_from_list(reslist,pos)
!
!  Rimuovi dalla lista il restraint in posizione pos 
!
   type(restraint_type), dimension(:), allocatable, intent(inout) :: reslist
   integer, intent(in)                                            :: pos
   integer                                                        :: n
!
   n = nrestraints(reslist)
   reslist(pos:n-1) = reslist(pos+1:)
   call reallocate_restr(reslist,n-1)
!
   end subroutine remove_restraint_from_list

!--------------------------------------------------------------------------

   integer function restraint_position(reslist,res)  result(pos)
   USE nr
   USE realloc
   type(restraint_type), dimension(:), allocatable, intent(in) :: reslist
   type(restraint_type), intent(in)                            :: res
   !integer, intent(in), optional                               :: numres
   integer                                                     :: i
   integer                                                     :: nrs
   integer, allocatable, dimension(:)                          :: iord1,iord2
   integer                                                     :: np1,np2
!
   pos = 0
   nrs = nrestraints(reslist)
   if (nrs == 0) return
   !if (present(numres)) then
   !    if (numres == 0 .or. numres > nrs) return
   !    nrs = numres
   !endif
!
   select case (res%code)
      case (RESDIST, ABUMP)
        do i=1,nrs
           if (res%code /= reslist(i)%code) cycle
           if ((reslist(i)%na(1) == res%na(1) .and. reslist(i)%na(2) == res%na(2)) .or.  &
                (reslist(i)%na(1) == res%na(2) .and. reslist(i)%na(2) == res%na(1)))then
               pos = i
               exit
           endif
        enddo

      case (RESANGLE)
        do i=1,nrs
           if (res%code /= reslist(i)%code) cycle
           if (reslist(i)%na(2) == res%na(2)) then
               if ((reslist(i)%na(1) == res%na(1) .and. reslist(i)%na(3) == res%na(3)) .or.  &
                    (reslist(i)%na(1) == res%na(3) .and. reslist(i)%na(3) == res%na(1)))then
                   pos = i
                   exit
               endif
            endif
        enddo

      case (RESPLANE)
        np1 = size(res%na)
        do i=1,nrs
           if (res%code /= reslist(i)%code) cycle
           np2 = size(reslist(i)%na)
           if (np1 == np2) then     ! se hanno lo stesso numero di atomi
               !allocate(iord1(np1))
               call reallocate(iord1,np1)
               call indexx(res%na,iord1)
               !allocate(iord2(np2))
               call reallocate(iord2,np2)
               call indexx(reslist(i)%na,iord2)
               if (all(reslist(i)%na(iord2) == res%na(iord1))) then  ! se hanno gli stessi atomi
                   pos = i
                   exit
               endif
           endif
        enddo

   end select
!
   end function restraint_position

!--------------------------------------------------------------------------

   subroutine stampa_restraints(kpr,restr,atom)
   USE atom_type_util
   integer, intent(in)                                         :: kpr
   type(restraint_type), allocatable, dimension(:), intent(in) :: restr
   type(atom_type), dimension(:), intent(in)                   :: atom
!
   if (nrestraints(restr) > 0) then
       call print_restraints(kpr,restr,RESDIST,atom)
       call print_restraints(kpr,restr,RESANGLE,atom)
       call print_restraints(kpr,restr,RESPLANE,atom)
   endif
!
   end subroutine stampa_restraints

!--------------------------------------------------------------------------

   subroutine print_restraints(kpr,restr,code,atom)
   USE atom_type_util
   USE strutil
   integer, intent(in)                                         :: kpr
   type(restraint_type), allocatable, dimension(:), intent(in) :: restr
   integer, intent(in)                                         :: code
   type(atom_type), dimension(:), intent(in)                   :: atom
   real                                                        :: diff,distadif,angledif,dista
   integer                                                     :: i
   character(len=200)                                          :: string
   integer                                                     :: n1,n2,n3,nrescode
!
   nrescode = count(restr(:)%code==code)
   if (nrescode == 0) return
!   
!  Title
   select case (code)
      case (RESDIST) 
       distadif = 0
       write(kpr,'(/a)')centra_str('Restraint(s) on distance',80)
       write(kpr,'(a)')'   Num.    Atoms              Observed  Expected       Weight'
      case (RESANGLE)
       angledif = 0
       write(kpr,'(/a)')centra_str('Restraint(s) on angle',80)
       write(kpr,'(a)')'   Num.    Atoms              Observed  Expected Angle            Weight'
      case (RESPLANE)
       write(kpr,'(/a)')centra_str('Restraint(s) on plane',80)
       write(kpr,'(a)')'   Num.  Distance      Weight              Atoms'
      case (ABUMP)
       write(kpr,'(/a)')centra_str('Anti-bump restraint(s)',80)
       write(kpr,'(a)')'   Num.    Atoms              Observed  Expected Distance    Standard Deviation'
   end select
!
   do i=1,nrestraints(restr)
      if (restr(i)%code == code) then
          select case (restr(i)%code)
            case (RESDIST) 
              n1 = restr(i)%na(1)
              n2 = restr(i)%na(2)
              diff = restr(i)%val - restr(i)%targ
              write(kpr,10) i,atom(n1)%lab,atom(n2)%lab,restr(i)%val,restr(i)%targ,restr(i)%wei   !!!!!wqq(1)
              10 format(2x,i3,')',2x,a8,2x,a8,2x,f8.3,2x,f8.3,2x,es15.3)
              distadif = distadif + abs(diff)

            case (RESANGLE)
              n1 = restr(i)%na(1)
              n2 = restr(i)%na(2)
              n3 = restr(i)%na(3)
              diff = restr(i)%val - restr(i)%targ
              write(kpr,20) i,atom(n1)%lab,atom(n2)%lab,atom(n3)%lab,restr(i)%val,restr(i)%targ,restr(i)%wei  !!!!!diff,wqq(2)
              !20 format(2x,i3,')',1x,a8,1x,a8,1x,a8,1x,f8.3,2x,f8.3,2x,f8.3,2x,es15.3)
              20 format(2x,i3,')',1x,a8,1x,a8,1x,a8,1x,f8.3,2x,f8.3,2x,es15.3)
              angledif = angledif + abs(diff)

            case (RESPLANE)
              dista = mdistance_lsqplane(atom(restr(i)%na))
              write(string,'(2x,i3,")",1x,f8.3,2x,f13.3)') i,dista,restr(i)%wei  !!!!!wqq(3)
              call write_svet(atom(restr(i)%na)%lab,size(restr(i)%na),kpr,string(:len_trim(string)+1))

            case (ABUMP)
              n1 = abs(restr(i)%na(1))
              n2 = restr(i)%na(2)
              write(kpr,'(2x,i5,")",2x,a8,2x,a8,2x,f8.3,5x,f8.3,10x,f8.3)') i,atom(n1)%lab,atom(n2)%lab,   &
                    restr(i)%val,restr(i)%targ,restr(i)%sigma
          end select
      endif
   enddo
!
   select case (code)
      case (RESDIST) 
          if (nrescode > 1) write(kpr,'(2x,a,f0.4)')'Average of differences on distances = ',distadif/nrescode
      case (RESANGLE)
          if (nrescode > 1) write(kpr,'(2x,a,f0.4)')'Average of differences on angles = ',angledif/nrescode
      case (RESPLANE)
      case (ABUMP)
   end select
!
   end subroutine print_restraints

!--------------------------------------------------------------------------

   subroutine read_restraints_from_file(filename,reslist)
!
!  Legge restraints da file esterno
!
   USE General, only: ERR_WINDOW,jhost
   USE strutil
   USE variables, only:atm
   USE Atomi, only:nats
   character(len=*), intent(in)                   :: filename
   type(restraint_type), dimension(:), allocatable, intent(inout) :: reslist
   type(restraint_type)                              :: res
   integer                                        :: jopen
   integer                                        :: nExitCode
   integer                                        :: ier
   character(len=500)                             :: line,line1,word
   integer                                        :: nlongl,nlongw
   real, dimension(100)                           :: vet
   integer, dimension(100)                        :: ivet
   integer                                        :: iv
!
   if (jopen(jhost,filename,2,1,ier) == 0) then
       call reset_restraints(reslist)
       do  
         read(jhost,'(a)',iostat=ier)line
         if (ier < 0) exit
         call s_filter(line)
         line1 = line
         call Cutst(line1,nlongl,word,nlongw)
         nlongw = min(4,nlongw)     ! bastano 4 caratteri per individuare la direttiva
         word = upper(word)
         select case(word(:nlongw))

            case ('WEID')       ! peso sulle distanze
              call Getnum(line1(:nlongl),vet,ivet,iv)
              if (iv > 0) wqq(1) = vet(1)

            case ('WEIA')       ! peso sugli angoli
              call Getnum(line1(:nlongl),vet,ivet,iv)
              if (iv > 0) wqq(2) = vet(1)

            case ('WEIP')       ! peso sui piani
              call Getnum(line1(:nlongl),vet,ivet,iv)
              if (iv > 0) wqq(3) = vet(1)

            case default        ! leggi restraints
              call get_restraint_from_string(line,atm(:nats),res,ier)
              if (ier == 0) then
                  call set_value_for_restraint(res,atm(:nats))    ! assegna valore corrente
                  call add_restraint_to_list(reslist,res)
              endif

         end select
       enddo

       close (jhost)
   else
       call MsgWinErr('ERROR','Open error for file '//trim(filename),  &
       ERR_WINDOW,nExitCode)
   endif
!
   end subroutine read_restraints_from_file

!--------------------------------------------------------------------------

   subroutine write_restraints_on_file(filename,reslist,atom,weir)
!
!  Scrive i restraints su un file esterno
!
   USE progtype
   USE General, only: jhost
        USE strutil
   character(len=*), intent(in)                                :: filename
   type(restraint_type), dimension(:), allocatable, intent(in) :: reslist
   type(atom_type), dimension(:), intent(in)                   :: atom
   real, dimension(3), intent(in), optional                    :: weir
   integer                                                     :: jopen
   integer                                                     :: i
   integer                                                     :: ier
   !character(len=100)                                          :: formt
   integer                                                     :: nap
   integer                                                     :: nrs
!
   if (jopen(jhost,filename,0,1,ier) < 0) return
!
   nrs = nrestraints(reslist)
   if (nrs > 0) then
       do i=1,nrs
         select case(reslist(i)%code)
             case (1)
               write(jhost,'(1x,2(a,2x),2f12.3)')trim(atom(reslist(i)%na(1))%lab), &
                    trim(atom(reslist(i)%na(2))%lab),reslist(i)%targ,reslist(i)%sigma

             case (2)
               write(jhost,'(1x,3(a,2x),2f12.3)')trim(atom(reslist(i)%na(1))%lab), &
                    trim(atom(reslist(i)%na(2))%lab),trim(atom(reslist(i)%na(3))%lab),  &
                    reslist(i)%targ,reslist(i)%sigma

             case (3)
               nap = size(reslist(i)%na)
               write(jhost,'(1x,a,2x,f12.3)')trim(catsvector2(atom(reslist(i)%na)%lab,' ')),reslist(i)%sigma
               !write(formt,'(a,i0,a)')"(1x,",nap,"(a,2x),f12.3)"
               ! non piace al gfortran
               !write(jhost,formt)(/(trim(atom(reslist(i)%na(j))%lab),j=1,nap)/),reslist(i)%sigma

         end select
       enddo
       if (present(weir)) then 
          ! write(jhost,'(a,3f20.6)')'weight  ',weir(:)
           if (any(reslist(:)%code==1))write(jhost,'(a,f20.5)')'weid ',weir(1)
           if (any(reslist(:)%code==2))write(jhost,'(a,f20.5)')'weia ',weir(2)
           if (any(reslist(:)%code==3))write(jhost,'(a,f20.5)')'weip ',weir(3)
       endif
   else
       write(jhost,*)   !svuota il file
   endif
!
   close(jhost)
!
   end subroutine write_restraints_on_file

!--------------------------------------------------------------------------

   subroutine get_restraint_from_string(string,atom,res,ier,sigd,weid)
!
!  Legge il restraint dalla stringa del tipo:
!    'atom1 atom2 target   esd'            for distance  (target and esd optional)
!    'atom1 atom2 atom3 target   esd'      for angle     (target and esd optional)
!    'atom1 atom2 atom3 atom4 ....  esd'   for plane   
!
   USE strutil
   USE atom_type_util
   character(len=*), intent(in)      :: string
   type(atom_type), dimension(:), intent(in) :: atom
   type(restraint_type), intent(out) :: res
   integer, intent(out)              :: ier
   real, optional                    :: sigd, weid
   character(len=len_trim(string))   :: line,word     
   integer                           :: nlongl,nlongw
   integer                           :: natom
   logical                           :: lval
   real                              :: value1     !!!!!,value2
   integer, dimension(100)           :: vpos
   real, parameter                   :: SIGDIST=0.01, SIGANG=2.0, SIGPLANE=0.1
   logical                           :: is_esd, is_wei
   real, dimension(20) :: vet
   integer :: iv
   real :: sigdis
!
   if (present(sigd)) then
       sigdis = sigd
   else
       sigdis = SIGDIST
   endif
   ier = 0
   line = trim(adjustl(string))
   call s_filter(line)
   if (len_trim(line) /= 0) then
       natom = 0
       lval = .false.
       is_esd = .false.
       is_wei = .false.
!
!      leggi gli atomi
       do 
           call Cutst(line,nlongl,word,nlongw)
           if (nlongw == 0) exit
           call s_is_r(word,value1,lval)  ! controlla se si tratta di numero e leggi target
           if (lval) exit
           natom = natom + 1
           vpos(natom) = position_stringv(word,atom(:)%lab)
           if (vpos(natom) == 0) then
               ier = 1
               exit
           endif
       enddo
       if (natom > 1 .and. ier == 0) then
           if (lval) then
!
!              leggi esd and weight
               if (natom <= 3) then  ! se distanza o angolo
                   call Getnum(line,vet,iv=iv)
                   select case (iv)
                     case (1)
                      is_esd = .true.
                      res%sigma = vet(1)
                     case (2)
                      is_esd = .true.
                      res%sigma = vet(1)
                      is_wei = .true.
                      res%wei = vet(2)
                   end select
               endif
           else
!
!              If distance is absent look up in the table and set default for esd
               if (natom == 2) then 
                   value1 = bond_distance(atom(vpos(1)),atom(vpos(2)))
               elseif (natom == 3) then
                   value1 = 110.0
               else
                   value1 = SIGPLANE
               endif
           endif
       else
           ier = 1
       endif
   else
       ier = 1
   endif
   if (ier == 0) then
       allocate(res%na(natom))
       res%na(:) = vpos(:natom)
       select case (natom)
         case (2)
            res%code = natom - 1
            res%targ = value1
            if (.not.is_esd)  res%sigma = sigdis
            if (present(weid) .and. .not.is_wei) res%wei = weid
         case (3)
            res%code = natom - 1
            res%targ = value1
            if (.not.is_esd)  res%sigma = SIGANG
         case (4:)
            res%code = 3
            res%sigma = value1
       end select
!corr      write(0,*)'res=',res%code,res%na(:natom),res%targ,res%sigma,res%wei
   endif
!
   end subroutine get_restraint_from_string

!--------------------------------------------------------------------------

   subroutine set_equivalent_for_angle(resv,legm)
!
!  For angle n1-n2-n3 assign value 1 to atom 
!  for which is not necessary compute equivalent
!
   USE connect_mod
   type(restraint_type), dimension(:), allocatable, intent(inout) :: resv
   type(bond_type), dimension(:), allocatable, intent(in)         :: legm
   integer                                                        :: i
   integer                                                        :: numrestr
!
   numrestr = nrestraints(resv)
   if (numrestr == 0) return
   do i=1,numrestr
      if (resv(i)%code == RESANGLE) then  ! res. on angle?
          resv(i)%sym(:) = [1,0,1]
          if (bond_position(legm,resv(i)%na(1),resv(i)%na(2)) > 0) then
              resv(i)%sym(1) = 0
     !         resv(i)%sym(2) = 0
          endif
          if (bond_position(legm,resv(i)%na(2),resv(i)%na(3)) > 0) then
     !         resv(i)%sym(2) = 0
              resv(i)%sym(3) = 0
          endif
!
!         if all atoms in restraints are not bound, fix the central atoms
     !     if (all(resv(i)%sym(:) == 1)) resv(i)%sym(2) = 0
          write(0,*)'res n.',i,'SYM=',resv(i)%sym(:)
      endif
   enddo
!
   end subroutine set_equivalent_for_angle

!--------------------------------------------------------------------------

   subroutine set_value_for_restraint(res,atom)
!
!  Calcola valore corrente per il restraint
!
   USE General, only: xgg,rtod
   USE progtype
   USE cgeom
   type(restraint_type), intent(inout)       :: res
   type(atom_type), dimension(:), intent(in) :: atom
!
   select case (res%code)
      case (1)
        res%val = distanzaC(atom(res%na(1))%xc,atom(res%na(2))%xc,xgg)

      case (2)
        res%val = rtod*angleC(atom(res%na(1))%xc,atom(res%na(2))%xc,atom(res%na(3))%xc,xgg)

      case (3)
        res%val = mdistance_lsqplane(atom(res%na))

   end select
!
   end subroutine set_value_for_restraint

!--------------------------------------------------------------------------

   subroutine set_weight(res,code,wei)
!
!  Assign weight to all restraints of type code
!
   type(restraint_type), dimension(:), allocatable, intent(inout)  :: res
   integer, intent(in)                                             :: code
   real, intent(in)                                                :: wei
   integer                                                         :: i
!
   do i=1,nrestraints(res)
      if (res(i)%code == code) res(i)%wei = wei
   enddo 
!
   end subroutine set_weight

!------------------------------------------------------------------------

   recursive subroutine conn_to_restraints(legm,atom,code,reslist)
!
!  Converte tutte le distanze e/o angoli in restraints
!  code = 1 (distanze), code = 2 (angoli), code = 3 (angoli+distanze)
!
   USE progtype
   USE connect_mod
   USE atom_type_util
   USE bondtmod
   USE variables, only:ortom
   type(bond_type), dimension(:), allocatable, intent(in) :: legm
   type(atom_type), dimension(:), intent(in)        :: atom
   type(atom_type), dimension(size(atom))           :: atomcart
   integer, intent(in)                              :: code
   type(restraint_type), dimension(:), allocatable  :: reslist
   type(connection_type), dimension(:), allocatable :: connt
   type(angle_type), dimension(:), allocatable      :: angle
   integer                                          :: natom
   integer                                          :: nleg,nang
   integer                                          :: i
   real                                             :: rtarget
   real                                             :: diffa,diffmina
!
   nleg = numbonds(legm)
   if (nleg == 0) return
   natom = size(atom)
   select case(code)
       case (1)
         atomcart(:) = atom
         call frac_to_cart(atomcart,ortom)
         do i=1,nleg
!
!           Ipotizza una distanza target
            call get_info_distance(atomcart(legm(i)%n1),atomcart(legm(i)%n2),dist=rtarget)
!!!!!prova
             !rtarget = legm(i)%dist
!!!!!prova
            !call set_restr(reslist,1,(/legm(i)%n1,legm(i)%n2/),rtarget,legm(i)%sigma,legm(i)%dist)
            call set_restr(reslist,1,(/legm(i)%n1,legm(i)%n2/),rtarget,0.01,legm(i)%dist)
         enddo

       case (2)
         call legm_to_connect(natom,legm,connt)
         call connect_to_ang(connt,atom,angle,nang,.true.)
         do i=1,nang
            rtarget = 110.0
            diffmina = abs (angle(i)%val - 110.0) 
            diffa = abs (angle(i)%val - 120.0) 
            if (diffa < diffmina) rtarget = 120.0
!!!!!prova
             !rtarget = angle(i)%val
!!!!!prova
            call set_restr(reslist,2,(/angle(i)%n1,angle(i)%n2,angle(i)%n3/),rtarget,angle(i)%sigma,angle(i)%val)
         enddo

       case (3)
         call conn_to_restraints(legm,atom,1,reslist)
         call conn_to_restraints(legm,atom,2,reslist)
                                 !call stampa_restraints(6,reslist)

   end select
!
   end subroutine conn_to_restraints

!--------------------------------------------------------------------------
 
   subroutine update_restraints(reslist,rmvet)
!
!  Aggiorna la lista dei restraints se ho eliminato degli atomi
!  rmvet = nuovo numero d'ordine degli atomi, vale 0 per atomi eliminati
!
   type(restraint_type), dimension(:), allocatable, intent(inout) :: reslist
   integer, dimension(:), intent(in)                              :: rmvet     
   integer                                                        :: i
   integer                                                        :: nat
!
   nat = size(rmvet)
!
!  Elimina i restraints nei quali sono coinvolti atomi eliminati
   do i=1,nat
      if (rmvet(i) == 0) then
      endif
   enddo
!
   end subroutine update_restraints

!--------------------------------------------------------------------------

   integer function nconstraints(constr)
   type(constraint_type), dimension(:), allocatable, intent(in) :: constr
!   
   if (allocated(constr)) then
       nconstraints = size(constr)
   else
       nconstraints = 0
   endif
!
   end function nconstraints

!--------------------------------------------------------------------------

   subroutine reset_constraints(constr)
!
!  Azzera constraints
!
   type(constraint_type), dimension(:), allocatable, intent(inout) :: constr
   if (allocated(constr)) deallocate(constr)
!
   end subroutine reset_constraints

!--------------------------------------------------------------------------
 
   subroutine set_restr(reslist,code,atr,targ,sig,val)
!
!  Definisce il restraint e lo aggiunge alla lista 
!
   USE realloc
   type(restraint_type), dimension(:), allocatable, intent(inout) :: reslist
   integer, intent(in)               :: code  ! codice restraints
   integer, dimension(:), intent(in) :: atr   ! n. d'ordine degli atomi coinvolti
   real, intent(in)                  :: targ  ! valore target
   real, intent(in)                  :: sig   ! sigma
   real, intent(in), optional        :: val   ! valore corrente
   integer                           :: numres
!
   numres = nrestraints(reslist) + 1
   call reallocate_restr(reslist,numres)    
   if (present(val)) then
       reslist(numres) = restraint_type(code,atr,val,targ,sig)
   else
       reslist(numres) = restraint_type(code,atr,0.0,targ,sig)
   endif
!
   end subroutine set_restr

!--------------------------------------------------------------------------

   subroutine save_restraints(unitbin,res)
!
!  Write restraints on binary file
!
   integer, intent(in)                                         :: unitbin
   type(restraint_type), dimension(:), allocatable, intent(in) :: res
   integer                                                     :: nres
   integer                                                     :: i
   nres = nrestraints(res)
   write(unitbin) nres
   do i=1,nres
      write(unitbin)size(res(i)%na)
      write(unitbin)res(i)%code,res(i)%na,res(i)%val,res(i)%targ,res(i)%sigma
   enddo
   end subroutine save_restraints

!--------------------------------------------------------------------------

   subroutine read_restraints(unitbin,res,err)
!
!  Read restraints from binary file
!
   USE errormod
   USE realloc
   integer, intent(in)                                            :: unitbin
   type(restraint_type), dimension(:), allocatable, intent(inout) :: res
   type(error_type), intent(out)                                  :: err
   integer                                                        :: nres
   integer                                                        :: ier
   integer                                                        :: natres
   integer                                                        :: i
!
   read(unitbin,iostat=ier,err=10)nres
   if (nres > 0 .and. ier == 0) then
       call reallocate_restr(res,nres)
       do i=1,nres
          read(unitbin,iostat=ier,err=10)natres
          call reallocate(res(i)%na,natres)
          read(unitbin,iostat=ier,err=10)res(i)%code,res(i)%na,res(i)%val,res(i)%targ,res(i)%sigma
       enddo
   endif
!
10 continue
   if (ier /= 0) then
       call err%set('Error on reading restraints information')
   endif
!
   end subroutine read_restraints

!--------------------------------------------------------------------------
 
   subroutine add_constraint(constr,code,vpar,vat,atom,coefi,npari,typec)
!
!  Aggiunge un nuovo constraint alla lista
!
   USE progtype
   type(constraint_type), dimension(:), allocatable, intent(inout) :: constr
   integer, intent(in)                                             :: code  ! codice constraint
   integer, dimension(:), intent(in)                               :: vpar  ! n. d'ordine dei parametri coinvolti
   integer, dimension(:), intent(in)                               :: vat   ! n. d'ordine degli atomi coinvolti
   type(atom_type), dimension(:), intent(inout)                    :: atom
   real, dimension(:), intent(in), optional                        :: coefi ! coefficienti
   integer, intent(in)                                             :: npari ! n. di parametri indipendenti
   integer, intent(in), optional                                   :: typec
   integer                                                         :: typecc
   real, dimension(size(vat))                                      :: coef
   integer                                                         :: nconstr
!
   if (present(coefi)) then
       coef(:) = coefi(:)
   else
       coef(:) = 1.0
   endif
!
   if (present(typec)) then
       typecc = typec
   else
       typecc = 2
   endif
!  
   nconstr = nconstraints(constr) + 1
   call reallocate_constr(constr,nconstr)    
   constr(nconstr) = constraint_type(code,vpar,vat,coef,size(vpar),npari,0,typecc) 
   call refine_constraint(constr(nconstr),atom,0)  ! azzera codice di affinamento e codice degli atomi
!
   end subroutine add_constraint

!--------------------------------------------------------------

   subroutine add_constraint_on_biso(atom,legm,elem,constr)
!
!  Imponi che atomi della stessa specia abbiano lo stesso fattore termico
!
   USE atom_type_util
   USE elements
   USE connect_mod
   USE contacts
!
   implicit none
   type(atom_type), dimension(:), intent(inout)                    :: atom
   type(bond_type), dimension(:), allocatable, intent(in)          :: legm
   type(element_type), dimension(:), allocatable                   :: elem
   type(constraint_type), dimension(:), allocatable, intent(inout) :: constr
   integer                                                         :: i,j
   integer, dimension(size(atom))                                  :: vats    
   integer                                                         :: nspec
   type(connection_type), dimension(:), allocatable :: connH
   integer :: nat
!corr   real, dimension(1) :: coef
!
   do i=1,numelem(elem)
      call get_atoms_of_specie(elem(i)%ptab,atom,vats,nspec)
      if (nspec > 1) then
          if (elem(i)%z > H_at) then
              call add_constraint(constr,1,(/(4,j=1,nspec)/),vats(:nspec),atom,npari=1,typec=2)
          endif
      endif
      !write(0,*)nspec,'VETS=',vats(:nspec)
   enddo
!
!!!!!FIXME - non affinare isolated hydrogens
   if (is_hydrogen(atom)) then
       nat = size(atom)
       call get_conn_hydrogens(connH,atom,nat,legm)
       call print_connect(atom%lab,connt=connH,kpri=0)
       do i=1,nat
          if (connH(i)%nat > 0) then
              call add_constraint(constr,4,(/(4,j=1,connH(i)%nat+1)/),[i,connH(i)%pos],    &
                   atom,(/(1.2,j=1,connH(i)%nat+1)/),npari=1,typec=2)
!corr   subroutine add_constraint(constr,code,vpar,vat,atom,coefi,npari,typec)
          endif
       enddo
   endif
!
   end subroutine add_constraint_on_biso

!--------------------------------------------------------------------------------------------------

   real function constr_aveb(constr,atom,bmin,bmax)  result(bmed)
!
!  Compute an average b for atom in constr
!
   USE atom_type_util
   type(constraint_type), intent(in)         :: constr
   type(atom_type), dimension(:), intent(in) :: atom
   real, intent(in)                          :: bmin,bmax
!
   bmed = sum(atom(constr%vat)%biso) / constr%npar
   if (bmed < bmin) bmed = bmin
   if (bmed > bmax) bmed = bmax
!
   end function constr_aveb
   
!--------------------------------------------------------------------------------------------------

   integer function par_in_constraint(constr,kat,npar) result(kconstr)
!
!  Controlla se un atomo e' implicato in un constraint
!
   type(constraint_type), dimension(:), allocatable, intent(in) :: constr
   integer, intent(in)                                          :: kat   ! n. dell'atomo
   integer, intent(in)                                          :: npar  ! n. del parametro
   integer                                                      :: i,j
!
   kconstr = 0
   loop_constr: do i=1,nconstraints(constr)     
      loop_par: do j=1,constr(i)%npar  
         if (constr(i)%vpar(j) == npar .and. constr(i)%vat(j) == kat) then
             kconstr = i
             exit loop_constr
         endif
      enddo loop_par
   enddo loop_constr
!
   end function par_in_constraint

!--------------------------------------------------------------------------------------------------

!corr   real function coef_of_constraint(kconstr,nat,npar) result(coef)
   real function coef_of_constraint(constr,nat,npar) result(coef)
!
!  Prende il coefficente del parametro npar del constraint
!
!corr   integer, intent(in) :: kconstr  !numero d'ordine del constraint
   type(constraint_type), intent(in) :: constr
   integer, intent(in) :: nat      !l'atomo
   integer, intent(in) :: npar     !il parametro
   integer             :: i
!
   coef = 0.0
   do i=1,constr%npar
      if (constr%vpar(i)==npar .and. constr%vat(i) == nat) then
          coef = constr%coef(i)
      endif
   enddo
!
   end function coef_of_constraint

!--------------------------------------------------------------------------------------------------

   subroutine refine_constraint(constr,atom,code)
!
!  Chiedo di affinare (code=1) o non affinare (code=0) un constraint
!
   USE progtype
   type(constraint_type), intent(inout)         :: constr
   type(atom_type), dimension(:), intent(inout) :: atom
   integer, intent(in)                          :: code
   integer                                      :: i
   integer                                      :: kat,kpar
!
   constr%ref = code
!
!  se affino il constraint tutti i parametri coinvolti nel constr. vanno fissati 
   do i=1,constr%npar
      kat=constr%vat(i)
      kpar=constr%vpar(i)
      atom(kat)%rcod(kpar) = 0
   enddo
!
   end subroutine refine_constraint

!--------------------------------------------------------------------------------------------------

   subroutine delete_constraint(constr,code,typec)
!
!  Elimina tutti i constrain di codice uguale a code
!
   type(constraint_type), dimension(:), allocatable, intent(inout) :: constr
   integer, intent(in)                       :: code
   integer, intent(in), optional             :: typec
   integer                                   :: nconstrn
   integer                                   :: i
!
   if (nconstraints(constr) == 0) return
!
   nconstrn = 0
   if (present(typec)) then
       do i=1,nconstraints(constr)
          if (constr(i)%code /= code .or. constr(i)%typec /= typec) then
              nconstrn = nconstrn + 1
              constr(nconstrn) = constr(i)
          endif 
       enddo
   else
       do i=1,nconstraints(constr)
          if (constr(i)%code /= code) then
              nconstrn = nconstrn + 1
              constr(nconstrn) = constr(i)
          endif 
       enddo
   endif
   call reallocate_constr(constr,nconstrn)
   !   write(0,*)nconstrn,'size constrv=',size(constrv)
   !do i=1,nconstrn
   !   constrv(i) = constrvn(i)
   !enddo
   !if (nconstrn > 0) constrv(:nconstrn) = constrvn(:nconstrn)
!corr   nconstr = nconstrn
!
   end subroutine delete_constraint

!--------------------------------------------------------------------------------------------------

   subroutine reallocate_restr(vetr,n,savevet)
!
!  Rialloca ad n un vettore di tipo restraint
!  Se savevet = .true. o non esiste si salva il suo contenuto.
!
   type(restraint_type), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)                         :: n
   logical, optional, intent(in)               :: savevet
   logical                                     :: savev
   integer                                     :: nv
   type(restraint_type), allocatable           :: vsav(:)
   integer                                     :: nsav
!
!  se n = 0 (riallocazione a 0): dealloca ed esci
   if (n == 0) then
       if (allocated(vetr)) deallocate(vetr)
       return
   endif
!
   if (.not.allocated(vetr)) then
       allocate(vetr(n))
   else
!
       nv = size(vetr)
       if (present(savevet)) then
           savev = savevet
       else
           savev = .true.
       endif
!
       if (savev) then
!
!          nsav contiene qual è la porzione di vetr da salvare
           select case(nv-n)
             case (1:)       ! compatta x ad n
               nsav = n
             case (:-1)      ! espandi x ad n
               nsav = nv
             case (0)
               return        ! n=nv non fare niente
           end select
!
!          salva vetr fino a nsav
           allocate(vsav(n))
           vsav(:nsav) = vetr(:nsav)
           call move_alloc(vsav,vetr)
       else
           if (nv /= n) then
               deallocate(vetr)
               allocate(vetr(n))
           endif
       endif
   endif
!
   end subroutine reallocate_restr

!--------------------------------------------------------------------------------------------------

   subroutine reallocate_constr(vetr,n,savevet)
!
!  Rialloca ad n un vettore di tipo restraint
!  Se savevet = .true. o non esiste si salva il suo contenuto.
!
   type(constraint_type), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)                              :: n
   logical, optional, intent(in)                    :: savevet
   logical                                          :: savev
   integer                                          :: nv
   type(constraint_type), allocatable                :: vsav(:)
   integer                                          :: nsav
!
!  se n = 0 (riallocazione a 0): dealloca ed esci
   if (n == 0) then
       if (allocated(vetr)) deallocate(vetr)
       return
   endif
!
   if (.not.allocated(vetr)) then
       allocate(vetr(n))
   else
!
       nv = size(vetr)
       if (present(savevet)) then
           savev = savevet
       else
           savev = .true.
       endif
!
       if (savev) then
!
!          nsav contiene qual è la porzione di vetr da salvare
           select case(nv-n)
             case (1:)       ! compatta x ad n
               nsav = n
             case (:-1)      ! espandi x ad n
               nsav = nv
             case (0)
               return        ! n=nv non fare niente
           end select
!
!          salva vetr fino a nsav
           allocate(vsav(n))
           vsav(:nsav) = vetr(:nsav)
           call move_alloc(vsav,vetr)
       else
           if (nv /= n) then
               deallocate(vetr)
               allocate(vetr(n))
           endif
       endif
   endif
!
   end subroutine reallocate_constr

!--------------------------------------------------------------------------------------------------

   subroutine calc_chi2res(reslist,npf)
!
!  Questa subroutine richiede che i restraints siano ordinati per classi (usa
!  ordina_res)
!
   USE GENLSQ,only:funcls
   type(restraint_type), dimension(:), intent(in) :: reslist
   integer, intent(in) :: npf          ! n di funzioni non di restraints 
   integer             :: i
   integer             :: posi,posf
   integer             :: nr
!
!  Calcola il contributi al chiquadro delle funzioni non di restraints 
   posi = 1
   posf = npf
   chi2func = dot_product(funcls(posi:posf),funcls(posi:posf))
!
!  Calcola il contributo al chiquadro per ogni classe di restraints
   do i=1,3
      nr = count(reslist(:)%code==i)
      if (nr == 0) then
          chi2res(i) = 0
      else
          posi = posf + 1
          posf = posf + nr
          chi2res(i) = dot_product(funcls(posi:posf),funcls(posi:posf))
      endif
   enddo
!
   end subroutine calc_chi2res

!--------------------------------------------------------------------------------------------------

   subroutine ordina_res(reslist)
!
!  Ordina i restraints per classe
!
   USE nr
   type(restraint_type), dimension(:), allocatable, intent(inout) :: reslist
   integer, dimension(size(reslist))                 :: iord
   integer                                           :: n
!
   n = nrestraints(reslist)  !!!!!size(reslist)
   if (n > 0) then
       call indexx(reslist%code,iord)
       reslist(:) = reslist(iord)
   endif
!
   end subroutine ordina_res

!--------------------------------------------------------------------------------------------------

   subroutine set_constr_on_position(constr)
!
!  Cerca atomi in posizione speciale generando nconstr constraints
!  e riempiendo  constrv da usare poi nel calcolo delle derivate dei constraints
!
   Use General
   Use Atomi
   Use Molcom
   USE variables
!
   implicit none
   type(constraint_type), dimension(:), allocatable, intent(inout) :: constr
   integer                  :: i,k,num
   real   , dimension(11)   :: xo,xn
   integer, dimension(10)   :: key
   integer                  :: ksp,kspecb,itype,iser,khead,iactn
   integer                  :: num_key_eq
   integer, dimension(3)    :: vat,vpar
   real, dimension(3)       :: vcoef
   integer                  :: npar
   integer                  :: kp
!
!  Non so se le seguenti  3 righe servono
   call xspinb()
   call inptf
!
   !!!!!call delete_constraint(code=2,typec=2)
   iactn = 2
   do i=1,nats
      xo(1:3)  = atm(i)%xc(:) 
      ksp=kspecb(newvet,xo,xn,key,itype,iser,khead,iactn)
      atm(i)%xc(:) = xn(1:3)
!
!     Se key(1:3) e' del tipo 1,2,3 e quindi la somma e' 6
!     l' atomo non e' in posizione speciale
      if (sum(key(1:3)) == 6) cycle
!
!     Se l'atomo e' in posizione speciale controllo se nel vettore key
!     c'e' la ripetizione dei numeri 1,2,3 (in valore assoluto)
!     num_key_eq = indica quante volte si ripete key
      num_key_eq = 0
      do num=1,3
         do k=1,3
            if (iabs(key(k)) == num) then
                num_key_eq = num_key_eq + 1
            elseif ( mod(iabs(key(k)),10) == num) then
                num_key_eq = num_key_eq + 1
                key(k) = sign(2,key(k))
            else
                key(k) = 0
            endif
         enddo
         if (num_key_eq > 1) then
             npar = 0
             do kp=1,3
                if (key(kp) /= 0) then
                    npar = npar + 1
                    vat(npar) = i
                    vpar(npar) = kp
                    vcoef(npar) = key(kp)
                endif
             enddo
             call add_constraint(constr,2,vpar(:npar),vat(:npar),atm(:nats),vcoef(:npar),npari=1,typec=2)
             exit
         endif
      enddo
   enddo
!
   end subroutine set_constr_on_position
!--------------------------------------------------------------
   subroutine set_constr_on_sof(atom,constr)
   USE atom_type_util
   USE General, only:cell
   USE unit_cell
   type(atom_type), dimension(:), intent(inout) :: atom
   type(constraint_type), dimension(:), allocatable, intent(inout) :: constr
   type(atom_type), dimension(size(atom))    :: atomc
   integer                                   :: nat
   integer, dimension(size(atom),size(atom)) :: softable
   integer, dimension(size(atom))            :: vet
   integer                                   :: i,j
   real, dimension(3)                        :: xdiff
   real                                      :: dist
   integer                                   :: npos,nposc
!
   nat = size(atom)   
   atomc(:) = atom(:)
   call frac_to_cart(atomc,orthomatrix(cell))
   softable(:,:) = 0
   vet(:) = 0
!
!  Riempi la tabella softable, usa vet per escudere gli atomi gia' presi
   do i=1,nat-1
      if (vet(i) == 1) cycle
      do j=i+1,nat
         if (vet(j) == 1) cycle
         xdiff(:) = atom(i)%xc - atom(j)%xc
         dist = sqrt(xdiff(1)**2+xdiff(2)**2+xdiff(3)**2)
         if (dist < 0.001) then
             softable(i,j) = 1
             vet(j) = 1
         endif
      enddo
   enddo
!
!  Aggiungi i constraints
   vet(:) = 0
   do i=1,nat
      nposc = sum(softable(i,:)) + 1
      if (nposc > 1) then
            !write(0,*)i,'sof=',softable(i,:)
!
!         In vet gli atomi che sono nella stessa posizione
          vet(1) = i
          npos = 1
          do j=1,nat
             if (softable(i,j) == 1) then
                 npos = npos + 1
                 vet(npos) = j
             endif
          enddo
!
!         constrain sull'occupanza
          call add_constraint(constr,3,(/(5,j=1,npos)/),vet(:npos),atom,npari=npos-1)
!
!         constrain sulla posizione
          if (atom(vet(1))%rcod(1) >= 0)call add_constraint(constr,2,(/(1,j=1,npos)/),vet(1:npos),atom,npari=1,typec=2)  ! sulle x
          if (atom(vet(1))%rcod(2) >= 0)call add_constraint(constr,2,(/(2,j=1,npos)/),vet(1:npos),atom,npari=1,typec=2)  ! sulle y
          if (atom(vet(1))%rcod(3) >= 0)call add_constraint(constr,2,(/(3,j=1,npos)/),vet(1:npos),atom,npari=1,typec=2)  ! sulle z
!
!         constrain sui b
          call add_constraint(constr,1,(/(4,j=1,npos)/),vet(:npos),atom,npari=1,typec=2)

!!!          write(0,*)'Atomi ',vet(:npos),' are in the same site'
      endif
   enddo
!
   end subroutine set_constr_on_sof

!--------------------------------------------------------------

   subroutine stampa_constraints(constr,atom,kpr,is_all)
   USE progtype
   USE strutil
   type(constraint_type), dimension(:), allocatable, intent(in) :: constr
   type(atom_type), dimension(:), intent(in)       :: atom
   integer, intent(in)                             :: kpr
   logical, intent(in), optional                   :: is_all ! stampa tutti i constraints
   integer                                         :: i,j
   integer, dimension(:), allocatable              :: vprint
   character(len=:), allocatable                   :: stringc
   integer                                         :: kat,kpar
   character(len=1), dimension(4) :: xyzstring = (/'x','y','z','B'/)
   integer                                         :: nconstr
!
   nconstr = nconstraints(constr)
   if (nconstr > 0) then
       allocate(vprint(nconstr))
       if (present(is_all)) then
           vprint(:) = 1
       else
           vprint(:) = constr(:)%ref ! stampa solo i constraints affinati
       endif
!
       if (any(vprint(:) > 0)) write(kpr,'(a)')centra_str('Contstraints on L.Sq. shifts',80)
   endif
!
   do i=1,nconstr
      if (vprint(i) > 0) then
          select case (constr(i)%code)

             case (1:2)
               stringc = ' '
               kat = constr(i)%vat(1)
               kpar = constr(i)%vpar(1)
               stringc = xyzstring(kpar)//'('//trim(atom(kat)%lab)//')'
               do j=2,size(constr(i)%vat)
                  kat = constr(i)%vat(j)
                  kpar = constr(i)%vpar(j)
                  stringc = trim(stringc)//' = '//xyzstring(kpar)//'('//trim(atom(kat)%lab)//')'
               enddo
               write(kpr,'(2x,a)')trim(stringc)

             case (3)
               stringc = ' '
               kat = constr(i)%vat(1)
               stringc = 'occ('//trim(atom(kat)%lab)//')'
               do j=2,size(constr(i)%vat)
                  kat = constr(i)%vat(j)
                  stringc = trim(stringc)//' + occ('//trim(atom(kat)%lab)//')'
               enddo
               stringc=trim(stringc)//' = 1'
               write(kpr,'(2x,a)')trim(stringc)
           
             case (4)
               write(kpr,'(2x,a)')'H = '//'B('//trim(atom(constr(i)%vat(1))%lab)//')'
          end select
      endif
   enddo
!
   end subroutine stampa_constraints

!--------------------------------------------------------------------------------------------------

 END MODULE RRESTR
